#pragma once

#include "CAFAna/Cuts.h"
#include "CAFAna/SpectrumLoaderBase.h"
#include "CAFAna/Spectrum.h"

class TDirectory;

namespace ana
{

  /// Standard interface to all decomposition techniques
  class IDecomp
  {
  public:
    virtual ~IDecomp() = default;
    virtual Spectrum NCComponent()       const = 0;
    virtual Spectrum NumuComponent()     const = 0;
    virtual Spectrum AntiNumuComponent() const = 0;
    virtual Spectrum NueComponent()      const = 0;
    virtual Spectrum AntiNueComponent()  const = 0;

    virtual void SaveTo(TDirectory* dir) const = 0;
  };

  /// Just return the ND truth spectra as the decomposition
  class BasicDecomp: public IDecomp
  {
  public:
    BasicDecomp(SpectrumLoaderBase& loader,
                std::string label,
                const Binning& bins,
                const Var& var,
                const Cut& cut = kNoCut,
                const Var& wei = kUnweighted);

    BasicDecomp(SpectrumLoaderBase& loader,
                const HistAxis& axis,
                const Cut& cut = kNoCut,
                const Var& wei = kUnweighted);

    Spectrum NCComponent()       const override {return fNC;}
    Spectrum NumuComponent()     const override {return fNumu;}
    Spectrum NueComponent()      const override {return fNue;}
    Spectrum AntiNumuComponent() const override {return fAntiNumu;}
    Spectrum AntiNueComponent()  const override {return fAntiNue;}

    void SaveTo(TDirectory* dir) const override;
    static std::unique_ptr<BasicDecomp> LoadFrom(TDirectory* dir);

  protected:
    BasicDecomp()
      : fNC   (0, 0),
        fNumu (0, 0),
        fNue  (0, 0),
        fAntiNumu(0, 0),
        fAntiNue (0, 0)
    {};

    Spectrum fNC;
    Spectrum fNumu;
    Spectrum fNue;
    Spectrum fAntiNumu;
    Spectrum fAntiNue;
  };
}
