#pragma once

#include <utility>
#include <vector>

#include "TTimeStamp.h"

class TGraph;

namespace ana
{
  /// Evolution of mass and beam power with time, calculation of exposure
  class ExposureAssumptions
  {
  public:
    ExposureAssumptions();

    void AddMassPt(TTimeStamp t, double m);
    void AddPowerPt(TTimeStamp t, double m);
    /// kton
    double MassAt(TTimeStamp t) const;
    /// kW
    double PowerAt(TTimeStamp t) const;
    /// kton*MW
    std::vector<std::pair<TTimeStamp, double>> PowerMassPts() const;
    /// kton*MW
    double PowerMassAt(TTimeStamp t) const;
    /// kton*MW*year
    std::vector<std::pair<TTimeStamp, double>> ExposurePts() const;
    /// kton*MW*year
    double ExposureAt(TTimeStamp t) const;

    /// Factor to scale a simulation with these parameters by to match the
    /// conditions at date \a t
    double ScaleFactor(TTimeStamp t,
                       double power,
                       double mass,
                       double years) const;

    TGraph* MassGraph() const;
    TGraph* PowerGraph() const;
    TGraph* PowerMassGraph() const;
    TGraph* ExposureGraph() const;

    void DrawMassPower() const;
    void DrawExposure() const;
    void DrawAll() const;
  protected:
    std::vector<std::pair<TTimeStamp, double>> fMassPts; ///< kton
    std::vector<std::pair<TTimeStamp, double>> fPowerPts; ///< kW

    mutable bool fDirtyPM, fDirtyExp;
    mutable std::vector<std::pair<TTimeStamp, double>> fPowerMassPts; ///< kton*MW
    mutable std::vector<std::pair<TTimeStamp, double>> fExposurePts;  ///< kton*MW*yr
  };

  /// Specific values for mass and beam power
  class NominalExposureAssumptions: public ExposureAssumptions
  {
  public:
    NominalExposureAssumptions();
  };
}
