#pragma once

class TTree;

namespace ana
{
  /// Represents the results of a single Feldman-Cousins pseudo-experiment
  class FCPoint
  {
  public:
    FCPoint(double tx, double ty, double tchi,
            double bx, double by, double bchi,
            unsigned int s)
      : fTrueX(tx), fTrueY(ty), fTrueChi(tchi),
        fBestX(bx), fBestY(by), fBestChi(bchi),
        fSeed(s)
    {
    }

    double TrueX() const {return fTrueX;}
    double TrueY() const {return fTrueY;}
    double Up() const {return fTrueChi-fBestChi;}

    bool operator<(const FCPoint& rhs) const{return Up() < rhs.Up();}

    static void InitToTree(TTree* tr);
    void ToTree(TTree* tr) const;

    static void InitFromTree(TTree* tr);
    static FCPoint FromTree(TTree* tr, int idx);
  protected:
    double fTrueX, fTrueY, fTrueChi;
    double fBestX, fBestY, fBestChi;
    unsigned int fSeed;
  };
} // namespace
