#pragma once

#include "CAFAna/SpectrumLoaderBase.h"

#include <set>

namespace ana
{
  /// \brief Create smaller CAFs
  ///
  /// This class can do two seperate things. It can cause non-selected
  /// variables to be NaN in the output file, causing it to compress very
  /// well. It can also cut out slices that fail a cut.
  class FileReducer: protected SpectrumLoaderBase
  {
  public:
    FileReducer(const std::string& wildcard, const std::string& outfile);
    FileReducer(const std::vector<std::string>& fnames,
                const std::string& outfile);
    virtual ~FileReducer();

    /// \brief Variables that will be filled in output file
    ///
    /// If you don't add any variables, everything will be copied.
    void AddRequiredVar(const Var& var);
    /// \brief Variables that will be filled in output file
    ///
    /// If you don't add any variables, everything will be copied.
    void AddRequiredCut(const Cut& cut);

    /// Only copy records to the output file if they pass this cut
    void AddEventCut(const Cut& cut);

    /// \brief If called, only events whose run/subrun/event occur in \a fname
    /// will be retained.
    void SetEventList(const std::string& fname);

    /// Set an optional parameter to omit bad files (zombie or no POT)
    void setOmitBadFiles(bool omit) {fOmitBadFiles = omit;}

    virtual void Go() override;

  protected:
    std::string fOutfile;
    Cut*        fEventCut;
    std::set<std::tuple<int, int, int>> fEventList;
    bool        fOmitBadFiles;
  };
}
