#pragma once

#include <memory>
#include <string>

#include "TFile.h"

class TDirectory;

namespace ana
{
  //----------------------------------------------------------------------
  // Most classes are happy to load themselves
  template<class T> std::unique_ptr<T> LoadFrom(TDirectory* dir)
  {
    return T::LoadFrom(dir);
  }

  //----------------------------------------------------------------------
  // But if you're trying to load a base class we need to figure out which
  // derived class is actually in the file and hand off to that
  class IDecomp;
  template<> std::unique_ptr<IDecomp> LoadFrom<IDecomp>(TDirectory* dir);
  class IExtrap;
  template<> std::unique_ptr<IExtrap> LoadFrom<IExtrap>(TDirectory* dir);
  class IPrediction;
  template<> std::unique_ptr<IPrediction> LoadFrom<IPrediction>(TDirectory* dir);
  class Experiment;
  template<> std::unique_ptr<Experiment> LoadFrom<Experiment>(TDirectory* dir);
  class SingleComponentExtrap;
  template<> std::unique_ptr<SingleComponentExtrap>
    LoadFrom<SingleComponentExtrap>(TDirectory* dir);

  //----------------------------------------------------------------------
  // For symmetry
  template<class T> void SaveTo(const T& x, TDirectory* dir)
  {
    x.SaveTo(dir);
  }

  //----------------------------------------------------------------------
  /// Helper for \ref LoadFromFile
  TDirectory* FileToDirectory(const std::string& fname,
                              const std::string& label);

  //----------------------------------------------------------------------
  template<class T> std::unique_ptr<T> LoadFromFile(const std::string& fname,
                                                    const std::string& label)
  {
    return LoadFrom<T>(FileToDirectory(fname, label));
  }

  //----------------------------------------------------------------------
  template<class T> void SaveToFile(const T& x,
                                    const std::string& fname,
                                    const std::string& label)
  {
    TFile fout(fname.c_str(), "RECREATE");
    x.SaveTo(fout.mkdir(label.c_str()));
  }
}
