#pragma once

#include "CAFAna/Surface.h"

class TPad;

namespace ana
{
  /// \brief Surface with axes \f$ \sin^2\theta_{23} \f$ and \f$ \Delta m^2_{32} \f$
  ///
  /// Two panels, for normal and inverted hierarchy.
  class NumuSurface
  {
  public:
    /// \param calc Values for oscillation parameters to be held fixed
    /// \param margVars Oscillation parameters to marginalize over
    /// \param margSysts Systematic parameters to marginalize over
    NumuSurface(const Experiment* expt,
		osc::IOscCalculatorAdjustable* calc,
		int nbinsx, double xmin, double xmax,
		int nbinsy, double ymin, double ymax,
		const std::vector<const FitVar*>& margVars = {},
		const std::vector<Syst>& margSysts = {});

    ~NumuSurface();

    /// Draw the surface itself
    void Draw() const;
    /// Draw the best fit point
    void DrawBestFit(Color_t color) const;
    double MinChi() const;
    /// \param style Line style for TAttLine, solid, dotted, dashed etc
    /// \param color Line color for TAttLine
    /// \param minchi \f$\chi^2\f$ of best fit to compare against.
    ///               Default: best fit from this surface.
    void DrawContour68(Style_t style, Color_t color, double minchi = -1);
    void DrawContour90(Style_t style, Color_t color, double minchi = -1);

    void SetTitle(const char* str);
  protected:
    void EnsurePad() const;

    void DrawContour(TH2* fcNorm, TH2* fcInv, Style_t style, Color_t color,
                     double minchi = -1);

    Surface *fSurfNorm, *fSurfInv;
    mutable TPad *fPadNorm, *fPadInv;
  };
}
