#pragma once

#include "CAFAna/ReweightableSpectrum.h"

#include "CAFAna/Binning.h"
#include "CAFAna/OscCurve.h"
#include "CAFAna/Spectrum.h"
#include "CAFAna/SpectrumLoaderBase.h"

#include <string>

class TH2;
class TH2D;

namespace osc{class IOscCalculator;}

namespace ana
{
  /// %Spectrum with true energy information, allowing it to be oscillated
  // Protected inheritance. Don't want to expose functions under the same names
  class OscillatableSpectrum: protected ReweightableSpectrum
  {
  public:
    friend class SpectrumLoader;
    friend class MRCCLoader;

    OscillatableSpectrum(std::string label,
                         const Binning& bins,
                         SpectrumLoaderBase& loader,
                         const Var& var,
                         const Cut& cut = kNoCut,
                         const Var& wei = kUnweighted);

    OscillatableSpectrum(SpectrumLoaderBase& loader,
                         const HistAxis& axis,
                         const Cut& cut = kNoCut,
                         const Var& wei = kUnweighted);

    OscillatableSpectrum(std::string label, const Binning& bins);
    OscillatableSpectrum(std::string label, double pot, const Binning& bins);
    OscillatableSpectrum(TH2* h, double pot, std::string varName = "");

    // Expose these ones directly
    using ReweightableSpectrum::Fill;
    using ReweightableSpectrum::ToTH2;
    using ReweightableSpectrum::Clear;

    // These under a different name
    Spectrum Unoscillated() const {return UnWeighted();}
    Spectrum TrueEnergy() const {return WeightingVariable();}

    Spectrum Oscillated(const OscCurve& curve) const;
    Spectrum Oscillated(osc::IOscCalculator* calc, int from, int to) const;

    /// Rescale bins so that \ref TrueEnergy will return \a target
    void ReweightToTrueSpectrum(const Spectrum& target);
    /// Recale bins so taht \ref Unoscillated will return \a target
    void ReweightToRecoSpectrum(const Spectrum& target);

    OscillatableSpectrum& operator+=(const OscillatableSpectrum& rhs);
    OscillatableSpectrum operator+(const OscillatableSpectrum& rhs) const;

    OscillatableSpectrum& operator-=(const OscillatableSpectrum& rhs);
    OscillatableSpectrum operator-(const OscillatableSpectrum& rhs) const;

    void SaveTo(TDirectory* dir) const;
    static std::unique_ptr<OscillatableSpectrum> LoadFrom(TDirectory* dir);
  };
}
