#pragma once

#include "CAFAna/Systs.h"

#include <vector>

class TLegend;

namespace osc{class IOscCalculator;}

namespace ana
{
  class IPrediction;
  class Spectrum;

  /// Overlay MC spectrum with data spectrum
  void DataMCComparison(const Spectrum& data, const Spectrum& mc);

  /// Overlay MC spectrum with data spectrum, area normalized
  void DataMCComparisonAreaNormalized(const Spectrum& data, const Spectrum& mc);

  /// Plot MC broken down into flavour components, overlayed with data
  void DataMCComparisonComponents(const Spectrum& data,
                                  const IPrediction* mc,
                                  osc::IOscCalculator* calc);

  /// Plot data/expected, compared with fit/expected
  void RatioPlot(const Spectrum& data,
                 const Spectrum& expected,
                 const Spectrum& fit);

  /// \brief Plot prediction with +/-1sigma error band
  ///
  /// When multiple systematics are used, the errors are the quadrature sum
  ///
  /// \param pred   The prediction. Must be able to generate shifted spectra
  /// \param systs  List of systematics to include in error band
  /// \param calc   Oscillation parameters
  /// \param pot    POT to evaluate prediction at
  /// \param col    Color of the prediction, default kRed
  /// \param errCol Color of the shading, default col-7 (kRed-7 is light red)
  void PlotWithSystErrorBand(IPrediction* pred,
                             const std::vector<Syst>& systs,
                             osc::IOscCalculator* calc,
                             double pot,
                             int col = -1, int errCol = -1);

  /// \brief Create a legend, maximizing distance from all histograms
  ///
  /// \param dx Width of the legend, fraction of pad width
  /// \param dy Height of the legend, fraction of pad height
  TLegend* AutoPlaceLegend(double dx, double dy);
}
