#pragma once

#include "CAFAna/Prediction.h"

namespace ana
{
  class IExtrap;

  /// Take the output of an extrapolation and oscillate it as required
  class PredictionExtrap: public IPrediction
  {
  public:
    PredictionExtrap(IExtrap* extrap,
                     SpectrumLoaderBase& loaderCosmic,
                     std::string label,
                     const Binning& bins,
                     const Var& var,
                     const Cut& cut = kNoCut,
                     const Var& wei = kUnweighted);

    PredictionExtrap(IExtrap* extrap,
                     SpectrumLoaderBase& loaderCosmic,
                     const HistAxis& axis,
                     const Cut& cut = kNoCut,
                     const Var& wei = kUnweighted);

    virtual Spectrum Predict(osc::IOscCalculator* calc) const override;

    virtual Spectrum PredictComponent(osc::IOscCalculator* calc,
                                      Flavors::Flavors_t flav,
                                      Current::Current_t curr,
                                      Sign::Sign_t sign) const override;
    virtual Spectrum PredictCosmicComponent() const override {return fCosmic;}

    virtual void SaveTo(TDirectory* dir) const;
    static std::unique_ptr<PredictionExtrap> LoadFrom(TDirectory* dir);
  protected:
    PredictionExtrap() : fCosmic(0, 0) {}

    IExtrap* fExtrap;
    Spectrum fCosmic;
  };
}
