#pragma once

#include "CAFAna/SpectrumLoaderBase.h"

namespace ana
{
  class Progress;
  class SystematicShifter;

  /// \brief Collaborates with \ref Spectrum and \ref OscillatableSpectrum to
  /// fill spectra from CAF files.
  ///
  /// The outside user should just pass a filename, wildcard, SAM dataset name,
  /// or query to the constructor. Then construct all the spectra you
  /// need. They will register with this loader. Finally, calling \ref Go will
  /// cause all the spectra to be filled at once. After this the loader may not
  /// be used again.
  class SpectrumLoader: public SpectrumLoaderBase
  {
  public:
    SpectrumLoader(const std::string& wildcard, DataSource src = kBeam);
    SpectrumLoader(const std::vector<std::string>& fnames,
                   DataSource src = kBeam);
    virtual ~SpectrumLoader();

    virtual void AddSpectrum(Spectrum& spect,
                             const Var& var,
                             const Cut& cut = kNoCut,
                             const Var& wei = kUnweighted) override;

    virtual void AddOscillatableSpectrum(OscillatableSpectrum& spect,
                                         const Var& var,
                                         const Cut& cut,
                                         const Var& wei) override;

    virtual void Go() override;

    /// \brief This shifter will be used for all subsequently added spectra
    ///
    /// Can alter between adding histograms. Earlier will have one, later the other.
    ///
    /// Call with zero to get unshifted (nominal) spectra
    void SetShifter(SystematicShifter* s);

  protected:
    virtual void HandleFile(const std::string& fname,
                            double& pot, double& livetime,
                            Progress* prog = 0);
    virtual void HandleRecord(caf::StandardRecord* sr);
    virtual void HandlePOT(double pot, double livetime);

    SystematicShifter* fShifter;
  };
}
