#pragma once

#include "CAFAna/SystShifts.h"

namespace caf{class StandardRecord;}
namespace cafrwgt{class CAFReweight;}

#include <map>
#include <set>

namespace ana
{
  /// \brief Apply systematic shifts to StandardRecord objects
  ///
  /// For use by SpectrumLoader
  class SystematicShifter
  {
  public:
    /// Shifts are 0=nominal, -1,+1 = 1 sigma shifts
    SystematicShifter(Syst syst, double shift);
    /// Shifts are 0=nominal, -1,+1 = 1 sigma shifts
    SystematicShifter(const std::map<Syst, double>& shifts);
    ~SystematicShifter();

    /// Modify \a sr, for reweightings, scale \a weight
    void Shift(caf::StandardRecord* sr, double& weight) const;
    /// Undo the most recent call to \ref Shift
    void Unshift(caf::StandardRecord* sr) const;

    /// Variables required to be loaded into the StandardRecord
    const std::set<std::string>& Requires() const {return fReqs;}
  protected:
    /// Helper function for constructors
    void SetShift(Syst syst, double shift);

    double GetSigma(Syst syst) const;

    void NormalizationShift(caf::StandardRecord* sr, double& weight) const;
    void CaloScaleShift(caf::StandardRecord* sr, double& weight) const;
    void NCBackgroundShift(caf::StandardRecord* sr, double& weight) const;
    void CCEShift(caf::StandardRecord* sr, double& weight) const;
    void QEEShift(caf::StandardRecord* sr, double& weight) const;
    void UCEShift(caf::StandardRecord* sr, double& weight) const;
    void GenieShiftsFromGenBranch(caf::StandardRecord* sr, double& weight) const;
    void GenieShiftsFromTable(caf::StandardRecord* sr, double& weight) const;


    SystShifts fShifts;
    bool fHaveGenie; // Are any of the systs GENIE systs?
    mutable cafrwgt::CAFReweight* fReweight;

    /// Backup of modified variables, for use by \ref Unshift
    mutable float fBakCalE, fBakCCE, fBakQEE, fBakUCE;

    std::set<std::string> fReqs;
  };
}
