////////////////////////////////////////////////////////////////////////
/// \brief   Definition of connection map class
/// \author  brebel@fnal.gov
/// \date    Sept. 2010
////////////////////////////////////////////////////////////////////////
#ifndef CMAP_H
#define CMAP_H

#include "DAQChannelMap/DAQChannelMap.h"

#include "art/Framework/Principal/Run.h"
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "fhiclcpp/ParameterSet.h"

namespace rawdata{class RawDigit;}


namespace cmap {
  
  /// Channel masks are defined in DAQChannelMap package
  
  class CMap {
  public:
    
    CMap(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg);
    ~CMap();
    
    daqchannelmap::DAQChannelMap* Map() const {
      return daqchannelmap::DAQChannelMap::getInstance(fDetID);
    }
    
    void reconfigure(fhicl::ParameterSet pset);
    void postBeginRun(art::Run const& run);
    
    unsigned short GetPlane(const rawdata::RawDigit* dig) const;
    unsigned short GetCell(const rawdata::RawDigit* dig)  const;
    
    int GetRun() const { return fRun; }
    
  public:
    ///
    /// For use only inside the input drivers which may or may not
    /// have seen this service properly configured by ART. Check is
    /// the service is configured for detector "detid". If yes, return true;
    /// if no return false. If force==true, force the configuration to match
    /// detid.
    /// 
    bool TestDetID(int detid, bool force=false);
    
  private:
    
    bool ChanToCellException(unsigned short board_id, 
			     unsigned short feb_id, unsigned short chan_id,
			     unsigned short& plane, unsigned short& cell);
    
    bool CellToChanException(unsigned short plane, unsigned short cell, 
			     unsigned short& dcm_id, unsigned short& feb_id, 
			     unsigned short& chan_id);
    
    
    
  private:
    int         fRun;     ///< current run number
    std::string fXMLFile; ///< xml file of the cmap table def
    int         fDetID;   ///< detector ID
    
  };
} // end namespace cmap

DECLARE_ART_SERVICE(cmap::CMap, LEGACY)
#endif // CMAP_H
////////////////////////////////////////////////////////////////////////
