////////////////////////////////////////////////////////////////////////
/// \file    EventSummary.h
/// \brief   Simple representation of event for LEM use
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_EVENTSUMMARY_H
#define LEM_EVENTSUMMARY_H

#include "LEM/func/LiteHit.h"

#include "TString.h"

#include <cstdlib>

class TH2;
class TTree;

namespace lem
{

  const int kVertexPlane = 64;
  const int kVertexCell = 128;

  /// Simple representation of event for %LEM use
  class EventSummary
  {
  public:
    EventSummary() {}
    EventSummary(const std::vector<LiteHit>& hs, int id, std::vector<int> pdgs);
    EventSummary(const EventSummary& evt);
    ~EventSummary();

    EventSummary& operator=(const EventSummary& evt);

    bool IsSig() const
    {
      return (abs(pdg) == 12 && ccnc == 0 && abs(origPdg) == 14);
    }

    std::string Description() const;
    void FillHists(TH2* h1, TH2* h2, bool flipEven = false, bool flipOdd = false, bool idxs = false) const;
    void SaveImage(TString fname, TString title = "") const;

    static void InitToTree(TTree* tr);
    void ToTree(TTree* tr) const;

    static void InitFromTree(TTree* tr);
    static EventSummary FromTree(TTree* tr, int evtIdx, bool enrich);

    int Length() const; ///< In planes

    unsigned short nhits;
    LiteHit* hits;

    double totalPE;
    double trueEVis;
    double y;
    int id;

    int run, subrun, event;
    int ccnc, pdg, origPdg;
    int mode;

    int trueVtxPlane, trueVtxCell, trueVtxCellOther;

    unsigned char nParts;
    int* partPdgs;

    double photL0, photL1;
    double photE0, photE1;

    bool enrich; ///< Was this event generated in a filtered-for-pi0 job?

    static double fgDecayRate;
    static double fgChargePower;

  protected:
    void Finalize();
  };
} // namespace

#endif
