////////////////////////////////////////////////////////////////////////
/// \file    FindMatches.h
/// \brief   Core of the LEM match-finding algorithm
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_FINDMATCHES_H
#define LEM_FINDMATCHES_H

#include "LEM/func/Match.h"
#include "LEM/func/Potential.h"

#include <set>
#include <vector>

namespace lem
{
  class Heads;

  /// Collection of Potential objects with odd and/or even view flipped
  struct FlippedPotentials
  {
    Potential V[2][2]; // [flipEven][flipOdd]
  };

  class EventSummary;
  class MatchableEvent;

  const unsigned int kMaxNumMatches = 1001;

  float CalcEnergy(const Potential& Va,
                   const MatchableEvent& a, const MatchableEvent& b);

  double FracChargeMatched(const EventSummary& a, const EventSummary& b,
                           bool flipEven, bool flipOdd);

  void FillPotential(const EventSummary& trial, Potential& V,
                     bool flipEven, bool flipOdd);

  void FillPotential(const EventSummary& trial, FlippedPotentials& V);

  std::vector<Match> FindMatches(const MatchableEvent& trial,
                                 unsigned int libSize,
                                 const MatchableEvent* lib,
                                 const MatchableEvent* libDownsample, int factor,
                                 const std::multiset<float>& already,
                                 unsigned int numMatches = kMaxNumMatches,
                                 int enrich = 2, // [no, only, both]
                                 const Heads* heads = 0,
                                 int headIdx = -1,
                                 bool flipHeadEven = false,
                                 bool flipHeadOdd = false,
                                 bool useDownsample = true);

} // namespace

#endif
