////////////////////////////////////////////////////////////////////////
/// \file    Match.h
/// \brief   Information about a LEM match
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_MATCH_H
#define LEM_MATCH_H

#include "TVector3.h"

namespace lem
{
  class MatchableEvent;

  /// Information about a %LEM match
  class Match
  {
  public:
    Match() : energy(-1e10), qfrac(-1), evt(0), flipEven(false), flipOdd(false), headSeqNo(-1) {}
    Match(float E, const MatchableEvent* e, bool even = false, bool odd = false, int headSeq = -1)
      : energy(E), evt(e), flipEven(even), flipOdd(odd), headSeqNo(headSeq) {}
    bool operator<(const Match& rhs) const {return energy < rhs.energy;}
    double energy;
    double qfrac;
    const MatchableEvent* evt;
    bool flipEven, flipOdd;

    int headSeqNo;

    TVector3 MaybeFlip(const TVector3& v) const;
  };
}

#endif
