/* 
 * File:   GlobalEventTime.h
 * Author: denis
 *
 * This class keeps track of the global event time for a simulated event
 * for the NOvA DAQ simulation
 *
 * Created on February 10, 2010, 11:31 AM
 */

#ifndef _GLOBALEVENTTIME_H
#define	_GLOBALEVENTTIME_H
#include <stdint.h>


// forward declarations
namespace daqdataformats{
    class RawNanoSlice;
}

namespace mcdatadaq {

  //static const uint32_t DEFAULT_TIME_DIFF = 3200000;     //In clock ticks . Equals to 50 millisec between events
  static const uint32_t DEFAULT_TIME_DIFF = 128000000; //In clock ticks . 2 seconds

  class GlobalEventTime {
    public:
        static GlobalEventTime* getInstance(); //!< Method to obtain the class instance
        ~GlobalEventTime();

        /// Increment the time of the next event time by fEventTimeDifference
        void     advanceEventTime();  

        /// Print methods 
        void     print() const;
        void     printEventTime() const;
        void     printNanoSliceTime(const daqdataformats::RawNanoSlice* Nanoslice) const;

        /// Get Global Time for the nanoslice
        uint64_t getNanoSliceGlobalTime(const daqdataformats::RawNanoSlice* Nanoslice) const;

        /// GetGlobal Time
        uint64_t getGlobalEventTime() const;

        /// GetGlobal Time
        uint64_t getNextEventGlobalTime() const;


    private:
        /// Constructors
        GlobalEventTime();
        GlobalEventTime(const uint64_t time_difference, const uint64_t global_event_time=0);

    private:
        static GlobalEventTime* finstanceID;           ///!< Instance pointer for singleton class

        uint64_t                fEventTimeDifference;  ///! Time difference between events (in ticks)
        uint64_t                fGlobalEventTime;      ///! Global event time (in ticks)


  };// end of GlobalEventTime
}// end of mcdatadaq namespace

#endif	/* _GLOBALEVENTTIME_H */

