////////////////////////////////////////////////////////////////////////
// \file    NumuSandObj.h
// \brief   Object collecting Numu Sandbox variables
// \version
// \author  Kirk Bays - bays@caltech.edu
////////////////////////////////////////////////////////////////////////
#ifndef NumuSandObj_h
#define NumuSandObj_h

namespace numusand
{
  class NumuSandObj {
  public:

    NumuSandObj();
    ~NumuSandObj();

    void               SetPiMuDecay(int pitomu);          ///< did slice contain pi->mu decay?
                                                          ///< -5 not filled (includes no truth)/ 0 no
                                                          ///< >0 yes and = remid sorted track#+1
    void               SetVtxDst(float dist);             ///< Perpendicular distance from muon vertex to detector edge
    void               SetMuDst(float dist);              ///< Projected active distance from muon end to detector edge
    void               SetMissingE(float misse);          ///< total E of true particles leaving detector (>1 MeV, not neutron)
    void               SetMissingE2(float misse2);        ///< missing E of true particles leaving detector (>1 MeV, not neutron)

    void               SetXMin2(float dxmin2);            ///< minimum position of hit second farthest from edge in that dimension
    void               SetYMin2(float dymin2);
    void               SetZMin2(float dzmin2);

    void               SetXMax2(float dxmax2);            ///< maximum position of hit second farthest from edge in that dimension
    void               SetYMax2(float dymax2);
    void               SetZMax2(float dzmax2);

    void               SetXMint(float dxmint);            /// minimum position of hit closest to edge in that dimension that has more than 100 photoelectons (not pecorr yet)
    void               SetYMint(float dymint);
    void               SetZMint(float dzmint);

    void               SetXMaxt(float dxmaxt);            /// minimum position of hit closest to edge in that dimension that has more than 100 photoelectons (not pecorr yet)
    void               SetYMaxt(float dymaxt);
    void               SetZMaxt(float dzmaxt);


    int                PiMuDecay() const;
    float              VtxDst() const;
    float              MuDst() const;
    float              MissingE() const;
    float              MissingE2() const;
    float              XMin2() const;
    float              YMin2() const;
    float              ZMin2() const;
    float              XMax2() const;
    float              YMax2() const;
    float              ZMax2() const;
    float              XMint() const;
    float              YMint() const;
    float              ZMint() const;
    float              XMaxt() const;
    float              YMaxt() const;
    float              ZMaxt() const;

  protected:

    int                fPiMuDecay;
    float              fVtxDst;
    float              fMuDst;
    float              fMissE;
    float              fMissE2;
    float              fXmin2;
    float              fYmin2;
    float              fZmin2;
    float              fXmax2;
    float              fYmax2;
    float              fZmax2;
    float              fXmint;
    float              fYmint;
    float              fZmint;
    float              fXmaxt;
    float              fYmaxt;
    float              fZmaxt;

  };

};
#endif
