///
/// \file  FEBRateAnalysis.h
/// \brief Summarize FEB rates produced by the online monitoring
///

// This must be one larger than the number of Diblocks/DCMs for the FD
// so that it can be easily accessed as fPixelsDCM[Diblock][DCM] instead
// of fPixelsDCM[Diblock - 1][DCM - 1].
static const unsigned int PixelsDCM_size = 16;

class TH2F;
class TFile;
class TTree;
namespace om             { class RateSummaryNt;   }
namespace daqchannelmap  { class HardwareDisplay;   }

namespace om {
  ///
  /// Summarize FEB rates produced by the online monitoring
  ///  
  class FEBRateAnalysis 
  {
  public:
    FEBRateAnalysis(std::string det);
    void  AnaFile(const TFile& f, int run, int subrun);
    virtual void StoreContent(TH2F*& histPix, TH2F*& histPixHigh, TH2F*& histPixLow, 
			      TH2F*& histPixMip, TH2F*& histPixMin,int run, int subrun, int db, int dcm);

  private:

    daqchannelmap::HardwareDisplay* fHardwareMapping; ///< Map pixel address to hardware space

    RateSummaryNt* fNt;
    TTree*         fRatesTree;

  };
}
////////////////////////////////////////////////////////////////////////
