#ifndef OM_HITMAPS_H
#define OM_HITMAPS_H
#include "OnlineMonitoring/producer/SummarySubscriber.h"

// This must be one larger than the number of Diblocks/DCMs for the FD
// so that it can be easily accessed as fPixelsDCM[Diblock][DCM] instead
// of fPixelsDCM[Diblock - 1][DCM - 1].
static const unsigned int PixelsDCM_size = 16;

class TH1F;
class TH2F;
namespace om            { class RawEventSummary;   }
namespace om            { class DataBlockSummary;  }
namespace om            { class MicroSliceSummary; }
namespace om            { class NanoSliceSummary;  }
namespace daqchannelmap { class HardwareDisplay;   }

namespace om {
  class HitMaps : public SummarySubscriber {
  public:
    HitMaps();
    ~HitMaps();
    virtual void GetNanoSliceSummary(const RawEventSummary&   r,
				     const DataBlockSummary&  dbs,
				     const MicroSliceSummary& mss,
				     const NanoSliceSummary&  nss);
    virtual void CopyHistContent(TH2F*& histFrom, 
				 TH2F*& histTo);

  private:
    daqchannelmap::HardwareDisplay* fHwMap;

    TH1F* fHitMapEvtCount;
    TH1F* fRecordedTime;
    TH2F* fDCMHitRateMap;
    TH1F* fDCMHitRateSpectrum;
    TH2F* fDCMHitRateMapLowADC;
    TH2F* fDCMHitRateMapMipADC;
    TH2F* fDCMHitRateMapHighADC;

    TH2F* fFEBHitRateMap;
    TH1F* fFEBHitRateSpectrum;
    TH2F* fFEBHitRateMapLowADC;
    TH2F* fFEBHitRateMapMipADC;
    TH2F* fFEBHitRateMapHighADC;

    TH2F* fPixelHitMap;
    TH2F* fPixelHitRateMap;
    TH1F* fPixelHitRateSpectrum;

    TH2F* fDCMHitMap;
    TH2F* fDCMHitMapLowADC;
    TH2F* fDCMHitMapMipADC;
    TH2F* fDCMHitMapHighADC;
    TH2F* fFEBHitMap;
    TH2F* fFEBHitMapLowADC;
    TH2F* fFEBHitMapMipADC;
    TH2F* fFEBHitMapHighADC;
    TH2F* fPlaneCellHitMapXviewAllHits;
    TH2F* fPlaneCellHitMapYviewAllHits;
    TH2F* fPlaneCellHitRateMapXviewAllHits;
    TH2F* fPlaneCellHitRateMapYviewAllHits;
    TH2F* fPlaneCellHitMapXviewCalib;
    TH2F* fPlaneCellHitMapYviewCalib;
    TH2F* fPlaneCellHitMapXviewNuMI;
    TH2F* fPlaneCellHitMapYviewNuMI;
    TH2F* fPlaneCellHitMapXviewBNB;
    TH2F* fPlaneCellHitMapYviewBNB;

    TH2F* fPixelsRateDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fLowADCPixelsRateDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fMipADCPixelsRateDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fHighADCPixelsRateDCM[PixelsDCM_size][PixelsDCM_size];
   
    TH2F* fPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fLowADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fMipADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fHighADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];

    TH2F* fMinADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fMaxADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fTotADCPixelsDCM[PixelsDCM_size][PixelsDCM_size];
    TH2F* fTotADCFEBHitMap;

  };
}

#endif
////////////////////////////////////////////////////////////////////////
