#ifndef OM_MICROERRORS_H
#define OM_MICROERRORS_H
#include "OnlineMonitoring/producer/SummarySubscriber.h"

// This must be one larger than the number of Diblocks/DCMs for the FD
// so that it can be easily accessed as fDCM[Diblock][DCM] instead
// of fDCM[Diblock - 1][DCM - 1].
static const unsigned int DCM_size_micro = 16;

class TH2F;
class TH1F;
namespace om            { class RawEventSummary;   }
namespace om            { class DataBlockSummary;  }
namespace om            { class MicroSliceSummary; }
namespace daqchannelmap { class HardwareDisplay;   }

namespace om {
  class MicroErrors : public SummarySubscriber {
  public:
    MicroErrors();
    ~MicroErrors();
    virtual void GetMicroSliceSummary(const RawEventSummary&   r,
				      const DataBlockSummary&  dbs,
				      const MicroSliceSummary& mss);
  private:
    daqchannelmap::HardwareDisplay* fHwMap;

    // Key for histogram abbreviations:
    //
    // DPE = "Data Present Error"
    // MC  = "Monte Carlo flag"
    // BC  = "Byte Count"
    // AE  = "All Errors"
    // AA  = "All Alerts"

    // histos displaying the entire detector

    TH2F* fDPEbyDCM;
    TH2F* fMCbyDCM;
    TH2F* fBCbyDCM;

    // histos for specific DCMs
    TH1F* fBCperDCM[DCM_size_micro][DCM_size_micro];

    TH2F* fAEVsHour;
    TH2F* fAAVsHour;    

  };
}

#endif
////////////////////////////////////////////////////////////////////////
