///
/// \file    RawEventSummary.h
/// \brief   All the interesting event-level data
/// \version $Id: RawEventSummary.h,v 1.6 2012-10-31 22:42:56 mbaird42 Exp $
/// \author  messier@indiana.edu
///
#ifndef OM_RAWEVENTSUMMARY_H
#define OM_RAWEVENTSUMMARY_H
#include <set>

namespace om {
  class RawEventSummary {
  public:
    RawEventSummary();
  public:
    //
    // All subscribers can expect this data to be filled correctly
    //
    unsigned int      fRun;             ///< Run number
    unsigned int      fSubrun;          ///< Subrun number
    unsigned int      fEvent;           ///< Event nunber
    unsigned int      fNdatablock;      ///< Number of data blocks
    unsigned int      fTrigger;         ///< Trigger type
    unsigned long int fT0;              ///< Trigger time in nova time units (64 Mhz ticks)
    unsigned int      fRange;           ///< Trigger length in units of 500 nsec
    float             fHour;            ///< A fractional hour of the day
    double            fT0lo32Usec;      ///< Just the lower 32 bits of T0 converted to Usec
    bool              fEventIncomplete; ///< Is the event incomplete?

    //
    // Only event summary subscribers can expect this data to be correct
    //
    unsigned int           fNhit;     ///< Total number of nanoslices (hits)
    unsigned int           fNfeberr;  ///< Number of FEB errors reported in event
    unsigned int           fNmicroSl; ///< Total number of micro slices
    std::set<unsigned int> fDiblocks; ///< The list of diblocks in event
    std::set<unsigned int> fDCMs;     ///< The list of DCMs contributing to the event
    std::set<unsigned int> fFEBs;     ///< The list of FEBs contributing to the event
  };
}

#endif
////////////////////////////////////////////////////////////////////////
