///
/// \file    RawEventUnpacker.h
/// \brief   Unpack the DAQ data, prepare summaries, and post data
/// \version $Id: RawEventUnpacker.h,v 1.4 2012-07-03 21:58:57 mbaird42 Exp $
/// \author  messier@indiana.edu
///
#ifndef OM_RAWEVENTUNPACKER_H
#define OM_RAWEVENTUNPACKER_H
#include <list>

namespace daqdataformats { class RawEvent;          }
namespace daqdataformats { class RawDataBlock;      }
namespace daqdataformats { class RawMicroSlice;     }
namespace daqdataformats { class RawNanoSlice;      }
namespace daqchannelmap  { class HardwareDisplay;   }
namespace om             { class RawEventSummary;   }
namespace om             { class DataBlockSummary;  }
namespace om             { class MicroSliceSummary; }
namespace om             { class NanoSliceSummary;  }
namespace om             { class SummarySubscriber; }

namespace om {
  ///
  /// Unpack the russian dolls that is an online event and publish
  /// results
  ///
  class RawEventUnpacker {
  public:
    static RawEventUnpacker& Instance();
  
  public:
    void SubscribeToRawEventSummary(SummarySubscriber*   s);
    void SubscribeToDataBlockSummary(SummarySubscriber*  s);
    void SubscribeToMicroSliceSummary(SummarySubscriber* s);
    void SubscribeToNanoSliceSummary(SummarySubscriber*  s);

  public:
    void UnpackRawEvent(daqdataformats::RawEvent& rawevt);

    void UnpackDataBlock(daqdataformats::RawEvent& rawevt,
			 unsigned int              idata,
			 RawEventSummary&          evtsum);

    void UnpackMicroSlice(daqdataformats::RawDataBlock& datablock,
			  unsigned int                  imicro, 
			  RawEventSummary&              evtsum,
			  DataBlockSummary&             dbsum);

    void UnpackNanoSlice(daqdataformats::RawMicroSlice& microslice,
			 unsigned int                   inano, 
			 RawEventSummary&               evtsum,
			 DataBlockSummary&              dbsum,
			 MicroSliceSummary&             microsum);
    
    void PublishRawEventSummary(const RawEventSummary& s);
    
    void PublishDataBlockSummary(const RawEventSummary&   res,
				 const DataBlockSummary&  dbs);
    
    void PublishMicroSliceSummary(const RawEventSummary&   res,
				  const DataBlockSummary&  dbs,
				  const MicroSliceSummary& mbs);
    
    void PublishNanoSliceSummary(const RawEventSummary&   res,
				 const DataBlockSummary&  dbs,
				 const MicroSliceSummary& mbs,
				 const NanoSliceSummary&  s);
    void SetDebug(bool debug);

  private:
    RawEventUnpacker();
    ~RawEventUnpacker();
  private:
    daqchannelmap::HardwareDisplay* fHardwareMapping; ///< Map pixel address to hardware space
    std::list<SummarySubscriber*>   fRESSubscriber;   ///< Raw event summary subscribers
    std::list<SummarySubscriber*>   fDBSSubscriber;   ///< Data block summary subscribers
    std::list<SummarySubscriber*>   fMSSSubscriber;   ///< Micro slice summary subscribers
    std::list<SummarySubscriber*>   fNSSSubscriber;   ///< Nano slice summary subscribers
    bool fDebug;                                      ///< Flag to print debugging statements

  };
}

#endif
////////////////////////////////////////////////////////////////////////

