/////////////////////////////////////////////////////////////////////////////////////////
/// \brief   Definition of basic raw Trigger information.
///          See definition of all trigger fields in the
///          DAQDataFormats documentation, NOvA-DocDB-4390 in the section "Trigger Formats".
/// \author  denis@fnal.gov
/// \date
/////////////////////////////////////////////////////////////////////////////////////////
#ifndef RAWTRIGGER_H
#define RAWTRIGGER_H

#include <iomanip>
#include <iosfwd>
#include <iostream>
#include <stdint.h>
#include <string>
#include <vector>

namespace rawdata {
  
  class RawTrigger 
  {
  public:
    RawTrigger(); // Default constructor
    ~RawTrigger();
    
    uint64_t TDCT0() const;
    
  public:
    uint8_t  fTriggerHeader_Source;
    uint16_t fTriggerHeader_SourceID;
    uint8_t  fTriggerHeader_SourceSubID;
    uint64_t fTriggerHeader_TriggerNumber;
    uint64_t fTriggerHeader_MasterTriggerNumber;
    uint64_t fTriggerTime_GenTime;
    uint8_t  fTriggerTime_ClockSource;
    uint64_t fTriggerTimingMarker_TimeStart;
    uint64_t fTriggerTimingMarker_ExtractionStart;
    uint32_t fTriggerRange_TriggerLength;
    bool     fTriggerMask_MCBit;
    uint8_t  fTriggerMask_TMod;
    uint8_t  fTriggerMask_TriggerType;
    uint32_t fTriggerMask_Prescale;
    
    friend std::ostream& operator << (std::ostream& o, const RawTrigger& rt);
    
  private:
    
  };
}

#endif // RAWTRIGGER_H
///////////////////////////////////////////////////////////////////////////////////////////
