////////////////////////////////////////////////////////////////////////
/// \brief  Store +/-1,2sigma shifts for all GENIE reweighting systematics
/// \author Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef RWGT_GENIEREWEIGHTTABLE_H
#define RWGT_GENIEREWEIGHTTABLE_H

#include <vector>

namespace rwgt
{
  /// \brief  Store +/-1,2sigma shifts for all GENIE reweighting systematics
  class GENIEReweightTable
  {
  public:
    unsigned int NShifts() const {return fMinus2Sigma.size();}
    double Minus2Sigma(int i) const {return fMinus2Sigma[i];}
    double Minus1Sigma(int i) const {return fMinus1Sigma[i];}
    double Plus1Sigma(int i) const {return fPlus1Sigma[i];}
    double Plus2Sigma(int i) const {return fPlus2Sigma[i];}

    void SetWeights(unsigned int i, double m2, double m1, double p1, double p2);
  protected:
    std::vector<double> fMinus2Sigma;
    std::vector<double> fMinus1Sigma;
    std::vector<double> fPlus1Sigma;
    std::vector<double> fPlus2Sigma;
  };
}

#endif
