///
/// \file:     SlicerAnaNt.h
/// \brief:    Ntuple produced by the SlicerAna module
/// \version:  $Id: SlicerAnaNt.h,v 1.3 2012-12-19 14:49:53 mbaird42 Exp $
/// \author:   mibaird@indiana.edu
///
#ifndef SLICER_SLICERANANT_H
#define SLICER_SLICERANANT_H

#include <vector>



namespace slicer {

  /// Ntuple produced by the SlicerAna module

  class SlicerAnaNt {
  public:

    int run;                ///< run number
    int subrun;             ///< subrun number
    int evt;                ///< event number
    int NumberOfNu;         ///< number of neutrino interactions per event
    int NumberOfGoodNu;     ///< number of neutrinos that left FLShits in the detector
    int NumberOfGoodNuIN;   ///< number of good nu with a vertex inside the detector
    int NumberOfGoodNuOUT;  ///< number of good nu with a vertex outside the detector (rock event)
    int NumberOfHitNu;      ///< number of nu that left hits in the detector
    int NumberOfNoiseNu;    ///< number of nu that best matched the noise slice
    int NumberOfSliceNu;    ///< number of nu that best matched a non-noise slice
    int NumberOfSlices;     ///< number of non-noise slices per event
    int NumberOfNoiseSL;    ///< number of noise slices per event


    // information about all slices
    std::vector<float>  nuX;       ///< x location for primary nu associated with slice
    std::vector<float>  nuY;       ///< y location for primary nu associated with slice
    std::vector<float>  nuZ;       ///< z location for primary nu associated with slice
    std::vector<float>  nuE;       ///< Energy for primary nu associated with slice
    std::vector<int>    nupdg;     ///< PDG code for primary nu associated with slice
    std::vector<int>    nuCCNC;    ///< CCNC for primary nu associated with slice
    std::vector<int>    nuIntType; ///< Interaction type for primary neutrino associated with slice
    std::vector<double> nuEff;     ///< efficiency for primary nu associated with slice
    std::vector<double> nuPur;     ///< purity for primary nu associated with slice
    std::vector<double> nu2Eff;    ///< efficiency for second nu associated with slice
    std::vector<double> nu2Pur;    ///< purity for second nu associated with slice
    std::vector<double> nu3Eff;    ///< efficiency for third nu associated with slice
    std::vector<double> nu3Pur;    ///< purity for third nu associated with slice

    std::vector<int> sliceNumMatches;   ///< number of MCTruth matches per slice
    std::vector<int> sliceNumHitsX;     ///< number of X hits per slice
    std::vector<int> sliceNumHitsY;     ///< number of Y hits per slice

    std::vector<double> noiseEff;        ///< efficiency for all nu contributing to the noise slice
    std::vector<double> noisePur;        ///< purity for all nu contributing to the noise slice
    std::vector<int>    noiseNumMatches; ///< number of MCTruth matches for noise slice
    std::vector<int>    noiseNumHitsX;   ///< number of X hits for the noise slice
    std::vector<int>    noiseNumHitsY;   ///< number of Y hits for the noise slice



    // information about all neutrinos
    std::vector<float>  ALLnuX;       ///< x location for ALL nu in the event
    std::vector<float>  ALLnuY;       ///< y location for ALL nu in the event
    std::vector<float>  ALLnuZ;       ///< z location for ALL nu in the event
    std::vector<float>  ALLnuE;       ///< Energy for ALL nu in the event
    std::vector<int>    ALLnupdg;     ///< PDG code for ALL nu in the event
    std::vector<int>    ALLnuCCNC;    ///< CCNC for ALL nu in the event
    std::vector<int>    ALLnuIntType; ///< Interaction type for ALL nu in the event
    std::vector<double> ALLnuEff;     ///< efficiency for ALL nu in the event
    std::vector<double> ALLnuPur;     ///< purity for ALL nu in the event
    std::vector<double> ALLnuVisEsl;  ///< visible energy for the nu in the best matched slice
    std::vector<double> ALLnuVisEtot; ///< total visible energy for the nu in the entire event
    std::vector<int>    ALLnuIsNoiseMatch;  ///< Is the best match for this nu the noise slice?

    std::vector<unsigned int> ALLnuNumSliceContributed; ///< The number of non-noise slices that this nu contributed to
    std::vector<unsigned int> ALLnuNumSliceMatch; ///< The number of non-noise slices that this nu matched


    // geometric info about all slices
    std::vector<double> Pave;  ///< average plane number for hits in a non-noise slice
    std::vector<double> Tave;  ///< average time for hits in a non-noise slice
    std::vector<double> CXave; ///< average cell number for X hits in a non-noise slice
    std::vector<double> CYave; ///< average cell number for Y hits in a non-noise slice

    std::vector<double> Psd;  ///< st. dev. of plane number for hits in a non-noise slice
    std::vector<double> Tsd;  ///< st. dev. time for hits in a non-noise slice
    std::vector<double> CXsd; ///< st. dev. cell number for X hits in a non-noise slice
    std::vector<double> CYsd; ///< st. dev. cell number for Y hits in a non-noise slice

    std::vector<double> PaveNoise;  ///< average plane number for hits in a non-noise slice
    std::vector<double> TaveNoise;  ///< average time for hits in a non-noise slice
    std::vector<double> CXaveNoise; ///< average cell number for X hits in a non-noise slice
    std::vector<double> CYaveNoise; ///< average cell number for Y hits in a non-noise slice

    std::vector<double> PsdNoise;  ///< st. dev. of plane number for hits in a non-noise slice
    std::vector<double> TsdNoise;  ///< st. dev. time for hits in a non-noise slice
    std::vector<double> CXsdNoise; ///< st. dev. cell number for X hits in a non-noise slice
    std::vector<double> CYsdNoise; ///< st. dev. cell number for Y hits in a non-noise slice

    std::vector<double> Phi;  ///< highest plane in the slice
    std::vector<double> Plo;  ///< lowest plane in the slice
    std::vector<double> Thi;  ///< highest time in the slice
    std::vector<double> Tlo;  ///< lowest plane in the slice
    std::vector<double> CXhi; ///< highest X cell in the slice
    std::vector<double> CXlo; ///< lowest X cell in the slice
    std::vector<double> CYhi; ///< highest Y cell in the slice
    std::vector<double> CYlo; ///< lowest Y cell in the slice

    std::vector<double> PhiNoise;  ///< highest plane in the Noise slice
    std::vector<double> PloNoise;  ///< lowest plane in the Noise slice
    std::vector<double> ThiNoise;  ///< highest time in the Noise slice
    std::vector<double> TloNoise;  ///< lowest plane in the Noise slice
    std::vector<double> CXhiNoise; ///< highest X cell in the Noise slice
    std::vector<double> CXloNoise; ///< lowest X cell in the Noise slice
    std::vector<double> CYhiNoise; ///< highest Y cell in the Noise slice
    std::vector<double> CYloNoise; ///< lowest Y cell in the Noise slice

  public:
    SlicerAnaNt();
  };
}

#endif
///////////////////////////////////////////////////////////////////
