////////////////////////////////////////////////////////////////////////
// \file    SRBeam.h
// \brief   This class contains information about the neutrino production
// \version $Id: SRLem.h,v 1.4 2012-12-07 06:37:29 gsdavies Exp $
// \author  $Author: gsdavies $
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////
#ifndef SRBEAM_H
#define SRBEAM_H

#include "TVector3.h"

namespace caf
{
  /// This class contains information about the neutrino production
  class SRBeam
  {
  public:
    SRBeam();
    ~SRBeam();

    void setDefault();

    // Docs from http://www.hep.utexas.edu/~zarko/wwwgnumi/v19/v19/output_gnumi.html

    TVector3 tv;    ///< "target vertex". Exit point of parent particle at the target
    TVector3 tp;    ///< "target momentum". Parent momentum exiting the target

    float runjob;   ///< Flux run number, indicates which FLUGG job created this flux
    int   potnum;     ///< Proton event number, indicates which proton was taken from the simulation 
    int tptype;     ///< "target particle type". Parent particle ID exiting the target (PDG code)
    float nimpwt;   ///< "neutrino importance weight". Weight of neutrino parent

    int ndecay;     ///< "neutrino decay". Decay mode that produced neutrino

    TVector3 v;     ///< "vertex". Position of hadron/muon decay
    TVector3 pdp;   ///< "parent decay momentum". Parent momentum at decay point

    float ppdxdz;   ///< "parent particle momentum dx/dx". Parent dx/dz direction at production
    float ppdydz;   ///< "parent particle momentum dy/dz". Parent dy/dz direction at production
    float pppz;     ///< "parent particle p_z". Parent Z momentum at production
    float ppenergy; ///< "parent particle energy". Parent energy at production
    int ppmedium;   ///< "parent particle medium". Tracking medium number where parent was produced
    int ptype;      ///< "parent type". Parent PDG code

    TVector3 ppv;   ///< "parent particle vertex". Parent production vertex
  };

} // end namespace

#endif // SRBEAM_H
//////////////////////////////////////////////////////////////////////////
