#ifndef SRCOSREJ_H
#define SRCOSREJ_H


namespace caf
{
    /// Class for storing output from Cosmic Rejection (CosRej) module
    class SRCosRej{
    public:
      ///  Default constructor
      SRCosRej();

      /// Default destuctor
      ~SRCosRej();
      void setDefault();

      /// \name Track Projections
      //@
      float   kalfwddist;       ///< distance projected forwards (track end to det wall) (kalmantrack)
      float   kalbakdist;       ///< distance projected backwards (track end to det wall) (kalmantrack)
      int     kalfwdcell;       ///< cells along distance projected forwards (kalmantrack)
      int     kalbakcell;       ///< cells along distance projected backwards (kalmantrack)
      float   cosfwddist;       ///< distance projected forwards (track end to det wall) (cosmictrack)
      float   cosbakdist;       ///< distance projected backwards (track end to det wall) (cosmictrack)
      int     cosfwdcell;       ///< cells along distance projected forwards (cosmictrack)
      int     cosbakcell;       ///< cells along distance projected backwards (cosmictrack)
      float   mindist;          ///< minimum projected distance to edge of any Kalman track with > 15 hits
      int     mincell;          ///< minimum projected cells to edge of any Kalman track with > 15 hits
      //@}

      /// \name Other Track Information
      //@
      float   anglekal;         ///< cos of angle of best ReMId Kalman track
      float   anglecos;         ///< cos of angle of Cosmic Track
      int     nkal3d;           ///< number of 3D tracks as determined by Kalman tracker
      float   costhetatrue;     ///< cosine of angle between cosmic trk dir and true dir of most contributing particle
      float   kalthetatrue;     ///< cosine of angle between kalman trk dir and true dir of most contributing particle
      int     pdgbest;          ///< pdg code of most contributing true particle to best kalman trk
      float   kaldist;          ///< perpendicular distance from best kal track end to det wall (negative means outside det)
      float   rpid;             ///< best ReMId pid of any 3D Kalman track
      //@}

      /// \name Cosmic Veto Variables
      //@
      bool    cvetocont;        ///< do we pass cosmic veto containment?
      bool    cvetoangleweak;   ///< do we pass the cosmic veto weak angle cut?
      bool    cvetoanglestrong; ///< do we pass the cosmic veto strong angle cut?
      float   cvetofwddist;     ///< forward projected distance
      float   cvetobakdist;     ///< backwards projected distance
      //@}

      /// \name PIDs
      //@
      float   numucontpid;      ///< cosmic rejection PID for contained events;
      float   numuuncontpid;    ///< cosmic rejection PID for uncontained events;
      //@}

      /// \name Scattering Variables
      //@
      float   scatt;            ///< sum of all all angular deviation (looping over kaltrk traj pts) / trk len
      float   fscattmax;        ///< maximum scattering angle (Fernanda)
      float   fscattsum;        ///< sum of scattering angles (Fernanda)
      float   fscattext;        ///< scattering variable (Fernanda)
      float   fscattsig;        ///< sigma of scattering variable distribution (Fernanda)
      //@}

      /// \name Energy and Activity
      //@
      float   eratio;           ///< ratio of best kalman track GeV / slice GeV
      float   hadE;             ///< GeV sum of non (Kalman best ReMId) track E
      //@}

      /// \name Containment Measures
      //@
      int     nbbc;             ///< number of hits outside of bounding box containment
      //@}

      //      track timing fits - disabled until fast timing is implemented
      float   kfitspeed;        ///< fit inverse speed (ns/cm) of Hough timing fit (TimingFit module) for best ReMId Kalman track
      float   cfitspeed;        ///< fit inverse speed (ns/cm) of Hough timing fit (TimingFit module) for cosmic track
      float   kdirscore;        ///< chisq score difference between +c and -c assumptions in Hough timing fit (TimingFit module) for best ReMId Kalman track
      float   cdirscore;        ///< chisq score difference between +c and -c assumptions in Hough timing fit (TimingFit module) for cosmic track
      float   kscorediff;       ///< chisq score difference between best +c/-c assumption and free fit in Hough timing fit (TimingFit module) for best ReMId Kalman track
      float   cscorediff;       ///< chisq score difference between best +c/-c assumption and free fit in Hough timing fit (TimingFit module) for Cosmic track
      //      float   kalslope;         ///< slope of track timing fit (kalmantrack)
      //      float   kalchisq;         ///< chisq value of slope fit (kalmantrack)
      //      float   kalchidiff;       ///< chisq difference of track timing fits (kalmantrack)
      //      float   cosslope;         ///< slope of track timing fit (cosmictrack)
      //      float   coschisq;         ///< chisq value of slope fit (cosmictrack)
      //      float   coschidiff;       ///< chisq difference of track timing fits (cosmictrack)

    };

} // end namespace

#endif // SRCOSREJ_H
//////////////////////////////////////////////////////////////////////////////
