////////////////////////////////////////////////////////////////////////
// \file    SRNuePresel.h
// \breif   This class contains the LEM PID output.  
// \version $Id: SRNuePresel.h,v 1.4 2012-12-07 06:37:29 gsdavies Exp $  
// \author  $Author: gsdavies $
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////
#ifndef SRNUEPRESEL_H
#define SRNUEPRESEL_H

#include "TVector3.h"	

namespace caf
{
  /// This class contains the LEM PID output.  
  class SRNuePresel
    {
    public:
      ///  Default constructor. 
      SRNuePresel();
      
      /// Default destuctor.
      ~SRNuePresel();

      float       calE;                   ///< Energy (sum of calibrated hit GeV in slice)
      int         ncell;                  ///< Number of cell hits in slice
      float       lengthdiscrete2d;       ///< Longest track (discrete)
      float       lengthdiscretemerge;    ///< Longest track (discretetrack merge)
      float       lengthkalman;           ///< Longest track (kalman)
      float       fracmip;                ///< Fraction of MIP hits

      TVector3    vtx;                    ///< Elastic vertex
      TVector3    mean;                   ///< Mean of slice
      int         cellstoedge;            ///< Number of cells to side of detector
      int         planestofront;          ///< Number of planes to front of detector
      int         planestoback;           ///< Number of planes to back end of detector

      bool        passfid;                ///< Bool saying if event passed Nue Fiducial (current guess)
      bool        passpre;                ///< Bool saying if event passed Nue Presel (current guess)


};
  
} // end namespace

#endif // SRNUEPRESEL_H
//////////////////////////////////////////////////////////////////////////
