////////////////////////////////////////////////////////////////////////
// \file    SRSpill.h
// \breif   An SRSpill contains information on the NuMI spill.
// \version $Id: SRSpill.h,v 1.5 2012-11-19 17:32:22 rocco Exp $  
// \author  $Author: edniner $
// \date    $Date: 2012-11-19 17:32:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRSPILL_H
#define SRSPILL_H

namespace caf
{
  /// The SRSpill contains information about the NuMI spill ant POT
  /// associated with the slice.
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRSpill
    {
    public:
      ///  Default constructor for SRSpill 
      SRSpill();
      
      /// Default destuctor.
      ~SRSpill();

      bool                 isgoodspill;         ///< Was the pot for a spill good? (only applicable to data, default true)
      unsigned long int    spilltimesec;        ///< Spill time in seconds [s] 
      unsigned long int    spilltimensec;       ///< Spill time in naonoseconds [ns]
      unsigned long int    gpsspilltimesec;     ///< Spill time from GPS [s]
      unsigned long int    gpsspilltimensec;    ///< Spill time from GPS [ns]
      signed long long int deltaspilltimensec;  ///< Delta time [ns]
      float               spillpot;            ///< POT in spill including factor of 1e12 so that a user does not have to apply this correction 
      float               hornI;               ///< Horn current 
      float               posx;                ///< x position on target 
      float               posy;                ///< y position on target 
      float               widthx;              ///< Spill width in x dimension
      float               widthy;              ///< Spill width in y dimension
    };
  
} // end namespace

#endif // SRSPILL_H
//////////////////////////////////////////////////////////////////////////////
