#! /usr/bin/env bash

# This a template script for submitting jobs
# to grid using a Cmake build and SAM files.
#
# Original script in /Utilities/batch/SAMSubmitTemplate.sh
# Current script in  /Utilities/batch/SAMSubmitTemplate_cmake.sh
#
# Questions, comments, concerns?
# Bother: Keith Matera (kmatera@fnal.gov)
#
# Last edited Sep 29, 2014

############################################
# Users EDIT THE BELOW PARAMETERS          #
############################################
# Setup the Job specific parameters
JOBNAME='mipana_ndmc_kmatera.sh'
DATASET='xbbu_ndgenie_recos140819_1'
NUMFILES=1018   #1018 max for the dataset used in this example
NUMJOBS=1       # if <= 0, it submits all of the jobs in the project 
OUTPUT_DIR='/nova/ana/users/kmatera/NEARDET_STUDIES/MC/Results/xbbu_new'
JOBFCL='mipanajob_ndmc.fcl' #Check your FHICL_FILE_PATH if you're having trouble
ROLE="" # Which role to use, options are blank (regular use), Production (production user)
GROUP=nova
USER=kmatera
DATA_TIER="reco" # Options are: reco, artcaf, artdaq, pclist, pcliststop
# If artcaf, make edit at end of this file also
#############################################
# USER INTERVENTION ENDS HERE, IN PRINCIPLE #
#############################################

# Should not have to change this...
MY_TEST_REL=${CETPKG_BUILD}/..

############################################
# Setup IFDH/grid-friendly environment     #
############################################
source /grid/fermiapp/products/nova/externals/setup
export PRODUCTS=/grid/fermiapp/products/common/db:/nova/data/pidlibs/products:${MY_TEST_REL}/localProducts_nova_devel_e6_debug/:${PRODUCTS}
setup jobsub_tools
source /grid/fermiapp/nova/novaart/novasvn/setup/setup_novagrid.sh

############################################
# Setup the SAM project here               #
############################################
if [[ "x$PROJECTNAME" == "x" ]]; then
    DATE="`date +%Y%m%d_%H%M`"
    PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
    if [ $? -eq 0 ]; then
	export SAM_PROJECT_NAME="${PROJECTNAME}"
    fi
else
  export SAM_PROJECT_NAME="${PROJECTNAME}"
fi

if [ ${NUMJOBS} -le 0 ]; then
  export NUMPROJECTFILES=`samweb -e nova count-definition-files ${DATASET}`
  if [ ${NUMFILES} -eq 1 ]; then
    export NUMJOBS=${NUMPROJECTFILES}
  else
    export NUMJOBS=`echo "(${NUMPROJECTFILES}/ ${NUMFILES}) + 1" | bc`
  fi
fi

############################################
# Configure Grid Proxy                     #
############################################
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

############################################
# Give log output names a sensible name    #
# based on the PROJECTNAME                 #
############################################
#SCRIPT is usually set with SRT_DIR input that depends on release type. No SRT-version dependent information
# used in art_sam_wrap.sh as of the writing of this template. For now then, cmake uses the development version
export SCRIPT=${NOVASOFT_FQ_DIR}/bin/art_sam_wrap.sh
ln -s $SCRIPT /nova/app/condor-exec/$USER/${PROJECTNAME}.sh
JOBSCRIPT=/nova/app/condor-exec/$USER/${PROJECTNAME}.sh

############################################
# Print out some useful information        #
############################################
echo "NUMJOBS IS $NUMJOBS with $NUMFILES per job"
echo "DATASET: " $DATASET
echo "Project name is: $SAM_PROJECT_NAME"
echo "Station Monitor: http://samweb.fnal.gov:8480/station_monitor/nova/stations/nova/projects/$SAM_PROJECT_NAME"
echo "$SAM_PROJECT_NAME" >> $0.projectname.log

############################################
# Make source_novasoft_cmake.sh            #
############################################
SOURCE_CMAKE=${PWD}/'source_novasoft_cmake.sh'
echo '#! /usr/bin/env bash' >> $SOURCE_CMAKE
echo ' ' >> $SOURCE_CMAKE
echo 'main(){' >> $SOURCE_CMAKE
echo '    source /grid/fermiapp/products/nova/externals/setup' >> $SOURCE_CMAKE
echo '    export PRODUCTS=/grid/fermiapp/products/common/db:/nova/data/pidlibs/products:'$MY_TEST_REL'/localProducts_nova_devel_e6_debug/:${PRODUCTS}' >> $SOURCE_CMAKE
echo '    setup novasoft devel -q e6:debug:grid' >> $SOURCE_CMAKE
echo '    setup jobsub_tools' >> $SOURCE_CMAKE
echo '    source /grid/fermiapp/nova/novaart/novasvn/setup/setup_novagrid.sh' >> $SOURCE_CMAKE
echo '}' >> $SOURCE_CMAKE
echo 'main $*' >> $SOURCE_CMAKE
chmod ug+x $SOURCE_CMAKE

############################################
# The Magic Jobsub command                 #
############################################
jobsub \
    -N ${NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -g \
    ${JOBSCRIPT} \
      --multifile \
      --export EXTERNALS='/grid/fermiapp/products/nova' \
      --inputfile '/nova/ana/calibration/csvs/v4/calib_atten_consts.nd.mc.v4.csv' \
      --inputfile '/nova/ana/calibration/csvs/v4/calib_atten_points.nd.mc.v4.csv' \
      --inputfile '/nova/ana/calibration/csvs/v4/calib_abs_consts.nd.mc.v4.csv' \
      --config $JOBFCL \
      --source ${SOURCE_CMAKE} \
      --limit ${NUMFILES} \
      --export DEST=${OUTPUT_DIR} \
      --postscript ${NOVASOFT_FQ_DIR}'/bin/copyOutFiles_cmake.sh' \
      -X ${NOVASOFT_FQ_DIR}'/bin/runNovaSAM_cmake.py' \

############################################
# Some options that you may wish to pass   #
############################################

# Note! --copyOut option only enables copying out of ART files.
# It will automatically exclude hist files.
#       --copyOut

# Looks for a path called out1 in mipanajob_ndmc, sends output
# through it
#       --outTier=out1:${DATA_TIER} \  
 
# Add if this is a CAF job. MAke sure DATA_TIER is set appropriately,
# in the USER INTERVENTION section
#  --cafLabel=cafmaker:${DATA_TIER} \ 