# Python std stuff
import sys, datetime
import os
# all of root 
from ROOT import *
# My utilities
import novaStyle, canMan, HTMLTools as html 
# Hist set for early data plots
from collections import OrderedDict
import psycopg2
from WatchdogFx import *


gEnv.SetValue("TFile.Recover", 0)


webDir = "/nusoft/app/web/htdoc/nova/datacheck/watchdogs/"


onMonDirs = OrderedDict()
onMonDirs["fardet"] = "/nova/data/nearline-OnMon/FarDet/S14-02-05/"
onMonDirs["neardet"] = "/nova/data/nearline-OnMon/NearDet/S14-06-20/"

fullNames = OrderedDict()
fullNames["fardet"] = "Far Detecor"
fullNames["neardet"] = "Near Detector"

now = datetime.datetime.now()
dateStr = str(now.year) + "-"+ str(now.month) + "-" + str(now.day) + "_" + str(now.hour) + "-" + str(now.minute)
outputDir = webDir + dateStr + "/"
print "Output results in : ", outputDir   
if not os.path.isdir(outputDir):
  os.mkdir(outputDir)

url = "http://nusoft.fnal.gov/nova/datacheck/watchdogs/" + dateStr + "/watchdog.html"


outputHtml = open(outputDir + "watchdog.html", 'w')
outputHtml.write(html.preventCaching())
files = os.listdir(outputDir)
outputHtml.write(html.title("Watchdog page for " + dateStr))
outputHtml.write(html.heading1("NOvA Offline Data Watchdog Summary"))
outputHtml.write(html.paragraph("This page generated at " + now.strftime("%Y/%m/%d at %H:%M") + ".  "+ html.link("Permalink", url)))
outputHtml.write(html.paragraph(html.link("Latest page", "http://nusoft.fnal.gov/nova/datacheck/watchdogs/latest/watchdog.html")))


outputHtml.write(html.paragraph("For older runs, see the " + html.link("All-time Processing History", "/nova/datacheck/processinghistory/history.html")+"."))

for detector, onMonDir in onMonDirs.items():
  (maxRunNumDB, minRunNumDB) = getMaxMinRunNumDB(detector) 
  onMonRunsHist =  getCompletedRunsHist(onMonDir, "OnMon Files", maxRunNumDB, minRunNumDB, lineStyle=7, lineColor=onMonColor)
  dbRunsHist = getDBRunsHist(minRunNumDB, maxRunNumDB, detector, lineStyle=1, lineColor=dbColor)
  os.system('export PGPASSWORD=' + pw + '; psql -h ifdbprod.fnal.gov -d nova_prod -p 5433 -U nova_reader  -H -c "select run, nsubruns, nevents, partition, tstart, shifter, begin_comment, end_comment  from fardet.runs where run > '+str(minRunNumDB) +' and run < '+str(maxRunNumDB) +' order by run asc;" > ' + outputDir + 'dbDump_'+detector+'.html')
  outputHtml.write(html.heading2(fullNames[detector] + " Processing History - Last Seven Days"))
  outputHtml.write(html.paragraph(html.link("Database table dump", "dbDump_"+detector+".html") + " for these runs."))


  for key, path in paths.items():
    if not detector in key:
      continue
    plotName = path.name + ".png"
    (can,hists) = path.getPlotCanvas(maxRunNumDB, minRunNumDB, dbRunsHist, onMonRunsHist)
    can.SaveAs(os.path.join(outputDir, plotName))
    outputHtml.write(html.heading3(path.title))
    outputHtml.write(html.img(plotName,""))
    onMonDiff = (onMonRunsHist - hists[0]).Integral()
    dbDiff = (dbRunsHist - hists[0]).Integral()
    outputHtml.write(html.paragraph("Missing raw files according to database: " + str(dbDiff)))
    outputHtml.write(html.paragraph("Missing raw files according to OnMon files: " + str(onMonDiff)))




outputHtml.close()
redirectUrl = dateStr + "/watchdog.html"

apacheFile = open(webDir + ".htaccess", 'w')
apacheFile.write("RewriteEngine on \n")
apacheFile.write("RewriteRule   latest.html  /nova/datacheck/watchdogs/" + redirectUrl + " [R] \n")
apacheFile.write("RewriteRule   latest/(.*)  /nova/datacheck/watchdogs/"+ dateStr +"/$1 [PT] \n")
apacheFile.close()


