#include "BreakPointFitter/func/dEdxTable.h"
#include <cmath>
#include <iostream>
#include "TH1F.h"
#include "TFile.h"

static const double gsMmu = 105.6583568E-3;
static const double gsMp  = 0.938;

int main(void) 
{
  unsigned int i;
  unsigned int nstep  = 600;
  double logtlo = -3.0;
  double logthi =  3.0;
  double t;

  TH1F* pvc      = new TH1F("pvc",     ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* iron     = new TH1F("iron",    ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* paraffin = new TH1F("paraffin",";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* minoil   = new TH1F("minoil",  ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);

  TH1F* pvcp      = new TH1F("pvcp",     ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* ironp     = new TH1F("ironp",    ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* paraffinp = new TH1F("paraffinp",";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* minoilp   = new TH1F("minoilp",  ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);

  bpfit::dEdxTable dedx_pvc  ("PVC");
  bpfit::dEdxTable dedx_iron ("Iron");
  bpfit::dEdxTable dedx_para ("Paraffin");
  bpfit::dEdxTable dedx_scint("Scintillator");

  for (i=0; i<nstep; ++i) {
    t = logtlo + (i+0.5)*(logthi-logtlo)/nstep;
    t = pow(10.0, t);
    
    pvc->     SetBinContent(i+1,dedx_pvc  (t,gsMmu)*1e3);
    iron->    SetBinContent(i+1,dedx_iron (t,gsMmu)*1e3);
    paraffin->SetBinContent(i+1,dedx_para (t,gsMmu)*1e3);
    minoil->  SetBinContent(i+1,dedx_scint(t,gsMmu)*1e3);

    pvcp->     SetBinContent(i+1,dedx_pvc  (t,gsMp)*1e3);
    ironp->    SetBinContent(i+1,dedx_iron (t,gsMp)*1e3);
    paraffinp->SetBinContent(i+1,dedx_para (t,gsMp)*1e3);
    minoilp->  SetBinContent(i+1,dedx_scint(t,gsMp)*1e3);
  }

  TFile* f = new TFile("dedx.root","RECREATE");
  pvc->     Write();
  iron->    Write();
  paraffin->Write();
  minoil->  Write();

  pvcp->     Write();
  ironp->    Write();
  paraffinp->Write();
  minoilp->  Write();

  f->Close();
}
