#pragma once

#include <cassert>
#include <memory>
#include <vector>

class TDirectory;

namespace ana
{
  /// \brief Represent the binning of a Spectrum's x-axis
  ///
  /// Maybe be "Simple" (equally spaced) or "Custom" (arbitrary binning)
  class Binning
  {
  public:
    static Binning Simple(int n, double lo, double hi);
    static Binning Custom(const std::vector<double>& edges);

    int NBins() const {return fNBins;}
    double Min() const {return fMin;}
    double Max() const {return fMax;}
    bool IsSimple() const {return fEdges.empty();}
    const std::vector<double>& Edges() const
    {
      assert(!IsSimple());
      return fEdges;
    }

    void SaveTo(TDirectory* dir) const;
    static std::unique_ptr<Binning> LoadFrom(TDirectory* dir);
  protected:
    std::vector<double> fEdges;
    int fNBins;
    double fMin, fMax;
  };

  /// Default true-energy bin edges
  Binning TrueEnergyBins();
  /// Default true-energy bin edges
  const Binning kTrueEnergyBins = TrueEnergyBins();

  // These are just guesses

  const Binning kNumuEnergyBinning = Binning::Simple(100, 0, 10);

  // Quarter GeV bins seem reasonable.
  // No optimization done for this binning.
  const Binning kNueEnergyBinning  = Binning::Simple( 40, 0, 10);
}
