#include "CAFAna/Cut.h"

namespace ana
{
  //----------------------------------------------------------------------
  Cut operator&&(const Cut& a, const Cut& b)
  {
    std::set<std::string> areqs = a.Requires();
    std::set<std::string> breqs = b.Requires();
    areqs.insert(breqs.begin(), breqs.end());
    return Cut(areqs,
               [a, b](const caf::StandardRecord* sr){return a(sr) && b(sr);});
  }

  //----------------------------------------------------------------------
  Cut operator||(const Cut& a, const Cut& b)
  {
    std::set<std::string> areqs = a.Requires();
    std::set<std::string> breqs = b.Requires();
    areqs.insert(breqs.begin(), breqs.end());
    return Cut(areqs,
               [a, b](const caf::StandardRecord* sr){return a(sr) || b(sr);});
  }

  //----------------------------------------------------------------------
  Cut operator!(const Cut& a)
  {
    return Cut(a.Requires(),
               [a](const caf::StandardRecord* sr){return !a(sr);});
  }

  //----------------------------------------------------------------------
  Cut operator>(const Var& v, double c)
  {
    return Cut(v.Requires(),
               [v, c](const caf::StandardRecord* sr){return v(sr) > c;});
  }

  //----------------------------------------------------------------------
  Cut operator>=(const Var& v, double c)
  {
    return Cut(v.Requires(),
               [v, c](const caf::StandardRecord* sr){return v(sr) >= c;});
  }

  //----------------------------------------------------------------------
  Cut operator<(const Var& v, double c)
  {
    return Cut(v.Requires(),
               [v, c](const caf::StandardRecord* sr){return v(sr) < c;});
  }

  //----------------------------------------------------------------------
  Cut operator<=(const Var& v, double c)
  {
    return Cut(v.Requires(),
               [v, c](const caf::StandardRecord* sr){return v(sr) <= c;});
  }

  //----------------------------------------------------------------------
  Cut operator==(const Var& v, double c)
  {
    return Cut(v.Requires(),
               [v, c](const caf::StandardRecord* sr){return v(sr) == c;});
  }

  //----------------------------------------------------------------------
  Cut operator>(const Var& a, const Var& b)
  {
    std::set<std::string> areqs = a.Requires();
    std::set<std::string> breqs = b.Requires();
    areqs.insert(breqs.begin(), breqs.end());
    return Cut(areqs,
               [a, b](const caf::StandardRecord* sr){return a(sr) > b(sr);});
  }

  //----------------------------------------------------------------------
  Cut operator>=(const Var& a, const Var& b)
  {
    std::set<std::string> areqs = a.Requires();
    std::set<std::string> breqs = b.Requires();
    areqs.insert(breqs.begin(), breqs.end());
    return Cut(areqs,
               [a, b](const caf::StandardRecord* sr){return a(sr) >= b(sr);});
  }

  //----------------------------------------------------------------------
  Cut operator==(const Var& a, const Var& b)
  {
    std::set<std::string> areqs = a.Requires();
    std::set<std::string> breqs = b.Requires();
    areqs.insert(breqs.begin(), breqs.end());
    return Cut(areqs,
               [a, b](const caf::StandardRecord* sr){return a(sr) == b(sr);});
  }

  // Build the rest up through simple logic

  Cut operator>(double c, const Var& v){return v < c;}
  Cut operator<(double c, const Var& v){return v > c;}
  Cut operator>=(double c, const Var& v){return v <= c;}
  Cut operator<=(double c, const Var& v){return v >= c;}

  Cut operator<(const Var& a, const Var& b){return !(b >= a);}
  Cut operator<=(const Var& a, const Var& b){return !(b > a);}
}
