#include "CAFAna/Decomp.h"

//#include "CAFAna/NumuDecomp.h"

#include <cassert>

#include "TDirectory.h"
#include "TObjString.h"

namespace ana
{
  //----------------------------------------------------------------------
  BasicDecomp::BasicDecomp(SpectrumLoaderBase& loader,
                           std::string label,
                           const Binning& bins,
                           const Var& var,
                           const Cut& cut,
                           const Var& wei)
    : BasicDecomp(loader, HistAxis(label, bins, var), cut, wei)
  {
  }

  //----------------------------------------------------------------------
  BasicDecomp::BasicDecomp(SpectrumLoaderBase& loader,
                           const HistAxis& axis,
                           const Cut& cut,
                           const Var& wei)
    : fNC      (loader, axis, cut && kIsNC, wei),
      fNumu    (loader, axis, cut && kIsNumuCC  && !kIsAntiNu, wei),
      fNue     (loader, axis, cut && kIsBeamNue && !kIsAntiNu, wei),
      fAntiNumu(loader, axis, cut && kIsNumuCC  &&  kIsAntiNu, wei),
      fAntiNue (loader, axis, cut && kIsBeamNue &&  kIsAntiNu, wei)
  {
  }

  //----------------------------------------------------------------------
  void BasicDecomp::SaveTo(TDirectory* dir) const
  {
    TDirectory* tmp = gDirectory;

    dir->cd();

    TObjString("BasicDecomp").Write("type");

    fNC.SaveTo(dir->mkdir("nc_comp"));
    fNumu.SaveTo(dir->mkdir("numu_comp"));
    fNue.SaveTo(dir->mkdir("nue_comp"));
    fAntiNumu.SaveTo(dir->mkdir("antinumu_comp"));
    fAntiNue.SaveTo(dir->mkdir("antinue_comp"));

    tmp->cd();
  }

  //----------------------------------------------------------------------
  std::unique_ptr<BasicDecomp> BasicDecomp::LoadFrom(TDirectory* dir)
  {
    std::unique_ptr<BasicDecomp> ret(new BasicDecomp);

    // This is a lot of repetitive typing. Define some macros
#define LOAD_SPECT(FIELD, LABEL) assert(dir->GetDirectory(LABEL)); ret->FIELD = *Spectrum::LoadFrom(dir->GetDirectory(LABEL));

    LOAD_SPECT(fNC,   "nc_comp");
    LOAD_SPECT(fNumu, "numu_comp");
    LOAD_SPECT(fNue,  "nue_comp");
    LOAD_SPECT(fAntiNumu, "antinumu_comp");
    LOAD_SPECT(fAntiNue,  "antinue_comp");

    return ret;
  }
}
