#include "CAFAna/EventList.h"

#include "CAFAna/Spectrum.h"
#include "CAFAna/SpectrumLoader.h"

#include "StandardRecord/StandardRecord.h"

namespace ana
{
  //----------------------------------------------------------------------
  void MakeEventListFile(const std::string& wildcard,
                         const Cut& cut,
                         const std::string& output,
                         bool includeSliceIndex)
  {
    /// Helper class for \ref MakeEventListFile
    class EventListMaker: public SpectrumLoader
    {
    public:
      EventListMaker(const std::string& wildcard, const Cut& cut, FILE* f, bool includeSliceIndex)
        : SpectrumLoader(wildcard, kBeam),
          fCut(cut), fFile(f),
          fIncludeSliceIndex(includeSliceIndex), fNPassed(0)
      {
        RequireVariables({"hdr.run", "hdr.subrun", "hdr.evt"});
        if(includeSliceIndex) RequireVariable("hdr.subevt");
        RequireVariables(cut.Requires());
      }

      ~EventListMaker()
      {
        std::cout << "Selected " << fNPassed << " slices." << std::endl;
      }

      virtual void HandleRecord(caf::StandardRecord* sr)
      {
        if(fCut(sr)){
          ++fNPassed;

          fprintf(fFile, "%d %d %d", sr->hdr.run, sr->hdr.subrun, sr->hdr.evt);

          if(fIncludeSliceIndex) fprintf(fFile, " %d", sr->hdr.subevt);

          fprintf(fFile, "\n");
        }
      }
    protected:
      Cut fCut;
      FILE* fFile;
      bool fIncludeSliceIndex;
      int fNPassed;
    };

    FILE* f = fopen(output.c_str(), "w");

    EventListMaker maker(wildcard, cut, f, includeSliceIndex);
    maker.Go();

    fclose(f);

    std::cout << "Wrote event list file " << output << std::endl;
  }
}
