#pragma once

#include "CAFAna/OscillatableSpectrum.h"

namespace ana
{
  /// Interface to extrapolation procedures
  class IExtrap
  {
  public:
    virtual ~IExtrap() {};

    /// Charged current electron neutrino survival (\f$\nu_e\to\nu_e\f$)
    virtual OscillatableSpectrum NueSurvComponent() = 0;
    /// Charged current electron antineutrino survival (\f$\bar\nu_e\to\bar\nu_e\f$)
    virtual OscillatableSpectrum AntiNueSurvComponent() = 0;

    /// Charged current muon neutrino survival (\f$\nu_\mu\to\nu_\mu\f$)
    virtual OscillatableSpectrum NumuSurvComponent() = 0;
    /// Charged current muon antineutrino survival (\f$\bar\nu_\mu\to\bar\nu_\mu\f$)
    virtual OscillatableSpectrum AntiNumuSurvComponent() = 0;

    /// Charged current electron neutrino appearance (\f$\nu_\mu\to\nu_e\f$)
    virtual OscillatableSpectrum NueAppComponent() = 0;
    /// Charged current electron antineutrino appearance (\f$\bar\nu_\mu\to\bar\nu_e\f$)
    virtual OscillatableSpectrum AntiNueAppComponent() = 0;

    /// Charged current muon neutrino appearance (\f$\nu_e\to\nu_\mu\f$)
    virtual OscillatableSpectrum NumuAppComponent() = 0;
    /// Charged current muon antineutrino appearance (\f$\bar\nu_e\to\bar\nu_\mu\f$)
    virtual OscillatableSpectrum AntiNumuAppComponent() = 0;

    /// Neutral currents
    virtual Spectrum NCComponent() = 0;

    virtual void SaveTo(TDirectory* dir) const;
  };
}
