#pragma once

#include <string>
#include <vector>

#include "CAFAna/Cut.h"
#include "CAFAna/Cuts.h"
#include "CAFAna/OscillatableSpectrum.h"
#include "CAFAna/Prediction.h"
#include "CAFAna/Spectrum.h"
#include "CAFAna/SpectrumLoader.h"
#include "CAFAna/Var.h"
#include "CAFAna/Vars.h"

class TCanvas;
class TH1;
class TH2;

namespace ana
{
  ///< Histogram multiplication with statistical errors. Doubles c1 and c2 provide scale factors.
  ///< The opt input can be set to "1off" or "2off" to set the errors for h1 or h2 to 0, respectively.
  TH1* MultiplyWithStats(TH1* h1, TH1* h2, double c1 = 1, double c2 = 1, std::string opt = "");

  ///< Histogram division with statistical errors. Doubles cn and cd provide scale factors.
  ///< The opt input can be set to "1off" or "2off" to set the errors for hn or hd to 0, respectively.
  TH1* DivideWithStats(TH1* hn, TH1* hd, double cn = 1, double cd = 1, std::string opt = "");

  ///< Given an oscillatable spectrum, create a Reco to True energy migration matrix,
  ///< normalizing each column of Reco Energy.
  TH2* RecoToTrueMatrix(OscillatableSpectrum ospecrt, bool stat_errors = false);

  ///< Given a TH2* create a migration matrix by normalizing each column
  TH2* RecoToTrueMatrix(TH2* rt, bool stat_errors = false);

  ///< Given an oscillatable spectrum, create a True to Reco energy migration matrix,
  ///< normalizing each row of True Energy.
  TH2* TrueToRecoMatrix(OscillatableSpectrum ospectr, bool stat_errors = false);

  ///< Given a TH2* create a migration matrix by normalizing each row
  TH2* TrueToRecoMatrix(TH2* tr, bool staterrors = false);

  void MigrateToTrue(TH1* hreco, TH2* hrt, TH1* htrue, bool stat_errors = false); ///< Migrate from reco to true energy
  void MigrateToReco(TH1* htrue, TH2* htr, TH1* hreco, bool stat_errors = false); ///< Migrate from true to reco energy

  ///< Create a purity or efficiency plot from two Spectrum objects
  TH1* PurityOrEfficiency(Spectrum numerator, Spectrum denominator, bool stat_errors = false);

  ///< Create a purity or efficiency plot from two OscillatableSpectrum objects
  TH1* PurityOrEfficiency(OscillatableSpectrum numerator, OscillatableSpectrum denominator, bool stat_errors = false);

  ///< Create a purity or efficiency plot from two TH1* objects
  TH1* PurityOrEfficiency(TH1* num, TH1* den, bool stat_errors = false);

  ///< Create a Far over Near ratio from two Spectrum objects
  TH1* FarOverNearRatio(Spectrum sFar, Spectrum sNear, bool stat_errors = false);
}
