#include "CAFAna/FCCollection.h"

#include "CAFAna/Progress.h"
#include "CAFAna/Utilities.h"

#include "TFile.h"
#include "TTree.h"

#include <cassert>

namespace ana
{
  //----------------------------------------------------------------------
  FCCollection::FCCollection()
  {
  }

  //----------------------------------------------------------------------
  void FCCollection::AddFiles(const std::string& wildcard)
  {
    Progress prog("Loading FCCollection from file(s) "+wildcard);

    const std::vector<std::string> fnames = Wildcard(wildcard);
    for(const std::string& fname: fnames){
      const double fileFrac = double(&fname - &fnames.front())/fnames.size();

      TFile fin(fname.c_str());
      assert(!fin.IsZombie());
      TTree* tr = (TTree*)fin.Get("fc");
      assert(tr);

      FCPoint::InitFromTree(tr);
      const unsigned int N = tr->GetEntries();

      fPts.reserve(fPts.size()+N);
      for(unsigned int n = 0; n < N; ++n){
        fPts.push_back(FCPoint::FromTree(tr, n));

        if(n % 1000 == 0)
          prog.SetProgress(fileFrac + double(n)/(N*fnames.size()));
      }
    } // end for fname
  }

  //----------------------------------------------------------------------
  void FCCollection::SaveToFile(const std::string& fname) const
  {
    TFile fout(fname.c_str(), "RECREATE");
    TTree tr("fc", "fc");
    FCPoint::InitToTree(&tr);
    for(const FCPoint& pt : fPts) pt.ToTree(&tr);
    tr.Write();
  }

  //----------------------------------------------------------------------
  std::unique_ptr<FCCollection> FCCollection::LoadFromFile(const std::string& wildcard)
  {
    std::unique_ptr<FCCollection> ret(new FCCollection);
    ret->AddFiles(wildcard);
    return ret;
  }
}
