#include "CAFAna/FitVars.h"

#include "OscLib/func/IOscCalculator.h"
#include "Utilities/func/MathUtil.h"

#include <cassert>
#include <cmath>

namespace ana
{
  //----------------------------------------------------------------------
  double ConstrainedFitVar::Penalty(double val) const
  {
    const double lo = LowLimit();
    const double hi = HighLimit();

    if(val >= lo && val <= hi) return 0;

    // Try to direct fit back towards centre of the space. Engineer penalty to
    // be zero at the limits.
    const double mean = (lo+hi)/2;
    const double rad = (hi-lo)/2;
    return util::sqr((val-mean)/rad)-1;


    //    if(val < lo) return util::sqr(lo-val);
    //    if(val > hi) return util::sqr(val-hi);
    //    return 0;
  }

  //----------------------------------------------------------------------
  double ConstrainedFitVar::Clamp(double val) const
  {
    return std::max(LowLimit(), std::min(val, HighLimit()));
  }

  //----------------------------------------------------------------------
  double FitTheta13::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return osc->GetTh13();
  }

  //----------------------------------------------------------------------
  void FitTheta13::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetTh13(val);
  }

  //----------------------------------------------------------------------
  double FitSinSq2Theta13::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return util::sqr(sin(2*osc->GetTh13()));
  }

  //----------------------------------------------------------------------
  void FitSinSq2Theta13::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetTh13(asin(sqrt(Clamp(val)))/2);
  }

  //----------------------------------------------------------------------
  double FitDeltaInPiUnits::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    double ret = osc->GetdCP()/M_PI;
    while(ret < 0) ret += 2;
    while(ret > 2) ret -= 2;
    return ret;
  }

  //----------------------------------------------------------------------
  void FitDeltaInPiUnits::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetdCP(M_PI*val);
  }

  //----------------------------------------------------------------------
  double FitSinSqTheta23::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return util::sqr(sin(osc->GetTh23()));
  }

  //----------------------------------------------------------------------
  void FitSinSqTheta23::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetTh23(asin(sqrt(Clamp(val))));
  }

  //----------------------------------------------------------------------
  double FitSinSq2Theta23::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return util::sqr(sin(2*osc->GetTh23()));
  }

  //----------------------------------------------------------------------
  void FitSinSq2Theta23::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetTh23(asin(sqrt(Clamp(val)))/2);
  }

  //----------------------------------------------------------------------
  double FitDmSq32::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return osc->GetDmsq32();
  }

  //----------------------------------------------------------------------
  void FitDmSq32::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetDmsq32(Clamp(val));
  }

  //----------------------------------------------------------------------
  double FitDmSq32Scaled::GetValue(const osc::IOscCalculatorAdjustable* osc) const
  {
    return osc->GetDmsq32()*1000.0;
  }

  //----------------------------------------------------------------------
  void FitDmSq32Scaled::SetValue(osc::IOscCalculatorAdjustable* osc, double val) const
  {
    osc->SetDmsq32(Clamp(val/1000.0));
  }

} // namespace
