#pragma once

#include "CAFAna/Binning.h"
#include "CAFAna/Var.h"

#include <string>

namespace ana
{
  /// \brief Collect information describing the x-axis of an analysis histogram
  ///
  /// That is, what it should be labelled, what the binning should be, and what
  /// variable will be being filled into it.
  struct HistAxis
  {
    HistAxis(const std::string& _label,
             const Binning& _bins,
             const Var& _var)
      : label(_label), bins(_bins), var(_var) {}

    /// Shortcut for simple binnings
    HistAxis(const std::string& _label,
             int nx, double x0, double x1,
             const Var& _var)
      : label(_label), bins(Binning::Simple(nx, x0, x1)), var(_var) {}

    const std::string label;
    const Binning bins;
    const Var& var;
  };
}
