#include "CAFAna/LoadFromFile.h"

#include "CAFAna/Utilities.h"


#include "CAFAna/Decomp.h"
#include "CAFAna/MichelDecomp.h"
#include "CAFAna/NumuDecomp.h"

#include "CAFAna/TrivialExtrap.h"
#include "CAFAna/FarToNearExtrap.h"
#include "CAFAna/NDDataMCRatioExtrap.h"
#include "CAFAna/ModularExtrap.h"

#include "CAFAna/PredictionExtrap.h"
#include "CAFAna/PredictionInterp.h"

#include "CAFAna/SingleSampleExperiment.h"
#include "CAFAna/MultiExperiment.h"

#include "CAFAna/ModularExtrapComponent.h"

#include "TDirectory.h"
#include "TObjString.h"


#include <iostream>

namespace ana
{
  //----------------------------------------------------------------------
  template<> std::unique_ptr<IDecomp> LoadFrom<IDecomp>(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);

    const TString tag = ptag->GetString();

    if(tag == "BasicDecomp") return BasicDecomp::LoadFrom(dir);
    if(tag == "MichelDecomp") return MichelDecomp::LoadFrom(dir);
    if(tag == "NumuDecomp") return NumuDecomp::LoadFrom(dir);

    std::cerr << "Unknown Decomp type '" << tag << "'" << std::endl;
    abort();
  }

  //----------------------------------------------------------------------
  template<> std::unique_ptr<IExtrap> LoadFrom<IExtrap>(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);

    const TString tag = ptag->GetString();

    if(tag == "TrivialExtrap") return TrivialExtrap::LoadFrom(dir);
    if(tag == "FarToNearExtrap") return FarToNearExtrap::LoadFrom(dir);
    if(tag == "NDDataMCRatioExtrap") return NDDataMCRatioExtrap::LoadFrom(dir);
    if(tag == "ModularExtrap") return ModularExtrap::LoadFrom(dir);

    std::cerr << "Unknown Extrap type '" << tag << "'" << std::endl;
    abort();
  }

  //----------------------------------------------------------------------
  template<> std::unique_ptr<IPrediction> LoadFrom<IPrediction>(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);

    const TString tag = ptag->GetString();

    if(tag == "PredictionExtrap") return PredictionExtrap::LoadFrom(dir);
    // PredictionNoExtrap doesn't need any extra handling over PredictionExtrap
    if(tag == "PredictionInterp") return PredictionInterp::LoadFrom(dir);

    std::cerr << "Unknown Prediction type '" << tag << "'" << std::endl;
    abort();
  }

  //----------------------------------------------------------------------
  template<> std::unique_ptr<Experiment> LoadFrom<Experiment>(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);

    const TString tag = ptag->GetString();

    if(tag == "SingleSampleExperiment")
      return SingleSampleExperiment::LoadFrom(dir);
    if(tag == "MultiExperiment")
      return MultiExperiment::LoadFrom(dir);

    std::cerr << "Unknown Experiment type '" << tag << "'" << std::endl;
    abort();
  }

  //----------------------------------------------------------------------
  template<> std::unique_ptr<SingleComponentExtrap>
    LoadFrom<SingleComponentExtrap>(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);

    const TString tag = ptag->GetString();

    if(tag == "ComponentFDMonteCarlo")
      return ComponentFDMonteCarlo::LoadFrom(dir);
    if(tag == "ComponentFarOverNear")
      return ComponentFarOverNear::LoadFrom(dir);
    if(tag == "ComponentRecoOverReco")
      return ComponentRecoOverReco::LoadFrom(dir);

    std::cerr << "Unknown Component Extrapolation type '"
              << tag << "'" << std::endl;
    abort();
  }

  //----------------------------------------------------------------------
  TDirectory* FileToDirectory(const std::string& fname,
                              const std::string& label)
  {
    TFile fin(fname.c_str());
    assert(!fin.IsZombie());
    DontAddDirectory guard;
    TDirectory* dir = fin.GetDirectory(label.c_str());
    if(!dir){
      std::cerr << "Didn't find '" << label << "' in " << fname << std::endl;
      abort();
    }
    return dir;
  }
}
