#pragma once

#include "CAFAna/SpectrumLoaderBase.h"

namespace ana
{
  class SystematicShifter;

  /// \brief Collaborates with \ref Spectrum and \ref OscillatableSpectrum to
  /// fill spectra from CAF files.
  ///
  /// The outside user should just pass a filename or wildcard to the
  /// constructor. Then construct all the spectra you need. They will register
  /// with this loader. Finally, calling \ref Go will cause all the spectra to
  /// be filled at once. After this the loader may not be used again.
  class MRCCLoader: public SpectrumLoaderBase
  {
  public:
    MRCCLoader(const std::string& childWildcard,
               const std::string& parentWildcard,
               const Cut& childCut,
               const Cut& parentCut);
    MRCCLoader(const std::vector<std::string>& childFileNames,
               const std::vector<std::string>& parentFileNames,
               const Cut& childCut,
               const Cut& parentCut);
    virtual ~MRCCLoader();

    virtual void AddSpectrum(Spectrum& spect,
                             const Var& var,
                             const Cut& cut = kNoCut,
                             const Var& wei = kUnweighted) override;

    virtual void AddOscillatableSpectrum(OscillatableSpectrum& spect,
                                         const Var& var,
                                         const Cut& cut,
                                         const Var& wei) override;

    virtual void Go() override;

    /// \brief This shifter will be used for all subsequently added spectra
    ///
    /// Can alter between adding histograms. Earlier will have one, later the other.
    ///
    /// Call with zero to get unshifted (nominal) spectra
    void SetShifter(SystematicShifter* s);

  protected:
    std::tuple<int, int, int, int> CurrentRecordTuple(const caf::StandardRecord* sr) const;
    std::tuple<int, int, int, int> ParentRecordTuple(const caf::StandardRecord* sr) const;

    void HandleRecord(caf::StandardRecord* childSR,
                      caf::StandardRecord* parentSR);

    SystematicShifter* fShifter;

    std::set<std::string> fParentBranchNames;
    std::vector<std::string> fParentFileNames;
    Cut fChildCut, fParentCut;
  };
}
