#include "CAFAna/LoadFromFile.h"
#include "CAFAna/ModularExtrap.h"
#include "CAFAna/Loaders.h"
#include "CAFAna/Cuts.h"
#include "TObjString.h"
#include "TDirectory.h"

namespace ana
{

  //----------------------------------------------------------------------------

  ModularExtrap ModularExtrap::Nue(
    Loaders& loaders,
    const IDecomp& nueDecomp,
    const IDecomp& numuDecomp,
    const HistAxis& axis,
    const Cut& fdcut,
    const Cut& nueNDcut,
    const Cut& numuNDcut,
    const Var& weight
  ){
    return ModularExtrap::Nue(
      loaders.GetLoader(
        caf::kNEARDET, Loaders::kMC, ana::kBeam, Loaders::kNonSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kFluxSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kNonSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kTauSwap),
      nueDecomp,
      numuDecomp,
      axis,
      fdcut,
      nueNDcut,
      numuNDcut,
      weight
    );
  }

  //----------------------------------------------------------------------------

  ModularExtrap ModularExtrap::Numu(
    Loaders& loaders,
    const IDecomp& numuDecomp,
    const HistAxis& axis,
    const Cut& fdcut,
    const Cut& ndcut,
    const Var& weight
  ){
    return ModularExtrap::Numu(
      loaders.GetLoader(
        caf::kNEARDET, Loaders::kMC, ana::kBeam, Loaders::kNonSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kFluxSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kNonSwap),
      loaders.GetLoader(
        caf::kFARDET,  Loaders::kMC, ana::kBeam, Loaders::kTauSwap),
      numuDecomp,
      axis,
      fdcut,
      ndcut,
      weight
    );
  }

  //----------------------------------------------------------------------------

  ModularExtrap ModularExtrap::Nue(
    SpectrumLoaderBase& nearMC,
    SpectrumLoaderBase& farMCswap,
    SpectrumLoaderBase& farMCnonswap,
    SpectrumLoaderBase& farMCtauswap,
    const IDecomp& nueDecomp,
    const IDecomp& numuDecomp,
    const HistAxis& axis,
    const Cut& fdcut,
    const Cut& nueNDcut,
    const Cut& numuNDcut,
    const Var& weight
  ){

    ModularExtrap extrap(
      farMCswap,
      farMCnonswap,
      farMCtauswap,
      axis,
      fdcut,
      weight
    );

    // mu -> mu  ----
    extrap.fMMextrap = new ComponentRecoOverReco(
      nearMC, axis, fdcut, weight, "mu -> mu",
      nueNDcut, nueDecomp,                         // nue selection in ND
      DecompResult::numu, kIsNumuCC && !kIsAntiNu, // numu truth in ND
      farMCnonswap, kIsNumuCC && !kIsAntiNu        // mu->mu in FD
    );
    extrap.fMMAntiextrap = new ComponentRecoOverReco(
      nearMC, axis, fdcut, weight, "mubar -> mubar",
      nueNDcut, nueDecomp,                           // nue selection in ND
      DecompResult::numubar, kIsNumuCC && kIsAntiNu, // numubar truth in ND
      farMCnonswap, kIsNumuCC && kIsAntiNu           // mubar->mubar in FD
    );

    // mu -> e ----
    extrap.fMEextrap = new ComponentFarOverNear(
      nearMC, axis, fdcut, weight, "mu -> e",
      numuNDcut, numuDecomp,                       // numu selection in ND
      DecompResult::numu, kIsNumuCC && !kIsAntiNu, // numu truth in ND
      farMCswap, kIsSig && !kIsAntiNu              // mu->e in FD
    );
    extrap.fMEAntiextrap = new ComponentFarOverNear(
      nearMC, axis, fdcut, weight, "mubar -> ebar",
      numuNDcut, numuDecomp,                         // numu selection in ND
      DecompResult::numubar, kIsNumuCC && kIsAntiNu, // numubar truth in ND
      farMCswap, kIsSig && kIsAntiNu                 // mubar->ebar in FD
    );

    // NC -> NC ----
    extrap.fNCextrap = new ComponentRecoOverReco(
      nearMC, axis, fdcut, weight, "NC -> NC",
      nueNDcut, nueDecomp,           // nue selection in ND
      DecompResult::NC, kIsNC,       // NC truth in ND
      farMCswap, kIsNC, farMCnonswap // NC->NC in FD
    );

    return extrap;

  }

  //----------------------------------------------------------------------------

  ModularExtrap ModularExtrap::Numu(
    SpectrumLoaderBase& nearMC,
    SpectrumLoaderBase& farMCswap,
    SpectrumLoaderBase& farMCnonswap,
    SpectrumLoaderBase& farMCtauswap,
    const IDecomp& numuDecomp,
    const HistAxis& axis,
    const Cut& fdcut,
    const Cut& numuNDcut,
    const Var& weight
  ){

    ModularExtrap extrap(
      farMCswap,
      farMCnonswap,
      farMCtauswap,
      axis,
      fdcut,
      weight
    );

    // mu -> mu ----
    extrap.fMMextrap = new ComponentFarOverNear(
      nearMC, axis, fdcut, weight, "mu -> mu",
      numuNDcut, numuDecomp,                       // numu selection in ND
      DecompResult::numu, kIsNumuCC && !kIsAntiNu, // numu truth in ND
      farMCnonswap, kIsNumuCC && !kIsAntiNu        // mu->mu in FD
    );
    extrap.fMMAntiextrap = new ComponentFarOverNear(
      nearMC, axis, fdcut, weight, "mubar -> mubar",
      numuNDcut, numuDecomp,                         // numu selection in ND
      DecompResult::numubar, kIsNumuCC && kIsAntiNu, // numubar truth in ND
      farMCnonswap, kIsNumuCC && kIsAntiNu           // mubar->mubar in FD
    );

    // mu -> e ----
    extrap.fMEextrap = new ComponentFDMonteCarlo(
      farMCswap, axis, fdcut, weight, kIsSig && !kIsAntiNu );
    extrap.fMEAntiextrap = new ComponentFDMonteCarlo(
      farMCswap, axis, fdcut, weight, kIsSig && kIsAntiNu);

    // NC -> NC ----
    extrap.fNCextrap = new ComponentFDMonteCarlo(
      farMCnonswap, axis, fdcut, weight, kIsNC, farMCswap);

    return extrap;

  }

  //----------------------------------------------------------------------------

  void ModularExtrap::SaveTo(TDirectory* dir) const
  {

    TDirectory* tmp = gDirectory;
    dir->cd();
    TObjString("ModularExtrap").Write("type");

    fEEextrap->SaveTo(dir->mkdir("EEextrap"));
    fEMextrap->SaveTo(dir->mkdir("EMextrap"));
    fMEextrap->SaveTo(dir->mkdir("MEextrap"));
    fMMextrap->SaveTo(dir->mkdir("MMextrap"));
    fEEAntiextrap->SaveTo(dir->mkdir("EEAntiextrap"));
    fEMAntiextrap->SaveTo(dir->mkdir("EMAntiextrap"));
    fMEAntiextrap->SaveTo(dir->mkdir("MEAntiextrap"));
    fMMAntiextrap->SaveTo(dir->mkdir("MMAntiextrap"));
    fNCextrap->SaveTo(dir->mkdir("NCextrap"));

    tmp->cd();

  }

  //----------------------------------------------------------------------------

  std::unique_ptr<ModularExtrap> ModularExtrap::LoadFrom(TDirectory* dir)
  {

    std::unique_ptr<ModularExtrap> ret(new ModularExtrap);

    assert(dir->GetDirectory("EEextrap"));
    ret->fEEextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("EEextrap")).release();
 
    assert(dir->GetDirectory("EMextrap"));
    ret->fEMextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("EMextrap")).release();
 
    assert(dir->GetDirectory("MEextrap"));
    ret->fMEextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("MEextrap")).release();
 
    assert(dir->GetDirectory("MMextrap"));
    ret->fMMextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("MMextrap")).release();
 
    assert(dir->GetDirectory("EEAntiextrap"));
    ret->fEEAntiextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("EEAntiextrap")).release();
 
    assert(dir->GetDirectory("EMAntiextrap"));
    ret->fEMAntiextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("EMAntiextrap")).release();
 
    assert(dir->GetDirectory("MEAntiextrap"));
    ret->fMEAntiextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("MEAntiextrap")).release();
 
    assert(dir->GetDirectory("MMAntiextrap"));
    ret->fMMAntiextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("MMAntiextrap")).release();
 
    assert(dir->GetDirectory("NCextrap"));
    ret->fNCextrap = ana::LoadFrom<SingleComponentExtrap>(
      dir->GetDirectory("NCextrap")).release();

  return ret;

  }
 
  //----------------------------------------------------------------------------

  ModularExtrap::ModularExtrap(
    SpectrumLoaderBase& farMCswap,
    SpectrumLoaderBase& farMCnonswap,
    SpectrumLoaderBase& farMCtauswap,
    const HistAxis& axis,
    const Cut& fdcut,
    const Var& weight
  ){

    // e -> e ----
    fEEextrap = new ComponentFDMonteCarlo(
      farMCnonswap, axis, fdcut, weight, kIsBeamNue && !kIsAntiNu);
    fEEAntiextrap = new ComponentFDMonteCarlo(
      farMCnonswap, axis, fdcut, weight, kIsBeamNue && kIsAntiNu);

    // e -> mu ----
    fEMextrap = new ComponentFDMonteCarlo(
      farMCswap, axis, fdcut, weight, kIsNumuApp && !kIsAntiNu);
    fEMAntiextrap = new ComponentFDMonteCarlo(
      farMCswap, axis, fdcut, weight, kIsNumuApp && kIsAntiNu);

    // mu -> tau ----
    fMTextrap = new ComponentFDMonteCarlo(
      farMCtauswap, axis, fdcut, weight, kIsTauFromMu && !kIsAntiNu);
    fMTAntiextrap = new ComponentFDMonteCarlo(
      farMCtauswap, axis, fdcut, weight, kIsTauFromMu && kIsAntiNu);

    // e -> tau ----
    fETextrap = new ComponentFDMonteCarlo(
      farMCtauswap, axis, fdcut, weight, kIsTauFromE && !kIsAntiNu);
    fETAntiextrap = new ComponentFDMonteCarlo(
      farMCtauswap, axis, fdcut, weight, kIsTauFromE && kIsAntiNu);

  }

  //----------------------------------------------------------------------------

  OscillatableSpectrum ModularExtrap::NueSurvComponent()
    {return fEEextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiNueSurvComponent()
    {return fEEAntiextrap->Return();}

  OscillatableSpectrum ModularExtrap::NumuSurvComponent()
    {return fMMextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiNumuSurvComponent()
    {return fMMAntiextrap->Return();}

  OscillatableSpectrum ModularExtrap::NueAppComponent()
    {return fMEextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiNueAppComponent()
    {return fMEAntiextrap->Return();}

  OscillatableSpectrum ModularExtrap::NumuAppComponent()
    {return fEMextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiNumuAppComponent()
    {return fEMAntiextrap->Return();}

  Spectrum ModularExtrap::NCComponent()
    {return fNCextrap->Return().Unoscillated();}

  OscillatableSpectrum ModularExtrap::TauFromMuComponent()
    {return fMTextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiTauFromMuComponent()
    {return fMTAntiextrap->Return();}

  OscillatableSpectrum ModularExtrap::TauFromEComponent()
    {return fETextrap->Return();}

  OscillatableSpectrum ModularExtrap::AntiTauFromEComponent()
    {return fETAntiextrap->Return();}

  //----------------------------------------------------------------------------

  ModularExtrap::~ModularExtrap()
  {
    delete fEEextrap;
    delete fMMextrap;
    delete fMEextrap;
    delete fEMextrap;
    delete fNCextrap;
    delete fEEAntiextrap;
    delete fMMAntiextrap;
    delete fMEAntiextrap;
    delete fEMAntiextrap;
    delete fMTextrap;
    delete fETextrap;
    delete fMTAntiextrap;
    delete fETAntiextrap;
  }

}
