#include "CAFAna/MultiExperiment.h"

#include "CAFAna/LoadFromFile.h"

#include "TDirectory.h"
#include "TObjString.h"

#include <cassert>

namespace ana
{
  //----------------------------------------------------------------------
  double MultiExperiment::ChiSq(osc::IOscCalculatorAdjustable* osc,
                                const SystShifts& syst) const
  {
    double ret = 0;
    for(unsigned int n = 0; n < fExpts.size(); ++n)
      ret += fExpts[n]->ChiSq(osc, syst);
    return ret;
  }

  //----------------------------------------------------------------------
  void MultiExperiment::SaveTo(TDirectory* dir) const
  {
    TDirectory* tmp = dir;

    dir->cd();
    TObjString("MultiExperiment").Write("type");

    for(unsigned int i = 0; i < fExpts.size(); ++i){
      fExpts[i]->SaveTo(dir->mkdir(TString::Format("expt%d", i)));
    }

    tmp->cd();
  }

  //----------------------------------------------------------------------
  std::unique_ptr<MultiExperiment> MultiExperiment::LoadFrom(TDirectory* dir)
  {
    TObjString* ptag = (TObjString*)dir->Get("type");
    assert(ptag);
    assert(ptag->GetString() == "MultiExperiment");

    std::vector<const Experiment*> expts;

    for(int i = 0; ; ++i){
      TDirectory* subdir = dir->GetDirectory(TString::Format("expt%d", i));
      if(!subdir) break;

      expts.push_back(ana::LoadFrom<Experiment>(subdir).release());
    }

    assert(!expts.empty());

    return std::unique_ptr<MultiExperiment>(new MultiExperiment(expts));
  }
}
