#include "CAFAna/Numu.h"

#include "CAFAna/Cuts.h"
#include "CAFAna/Vars.h"
#include "CAFAna/MultiExperiment.h"
#include "CAFAna/SingleSampleExperiment.h"

namespace ana
{
  //----------------------------------------------------------------------
  NumuAnalysis::NumuAnalysis(SpectrumLoaderBase& loader,
                             SpectrumLoaderBase& loaderSwap,
                             SpectrumLoaderBase& loaderCosmic)
    : fPredCC(loader, loaderSwap, loaderCosmic,
              "Simple Energy (GeV)", kNumuEnergyBinning,  // doesn't include extra cosmic rejection cuts, would add some inefficiency
              kCCE, kCC && kContain && !kQE),
      fPredQE(loader, loaderSwap, loaderCosmic,
              "QE Energy (GeV)", kNumuEnergyBinning,
              kQEE, kCC && kContain && kQE),
      fPredUC(loader, loaderSwap, loaderCosmic,
              "Simple Energy (GeV)", kNumuEnergyBinning,
              kUCE, kCC && !kContain)
  {
  }

  //----------------------------------------------------------------------
  Experiment* NumuAnalysis::MakeExperiment(osc::IOscCalculator* calc,
                                           double pot)
  {
    const Spectrum fakeCC = fPredCC.Predict(calc).FakeData(pot);
    const Spectrum fakeQE = fPredQE.Predict(calc).FakeData(pot);
    const Spectrum fakeUC = fPredUC.Predict(calc).FakeData(pot);

    MultiExperiment* ret = new MultiExperiment;

    ret->Add(new SingleSampleExperiment(&fPredCC, fakeCC));
    ret->Add(new SingleSampleExperiment(&fPredQE, fakeQE));
    ret->Add(new SingleSampleExperiment(&fPredUC, fakeUC));

    return ret;
  }
} // namespace

