#include "CAFAna/PredictionExtrap.h"

#include "CAFAna/Extrap.h"
#include "CAFAna/LoadFromFile.h"

#include "TDirectory.h"
#include "TObjString.h"

namespace ana
{
  //----------------------------------------------------------------------
  PredictionExtrap::PredictionExtrap(IExtrap* extrap,
                                     SpectrumLoaderBase& loaderCosmic,
                                     std::string label,
                                     const Binning& bins,
                                     const Var& var,
                                     const Cut& cut,
                                     const Var& wei)
    : PredictionExtrap(extrap, loaderCosmic, HistAxis(label, bins, var), cut, wei)
  {
  }

  //----------------------------------------------------------------------
  PredictionExtrap::PredictionExtrap(IExtrap* extrap,
                                     SpectrumLoaderBase& loaderCosmic,
                                     const HistAxis& axis,
                                     const Cut& cut,
                                     const Var& wei)
    : fExtrap(extrap),
      fCosmic(loaderCosmic, axis, cut, wei)
  {
  }

  //----------------------------------------------------------------------
  Spectrum PredictionExtrap::Predict(osc::IOscCalculator* calc) const
  {
    return PredictComponent(calc,
                            Flavors::kAll,
                            Current::kBoth,
                            Sign::kBoth) + PredictCosmicComponent();
  }

  //----------------------------------------------------------------------
  Spectrum PredictionExtrap::PredictComponent(osc::IOscCalculator* calc,
                                              Flavors::Flavors_t flav,
                                              Current::Current_t curr,
                                              Sign::Sign_t sign) const
  {
    Spectrum ret = fExtrap->NCComponent(); // Get binning
    ret.Clear();

    if(curr & Current::kCC){
      if(flav & Flavors::kNuEToNuE   && sign & Sign::kNu)     ret += fExtrap->NueSurvComponent().    Oscillated(calc, +12, +12);
      if(flav & Flavors::kNuEToNuE   && sign & Sign::kAntiNu) ret += fExtrap->AntiNueSurvComponent().Oscillated(calc, -12, -12);

      if(flav & Flavors::kNuEToNuMu  && sign & Sign::kNu)     ret += fExtrap->NumuAppComponent().    Oscillated(calc, +12, +14);
      if(flav & Flavors::kNuEToNuMu  && sign & Sign::kAntiNu) ret += fExtrap->AntiNumuAppComponent().Oscillated(calc, -12, -14);

      if(flav & Flavors::kNuEToNuTau)  /* nothing */;

      if(flav & Flavors::kNuMuToNuE  && sign & Sign::kNu)     ret += fExtrap->NueAppComponent().    Oscillated(calc, +14, +12);
      if(flav & Flavors::kNuMuToNuE  && sign & Sign::kAntiNu) ret += fExtrap->AntiNueAppComponent().Oscillated(calc, -14, -12);

      if(flav & Flavors::kNuMuToNuMu && sign & Sign::kNu)     ret += fExtrap->NumuSurvComponent().    Oscillated(calc, +14, +14);
      if(flav & Flavors::kNuMuToNuMu && sign & Sign::kAntiNu) ret += fExtrap->AntiNumuSurvComponent().Oscillated(calc, -14, -14);

      if(flav & Flavors::kNuMuToNuTau) /* nothing */;
    }
    if(curr & Current::kNC){
      assert(flav == Flavors::kAll); // Don't know how to calculate anything else
      assert(sign == Sign::kBoth);   // Why would you want to split NCs out by sign?

      ret += fExtrap->NCComponent();
    }

    return ret;
  }

  //----------------------------------------------------------------------
  void PredictionExtrap::SaveTo(TDirectory* dir) const
  {
    TDirectory* tmp = gDirectory;

    dir->cd();

    TObjString("PredictionExtrap").Write("type");

    fExtrap->SaveTo(dir->mkdir("extrap"));
    fCosmic.SaveTo(dir->mkdir("cosmic"));

    tmp->cd();
  }

  //----------------------------------------------------------------------
  std::unique_ptr<PredictionExtrap> PredictionExtrap::LoadFrom(TDirectory* dir)
  {
    std::unique_ptr<PredictionExtrap> ret(new PredictionExtrap);

    assert(dir->GetDirectory("extrap"));
    ret->fExtrap = ana::LoadFrom<IExtrap>(dir->GetDirectory("extrap")).release();

    assert(dir->GetDirectory("cosmic"));
    ret->fCosmic = *ana::LoadFrom<Spectrum>(dir->GetDirectory("cosmic"));

    return ret;
  }
}
