#pragma once

#include "CAFAna/Spectrum.h"

#include <string>

class TDirectory;
class TH2;
class TH2D;

namespace ana
{
  /// %Spectrum with the value of a second variable, allowing for reweighting
  class ReweightableSpectrum
  {
  public:
    ReweightableSpectrum(std::string xlabel, std::string ylabel, double pot,
                         int nbinsx, double xmin, double xmax,
                         int nbinsy, double ymin, double ymax);
    ReweightableSpectrum(TH2* h, double pot, std::string varName = "");
    virtual ~ReweightableSpectrum();

    ReweightableSpectrum(const ReweightableSpectrum& rhs);
    ReweightableSpectrum& operator=(const ReweightableSpectrum& rhs);

    void Fill(double x, double y, double w = 1);

    TH2D* ToTH2(double pot) const;

    Spectrum UnWeighted() const;

    Spectrum WeightingVariable() const;

    Spectrum WeightedBy(const TH1* weights) const;

    void Clear();

    void SaveTo(TDirectory* dir) const;
    static std::unique_ptr<ReweightableSpectrum> LoadFrom(TDirectory* dir);
  protected:
    // Derived classes can be trusted take care of their own construction
    ReweightableSpectrum() {}

    TH2D* fHist;
    double fPOT;
  };
}
