#pragma once

#include "CAFAna/SpectrumLoader.h"

namespace osc{class IOscCalculator;}

namespace ana
{
  /// \brief SpectrumLoader for producing mock data and fake data
  ///
  /// Place in the "data" position. Takes in MC files, scales to the requested
  /// POT, and optionally fluctuates. In the FD, takes in nonswap and swap
  /// files, plus oscillation parameters.
  ///
  /// Data produced in this way with fluctuations is called "mock data",
  /// without fluctuations it's called "fake data".
  ///
  /// Note that oscillations are applied event-by-event, so small differences
  /// with the histogram-based PredictionNoExtrap are possible, and interesting
  /// for studying the effects of binning.
  class SpectrumLoaderMockData: public SpectrumLoader
  {
  public:
    /// \name NearDet constructors
    //@{
    SpectrumLoaderMockData(double pot,
                           const std::string& wildcard,
                           bool fluctuate  = true);
    SpectrumLoaderMockData(double pot,
                           const std::vector<std::string>& fnames,
                           bool fluctuate = true);
    //@}

    /// \name FarDet constructors
    //@{
    SpectrumLoaderMockData(double pot,
                           const std::string& wildcardNonswap,
                           const std::string& wildcardSwwap,
                           osc::IOscCalculator* calc,
                           bool fluctuate = true);
    SpectrumLoaderMockData(double pot,
                           const std::vector<std::string>& fnamesNonswap,
                           const std::vector<std::string>& fnamesSwap,
                           osc::IOscCalculator* calc,
                           bool fluctuate = true);
    //@}

    virtual ~SpectrumLoaderMockData();

    virtual void AddSpectrum(Spectrum& spect,
                             const Var& var,
                             const Cut& cut,
                             const Var& wei);

    virtual void SetShifter(SystematicShifter* s);

    virtual void Go() override;
  protected:
    /// Doesn't make sense to call for Mock Data
    virtual void AddOscillatableSpectrum(OscillatableSpectrum&,
                                         const Var&,
                                         const Cut&,
                                         const Var&) override {}

    double fDataPOT; ///< Level of stats histograms will be fluctuated to
    bool fFluctuate; ///< Apply Poisson fluctuations?

    SpectrumLoader* fSwapLoader; ///< Loader for swapped files in FD, else NULL
    /// Swapped spectra, matching non-swapped spectra in base class. FD only
    std::vector<Spectrum*> fSwapSpectra;
    /// Multipied into "Var wei" to provide oscillations, FD only
    Var* fOscWeight;
  };
}
