#include "CAFAna/SystShifts.h"

#include "Utilities/func/MathUtil.h"

#include <cassert>

namespace ana
{
  //----------------------------------------------------------------------
  std::string ShortName(Syst s)
  {
    assert(s != kGENIESystInit && s != kGENIESystEnd);
    if(s > kGENIESystInit && s < kGENIESystEnd){
      return genie::rew::GSyst::AsString(genie::rew::EGSyst(s-kGENIESystInit));
    }

    switch(s){
    case kNormSyst: return "norm";
    case kCaloESyst: return "caloE";
    case kNCScaleSyst: return "ncScale";
    case kCCESyst: return "cce";
    case kQEESyst: return "qee";
    case kUCESyst: return "uce";
    default: assert(0 && "Unknown syst");
    }
  }

  //----------------------------------------------------------------------
  std::string LatexName(Syst s)
  {
    assert(s != kGENIESystInit && s != kGENIESystEnd);
    if(s > kGENIESystInit && s < kGENIESystEnd){
      // We're going to have to add prettier versions of at least the most
      // popular ones
      return genie::rew::GSyst::AsString(genie::rew::EGSyst(s-kGENIESystInit));
    }

    switch(s){
    case kNormSyst: return "Normalization";
    case kCaloESyst: return "Calorimetric energy";
    case kNCScaleSyst: return "Neutral current scale";
    case kCCESyst: return "CC Contained Energy";
    case kQEESyst: return "QE Contained Energy";
    case kUCESyst: return "UC Contained Energy";
    default: assert(0 && "Unknown syst");
    }
  }

  //----------------------------------------------------------------------
  SystShifts::SystShifts()
  {
  }

  //----------------------------------------------------------------------
  void SystShifts::SetShift(Syst syst, double shift)
  {
    if(shift == 0){
      auto it = fShifts.find(syst);
      if(it != fShifts.end()) fShifts.erase(it);
    }
    else{
      fShifts[syst] = shift;
    }
  }

  //----------------------------------------------------------------------
  double SystShifts::GetShift(Syst syst) const
  {
    auto it = fShifts.find(syst);
    if(it == fShifts.end()) return 0;
    return it->second;
  }

  //----------------------------------------------------------------------
  void SystShifts::ResetToNominal()
  {
    fShifts.clear();
  }

  //----------------------------------------------------------------------
  double SystShifts::Penalty() const
  {
    double ret = 0;
    // Systematics are all expressed in terms of sigmas
    for(auto it: fShifts) ret += util::sqr(it.second);
    return ret;
  }
}
