#pragma once

#include "NuReweight/ReweightLabels.h"

namespace ana
{
  // TODO: the degree of copy-and paste from NuReweight into here is
  // regretable. But I don't see how else to do it while keeping a single
  // unified type for GENIE reweights and everything else. Further thought is
  // required...

  /// Enumeration of available systematic parameters
  enum Syst{
    kNormSyst,
    kCaloESyst,
    kNCScaleSyst,
    kCCESyst,
    kQEESyst,
    kUCESyst,


    kGENIESystInit, // All GENIE systs must come after this


    // NCEL tweaking parameters:
    kReweightMaNCEL  = kGENIESystInit + rwgt::fReweightMaNCEL,
    kReweightEtaNCEL = kGENIESystInit + rwgt::fReweightEtaNCEL,

    // CCQE tweaking parameters:
    kReweightNormCCQE     = kGENIESystInit + rwgt::fReweightNormCCQE,
    kReweightNormCCQEenu  = kGENIESystInit + rwgt::fReweightNormCCQEenu,
    kReweightMaCCQEshape  = kGENIESystInit + rwgt::fReweightMaCCQEshape,
    kReweightMaCCQE       = kGENIESystInit + rwgt::fReweightMaCCQE,
    kReweightVecCCQEshape = kGENIESystInit + rwgt::fReweightVecCCQEshape,

    // Resonance neutrino-production tweaking parameters:
    kReweightNormCCRES    = kGENIESystInit + rwgt::fReweightNormCCRES,
    kReweightMaCCRESshape = kGENIESystInit + rwgt::fReweightMaCCRESshape,
    kReweightMvCCRESshape = kGENIESystInit + rwgt::fReweightMvCCRESshape,
    kReweightMaCCRES      = kGENIESystInit + rwgt::fReweightMaCCRES,
    kReweightMvCCRES      = kGENIESystInit + rwgt::fReweightMvCCRES,

    kReweightNormNCRES    = kGENIESystInit + rwgt::fReweightNormNCRES,
    kReweightMaNCRESshape = kGENIESystInit + rwgt::fReweightMaNCRESshape,
    kReweightMvNCRESshape = kGENIESystInit + rwgt::fReweightMvNCRESshape,
    kReweightMaNCRES      = kGENIESystInit + rwgt::fReweightMaNCRES,
    kReweightMvNCRES      = kGENIESystInit + rwgt::fReweightMvNCRES,

    // Coherent pion production tweaking parameters:
    kReweightMaCOHpi = kGENIESystInit + rwgt::fReweightMaCOHpi,
    kReweightR0COHpi = kGENIESystInit + rwgt::fReweightR0COHpi,

    // Non-resonance background tweaking parameters:
    kReweightRvpCC1pi    = kGENIESystInit + rwgt::fReweightRvpCC1pi,
    kReweightRvpCC2pi    = kGENIESystInit + rwgt::fReweightRvpCC2pi,
    kReweightRvpNC1pi    = kGENIESystInit + rwgt::fReweightRvpNC1pi,
    kReweightRvpNC2pi    = kGENIESystInit + rwgt::fReweightRvpNC2pi,
    kReweightRvnCC1pi    = kGENIESystInit + rwgt::fReweightRvnCC1pi,
    kReweightRvnCC2pi    = kGENIESystInit + rwgt::fReweightRvnCC2pi,
    kReweightRvnNC1pi    = kGENIESystInit + rwgt::fReweightRvnNC1pi,
    kReweightRvnNC2pi    = kGENIESystInit + rwgt::fReweightRvnNC2pi,
    kReweightRvbarpCC1pi = kGENIESystInit + rwgt::fReweightRvbarpCC1pi,
    kReweightRvbarpCC2pi = kGENIESystInit + rwgt::fReweightRvbarpCC2pi,
    kReweightRvbarpNC1pi = kGENIESystInit + rwgt::fReweightRvbarpNC1pi,
    kReweightRvbarpNC2pi = kGENIESystInit + rwgt::fReweightRvbarpNC2pi,
    kReweightRvbarnCC1pi = kGENIESystInit + rwgt::fReweightRvbarnCC1pi,
    kReweightRvbarnCC2pi = kGENIESystInit + rwgt::fReweightRvbarnCC2pi,
    kReweightRvbarnNC1pi = kGENIESystInit + rwgt::fReweightRvbarnNC1pi,
    kReweightRvbarnNC2pi = kGENIESystInit + rwgt::fReweightRvbarnNC2pi,

    // DIS tweaking parameters - applied for DIS events with (Q2>Q2o, W>Wo),
    // typically Q2okReweight =1GeV^2, WokReweight =1.7-2.0GeV
    kReweightAhtBY       = kGENIESystInit + rwgt::fReweightAhtBY,
    kReweightBhtBY       = kGENIESystInit + rwgt::fReweightBhtBY,
    kReweightCV1uBY      = kGENIESystInit + rwgt::fReweightCV1uBY,
    kReweightCV2uBY      = kGENIESystInit + rwgt::fReweightCV2uBY,
    kReweightAhtBYshape  = kGENIESystInit + rwgt::fReweightAhtBYshape,
    kReweightBhtBYshape  = kGENIESystInit + rwgt::fReweightBhtBYshape,
    kReweightCV1uBYshape = kGENIESystInit + rwgt::fReweightCV1uBYshape,
    kReweightCV2uBYshape = kGENIESystInit + rwgt::fReweightCV2uBYshape,
    kReweightNormDISCC   = kGENIESystInit + rwgt::fReweightNormDISCC,
    kReweightRnubarnuCC  = kGENIESystInit + rwgt::fReweightRnubarnuCC,
    kReweightDISNuclMod  = kGENIESystInit + rwgt::fReweightDISNuclMod,

    //
    kReweightNC = kGENIESystInit + rwgt::fReweightNC,

    // Hadronization (free nucleon target)
    kReweightAGKY_xF1pi = kGENIESystInit + rwgt::fReweightAGKY_xF1pi,
    kReweightAGKY_pT1pi = kGENIESystInit + rwgt::fReweightAGKY_pT1pi,

    // Medium-effects to hadronization
    kReweightFormZone = kGENIESystInit + rwgt::fReweightFormZone,

    // Intranuclear rescattering systematics.
    // There are 2 sets of parameters:
    // - parameters that control the total rescattering probability, P(total)
    // - parameters that control the fraction of each process (`fate'), given a total rescat. prob., P(fate|total)
    // These parameters are considered separately for pions and nucleons.
    kReweightMFP_pi      = kGENIESystInit + rwgt::fReweightMFP_pi,
    kReweightMFP_N       = kGENIESystInit + rwgt::fReweightMFP_N,
    kReweightFrCEx_pi    = kGENIESystInit + rwgt::fReweightFrCEx_pi,
    kReweightFrElas_pi   = kGENIESystInit + rwgt::fReweightFrElas_pi,
    kReweightFrInel_pi   = kGENIESystInit + rwgt::fReweightFrInel_pi,
    kReweightFrAbs_pi    = kGENIESystInit + rwgt::fReweightFrAbs_pi,
    kReweightFrPiProd_pi = kGENIESystInit + rwgt::fReweightFrPiProd_pi,
    kReweightFrCEx_N     = kGENIESystInit + rwgt::fReweightFrCEx_N,
    kReweightFrElas_N    = kGENIESystInit + rwgt::fReweightFrElas_N,
    kReweightFrInel_N    = kGENIESystInit + rwgt::fReweightFrInel_N,
    kReweightFrAbs_N     = kGENIESystInit + rwgt::fReweightFrAbs_N,
    kReweightFrPiProd_N  = kGENIESystInit + rwgt::fReweightFrPiProd_N,

    // Nuclear model
    kReweightCCQEPauliSupViaKF   = kGENIESystInit + rwgt::fReweightCCQEPauliSupViaKF,
    kReweightCCQEMomDistroFGtoSF = kGENIESystInit + rwgt::fReweightCCQEMomDistroFGtoSF,

    // Resonance decays
    kReweightBR1gamma        = kGENIESystInit + rwgt::fReweightBR1gamma,
    kReweightBR1eta          = kGENIESystInit + rwgt::fReweightBR1eta,
    kReweightTheta_Delta2Npi = kGENIESystInit + rwgt::fReweightTheta_Delta2Npi,


    kGENIESystEnd, // Must come after all GENIE systs
  };
} // namespace
