#include "CAFAna/TrivialExtrap.h"

#include "CAFAna/Cuts.h"
#include "CAFAna/Loaders.h"

#include "TDirectory.h"
#include "TObjString.h"

namespace ana
{
  //----------------------------------------------------------------------
  TrivialExtrap::TrivialExtrap(Loaders& loaders,
                               std::string label,
                               const Binning& bins,
                               const Var& var,
                               const Cut& cut,
                               const Var& wei)
    : TrivialExtrap(loaders, HistAxis(label, bins, var), cut, wei)
  {
  }

  //----------------------------------------------------------------------
  TrivialExtrap::TrivialExtrap(Loaders& loaders,
                               const HistAxis& axis,
                               const Cut& cut,
                               const Var& wei)
    : TrivialExtrap(loaders.GetLoader(caf::kFARDET, Loaders::kMC, ana::kBeam, Loaders::kNonSwap),
                    loaders.GetLoader(caf::kFARDET, Loaders::kMC, ana::kBeam, Loaders::kFluxSwap),
                    axis.label, axis.bins, axis.var, cut, wei)
  {
  }

  //----------------------------------------------------------------------
  TrivialExtrap::TrivialExtrap(SpectrumLoaderBase& loader,
                               SpectrumLoaderBase& loaderSwap,
                               std::string label,
                               const Binning& bins,
                               const Var& var,
                               const Cut& cut,
                               const Var& wei) :
    fNueApp      (label, bins, loaderSwap, var, cut && kIsSig     && !kIsAntiNu, wei),
    fNueAppAnti  (label, bins, loaderSwap, var, cut && kIsSig     &&  kIsAntiNu, wei),
    fNumuSurv    (label, bins, loader,     var, cut && kIsNumuCC  && !kIsAntiNu, wei),
    fNumuSurvAnti(label, bins, loader,     var, cut && kIsNumuCC  &&  kIsAntiNu, wei),
    fNumuApp     (label, bins, loaderSwap, var, cut && kIsNumuApp && !kIsAntiNu, wei),
    fNumuAppAnti (label, bins, loaderSwap, var, cut && kIsNumuApp &&  kIsAntiNu, wei),
    fNueSurv     (label, bins, loader,     var, cut && kIsBeamNue && !kIsAntiNu, wei),
    fNueSurvAnti (label, bins, loader,     var, cut && kIsBeamNue &&  kIsAntiNu, wei),
    fNC          (label, bins, loader,     var, cut && kIsNC,                    wei)
  {
    // Swapped files are just as valid a source of NCs as unswapped. This
    // approximately doubles our statistics. SpectrumLoader just adds events
    // and POT for both cases, which is the right thing to do.
    loaderSwap.AddSpectrum(fNC, var, cut && kIsNC, wei);
  }

  //----------------------------------------------------------------------
  void TrivialExtrap::SaveTo(TDirectory* dir) const
  {
    TDirectory* tmp = gDirectory;

    dir->cd();

    TObjString("TrivialExtrap").Write("type");

    fNueApp.SaveTo(dir->mkdir("nue_app"));
    fNueAppAnti.SaveTo(dir->mkdir("nue_app_anti"));
    fNC.SaveTo(dir->mkdir("nc"));
    fNumuSurv.SaveTo(dir->mkdir("numu_surv"));
    fNumuSurvAnti.SaveTo(dir->mkdir("numu_surv_anti"));
    fNumuApp.SaveTo(dir->mkdir("numu_app"));
    fNumuAppAnti.SaveTo(dir->mkdir("numu_app_anti"));
    fNueSurv.SaveTo(dir->mkdir("nue_surv"));
    fNueSurvAnti.SaveTo(dir->mkdir("nue_surv_anti"));

    tmp->cd();
  }

  //----------------------------------------------------------------------
  std::unique_ptr<TrivialExtrap> TrivialExtrap::LoadFrom(TDirectory* dir)
  {
    std::unique_ptr<TrivialExtrap> ret(new TrivialExtrap);

    // This is a lot of repetitive typing. Define some macros
#define LOAD_OSC(FIELD, LABEL) assert(dir->GetDirectory(LABEL)); ret->FIELD = *OscillatableSpectrum::LoadFrom(dir->GetDirectory(LABEL));
#define LOAD_SPECT(FIELD, LABEL) assert(dir->GetDirectory(LABEL)); ret->FIELD = *Spectrum::LoadFrom(dir->GetDirectory(LABEL));

    LOAD_OSC(fNueApp,       "nue_app");
    LOAD_OSC(fNueAppAnti,   "nue_app_anti");
    LOAD_OSC(fNumuSurv,     "numu_surv");
    LOAD_OSC(fNumuSurvAnti, "numu_surv_anti");
    LOAD_OSC(fNumuApp,      "numu_app");
    LOAD_OSC(fNumuAppAnti,  "numu_app_anti");
    LOAD_OSC(fNueSurv,      "nue_surv");
    LOAD_OSC(fNueSurvAnti,  "nue_surv_anti");

    LOAD_SPECT(fNC, "nc");

    return ret;
  }
}
