#pragma once

#include "CAFAna/Extrap.h"

namespace ana
{
  class Loaders;

  /// "Extrapolation" that simply returns the FD MC prediction
  class TrivialExtrap: public IExtrap
  {
  public:
    TrivialExtrap(Loaders& loaders,
                  std::string label,
                  const Binning& bins,
                  const Var& var,
                  const Cut& cut = kNoCut,
                  const Var& wei = kUnweighted);

    TrivialExtrap(Loaders& loaders,
                  const HistAxis& axis,
                  const Cut& cut = kNoCut,
                  const Var& wei = kUnweighted);

    TrivialExtrap(SpectrumLoaderBase& loader,
                  SpectrumLoaderBase& loaderSwap,
                  std::string label,
                  const Binning& bins,
                  const Var& var,
                  const Cut& cut = kNoCut,
                  const Var& wei = kUnweighted);

    virtual OscillatableSpectrum NueSurvComponent()      {return fNueSurv;}
    virtual OscillatableSpectrum AntiNueSurvComponent()  {return fNueSurvAnti;}

    virtual OscillatableSpectrum NumuSurvComponent()     {return fNumuSurv;}
    virtual OscillatableSpectrum AntiNumuSurvComponent() {return fNumuSurvAnti;}

    virtual OscillatableSpectrum NueAppComponent()       {return fNueApp;}
    virtual OscillatableSpectrum AntiNueAppComponent()   {return fNueAppAnti;}

    virtual OscillatableSpectrum NumuAppComponent()      {return fNumuApp;}
    virtual OscillatableSpectrum AntiNumuAppComponent()  {return fNumuAppAnti;}

    virtual Spectrum NCComponent() {return fNC;}

    virtual void SaveTo(TDirectory* dir) const;
    static std::unique_ptr<TrivialExtrap> LoadFrom(TDirectory* dir);

  protected:
    TrivialExtrap()
      : fNueApp(0, 0),   fNueAppAnti(0, 0),
        fNumuSurv(0, 0), fNumuSurvAnti(0, 0),
        fNumuApp(0, 0),  fNumuAppAnti(0, 0),
        fNueSurv(0, 0),  fNueSurvAnti(0, 0),
        fNC(0, 0)
    {}

    OscillatableSpectrum fNueApp,   fNueAppAnti;
    OscillatableSpectrum fNumuSurv, fNumuSurvAnti;
    OscillatableSpectrum fNumuApp,  fNumuAppAnti;
    OscillatableSpectrum fNueSurv,  fNueSurvAnti;
    Spectrum fNC;
  };
}
