#pragma once

// Specific variables. When this file gets too crowded it should be split up

// See Cuts.h for a walkthrough of how to define this kind of object

#include "Var.h"

#include "StandardRecord/StandardRecord.h"

#include <cassert>

namespace ana
{
  /// \f$ \nu_e \f$ PID
  const Var kLEM({"sel.lem.pid"},
                 [](const caf::StandardRecord* sr){return sr->sel.lem.pid;});

  /// \f$ \nu_e \f$ PID
  const Var kElecID({"sel.elecid.ann"},
		 [](const caf::StandardRecord* sr){return sr->sel.elecid.ann;});

  /// \f$ \nu_e \f$ PID
  const Var kRVP({"sel.rvp.pid"},
		 [](const caf::StandardRecord* sr){return sr->sel.rvp.pid;});

  /// Summed calorimetric energy of all hits in slice
  const Var kCaloE({"slc.calE"},
                   [](const caf::StandardRecord* sr){return 2*sr->slc.calE;});

  /// \f$ \nu_\mu \f$ PID
  const Var kRemID({"sel.remid.pid"},
                   [](const caf::StandardRecord* sr){return sr->sel.remid.pid;});

  /// Energy estimator for CC events
  const Var kCCE({"energy.numusimp.trknonqeE"},
                 [](const caf::StandardRecord* sr)
                 {return sr->energy.numusimp.trknonqeE;});

  /// Energy estimator for quasielastic CC events
  const Var kQEE({"energy.numusimp.trkqeE"},
                 [](const caf::StandardRecord* sr)
                 {return sr->energy.numusimp.trkqeE;});

  /// Energy estimator for uncontained CC events
  const Var kUCE({"energy.numusimp.ucccE"},
                 [](const caf::StandardRecord* sr)
                 {return sr->energy.numusimp.ucccE;});

  /// True neutrino energy
  const Var kTrueE({"mc.nu", "mc.nnu", "mc.nu.E"},
                   [](const caf::StandardRecord* sr)
                   {return (sr->mc.nnu == 0) ? 0 : sr->mc.nu[0].E;});
} // namespace
