////////////////////////////////////////////////////////////////////////
/// \brief   cmap::CMap class source code
/// \author  brebel@fnal.gov
/// \date    Sept. 2010
////////////////////////////////////////////////////////////////////////
#include "CMap/CMap.h"

// C/C++ includes
#include <string>

// NOvA includes
#include "RawData/RawDigit.h"
#include "SummaryData/RunData.h"
#include "NovaDAQConventions/DAQConventions.h"


// ART includes
#include "cetlib/search_path.h"

namespace cmap {

  //----------------------------------------------------------------------
  CMap::CMap(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg) :
    fRun(0),
    fDetID(novadaq::cnv::kUNKNOWN_DET)
  {
    this->reconfigure(pset);
    reg.sPostBeginRun.watch(this, &CMap::postBeginRun);
  }

  //----------------------------------------------------------------------  
  bool CMap::TestDetID(int detid, bool force) 
  {
    if (force) fDetID = detid;
    return (fDetID == detid);
  }
  
  //----------------------------------------------------------------------  
  CMap::~CMap() { }

  //----------------------------------------------------------------------  
  void CMap::reconfigure(fhicl::ParameterSet )
  {
    return;
  }

  //----------------------------------------------------------------------  
  void CMap::postBeginRun(art::Run const& run)
  {
    cet::search_path sp("FW_SEARCH_PATH");

    std::string fname = "CMap/";
    
    //
    // insert check hear to see if we need to load a new geometry.
    // get the detector id from the run object
    //
    std::vector< art::Handle<sumdata::RunData> > rdcol;
    run.getManyByType(rdcol);
    assert(!rdcol.empty());

    const int detId = rdcol[0]->DetId();
    this->TestDetID(rdcol[0]->DetId(), true);

    if      (detId == novadaq::cnv::kNEARDET) fname += "cmapnd.xml"; 
    else if (detId == novadaq::cnv::kFARDET)  fname += "cmapfd.xml";
    else if (detId == novadaq::cnv::kNDOS)    fname += "cmapipnd.xml";

    sp.find_file(fname, fXMLFile);

    return;
  }

  //----------------------------------------------------------------------
    unsigned short CMap::GetPlane(const rawdata::RawDigit* dig) const
  {
    if (!dig) return 30000;

    const daqchannelmap::lchan local_channel  = dig->Channel();

    return this->Map()->getPlane(local_channel);
    
  }
  
  //----------------------------------------------------------------------
  unsigned short CMap::GetCell(const rawdata::RawDigit* dig) const
  {
    if (!dig) return 30000;
    
    const daqchannelmap::lchan local_channel = dig->Channel();

    return this->Map()->getCell(local_channel);
  }
  
  //----------------------------------------------------------------------
  bool CMap::ChanToCellException(unsigned short dcs_id, 
				 unsigned short feb_id,
				 unsigned short chan_id, 
				 unsigned short& plane, 
				 unsigned short& cell)
  {
    /*
    int did, cid, fid;
    
    // look for exceptions in the table... if found, set plane/cell 
    // according to table entries
    for (unsigned int i=0; i<this->size(); ++i) {
      dbi::Row& row = (*this)[i];
      row.Get(std::string("dcs_id"),did);
      row.Get(std::string("chan_id"),cid);
      row.Get(std::string("chan_id"),fid);
      if (did == dcs_id && cid == chan_id && fid == feb_id) {
	row.Get(std::string("plane"),plane);
	row.Get(std::string("cell"),cell);
	return true;
      }
    }
    */    
    return false;
  }
  
  //----------------------------------------------------------------------
  
  bool CMap::CellToChanException(unsigned short plane, 
				 unsigned short cell,
				 unsigned short& dcs_id, 
				 unsigned short& feb_id,
				 unsigned short& chan_id)
  {
    /*
    int pl, ce;
    
    // look for exceptions in the table... if found, set plane/cell 
    // according to table entries
    for (unsigned int i=0; i<this->size(); ++i) {
      dbi::Row& row = (*this)[i];
      row.Get(std::string("plane"),pl);
      row.Get(std::string("cell"),ce);
      if (pl == plane && ce == cell) {
	row.Get(std::string("dcs_id"),dcs_id);
	row.Get(std::string("feb_id"),feb_id);
	row.Get(std::string("chan_id"),chan_id);
	return true;
      }
    }
    */    
    return false;
  }

  DEFINE_ART_SERVICE(CMap);

} // end namespace cmap
////////////////////////////////////////////////////////////////////////

