////////////////////////////////////////////////////////////////////////
// Class:       DCMTimingOffset
// Module Type: producer
// File:        DCMTimingOffset_module.cc
//
// Generated at Mon Feb 11 09:01:32 2013 by Evan Niner using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

//NOvASoft includes
#include "Geometry/Geometry.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "Utilities/func/MathUtil.h"
#include "CalibrationDataProducts/PCHit.h"
#include "CalibrationDataProducts/TCTrack.h"
#include "CalibrationDataProducts/DCMSummary.h"
#include "CalibrationDataProducts/DCMOffset.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "RunHistory/service/RunHistoryService.h"
#include "DAQChannelMap/DAQChannelMap.h"

//framework includes
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Core/FindManyP.h"
#include "Utilities/AssociationUtil.h"
#include "cetlib/exception.h"
#include "art/Persistency/Provenance/Timestamp.h"

#include <stdint.h>
#include <sstream>
#include <map>
#include <iterator>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <string>

//ROOT includes
#include "TTree.h"
#include "TH2F.h"
#include "TH1F.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"
#include "TDecompLU.h"
#include "TDirectory.h"

namespace calib {

  class DCMTimingOffset : public art::EDFilter {
  public:
    explicit DCMTimingOffset(fhicl::ParameterSet const & p);
    virtual ~DCMTimingOffset();

    //void produce(art::Event & e) override;
    virtual bool filter(art::Event& evt) {return false;}

    void reconfigure(const fhicl::ParameterSet& pset);
    void beginJob();
    void endJob();
    virtual bool beginRun(art::Run& run);
    virtual bool endSubRun(art::SubRun& srun);
    virtual bool endRun(art::Run & r);
    //virtual void respondToCloseOutputFiles(art::FileBlock const& fb);


  private:

    std::string   fDCMLabel;        ///< Where to find dcm stats
    int           fNumDCM;          ///< Number of DCMs calibration will be performed on
    int           fMinDCM;          ///< Number of lowest DCM in calibration set
    int           fMaxDCM;          ///< Number of highest DCM in calibration set
    int           fMinCount;        ///< Minimum number of counts for a DCM pair
    bool          fOverrideRef; ///<override default of fixing calibration of dcm 6 with an offset of 0
    int           fRefDCM = 6;    ///< DCM number to use as refence, default is 6
    float         fRefOffset = 0.0;   ///<absolute offset, in ns, to fix refence DCM to, default is 0.0
    bool          fIsMC;
    

    //keep stats for entire group of runs
    std::vector<std::vector<int> >    countDCMOffsetArray;
    std::vector<std::vector<float> > sumSquareDCMOffsetArray;
    std::vector<std::vector<float> > sumDCMOffsetArray;
    std::vector<std::vector<float> > meanDCMOffsetArray;
    std::vector<std::vector<float> > weightDCMOffsetArray;
    std::vector<float>               absoluteDCMOffset;
    std::vector<float>               absoluteDCMErrors;
    std::vector<int>                  countDCMUse;
    uint32_t minTime;
    uint32_t maxTime;

    //keep stats for an individual run
    std::vector<std::vector<int> >    countDCMOffsetArrayRun;
    std::vector<std::vector<float> > sumSquareDCMOffsetArrayRun;
    std::vector<std::vector<float> > sumDCMOffsetArrayRun;
    std::vector<std::vector<float> > meanDCMOffsetArrayRun;
    std::vector<std::vector<float> > weightDCMOffsetArrayRun;
    std::vector<float>               absoluteDCMOffsetRun;
    std::vector<float>               absoluteDCMErrorsRun;
    std::vector<int>                  countDCMUseRun;
    uint32_t minTimeRun;
    uint32_t maxTimeRun;
    
    std::map<int, std::vector<caldp::DCMOffset> > offsetMap;

  };
}

namespace calib {

  //define speed of light in cm/ns
  static const double sCmPerNsec = 29.9792458;

  DCMTimingOffset::DCMTimingOffset(fhicl::ParameterSet const & pset)
  // :
  // Initialize member data here.
  {
    this->reconfigure(pset); 

    produces< std::vector<caldp::DCMOffset>, art::InRun >();
    produces< caldp::DCMSummary, art::InRun >();

  }

//---------------------------------------------------------------------------
  void DCMTimingOffset::reconfigure(const fhicl::ParameterSet& pset)
  {
    fDCMLabel  = pset.get< std::string >("DCMLabel");
    fMinDCM    = pset.get< int         >("MinDCM");
    fMaxDCM    = pset.get< int         >("MaxDCM");
    fMinCount  = pset.get< int         >("MinCount");
    fOverrideRef = pset.get< bool >("OverrideRef");

    if (fOverrideRef){
      fRefDCM = pset.get< int >("RefDCM");
      fRefOffset = pset.get< float >("RefOffset");
    }

    assert(fMaxDCM >= fMinDCM);
    assert((fRefDCM >= fMinDCM) && (fRefDCM <= fMaxDCM));
    fNumDCM = fMaxDCM - fMinDCM + 1;

    countDCMOffsetArray.resize(fNumDCM);
    sumSquareDCMOffsetArray.resize(fNumDCM);
    sumDCMOffsetArray.resize(fNumDCM);
    meanDCMOffsetArray.resize(fNumDCM);
    weightDCMOffsetArray.resize(fNumDCM);
    absoluteDCMOffset.resize(fNumDCM);
    absoluteDCMErrors.resize(fNumDCM);
    countDCMUse.resize(fNumDCM);

    for (int i=0; i<fNumDCM; ++i){
      countDCMOffsetArray[i].resize(fNumDCM);
      sumSquareDCMOffsetArray[i].resize(fNumDCM);
      sumDCMOffsetArray[i].resize(fNumDCM);
      meanDCMOffsetArray[i].resize(fNumDCM);
      weightDCMOffsetArray[i].resize(fNumDCM);
    }
    

  }  

  //---------------------------------------------------------------------------
  DCMTimingOffset::~DCMTimingOffset()
  {
    // Clean up dynamic memory and other resources here.
  }

  //---------------------------------------------------------------------------
  void DCMTimingOffset::beginJob()
  {
    //We want to be able to handle any time possible.
    minTime =  std::numeric_limits<uint32_t>::max()-1;
    maxTime =  std::numeric_limits<uint32_t>::min();
  }

  //---------------------------------------------------------------------------
  bool DCMTimingOffset::beginRun(art::Run& run)
  {
    art::ServiceHandle<nova::dbi::RunHistoryService> rh;
    fIsMC = (rh->GetDataType() == nova::dbi::kMCOnly);

    minTimeRun =  std::numeric_limits<uint32_t>::max()-1;
    maxTimeRun =  std::numeric_limits<uint32_t>::min();

    countDCMOffsetArrayRun.clear();
    sumSquareDCMOffsetArrayRun.clear();
    sumDCMOffsetArrayRun.clear();
    meanDCMOffsetArrayRun.clear();
    weightDCMOffsetArrayRun.clear();
    absoluteDCMOffsetRun.clear();
    absoluteDCMErrorsRun.clear();
    countDCMUseRun.clear();

    countDCMOffsetArrayRun.resize(fNumDCM);
    sumSquareDCMOffsetArrayRun.resize(fNumDCM);
    sumDCMOffsetArrayRun.resize(fNumDCM);
    meanDCMOffsetArrayRun.resize(fNumDCM);
    weightDCMOffsetArrayRun.resize(fNumDCM);
    absoluteDCMOffsetRun.resize(fNumDCM);
    absoluteDCMErrorsRun.resize(fNumDCM);
    countDCMUseRun.resize(fNumDCM);

    for (int i=0; i<fNumDCM; ++i){
      countDCMOffsetArrayRun[i].resize(fNumDCM);
      sumSquareDCMOffsetArrayRun[i].resize(fNumDCM);
      sumDCMOffsetArrayRun[i].resize(fNumDCM);
      meanDCMOffsetArrayRun[i].resize(fNumDCM);
      weightDCMOffsetArrayRun[i].resize(fNumDCM);
    }

    return true;
  }

  //---------------------------------------------------------------------------
  bool DCMTimingOffset::endSubRun(art::SubRun& srun)
  {
    //get out summary dataproduct
    //get the cosmic tracks associated with each event
    art::Handle<caldp::DCMSummary > dcmsum;
    srun.getByLabel(fDCMLabel,dcmsum);

    for (int j=0; j<fNumDCM; ++j){
      int jabs = j + fMinDCM - 1;
      countDCMUseRun[j] += dcmsum->DCMUseIndx(jabs);
      countDCMUse[j] += dcmsum->DCMUseIndx(jabs);
      for (int k=j+1; k<fNumDCM; ++k){
        int kabs = k +fMinDCM - 1;
        sumDCMOffsetArrayRun[j][k]+= dcmsum->DCMOffsetSumIndx(jabs,kabs);
        sumDCMOffsetArray[j][k]+= dcmsum->DCMOffsetSumIndx(jabs,kabs);
        sumSquareDCMOffsetArrayRun[j][k] += dcmsum->DCMOffsetSumSquareIndx(jabs,kabs);
        sumSquareDCMOffsetArray[j][k] += dcmsum->DCMOffsetSumSquareIndx(jabs,kabs);
        countDCMOffsetArrayRun[j][k] += dcmsum->DCMOffsetCountIndx(jabs,kabs);
        countDCMOffsetArray[j][k] += dcmsum->DCMOffsetCountIndx(jabs,kabs);
      }
    }
 
    uint32_t min = dcmsum->StartTime();
    uint32_t max = dcmsum->EndTime();
    if (max>maxTime) maxTime = max;
    if (min<minTime) minTime = min;
    if (max>maxTimeRun) maxTimeRun = max;
    if (min<minTimeRun) minTimeRun = min;
      

    return false;
  }



  //---------------------------------------------------------------------------
  bool DCMTimingOffset::endRun(art::Run& run)
  {

    //Ptr to DriftResp information:
    std::unique_ptr<std::vector<caldp::DCMOffset > >runOffsets(new std::vector<caldp::DCMOffset >);

    caldp::DCMSummary runsum;
    runsum.SetStartTime(minTimeRun);
    runsum.SetEndTime(maxTimeRun);

    for(int i=0; i<fNumDCM-1; ++i){
      int iabs = i + fMinDCM - 1;
      runsum.SetDCMUseIndx(iabs, countDCMUseRun[i]);
      for(int j=i+1; j<fNumDCM; ++j){
        int jabs = j + fMinDCM - 1;
        float sum = sumDCMOffsetArrayRun[i][j];
        int count = countDCMOffsetArrayRun[i][j];
        float sumSqr = sumSquareDCMOffsetArrayRun[i][j];
        runsum.SetDCMOffsetSumIndx(iabs,jabs,sum);
        runsum.SetDCMOffsetSumSquareIndx(iabs,jabs,sumSqr);
        runsum.SetDCMOffsetCountIndx(iabs,jabs,count);
        if (count >= fMinCount){
          meanDCMOffsetArrayRun[i][j] = sum/count;
          float sigma = sqrt((count*sumSqr - sum*sum)/(count*(count-1.0)));
          weightDCMOffsetArrayRun[i][j] = 1.0/(sigma*sigma);
        }
        else {
          meanDCMOffsetArrayRun[i][j] = 0;
          weightDCMOffsetArrayRun[i][j] = 0;
        }
      }
    }

    const int num= fNumDCM;
    int refIndx = fRefDCM - fMinDCM;
    TMatrixD A;
    A.ResizeTo(fNumDCM, fNumDCM);
    double y[num];
    for (int x=0; x<num; ++x) y[x]=0.;


    for(int i=0; i<fNumDCM; ++i){
      double temp[num];
      for (int x=0; x<num; ++x) temp[x]=0.;
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i){
          temp[i] += weightDCMOffsetArrayRun[j][i]*2.0;
          temp[j] += -weightDCMOffsetArrayRun[j][i]*2.0;
          y[i] += meanDCMOffsetArrayRun[j][i]*weightDCMOffsetArrayRun[j][i]*2.0; 
        }
        if (j>i){
          temp[i] += weightDCMOffsetArrayRun[i][j]*2.0;
          temp[j] += -weightDCMOffsetArrayRun[i][j]*2.0;
          y[i] += -meanDCMOffsetArrayRun[i][j]*weightDCMOffsetArrayRun[i][j]*2.0;
        }
      }

      if (i == refIndx){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndx]=1.0;
      } 

      y[i]*=-1.0;
      TVectorD tempvec;
      tempvec.Use(fNumDCM,temp);
      TMatrixDRow(A,i) = tempvec;
    }

    y[refIndx] = fRefOffset;
    TVectorD yvec;
    yvec.Use(fNumDCM,y);

    //calculate uncertainties
    for(int i=0; i<fNumDCM; ++i){
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i) absoluteDCMErrorsRun[i] += 2.0*weightDCMOffsetArrayRun[j][i];
        else if (j>i) absoluteDCMErrorsRun[i] += 2.0*weightDCMOffsetArrayRun[i][j];
      }
      absoluteDCMErrorsRun[i] = sqrt(2.0/absoluteDCMErrorsRun[i]);
    }
    absoluteDCMErrorsRun[refIndx] = 0.0;

    //solve for absolute offsets
    TDecompLU lu(A);
    bool inv = lu.Decompose();
    if (inv) lu.Invert(A);
    //std::cout<<A.IsValid()<<std::endl;
    if (inv) yvec *= A;

    for (int i=0; i<fNumDCM; ++i){
      if (!inv) break;
      caldp::DCMOffset dcmOffset;
      dcmOffset.SetDCM(i+fMinDCM);
      dcmOffset.SetOffset(yvec[i]);
      dcmOffset.SetSigma(absoluteDCMErrorsRun[i]);
      dcmOffset.SetStats(countDCMUseRun[i]);
      dcmOffset.SetStartTime(minTimeRun);
      dcmOffset.SetEndTime(maxTimeRun);
      runOffsets->push_back(dcmOffset);
      offsetMap[run.run()].push_back(dcmOffset);
    }

    countDCMOffsetArrayRun.clear();
    sumSquareDCMOffsetArrayRun.clear();
    sumDCMOffsetArrayRun.clear();
    meanDCMOffsetArrayRun.clear();
    weightDCMOffsetArrayRun.clear();
    absoluteDCMOffsetRun.clear();
    absoluteDCMErrorsRun.clear();
    countDCMUseRun.clear();

    std::unique_ptr<caldp::DCMSummary> dsum(new caldp::DCMSummary(runsum));
    run.put(std::move(dsum));

    run.put(std::move(runOffsets));

    return true;
  }

  //---------------------------------------------------------------------------
  void DCMTimingOffset::endJob()
  {
    //make summary plots of calibration performance
    art::ServiceHandle<art::TFileService> tfs;

    art::ServiceHandle<geo::Geometry>         geom;
    daqchannelmap::DAQChannelMap* cmap = daqchannelmap::DAQChannelMap::getInstance(geom->DetId());
    const int NDCMDB = cmap->getNumberOfDCM(daqchannelmap::AFULLAFULL_DIBLOCK);

    const int runs = offsetMap.size();
    const int num= fNumDCM;
    TGraphErrors* grAbsDCM[num];
    TGraphErrors* grAbsRuns[runs];
    TGraph* grStatsDCM[num];
    TGraph* grStatsRuns[runs];
    TGraphErrors* grAbs;
    TGraph* grStats;
    TH2F* fHistoRelMap;

    double runDCM[num][runs];
    double runDCMOffset[num][runs];
    double runDCMOffsetE[num][runs];
    double runDCMStats[num][runs];

    grAbs = tfs->make<TGraphErrors>();
    gDirectory->Append(grAbs);
    grStats = tfs->make<TGraph>();
    gDirectory->Append(grStats);
    fHistoRelMap = tfs->make<TH2F>("fHistoRelMap","Relative offsets (ns), all runs, DCM Y- DCM X;dcm X;dcm Y",
                                   fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5,fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5);
    for (int i=0; i< runs; ++i){
      grAbsRuns[i] = tfs->make<TGraphErrors>();
      gDirectory->Append(grAbsRuns[i]); 
      grStatsRuns[i] = tfs->make<TGraph>();
      gDirectory->Append(grStatsRuns[i]); 
    }
    for (int i=0; i< num; ++i){
      grAbsDCM[i] = tfs->make<TGraphErrors>();
      gDirectory->Append(grAbsDCM[i]); 
      grStatsDCM[i] = tfs->make<TGraph>();
      gDirectory->Append(grStatsDCM[i]); 
    }


    for(int i=0; i<fNumDCM-1; ++i){
      for(int j=i+1; j<fNumDCM; ++j){
        float sum = sumDCMOffsetArray[i][j];
        int count = countDCMOffsetArray[i][j];
        float sumSqr = sumSquareDCMOffsetArray[i][j];
        if (count >= fMinCount){
          meanDCMOffsetArray[i][j] = sum/count;
          float sigma = sqrt((count*sumSqr - sum*sum)/(count*(count-1.0)));
          weightDCMOffsetArray[i][j] = 1.0/(sigma*sigma);
          fHistoRelMap->SetBinContent(j+1,i+1,sum/count);
        }
        else {
          meanDCMOffsetArray[i][j] = 0;
          weightDCMOffsetArray[i][j] = 0;
        }
      }
    }


    int refIndx = fRefDCM - fMinDCM;
    TMatrixD A;
    A.ResizeTo(fNumDCM, fNumDCM);
    double y[num];
    for (int x=0; x<num; ++x) y[x]=0.;

    for(int i=0; i<fNumDCM; ++i){
      double temp[num];
      for (int x=0; x<num; ++x) temp[x]=0.;
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i){
          temp[i] += weightDCMOffsetArray[j][i]*2.0;
          temp[j] += -weightDCMOffsetArray[j][i]*2.0;
          y[i] += meanDCMOffsetArray[j][i]*weightDCMOffsetArray[j][i]*2.0; 
        }
        if (j>i){
          temp[i] += weightDCMOffsetArray[i][j]*2.0;
          temp[j] += -weightDCMOffsetArray[i][j]*2.0;
          y[i] += -meanDCMOffsetArray[i][j]*weightDCMOffsetArray[i][j]*2.0;
        }
      }

      if (i == refIndx){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndx]=1.0;
      } 

      y[i]*=-1.0;
      TVectorD tempvec;
      tempvec.Use(fNumDCM,temp);
      TMatrixDRow(A,i) = tempvec;
    }

    y[refIndx] = fRefOffset;
    TVectorD yvec;
    yvec.Use(fNumDCM,y);

    //calculate uncertainties
    for(int i=0; i<fNumDCM; ++i){
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i) absoluteDCMErrors[i] += 2.0*weightDCMOffsetArray[j][i];
        else if (j>i) absoluteDCMErrors[i] += 2.0*weightDCMOffsetArray[i][j];
      }
      absoluteDCMErrors[i] = sqrt(2.0/absoluteDCMErrors[i]);
    }
    absoluteDCMErrors[refIndx] = 0.0;

    //solve for absolute offsets
    A.Invert();
    yvec *= A;
  
    //write results to csv file
    const TString mcStr = fIsMC ? "_mc" : "";
    FILE* fConstsCSV = fopen("calib_abs_dcm_delay_consts"+mcStr+".csv","w");

    fprintf(fConstsCSV, "#dcmoffset,sigma,stats,channel,tv\n");

    //Multipy by -1 since the correction facto is the opposite sign of the offset
    //store channel as diblock*1000+dcm
    for (int i=0; i<fNumDCM; ++i){
      absoluteDCMOffset[i] = yvec[i];
      int db = ((i+fMinDCM)/NDCMDB)+1;
      int dcm = (i+fMinDCM)%NDCMDB;
      if ((i+fMinDCM)%NDCMDB == 0){
        db--;
        dcm = 12;
      }
      int chan = db*1000 + dcm;
      fprintf(fConstsCSV, "%g,%g,%d,%d,%d\n",
              (-1*absoluteDCMOffset[i]), absoluteDCMErrors[i], countDCMUse[i], chan,  minTime);
    }

    fclose(fConstsCSV);

    int runcount= 0;
    for (std::map<int, std::vector<caldp::DCMOffset> >::iterator itr = offsetMap.begin(); itr != offsetMap.end(); ++itr){
      int rnum = itr->first;
      std::vector<caldp::DCMOffset> offsets = itr->second;
      
      TString dummy;
      std::ostringstream convert;
      convert << rnum;
      dummy = convert.str();
      grAbsRuns[runcount]->SetName("grAbsOffset_run_"+dummy);
      grAbsRuns[runcount]->SetTitle("Absolute DCM timing offsets (ns) for run: "+dummy);
      grAbsRuns[runcount]->Set(offsets.size());
      grStatsRuns[runcount]->SetName("grStatsOffset_run_"+dummy);
      grStatsRuns[runcount]->SetTitle("Number of track segments used by dcm for run: "+dummy);
      grStatsRuns[runcount]->Set(offsets.size());
      for (unsigned int i=0; i<offsets.size(); ++i){
        int tdcm = offsets[i].DCM();
        double toffset = offsets[i].Offset();
        double tsigma = offsets[i].Sigma(); 
        int tstats = offsets[i].Stats();
        grStatsRuns[runcount]->SetPoint(i,tdcm,tstats);
        grAbsRuns[runcount]->SetPoint(i,tdcm,toffset);
        grAbsRuns[runcount]->SetPointError(i,0.0,tsigma);
        runDCM[tdcm-fMinDCM][runcount]=rnum;
        runDCMOffset[tdcm-fMinDCM][runcount]=toffset;
        runDCMOffsetE[tdcm-fMinDCM][runcount]=tsigma;
        runDCMStats[tdcm-fMinDCM][runcount]=tstats;
      }
      runcount++;
    }


    for(int i=0; i<num; ++i){
      int tdcm = i + fMinDCM;
      TString dummy;
      std::ostringstream convert;
      convert << tdcm;
      dummy = convert.str();
      grAbsDCM[i]->SetName("grAbsOffset_dcm_"+dummy);
      grAbsDCM[i]->SetTitle("Absolute DCM timing offsets (ns) across all run for dcm: "+dummy);
      grAbsDCM[i]->Set(runs);
      grStatsDCM[i]->SetName("grStatsOffset_dcm_"+dummy);
      grStatsDCM[i]->SetTitle("Number of track segments used for all runs by dcm: "+dummy);
      grStatsDCM[i]->Set(runs);
      for(int j=0; j<runs; ++j){
        grAbsDCM[i]->SetPoint(j,runDCM[i][j],runDCMOffset[i][j]);
        grAbsDCM[i]->SetPointError(j,0.0,runDCMOffsetE[i][j]);
        grStatsDCM[i]->SetPoint(j,runDCM[i][j],runDCMStats[i][j]);
      }
    }

    grAbs->SetName("grAbsOffset_total");
    grAbs->SetTitle("Absolute DCM timing offsets (ns), all runs");
    grAbs->Set(num);
    grStats->SetName("grStatsOffset_total");
    grStats->SetTitle("Number of track segments used by dcm, all runs");
    grStats->Set(num);

    for(int i=0; i< fNumDCM; ++i){
      grAbs->SetPoint(i,i+fMinDCM,absoluteDCMOffset[i]);
      grAbs->SetPointError(i,0.0,absoluteDCMErrors[i]);
      grStats->SetPoint(i,i+fMinDCM,countDCMUse[i]);
    }
    

  }

} //end namespace

//-----------------------------------------------------------------------------

DEFINE_ART_MODULE(calib::DCMTimingOffset)
