///////////////////////////////////////////////////////////////////////
// Class:       RawDigit2DAQHit
// Module Type: producer
// File:        RawDigit2DAQHit_module.cc
//
// Generated at Mon Apr 29 07:46:14 2013 by Matthew Tamsett using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"

#include "CMap/CMap.h"
#include "DAQChannelMap/DAQChannelMap.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/BaseProducts.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "RawData/RawDigit.h"
#include "RawData/RawTrigger.h"
#include "SummaryData/RunData.h"

#include <string>
#include <vector>

namespace raw2daq {
  class RawDigit2DAQHit;
}

//--------------------------------------------------------------------
class raw2daq::RawDigit2DAQHit : public art::EDProducer {
public:
  explicit RawDigit2DAQHit(fhicl::ParameterSet const & p);
  virtual ~RawDigit2DAQHit();
  void produce(art::Event & event) override;
  void beginRun(art::Run & run);

private:
  std::string fRawDigitLabel;  ///< label of where to find Raw Digits
  novadaq::cnv::DetId fDetId;

protected:
  void FillDAQHits(art::Handle< std::vector<rawdata::RawDigit> > & RawDigits,
                   std::unique_ptr<std::vector<novaddt::DAQHit> > & DAQHits,
                   uint64_t event_time
                   );
};
//---------------------------------------------------------------------
raw2daq::RawDigit2DAQHit::RawDigit2DAQHit(fhicl::ParameterSet const & p)
    : fRawDigitLabel  (p.get< std::string >("RawDigitLabel"))
{
  // Call appropriate Produces<>() functions here.
  mf::LogInfo("RawDigit2DAQHit") << "=== raw2daq:::RawDigit2DAQHit instantiate";
  mf::LogInfo("RawDigit2DAQHit") << "\t RawDigitLabel: " << fRawDigitLabel;
  produces< std::vector<novaddt::DAQHit>,   art::InEvent>();

}
//---------------------------------------------------------------------
raw2daq::RawDigit2DAQHit::~RawDigit2DAQHit()
{
  // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
void raw2daq::RawDigit2DAQHit::produce(art::Event & event)
{
  // Implementation of required member function here.
  //mf::LogDebug("RawDigit2DAQHit") << "=== raw2daq:::RawDigit2DAQHit produce. Event: " 
  //                                << event.id().event();
  
  // get the RawDigit list
  art::Handle< std::vector<rawdata::RawDigit> > RawDigits;
  event.getByLabel(fRawDigitLabel, RawDigits);
  // make a vector to store the DAQHits into
  std::unique_ptr<std::vector<novaddt::DAQHit> > DAQHits(new std::vector<novaddt::DAQHit>);
  
  // Get the raw trigger to get the event time
  art::Handle< std::vector<rawdata::RawTrigger> > RawTriggers; 
  event.getByLabel(fRawDigitLabel, RawTriggers);    
  //mf::LogDebug("RawDigit2DAQHit") << "\tgot " << RawTriggers->size() << " RawTriggers";
  // Currently no triggers should be configured to have more than one raw trigger per event
  // The vector nature is reserved for exotic supernovae triggers
  assert(RawTriggers->size()==1);
  uint64_t event_time = RawTriggers->at(0).fTriggerTimingMarker_TimeStart;
  //mf::LogDebug("RawDigit2DAQHit") << "\tRawTriggers[0] offset time: " << event_time;
  
  // Make a DAQHit for every RawDigit and write it out
  FillDAQHits(RawDigits, DAQHits, event_time);
  // add DAQHits into the event
  event.put(std::move(DAQHits));
}
//---------------------------------------------------------------------
void raw2daq::RawDigit2DAQHit::FillDAQHits(art::Handle< std::vector<rawdata::RawDigit> > & RawDigits,
                                           std::unique_ptr<std::vector<novaddt::DAQHit> > & DAQHits,
                                           uint64_t event_time
                                           )
{
  const unsigned int nDigits = RawDigits->size();
  //mf::LogDebug("RawDigit2DAQHit") << "\t got " << nDigits << " Raw Digits";
  // Map between channel numbers and cell and plane numbers
  art::ServiceHandle<cmap::CMap> cmap;

  // loop over digits
  for(unsigned int iDigit=0; iDigit < nDigits; ++iDigit){
    const rawdata::RawDigit* digit = &(*RawDigits)[iDigit];
    //const int plane = cmap->GetPlane(digit);
    //const int cell  = cmap->GetCell(digit); 
    const daqchannelmap::lchan channel       = digit->Channel();
    const daqchannelmap::dchan daqchannel    = cmap->Map()->encodeDChan(channel);
    //const novaddt::View view = daqchannelmap::DAQChannelMap::getInstance()->getView(daqchannel);
    // Take a look at the RAW Digit object
    //mf::LogDebug("RawDigit2DAQHit") << "\t\t digit["     << iDigit
				    //<< "], plane: "      << plane
				    //<< ", cell: "        << cell
				    //<< ", Channel: "     << digit->Channel()        
				    //<< ", DAQ Channel: " << daqchannel
				    //<< ", nADC hits: "   << digit->NADC()
				    //<< ", nTDC hits: "   << digit->NTDC();

    //for (unsigned int iADC=0; iADC<digit->NADC(); ++iADC){
      //mf::LogDebug("RawDigit2DAQHit") << "\t\t\t ADC[" << iADC
				      //<< "]: "         << digit->fADC[iADC];
    //} // end of loop on ADC
    //for (unsigned int iTDC=0; iTDC<digit->NTDC(); ++iTDC){
      //mf::LogDebug("RawDigit2DAQHit") << "\t\t\t TDC[" << iTDC
				      //<< "]: "         << digit->fTDC[iTDC] + event_time;
    //} // end of loop on TDC
    // emplace a DAQHit into our return vector        
    DAQHits->emplace_back(fDetId,
			  daqchannel,
			  //novaddt::Plane(plane),
			  //novaddt::Cell(cell),
			  //view,
			  novaddt::ADC(digit->ADC()),
			  novaddt::TDC(digit->TDC() + event_time));
    //// Take a look at the DAQ Hit object
    //mf::LogDebug("RawDigit2DAQHit") << "\t\t - DAQHit[" << iDigit
				    //<< "], plane: "     << DAQHits->at(iDigit).Plane().val 
				    //<< ", cell: "       << DAQHits->at(iDigit).Cell().val 
				    //<< ", x view: "     << (DAQHits->at(iDigit).View().val == daqchannelmap::X_VIEW) 
				    //<< ", y view: "     << (DAQHits->at(iDigit).View().val == daqchannelmap::Y_VIEW) 
				    //<< ", ADC: "        << DAQHits->at(iDigit).ADC().val
				    //<< ", TDC: "        << DAQHits->at(iDigit).TDC().val;
  } // end of loop on raw digits
}

//---------------------------------------------------------------------
void raw2daq::RawDigit2DAQHit::beginRun(art::Run & run)
{
  art::ServiceHandle<cmap::CMap> cmap;
  
  std::vector< art::Handle<sumdata::RunData> > rdcol;
  run.getManyByType(rdcol);
  assert(!rdcol.empty());
  
  fDetId = rdcol[0]->DetId();
  cmap->TestDetID(rdcol[0]->DetId(),true);
  
  return;
}
//---------------------------------------------------------------------
DEFINE_ART_MODULE(raw2daq::RawDigit2DAQHit)
