////////////////////////////////////////////////////////////////////////////
/// \file    ColorScaleTable.cxx
/// \brief   The color scales used by the event display
/// \author  messier@indiana.edu
/// \version $Id: ColorScaleTable.cxx,v 1.9 2011-10-21 20:37:50 gsdavies Exp $ 
///////////////////////////////////////////////////////////////////////////
#include "EventDisplay/ColorScaleTable.h"

static evdb::ColorScale* gsRawT = 0;
static evdb::ColorScale* gsCalT = 0;
static evdb::ColorScale* gsRawQ = 0;
static evdb::ColorScale* gsCalQ = 0;

namespace evd
{
  //......................................................................
  void ColorScaleTable::MakeRawT(double lo, double hi) 
  {
    if (gsRawT) { delete gsRawT; gsRawT = 0; }

    gsRawT = new evdb::ColorScale(lo, hi, evdb::kFocus);
    gsRawT->SetUnderFlowColor(kGhostedColor);
    gsRawT->SetOverFlowColor(kGhostedColor);

  }

  //......................................................................
  void ColorScaleTable::MakeCalT(double lo, double hi) 
  {
    if (gsCalT) { delete gsCalT; gsCalT = 0; }

    gsCalT = new evdb::ColorScale(lo, hi, evdb::kFocus);
    gsCalT->SetUnderFlowColor(kGhostedColor);
    gsCalT->SetOverFlowColor(kGhostedColor);
  }

  //......................................................................
  void ColorScaleTable::MakeRawQ(double lo, double hi) 
  {
    if (gsRawQ) { delete gsRawQ; gsRawQ = 0; }
  
    gsRawQ = new evdb::ColorScale(lo, hi,
				  evdb::kCustom, evdb::kLog, 
				  20,
				  60.0, -120.0,
				  0.25, 0.75);

    gsRawQ->SetUnderFlowColor(kGhostedColor);
    gsRawQ->SetOverFlowColor(kGhostedColor);
  }

  //......................................................................
  void ColorScaleTable::MakeCalQ(double lo, double hi) 
  {
    if (gsCalQ) { delete gsCalQ; gsCalQ = 0; }
  
    gsCalQ = new evdb::ColorScale(lo, hi,
				  evdb::kCustom, evdb::kLog, 
				  20,
				  60.0, -120.0,
				  0.25, 0.75);

    gsCalQ->SetUnderFlowColor(kGhostedColor);
    gsCalQ->SetOverFlowColor(kGhostedColor);
  }

  //......................................................................
  const evdb::ColorScale& ColorScaleTable::RawQ() 
  {
    if (gsRawQ==0) ColorScaleTable::MakeRawQ(0.0, 4096.0);
    return (*gsRawQ);
  }

  //......................................................................
  const evdb::ColorScale& ColorScaleTable::CalQ() 
  {
    if (gsCalQ==0) ColorScaleTable::MakeCalQ(0.0, 4096.0);
    return (*gsCalQ);
  }

  //......................................................................
  const evdb::ColorScale& ColorScaleTable::RawT()
  {
    if (gsRawT==0) ColorScaleTable::MakeRawT(-100,400);
    return *gsRawT;
  }

  //......................................................................
  const evdb::ColorScale& ColorScaleTable::CalT()
  {
    if (gsCalT==0) ColorScaleTable::MakeCalT(-100.0,400.0);
    return *gsCalT;
  }

} // end namespace evd
////////////////////////////////////////////////////////////////////////
