////////////////////////////////////////////////////////////////////////
// \file    LEMAssociator_module.cc
// \brief   TODO
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#include "LEM/PIDDetails.h"

// NOvASoft includes
#include "RecoBase/Cluster.h"
#include "RecoBase/FilterList.h"

// Framework includes
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "fhiclcpp/ParameterSet.h"

namespace lem
{
  class LEMAssociator: public art::EDProducer
  {
  public:
    explicit LEMAssociator(const fhicl::ParameterSet& pset);
    ~LEMAssociator();

    virtual void reconfigure(const fhicl::ParameterSet& pset);
    virtual void produce(art::Event& evt);

  protected:
    std::string fPIDLabel;
    std::string fSlicerLabel;
  };

  //......................................................................
  LEMAssociator::LEMAssociator(const fhicl::ParameterSet& pset)
  {
    reconfigure(pset);

    produces<art::Assns<lem::PIDDetails, rb::Cluster>>();
  }

  //......................................................................
  LEMAssociator::~LEMAssociator()
  {
  }

  //......................................................................
  void LEMAssociator::reconfigure(const fhicl::ParameterSet& pset)
  {
    fPIDLabel = pset.get<std::string>("PIDLabel");
    fSlicerLabel = pset.get<std::string>("SlicerLabel");
  }

  //......................................................................
  void LEMAssociator::produce(art::Event& evt)
  {
    std::unique_ptr<art::Assns<lem::PIDDetails, rb::Cluster>> assns(new art::Assns<lem::PIDDetails, rb::Cluster>);

    art::Handle<std::vector<lem::PIDDetails> > pids;
    evt.getByLabel(fPIDLabel, pids);

    art::Handle<std::vector<rb::Cluster> > slices;
    evt.getByLabel(fSlicerLabel, slices);

    unsigned int pidIdx = 0;

    const int sliceMax = slices->size();
    for(int sliceIdx = 0; sliceIdx < sliceMax; ++sliceIdx){
      if(rb::IsFiltered(evt, slices, sliceIdx)) continue;

      const art::Ptr<rb::Cluster> slice(slices, sliceIdx);

      if(slice->IsNoise()) continue;

      assert(pidIdx < pids->size());
      const art::Ptr<lem::PIDDetails> pid(pids, pidIdx);
      assns->addSingle(pid, slice);
      ++pidIdx;
    } // end for sliceIdx

    assert(pidIdx == pids->size());

    evt.put(std::move(assns));
  }

  DEFINE_ART_MODULE(LEMAssociator);

} //namespace lem
////////////////////////////////////////////////////////////////////////

