////////////////////////////////////////////////////////////////////////
/// \file    DistanceMap.h
/// \brief   Calculate and cache electrostatic potential between cells
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_DISTANCEMAP_H
#define LEM_DISTANCEMAP_H

#include "LEM/func/LiteHit.h"

class TH2;

namespace lem
{
  const int kNumPlanes = 256;
  const int kNumCells = 256;

  /// Calculate and cache electrostatic potential between cells
  class DistanceMap
  {
  public:
    /// Singleton
    const static DistanceMap& Instance();

    double InvDist(int aplane, int bplane, int acell, int bcell) const;
    double InvDist(const LiteHit& a, const LiteHit& b) const;

    TH2* MakeHist() const;

    /// Must call before first call to Instance()
    static void SetPlaneScale(double ps){fgPlaneScale = ps;}
    static void SetCellScale(double cs){fgCellScale = cs;}
    static void SetDecayPower(double dp){fgDecayPower = dp;}
    static void SetExpMode(bool em = true){fgExpMode = em;}

  protected:
    static double DistFunc(double dp, double dc);

    static double fgPlaneScale;
    static double fgCellScale;
    static double fgDecayPower;
    static bool fgExpMode;

    double fInvDistMap[kNumPlanes][kNumCells];

  private:
    DistanceMap();
  };
} //namespace

#endif
