#ifndef LEM_FINDMATCHESALG_H
#define LEM_FINDMATCHESALG_H

#include "LEM/func/FindMatches.h" // For FlippedPotentials
#include "LEM/func/Heads.h"
#include "LEM/func/Library.h"
#include "LEM/func/Match.h"
#include "LEM/func/MatchableEvent.h"

// TODO: forward declare?
#include <string>
#include <set>
#include <vector>

namespace lem
{
  class FindMatchesAlg
  {
  public:
    FindMatchesAlg(const std::string& libDir, bool preload, bool useHeads);
    ~FindMatchesAlg();

    std::vector<Match> FindMatches(const MatchableEvent& trial,
                                   unsigned int numMatches,
                                   int enrich) const;

    const Library* GetLibrary() const {return fLib;}

  protected:
    std::vector<Match> FindMatchesNoHeads(const MatchableEvent& trial,
                                          unsigned int numMatches,
                                          int enrich) const;

    std::vector<Match> FindMatchesHeads(const MatchableEvent& trial,
                                        unsigned int numMatches,
                                        int enrich) const;

    int BestHeadFor(const FlippedPotentials& trialVs,
                    const MatchableEvent& trial,
                    bool& bestFlipEven, bool& bestFlipOdd,
                    double& bestEnergy) const;

    std::vector<Match> PackageMatches(const std::multiset<Match>& matches,
                                      const MatchableEvent& trial) const;

    Library* fLib;
    Heads* fHeads;
  };
}

#endif
