////////////////////////////////////////////////////////////////////////
/// \file    Heads.h
/// \brief   Mapping from a subset of the library to their best matches
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_HEADS_H
#define LEM_HEADS_H

#include <ostream>
#include <string>

namespace lem
{
  /// Mapping from a subset of the library to their best matches
  class Heads
  {
  public:
    static Heads* FromMMap(const std::string& libPath, bool touchAll);
    static Heads* FromMMapOnDemand(const std::string& libPath);

    ~Heads();

    friend std::ostream& operator<<(std::ostream&, const lem::Heads&);

    int NHeads() const {return fNHeads;}
    int HeadIdx(int i) const {return fHeadIdxs[i];}
    int HeadSeqsLen() const {return fHeadSeqsLen;}
    int ChildIdxFor(int head, int child) const;
    bool FlipEvenFor(int head, int child) const;
    bool FlipOddFor(int head, int child) const;

  protected:
    Heads() {}
    Heads(const Heads&);
    Heads& operator=(const Heads&);

    int fNHeads;
    int* fHeadIdxs;
    int fHeadSeqsLen;
    int** fHeadSeqs;

    long fMappingSize;
  };

  class HeadsTranspose
  {
  public:
    static HeadsTranspose* FromMMap(const std::string& libPath);
    static HeadsTranspose* FromMMapOnDemand(const std::string& libPath);

    ~HeadsTranspose();

    friend std::ostream& operator<<(std::ostream&, const lem::HeadsTranspose&);

    int NHeads() const {return fNHeads;}
    int HeadIdx(int i) const {return fHeadIdxs[i];}

    int* pHeadSeqs() const {return fPSeqs;}

  protected:
    HeadsTranspose() {}
    HeadsTranspose(const HeadsTranspose&);
    HeadsTranspose& operator=(const HeadsTranspose&);

    int fNHeads;
    int fLibSize;
    int* fHeadIdxs;
    int* fPSeqs;

    long fMappingSize;
  };

  std::ostream& operator<<(std::ostream& os, const lem::Heads& hs);
  std::ostream& operator<<(std::ostream& os, const lem::HeadsTranspose& hs);
}

#endif
