////////////////////////////////////////////////////////////////////////
/// \file    Library.h
/// \brief   Collection of events for matching
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_LIBRARY_H
#define LEM_LIBRARY_H

#include <string>

#include "LEM/func/LibrarySummary.h"
#include "LEM/func/MatchableEvent.h"

namespace lem
{
  /// Collection of events for matching
  class Library
  {
  public:
    static Library* FromMMap(const std::string& libPath, bool touchAll);
    static Library* FromMMapOnDemand(const std::string& libPath);
    static Library* FromTrees(const std::string& libPath,
                              int splitLibFactor,
                              int splitLibIndex,
                              int downsampleFactor);

    ~Library();

    const LibrarySummary& Summary() const {return fSummary;}
    unsigned int NEvents() const {return fLibSize;}
    const MatchableEvent& Event(int i) const {return fLib[i];}
    const MatchableEvent& DownsampledEvent(int i) const {return fLibDownsample[i];}
    const MatchableEvent* Events() const {return fLib;}
    const MatchableEvent* DownsampledEvents() const {return fLibDownsample;}

  protected:
    Library() {}
    Library(const Library&);
    Library& operator=(const Library&);

    static double SimpleSurvivalProb(const EventSummary& evt);

    LibrarySummary fSummary;
    unsigned int fLibSize;
    MatchableEvent* fLib;
    MatchableEvent* fLibDownsample;

    long fMappingSize;
  };
}

#endif
