////////////////////////////////////////////////////////////////////////
/// \file    LibrarySummary.cxx
/// \brief   Details of the library LEM matches were made against
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#include "LEM/func/LibrarySummary.h"

#include <ostream>

namespace lem
{
  //......................................................................
  LibrarySummary::LibrarySummary()
    : N(0), nTrueNC(0), nSwapNC(0), totBkg(0), totSig(0), totEnrich(0)
  {
  }

  //......................................................................
  LibrarySummary& LibrarySummary::operator+=(const LibrarySummary& s)
  {
    N += s.N;
    nTrueNC += s.nTrueNC;
    nSwapNC += s.nSwapNC;
    totBkg += s.totBkg;
    totSig += s.totSig;
    totEnrich += s.totEnrich;
    return *this;
  }

  //......................................................................
  std::ostream& operator<<(std::ostream& os, const lem::LibrarySummary& ls)
  {
    os << ls.N << " events, weighted to "
       << ls.totSig << " signal, "
       << ls.totBkg << " background and "
       << ls.totEnrich << " enriched background";

    return os;
  }

} // namespace
