////////////////////////////////////////////////////////////////////////
/// \file    LiteHit.h
/// \brief   Compressed hit info, basic component of LEM events
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_LITEHIT_H
#define LEM_LITEHIT_H

#ifndef __GCCXML__
#include <cassert>
#include <ostream>
#endif // __GCCXML__

namespace lem
{
  /// Compressed hit info, basic component of %LEM events
  class LiteHit
  {
  public:
    LiteHit() {}

    // Saving space
    float pecorr;
    unsigned short cellIdx;
    unsigned char partIdx;

#ifndef __GCCXML__
    LiteHit(unsigned int p, unsigned int c, double pe, unsigned char i) : pecorr(pe), partIdx(i)
    {
      assert(pecorr > 0);

      assert(sizeof(short) == 2);

      SetPlaneCell(p, c);
    }

    int Plane() const {return cellIdx/256;}
    int Cell() const {return cellIdx%256;}
    void SetPlaneCell(int p, int c) {cellIdx = int(p)*256+int(c);}
    void SetPlane(int p){SetPlaneCell(p, Cell());}
    void SetCell(int c){SetPlaneCell(Plane(), c);}

    bool operator<(const LiteHit& rhs) const {return cellIdx < rhs.cellIdx;}
#endif // __GCCXML__
  };

#ifndef __GCCXML__
  bool compareByCell(const LiteHit& a, const LiteHit& b);
  bool compareByPlane(const LiteHit& a, const LiteHit& b);

  std::ostream& operator<<(std::ostream& os, const lem::LiteHit& h);
#endif // __GCCXML__
} // namespace

#endif
