////////////////////////////////////////////////////////////////////////
/// \file    MatchSummary.h
/// \brief   Simplified Match information, suitable for serialization
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef LEM_MATCHSUMMARY_H
#define LEM_MATCHSUMMARY_H

#include "LEM/func/Match.h"
#include "LEM/func/MatchableEvent.h"

namespace lem
{
  /// Simplified Match information, suitable for serialization
  struct MatchSummary
  {
    MatchSummary() : isSig(false), weight(-1), potential(-1), flipEven(false), flipOdd(false), pe(-1), nhits(-1), length(-1), y(-1), qFrac(-1), trueEVis(-1), selfEnergy(-1), id(-1), run(-1), subrun(-1), event(-1), ccnc(-1), pdg(-1), origPdg(-1), headSeqNo(-1) {}

    explicit MatchSummary(const Match& m)
      : isSig(m.evt->IsSig()),
        weight(m.evt->weight),
        potential(m.energy),
        flipEven(m.flipEven),
        flipOdd(m.flipOdd),
        pe(m.evt->totalPE),
        nhits(m.evt->nhits),
        length(m.evt->Length()),
        y(m.evt->y),
        qFrac(m.qfrac),
        trueEVis(m.evt->trueEVis),
        selfEnergy(m.evt->selfEnergy),
        id(m.evt->id),
        run(m.evt->run),
        subrun(m.evt->subrun),
        event(m.evt->event),
        ccnc(m.evt->ccnc),
        pdg(m.evt->pdg),
        origPdg(m.evt->origPdg),
        mode(m.evt->mode),
        photL0(m.evt->photL0),
        photL1(m.evt->photL1),
        photE0(m.evt->photE0),
        photE1(m.evt->photE1),
        enrich(m.evt->enrich),
        headSeqNo(m.headSeqNo)
    {
    }

    bool operator<(const MatchSummary& rhs) const {return potential < rhs.potential;}

    bool isSig;
    float weight;
    float potential;
    bool flipEven, flipOdd;
    float pe;
    int nhits;
    int length; ///< In planes
    float y;
    float qFrac;
    float trueEVis;
    float selfEnergy;
    int id;

    int run, subrun, event;
    int ccnc, pdg, origPdg;
    int mode;

    float photL0, photL1;
    float photE0, photE1;

    bool enrich;

    int headSeqNo;
  };
} // namespace

#endif
