////////////////////////////////////////////////////////////////////////
/// \file    Potential.cxx
/// \brief   Map of electrostatic potential at each cell
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#include "LEM/func/Potential.h"

#include <algorithm>
#include <cassert>

namespace lem
{
  //......................................................................
  Potential::Potential() : V()
  {
    // That "V()" should have taken care of this, check.
    assert(V[0] == 0);
    assert(V[256*256-1] == 0);
  }

  //......................................................................
  Potential Potential::Downsampled(int factor) const
  {
    Potential ret;

    for(int p = 0; p < 256; ++p){
      for(int c = 0; c < 256; ++c){
        const int idx = p*256+c;

        const int newp = ((p>>(factor+1))<<1)+p%2;
        const int newc = c>>factor;

        // Matches indexing in MatchableEvent::Downsampled()
        const int newidx = (newp<<(8-factor))+newc;

        // Keeping the largest value ensures that the energy calculated from
        // this potential is a lower bound, as required.
        ret.V[newidx] = std::max(ret.V[newidx], V[idx]);
      }
    }

    return ret;
  }
}
