////////////////////////////////////////////////////////////////////////
/// \file    Util.cxx
/// \brief   Utilities for mmap
/// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#include "LEM/func/Util.h"

#include <sys/file.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <cassert>
#include <iostream>

namespace lem
{
  long MMapFileAtAddress(const std::string& fname, void* addr, bool touchAll,
                         const std::string& lockName)
  {
    FILE* lockFile = 0;
    if(touchAll && !lockName.empty()){
      lockFile = fopen((lockName+".done").c_str(), "w");
      int ret = flock(fileno(lockFile), LOCK_EX | LOCK_NB);
      // If we succesfully get this lock, that means that no running processes
      // have preloaded the file for us.
      if(ret == 0){
        // Drop it again, we were just testing
        fclose(lockFile);
        lockFile = 0;

        // So we need to take the preload lock, because our preload is about to
        // actually do some work.
        lockFile = fopen(lockName.c_str(), "w");
        // Wait for other preloaders to finish so we don't fight
        flock(fileno(lockFile), LOCK_EX);
      }
    }


    FILE* f = fopen(fname.c_str(), "r");
    assert(f);

    struct stat st;
    fstat(fileno(f), &st);
    const long fsize = st.st_size;

    void* ret = mmap(addr, fsize, PROT_READ, MAP_SHARED | MAP_FIXED, fileno(f), 0);
    fclose(f);
    assert(ret != (void*)-1);


    if(touchAll){
      // Touch all the memory, one byte on each page
      char tot = 0;
      for(long i = 0; i < fsize; i += 4096){
        if((i/4096)%(fsize/409600) == 0) std::cout << (100*i)/fsize << "%" << std::endl;
        tot += ((char*)addr)[i];
      }
      // Printing forces it not to get optimized out
      std::cout << "Fully loaded: " << int(tot) << std::endl;

      if(!lockName.empty()){
        if(lockFile) fclose(lockFile); // Releases the main lock if we had it

        lockFile = fopen((lockName+".done").c_str(), "w");
        // Mark that we succesfully preloaded the file, so long as someone has
        // done this later people don't need to wait on the first lock. This
        // indication lasts until the end of this process.
        flock(fileno(lockFile), LOCK_SH);
      }
    }

    return fsize;
  }
}
