////////////////////////////////////////////////////////////////////////
// Class:       SlowMonopoleAna
// Module Type: analyzer
// File:        SlowMonopoleAna_module.cc
//
// Generated at Mon Aug 11 11:18:01 2014 by Martin Frank using artmod
// from cetpkgsupport v1_06_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include "RawData/RawTrigger.h"

namespace mono {
  class SlowMonopoleAna;
}

class mono::SlowMonopoleAna : public art::EDAnalyzer {
public:
  explicit SlowMonopoleAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  SlowMonopoleAna(SlowMonopoleAna const &) = delete;
  SlowMonopoleAna(SlowMonopoleAna &&) = delete;
  SlowMonopoleAna & operator = (SlowMonopoleAna const &) = delete;
  SlowMonopoleAna & operator = (SlowMonopoleAna &&) = delete;
  void analyze(art::Event const & e) override;
};



mono::SlowMonopoleAna::SlowMonopoleAna(fhicl::ParameterSet const & p) :
  EDAnalyzer(p)
{}



void mono::SlowMonopoleAna::analyze(art::Event const & e)
{
  art::Handle<std::vector<rawdata::RawTrigger> > raw_triggers;
  e.getByLabel("daq", raw_triggers);
  assert(raw_triggers->size() == 1);
  novaddt::TDC event_time(raw_triggers->at(0).fTriggerTimingMarker_TimeStart);

  art::Handle<novaddt::HitList> hits;
  e.getByLabel("smmhighadcfilter", "HEHits", hits);
  
  art::Handle<std::vector<novaddt::TriggerDecision> > decisions;
  e.getByLabel("smmtrigger", decisions);

  if (decisions.failedToGet())
  {
    std::cout << "MF: No Trigger Decision!" << std::endl;
  } else {
    double tdc_to_us = 1.0 / 64e6 * 1e6;
    unsigned deltaT = 300;

    std::cout << "MF: Triggered Event " << e.event()
	      << "\nMF:  N Hits      = " << hits->size()
	      << "\nMF:  N Decisions = " << decisions->size();

    for (auto const& decision : *decisions)
    {
      auto t0 = 
	decision.start() + 2 * deltaT - event_time;
      auto t1 = 
	decision.start() + decision.duration() - 2 * deltaT - event_time;

      std::cout << "\nMF:     t0 = " << t0 * tdc_to_us
		<< "\nMF:     t1 = " << t1 * tdc_to_us
		<< std::endl;
    }

    std::cout << std::endl;
  }
}

DEFINE_ART_MODULE(mono::SlowMonopoleAna)
