////////////////////////////////////////////////////////////////////////
// Class:       SlowMonopoleFilt
// Module Type: filter
// File:        SlowMonopoleFilt_module.cc
//
// Generated at Mon Aug 11 16:08:14 2014 by Martin Frank using artmod
// from cetpkgsupport v1_06_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include "Monopole/SlowMonopoleTrigger.h"
#include "RawData/RawDigit.h"
#include "RawData/RawTrigger.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"

#include <boost/format.hpp>

#include <memory>


namespace mono {
  class SlowMonopoleFilt;
}

class mono::SlowMonopoleFilt : public art::EDFilter {
public:
  explicit SlowMonopoleFilt(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  SlowMonopoleFilt(SlowMonopoleFilt const &) = delete;
  SlowMonopoleFilt(SlowMonopoleFilt &&) = delete;
  SlowMonopoleFilt & operator = (SlowMonopoleFilt const &) = delete;
  SlowMonopoleFilt & operator = (SlowMonopoleFilt &&) = delete;
  bool filter(art::Event & e) override;

private:
  bool is_equal
  (novaddt::DAQHit const& ddthit, art::Ptr<rb::CellHit> const& cellhit,
   novaddt::TDC const& event_time) const;
  bool is_equal
  (novaddt::DAQHit const& ddthit, rawdata::RawDigit const& digit,
   novaddt::TDC const& event_time) const;
  bool is_equal
  (novaddt::DAQHit const& lhs, novaddt::DAQHit const& rhs) const;

  void print(novaddt::DAQHit const& h, novaddt::TDC const& event_time) const;
  void print(art::Ptr<rb::CellHit> const& h) const;
  void print(rawdata::RawDigit const& h) const;
};



mono::SlowMonopoleFilt::SlowMonopoleFilt(fhicl::ParameterSet const & p)
{
  produces<std::vector<rb::Cluster> >();
}



bool mono::SlowMonopoleFilt::filter(art::Event & e)
{
  //
  // Extract Objects from the Event
  //
  // SMM Trigger Decisions
  art::Handle<std::vector<novaddt::TriggerDecision> > ddt_decisions;
  e.getByLabel("smmtrigger", ddt_decisions);
  assert(ddt_decisions->size() == 1);

  // High ADC Filtered Hits
  art::Handle<novaddt::HitList> ddt_hits;
  e.getByLabel("smmhighadcfilter", "HEHits", ddt_hits);

  // Raw Trigger Header
  art::Handle<std::vector<rawdata::RawTrigger> > raw_triggers;
  e.getByLabel("daq", raw_triggers);
  assert(raw_triggers->size() == 1);

  // Raw Digits
  art::Handle<std::vector<rawdata::RawDigit> > digits;
  e.getByLabel("daq", digits);

  // Offline Cell Hits
  art::Handle<std::vector<rb::CellHit> > hits;
  e.getByLabel("calhit", hits);
  std::vector<art::Ptr<rb::CellHit> > hitptrs;
  for (unsigned i = 0; i < hits->size(); ++i)
    hitptrs.emplace_back(hits, i);


  // Run Zukai's Trigger Algorithm
  SlowMonopoleTrigger smt(*ddt_hits);
  std::vector<novaddt::TriggerDecision> 
    local_decisions = smt.trigger_decisions();
  assert(local_decisions.size() == 1);
  

  // Let's compare the local and ddt trigger decisions:
  assert(is_subset_of(ddt_decisions->at(0), local_decisions.at(0)));
  // assert(local_decisions.at(0) == ddt_decisions->at(0));


  // Figure out the DDT Trigger Decision Times:
  double tdc_to_ns = 1.0 / 64e6 * 1e9;
  unsigned deltaT = 300;
  novaddt::TDC event_time(raw_triggers->at(0).fTriggerTimingMarker_TimeStart);
  auto t0 = tdc_to_ns * 
    (ddt_decisions->at(0).start() + 2 * deltaT - event_time);
  auto t1 = tdc_to_ns * 
    (ddt_decisions->at(0).start() + ddt_decisions->at(0).duration() 
     - 2 * deltaT - event_time);


  // std::unique_vector<std::vector<TriggerDecisions> > 
  //   local_decisions(new std::vector<TriggerDecisions>);
  // zukai_algorithm(ddt_hits, local_decisions);

  std::unique_ptr<std::vector<rb::Cluster> > 
    clusters(new std::vector<rb::Cluster>);
  rb::Cluster cluster, essential_cluster;

  for (auto const& hitptr : hitptrs)
  {
    if (hitptr->TNS() >= t0 && hitptr->TNS() <= t1 && hitptr->ADC() >= 200)
      cluster.Add(hitptr);
  }

  // clusters->push_back(cluster);

  std::cout << "\nMF: Find Hit in Offline Cluster" << std::endl;
  for (auto const& essential_hit : smt.essential_hits())
  {
    auto matched_offline_hit = std::find_if
      (cluster.AllCells().begin(), cluster.AllCells().end(),
       [&](art::Ptr<rb::CellHit> const& cell_hit)
       { 
    	 return is_equal(essential_hit, cell_hit, event_time); 
       });
    
    if (matched_offline_hit != cluster.AllCells().end())
    {
      std::cout << "MF: Found the matching hit!" << std::endl;
      print(essential_hit, event_time);
      print(*matched_offline_hit);

      essential_cluster.Add(*matched_offline_hit);
    } else {
      std::cout << "MF: No matching hit found for:" << std::endl;
      print(essential_hit, event_time);
    }
  }

  std::cout << "\nMF: Find Hit in Offline Cell Hit Collection" << std::endl;
  for (auto const& essential_hit : smt.essential_hits())
  {
    for (auto const& hitptr : hitptrs)
    {
      if (is_equal(essential_hit, hitptr, event_time))
      {
	std::cout << "MF: Found the matching hit!" << std::endl;
	print(essential_hit, event_time);
	print(hitptr);

  	// essential_cluster.Add(cellhit);
      }
    } 
  }

  std::cout << "\nMF: Find Hit in Raw Digits" << std::endl;
  for (auto const& essential_hit : smt.essential_hits())
  {
    for (auto const& digit : *digits)
    {
      if (is_equal(essential_hit, digit, event_time))
      {
	std::cout << "MF: Found the matching hit!" << std::endl;
	print(essential_hit, event_time);
	print(digit);
      }
    } 
  }

  std::cout << "\nMF: Find Hit in DDT HitList" << std::endl;
  for (auto const& essential_hit : smt.essential_hits())
  {
    for (auto const& ddt_hit : *ddt_hits)
    {
      if (is_equal(essential_hit, ddt_hit))
      {
	std::cout << "MF: Found the matching hit!" << std::endl;
	print(essential_hit, 0);
	print(ddt_hit, 0);
      }
    } 
  }

  std::cout << "MF: Essential Hits Size    = " << smt.essential_hits().size()
	    << std::endl;
  std::cout << "MF: Essential Cluster Size = " << essential_cluster.NCell()
	    << std::endl;

  assert(essential_cluster.NCell() == 2);
  
  clusters->push_back(essential_cluster);

  e.put(std::move(clusters));

  return true;
}



bool mono::SlowMonopoleFilt::is_equal
(novaddt::DAQHit const& ddthit, art::Ptr<rb::CellHit> const& cellhit,
 novaddt::TDC const& event_time) const
{
  int64_t ddt_time = ddthit.TDC().val - event_time;

  if (ddt_time != cellhit->TDC())
    return false;
  
  if (ddthit.ADC().val != cellhit->ADC())
    return false;

  if (ddthit.Plane().val != cellhit->Plane())
    return false;

  if (ddthit.Cell().val != cellhit->Cell())
    return false;

  return true;
}



bool mono::SlowMonopoleFilt::is_equal
(novaddt::DAQHit const& ddthit, rawdata::RawDigit const& digit,
 novaddt::TDC const& event_time) const
{
  int64_t ddt_time = ddthit.TDC().val - event_time;

  if (ddt_time != digit.TDC())
    return false;
  
  if (ddthit.ADC().val != digit.ADC(0))
    return false;

  return true;
}



bool mono::SlowMonopoleFilt::is_equal
(novaddt::DAQHit const& lhs, novaddt::DAQHit const& rhs) const
{
  if (lhs.TDC().val != rhs.TDC().val)
    return false;

  if (lhs.ADC().val != rhs.ADC().val)
    return false;

  return true;
}



void mono::SlowMonopoleFilt::print
(novaddt::DAQHit const& h, 
 novaddt::TDC const& event_time) const
{
  auto relative_time = h.TDC().val - event_time;

  std::string view;
  if (h.View().val == daqchannelmap::DetView_TYPE::X_VIEW)
    view = "xz";
  else if (h.View().val == daqchannelmap::DetView_TYPE::Y_VIEW)
    view = "yz";
  else
    view = "bad";

  boost::format form("%3s %4u %4u %5u %20u");
  std::cout << "MF: " << 
    form % view % h.Plane().val % h.Cell().val % 
    h.ADC().val % relative_time
	    << std::endl;
}



void mono::SlowMonopoleFilt::print(art::Ptr<rb::CellHit> const& h) const
{
  boost::format form("%3s %4u %4u %5u %20u");

  std::string view;
  if (h->View() == geo::View_t::kX)
    view = "xz";
  else if (h->View() == geo::View_t::kY)
    view = "yz";
  else
    view = "bad";

  std::cout << "MF: " << 
    form % view % h->Plane() % h->Cell() % 
    h->ADC() % h->TDC()
	    << std::endl;
}



void mono::SlowMonopoleFilt::print(rawdata::RawDigit const& h) const
{
  boost::format form("%3s %4u %4u %5u %20u");

  std::cout << "MF: " << 
    form % "?" % 0 % 0 % 
    h.ADC(0) % h.TDC()
	    << std::endl;
}



DEFINE_ART_MODULE(mono::SlowMonopoleFilt)
