#ifndef GUARD_OFFLINE_SLOW_MONOPOLE_TRIGGER_HH
#define GUARD_OFFLINE_SLOW_MONOPOLE_TRIGGER_HH


#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/TriggerDecision.h"

#include <map>


namespace novaddt
{
  bool operator== 
  (TriggerDecision const& lhs, TriggerDecision const& rhs);

  bool is_subset_of
  (TriggerDecision const& subset, TriggerDecision const& set);
}


namespace mono
{
  class SlowMonopoleTrigger;
}


class mono::SlowMonopoleTrigger
{
public:
  SlowMonopoleTrigger(novaddt::HitList const& h);

  std::vector<novaddt::TriggerDecision> trigger_decisions() const;
  novaddt::HitList essential_hits() const;
  
  bool mf_algorithm();
  bool zukai_algorithm();

private:
  /*CAUTION:
    The current signature causes the hits to be copied into the function.
    While we are comparing algorithms, this ensures that hits_ does not get
    overwritten.  I am not sure whether we can avoid the copy depending on 
    how we pull the hits out of the event. */
  void split_by_view(novaddt::HitList hits,
		     std::map<std::string, novaddt::HitList> & hitmap) const;

  void find_view_matched_hits
  (std::map<std::string, novaddt::HitList> & hitmap) const;

  bool hit_is_on_surface(novaddt::DAQHit const& hit) const;

  bool hit_is_on_track(novaddt::DAQHit const& hit,
		       novaddt::DAQHit const& track_start, 
		       novaddt::DAQHit const& track_end) const;

  bool hit_is_in_time(double hit_time, double time_to_hit) const;

  double distance(novaddt::DAQHit const& hit1, 
		  novaddt::DAQHit const& hit2) const;

  novaddt::HitList::const_iterator find(novaddt::HitList const& hits, 
					novaddt::TDC tdc) const;

  void print(novaddt::DAQHit const& h, std::string prefix = "MF:") const;
  void print(std::vector<novaddt::TriggerDecision> const& decisions, 
	     std::string prefix = "MF:") const;
  
  unsigned min_hits_,
    view_match_time_cut_, view_match_plane_cut_,
    total_planes_, total_cells_, surface_plane_cut_, surface_cell_cut_,
    trigger_decision_time_buffer_;
  double aspect_ratio_, track_time_deviation_, hits_on_track_fraction_;

  novaddt::HitList const& hits_;
  novaddt::HitList essential_hits_;
  std::vector<novaddt::TriggerDecision> 
  mf_trigger_decisions_, zukai_trigger_decisions_;


  // Zukai's methods and parameters:  
  bool SurfAssign(novaddt::DAQHit const & hit);
  bool ViewCheck(unsigned p1, novaddt::TDC::value_type T1,
		 unsigned p2, novaddt::TDC::value_type T2);
  unsigned IndexLocator(novaddt::HitList & hl, novaddt::TDC::value_type T);
  float Dist_(float c1, float c2, float p1, float p2);

  float    _hf_min;
  unsigned _minHits;
  float    _deltP;
  float    _deltC;
  float    _sigmaL;
  float    _Pmax;
  float    _Cmax;
  float    _DistMin;
  float    _vproMin;

  unsigned _deltaT;
  unsigned _sigmaT;
  float    _vproMax;
};


#endif
