#include "Calibrator/Calibrator.h"
#include "Geometry/Geometry.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/RecoHit.h"

// Framework includes
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Principal/Event.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include <cmath>
#include <vector>
#include <string>

namespace ncs {
  class Skim : public art::EDFilter {
  public:
    explicit Skim(fhicl::ParameterSet const& pset);
    virtual ~Skim();
    virtual bool filter(art::Event& evt);

    };
}

namespace ncs{

  //......................................................................
  Skim::Skim(fhicl::ParameterSet const& pset)
  {
  }
  
  //......................................................................
  Skim::~Skim()
  {
  }
  
  //......................................................................
  bool Skim::filter(art::Event& evt) 
  {
    art::ServiceHandle<calib::Calibrator> cal;
    art::ServiceHandle<geo::Geometry> geom;

    //get reco slicer info  
    art::Handle< std::vector<rb::Cluster> > slices;
    evt.getByLabel("slicer", slices);

    int bestSlice = -1;
    unsigned int mostCells = 0;
    for(unsigned int sliceIdx = 0; sliceIdx < slices->size(); ++sliceIdx){
      const rb::Cluster& slice = (*slices)[sliceIdx];
      if(slice.IsNoise()) continue;
      if(slice.NCell() > mostCells){        
        mostCells = slice.NCell();
        bestSlice = sliceIdx;
      }
    }
    if(bestSlice < 0) return false;
    const rb::Cluster& slice = (*slices)[bestSlice];
    int nCells = slice.NCell();
    int lengthPlanes=slice.ExtentPlane();

    double recoEnergy=0.;
    for( unsigned int icell=0; icell<slice.NCell(); ++ icell){
       const art::Ptr<rb::CellHit>& chit = slice.Cell(icell);
       const TVector3 mean = slice.MeanXYZ();
       const double wx = (slice.NXCell() > 0) ? slice.W(slice.XCell(0).get()) : 0;
       const double wy = (slice.NYCell() > 0) ? slice.W(slice.YCell(0).get()) : 0;

       const rb::RecoHit rhit(cal->MakeRecoHit(*chit, chit->View() == geo::kX ? wx : wy));

       if( !rhit.IsCalibrated() ) continue;

       recoEnergy += rhit.GeV();
    }

    //preselection cuts
    //if( recoEnergy>0.35&&recoEnergy<1.75 && nCells>20 && nCells<300 && lengthPlanes<140 ) return true;
    //else return false;
    if( recoEnergy<0.1 || recoEnergy>1.75 ) return false;
    if( nCells<10 || nCells>300 ) return false;
    if( lengthPlanes>140 ) return false;
    //std::cout<<"Testing: "<<recoEnergy<<" "<<nCells<<" "<<lengthPlanes<<std::endl; 
    return true;
  }//end filter
  
}//end namespace

namespace ncs{

  DEFINE_ART_MODULE(Skim);

}

