#ifndef NumuSandFxs_h
#define NumuSandFxs_h

#include "RecoBase/Track.h"
#include "TrackFit/KalmanGeoHelper.h"

namespace numusand {

  class NumuSandFxs {// class for various functions
  public:
    explicit NumuSandFxs();
    virtual ~NumuSandFxs();

    float getAngle(TVector3 trackdir);  // get cos(theta) of muon angle wrt neutrino angle
    float getERemain(const rb::Cluster& clus1, const rb::Cluster& clus2); // get recalibrating remaining E
    void getContainment(art::Ptr<rb::Track> track, float &vtxdist, float &mudist);
    void getCosmicVars(const art::Ptr<rb::Track> track, double &fwddist, double &bakdist, int &fwdcell, int &bakcell);
    void getFits(const art::Ptr<rb::Track> track, double &slope, double &chisq, double &chisqdiff);
    float getScatt(const art::Ptr<rb::Track> track);
    void getActivity(const art::Ptr<rb::Track> track, double &startact, double &endact);
    float fitslope(const std::vector< double > &l, const std::vector< double > &t, const std::vector< double > &q, double &chisq, double &inter, bool linflag);
    float fitchisq(const std::vector< double > &t, const std::vector< double > &l, int flag, double c);
    bool getMissingE(const art::Ptr<rb::Cluster> slice, float &missE1, float &missE2);
    bool getNewContain(const art::Ptr<rb::Cluster> slice, int &bbc, float &dfe2, float &dfe3, float &dfeY2, float &dfeY3, float &dfeth1, float &dfeth2, float &dfeYth1, float &dfeYth2);

  private:
    trk::KalmanGeoHelper*      fKGeo;

  };
}

#endif
