#include <TFile.h>
#include <TTree.h>
#include <TChain.h>
#include <TMarker.h>
#include <TCanvas.h>
#include <TNtuple.h>
#include <TLegend.h>
#include <TH1.h>
#include <TH2.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TGraph.h>
#include <TH1D.h>
#include <TProfile.h>
#include <TColor.h>
#include <TMultiGraph.h> 
#include "TStyle.h"
#include <iostream>
#include <vector>
#include "RateSummaryNt.h"

// before running on root do gSystem->Load("libOnlineMonitoringAna.so"); 

void rates_by_run()
{

  int runlo=10230;
  int runhi=10579;

  int colors[33];
  colors[0]=632;//red
  colors[1]=920+2;//gray
  colors[2]=632-7;//red
  colors[3]=900+5;//pink
  colors[4]=400-6;//yellow
  colors[5]=616;//mgnt
  colors[6]=616+2;//mgnt
  colors[7]=616-6;//mgnt
  colors[8]=800+10;//orange
  colors[9]=880-7;//violet
  colors[10]=600;//blue
  colors[11]=600+2;//blue
  colors[12]=600-3;//blue
  colors[13]=600+3;//blue
  colors[14]=600-6;//blue
  colors[15]=860+7;//azure
  colors[16]=432+1;//cyan
  colors[17]=880-5;//violet
  colors[18]=432-6;//cyan
  colors[19]=416;//green
  colors[20]=416+3;//green
  colors[21]=416-6;//green
  colors[22]=416+2;//green
  colors[23]=820-5;//spring
  colors[24]=820-7;//spring
  colors[25]=820+4;//spring
  colors[26]=400+2;//yellow
  colors[27]=400+3;//yellow
  colors[28]=800+7;//orange
  colors[29]=632+2;//red
  colors[30]=860+7;//azure
  colors[31]=900+6;//pink
  colors[32]=800+4;//orange

  const Int_t NRGBs = 3;	
  const Int_t NCont = 255;
  Double_t stops[NRGBs] = { 0.00, 0.34, 1.00};
  Double_t red[NRGBs]   = { 1.00, 1.00, 0.60};
  Double_t green[NRGBs] = { 0.97, 0.40, 0.00};
  Double_t blue[NRGBs]  = { 0.99, 0.70, 0.00};

  TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
  gStyle->SetNumberContours(NCont);

  //...............
  TChain ch("OnMonSummary");
  for(int i=runlo; i<runhi+1; i++ ){
    char anafilename[200];
    sprintf(anafilename, "/nova/ana/users/psihas/OnMonSummary/Summary-onmon_ana-subrun-%.5u-*.root", i);
    ch.Add(anafilename);
    std::cout<<" Chained  ana file: "<<anafilename<<std::endl;
  }  
  //...............
  om::RateSummaryNt* Nt = new om::RateSummaryNt();
  ch.SetBranchAddress("fRateSummaryNt", &Nt);

  unsigned int DB_size  = 16;
  unsigned int DCM_size = 13;
  unsigned int FEB_size = 64;
  unsigned int PIX_size = 33;
  TH2D* HighRatesDCMs[DB_size];
  TH2D* AllTheDCMs[DB_size];
  TH2D* HighRatesFEBs[DB_size][DCM_size];
  TH2D* AllTheFEBs[DB_size][DCM_size];

  std::cout << "Initializing histos... " << std::endl;
  for(unsigned int i = 0; i < DB_size; ++i) {
    HighRatesDCMs[i] = 0;
    AllTheDCMs[i]    = 0;
    for(unsigned int j = 0; j < DCM_size; ++j) {
      HighRatesFEBs[i][j] = 0;
      AllTheFEBs[i][j]    = 0;
    }
  }
  
  int n_entries = ch.GetEntries();
  std::cout << "Got " << n_entries << std::endl;

  // Get run range if none is given
  if(runlo==0 && runhi==0){
    ch.GetEntry(0);
    runlo=Nt->run;
    ch.GetEntry(n_entries);
    runhi=Nt->run;
  }
  runhi++;
  int runsperbin = 10;
  int runbins    = (runhi-runlo)/runsperbin;
  int runhitbins    = (runhi-runlo)*64;

  TH1D* AllRates       = new TH1D("AllPixelHitRates", 
				  "Rate in all Pixels;log10(rate (Hz));pixels*subrun", 70, -1, 6);
  TH2D* RatesPerRun    = new TH2D("RatesPerRun", 
				  "Rate in all Pixels per run;run;log10(rate (Hz))",runbins, runlo, runhi, 70,-1,7);

  int pastrun=0;
  // Fill Pixel and FEB level graphs with rates
  for (int i = 0; i < n_entries; ++i)
  {
    ch.GetEntry(i);
    if(Nt->run!= pastrun && Nt->run % 5 == 0){
      std::cout << "RUN:  " << Nt->run << std::endl;
      pastrun=Nt->run;
    }
    double RUN = (double)(Nt->run)+((double)(Nt->subrun)/64.0);
    if(Nt->rate > 0){
      double lograte = log10(Nt->rate);
      AllRates->Fill(lograte);
      RatesPerRun->Fill(RUN,lograte);
    }
    if(AllTheDCMs[Nt->db] == 0){
      char ratehistoname[50];
      sprintf(ratehistoname, "AllTheDCMs_%.2u", Nt->db);
      char ratehistotitle[100];
      sprintf(ratehistotitle, "All rates on DCMs for DB%.2u;run;DCM", Nt->db);
      AllTheDCMs[Nt->db]     = new TH2D(ratehistoname, ratehistotitle, runbins, runlo, runhi, 12, 1, 13);
    }
    if(AllTheFEBs[Nt->db][Nt->dcm] == 0){
      char ratehistoname[50];
      sprintf(ratehistoname, "AllTheFEBs_%.2u_%.2u", Nt->db, Nt->dcm);
      char ratehistotitle[100];
      sprintf(ratehistotitle, "All rates on FEBs for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      AllTheFEBs[Nt->db][Nt->dcm]     = new TH2D(ratehistoname, ratehistotitle, runbins, runlo, runhi, 64, 0, 64);
    }
    AllTheDCMs[Nt->db]->Fill(RUN,Nt->dcm);
    AllTheFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    if(Nt->rate > 500){
      if(HighRatesFEBs[Nt->db][Nt->dcm] == 0){
	char hratehistoname[50];
	sprintf(hratehistoname, "HighRateFEB_%.2u_%.2u", Nt->db, Nt->dcm);
	char ratehistotitle[100];
	sprintf(ratehistotitle, "Fraction of noise incidents per FEB (DB%.2u DCM%.2u);run;FEB", Nt->db, Nt->dcm);
	HighRatesFEBs[Nt->db][Nt->dcm]  = new TH2D(hratehistoname, ratehistotitle, runbins, runlo, runhi, 64, 0, 64);
      }
      if(HighRatesDCMs[Nt->db] == 0){
	char hratehistoname[50];
	sprintf(hratehistoname, "HighRateDCM_%.2u", Nt->db);
	char ratehistotitle[100];
	sprintf(ratehistotitle, "Fraction of noise incidents per DCM (DB%.2u);run;DCM", Nt->db);
	HighRatesDCMs[Nt->db]  = new TH2D(hratehistoname, ratehistotitle, runbins, runlo, runhi, 12, 1, 13);
      }
      HighRatesDCMs[Nt->db]->Fill(RUN,Nt->dcm);
      HighRatesFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    }
  }

  //  Open output file for this subrun
  char outfilename[100];
  sprintf(outfilename, "Onmon_ana-Histos-run%.2u-%.2u.root", runlo, runhi-1);
  TFile* out_file = new TFile(outfilename, "RECREATE");
  out_file->mkdir("DCMPlots");
  std::cout << "Writting " << outfilename << std::endl;

  AllRates->Write();
  RatesPerRun->Write();

  for(unsigned int i = 1; i < DB_size; ++i) {
    if( AllTheDCMs[i] !=0  && HighRatesDCMs[i] !=0) {
      HighRatesDCMs[i]->Divide(AllTheDCMs[i]);
      AllTheDCMs[i]->GetYaxis()->SetNdivisions(12);
      HighRatesDCMs[i]->GetYaxis()->SetNdivisions(12);
      HighRatesDCMs[i]->Write();
    }
  }

  out_file->cd("DCMPlots");
  for(unsigned int i = 1; i < DB_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( AllTheFEBs[i][j] !=0 && HighRatesFEBs[i][j]) {
	HighRatesFEBs[i][j]->Divide(AllTheFEBs[i][j]);
	AllTheFEBs[i][j]->GetYaxis()->SetNdivisions(64);
	AllTheFEBs[i][j]->GetYaxis()->SetLabelSize(0.015);
	HighRatesFEBs[i][j]->GetYaxis()->SetNdivisions(64);
	HighRatesFEBs[i][j]->GetYaxis()->SetLabelSize(0.015);
	HighRatesFEBs[i][j]->GetXaxis()->SetLabelSize(0.025);
	HighRatesFEBs[i][j]->Write();
      }
    }
  }

  out_file->Close();

  //Show some examples of the 2D histos
  TCanvas* cAllRates   = new TCanvas("AllRates","AllRates",800,500);
  AllRates->Draw();

  TCanvas* cRatePerRun = new TCanvas("RatePerRun","RatePerRun",800,500);  
  RatesPerRun->Draw("colz");

  TCanvas* cAllDCMsDB1 = new TCanvas("AllDCMsDB1","AllDCMsDB1",1000,500);
  cAllDCMsDB1->Divide(2);
  cAllDCMsDB1->cd(1);
  AllTheFEBs[1][7]->Draw("colz");
  AllTheFEBs[1][7]->SetStats(0);
  cAllDCMsDB1->cd(2);
  HighRatesFEBs[1][7]->Draw("colz");
  HighRatesFEBs[1][7]->SetStats(0);

  TCanvas* cHighRatesDCMsDB1Ratio = new TCanvas("HighRatesDCMsDB1Ratio","HighRatesDCMsDB1Ratio",800,500); 
  HighRatesDCMs[1]->Draw("colz");
  HighRatesDCMs[1]->SetStats(0);
  cHighRatesDCMsDB1Ratio->SetGridy();

  TCanvas* cHighRatesFEBDCM1DB1Ratio = new TCanvas("HighRatesFEBDCM1DB1Ratio","HighRatesFEBDCM1DB1Ratio",800,500); 
  HighRatesFEBs[1][7]->Draw("colz");
  HighRatesFEBs[1][7]->SetStats(0);
  cHighRatesFEBDCM1DB1Ratioo->SetGridy();


}

