#include <TFile.h>
#include <TTree.h>
#include <TChain.h>
#include <TMarker.h>
#include <TCanvas.h>
#include <TNtuple.h>
#include <TLegend.h>
#include <TH1.h>
#include <TH2.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TGraph.h>
#include <TH1D.h>
#include <TProfile.h>
#include <TColor.h>
#include <TMultiGraph.h> 
#include <TPaveText.h>
#include "TStyle.h"
#include <iostream>
#include <vector>
#include "RateSummaryNt.h"

// do     gSystem->Load("libOnlineMonitoringAna.so");

// TChain select_runs(int runlo, int runhi)
// {
//   //  Chain files 

//   //missing check that chain doesn't exist already and if so delete it

//   TChain chain("OnMonSummary");
//   if(runlo==0 && runhi==0){
//     chain.Add("/nova/ana/users/psihas/OnMonSummary/Summary-onmon_ana-subrun-*.root");
//     std::cout<<" Chained EVERY ana file "<<std::endl;
//   }
//   else{
//     for(int i=runlo; i<runhi+1; i++ ){
//       char anafilename[200];
//       sprintf(anafilename, "/nova/ana/users/psihas/OnMonSummary/Summary-onmon_ana-subrun-0%.5u-*.root", i);
//       chain.Add(anafilename);
//       std::cout<<" Chained  ana file: "<<anafilename<<std::endl;
//     }
//   }

//   return(chain);
// }
//....................................................
//////////////////////////////////////////////////////

void PIXrates_by_run()
{

  int runlo=10230;
  //int runhi=10259;
  int runhi=10250;

  int colors[33];
  colors[0]=632;//red
  colors[1]=920+2;//gray
  colors[2]=632-7;//red
  colors[3]=900+5;//pink
  colors[4]=400-6;//yellow
  colors[5]=616;//mgnt
  colors[6]=616+2;//mgnt
  colors[7]=616-6;//mgnt
  colors[8]=800+10;//orange
  colors[9]=880-7;//violet
  colors[10]=600;//blue
  colors[11]=600+2;//blue
  colors[12]=600-3;//blue
  colors[13]=600+3;//blue
  colors[14]=600-6;//blue
  colors[15]=860+7;//azure
  colors[16]=432+1;//cyan
  colors[17]=880-5;//violet
  colors[18]=432-6;//cyan
  colors[19]=416;//green
  colors[20]=416+3;//green
  colors[21]=416-6;//green
  colors[22]=416+2;//green
  colors[23]=820-5;//spring
  colors[24]=820-7;//spring
  colors[25]=820+4;//spring
  colors[26]=400+2;//yellow
  colors[27]=400+3;//yellow
  colors[28]=800+7;//orange
  colors[29]=632+2;//red
  colors[30]=860+7;//azure
  colors[31]=900+6;//pink
  colors[32]=800+4;//orange

  //...............
  TChain ch("OnMonSummary");
  for(int i=runlo; i<runhi+1; i++ ){
    char anafilename[200];
    sprintf(anafilename, "/nova/ana/users/psihas/OnMonSummary/Summary-onmon_ana-subrun-%.5u-*.root", i);
    ch.Add(anafilename);
    std::cout<<" Chained  ana file: "<<anafilename<<std::endl;
  }  
  //...............
  om::RateSummaryNt* Nt = new om::RateSummaryNt();
  ch.SetBranchAddress("fRateSummaryNt", &Nt);

  unsigned int DB_size  = 16;
  unsigned int DCM_size = 13;
  unsigned int FEB_size = 64;
  unsigned int PIX_size = 33;
  TH1D* PIXRates[DB_size][DCM_size][FEB_size][PIX_size];
  TH1D* PIXMin[DB_size][DCM_size][FEB_size][PIX_size];

  std::cout << "Initializing histos... " << std::endl;
  for(unsigned int i = 0; i < DB_size; ++i) {
    for(unsigned int j = 0; j < DCM_size; ++j) {
      for(unsigned int k = 0; k < FEB_size; ++k) {
      	for(unsigned int l = 0; l < PIX_size; ++l) {
      	  PIXRates[i][j][k][l]  = 0;
      	  PIXMin[i][j][k][l]  = 0;
      	}
      }
    }
  }
  
  int n_entries = ch.GetEntries();
  std::cout << "Got " << n_entries << std::endl;

  // Get run range if none is given
  if(runlo==0 && runhi==0){
    ch.GetEntry(0);
    runlo=Nt->run;
    ch.GetEntry(n_entries);
    runhi=Nt->run;
  }
  runhi++;
  int runsperbin = 10;
  int runbins    = (runhi-runlo)/runsperbin;
  int runhitbins    = (runhi-runlo)*64;

  TH1D* AllRates       = new TH1D("AllPixelHitRates", "Rate in all Pixels;log10(rate (Hz));pixels*subrun", 70, -1, 6);
  TH2D* RatesPerRun    = new TH2D("RatesPerRun", "Rate in all Pixels per run;run;log10(rate (Hz))",runbins, runlo, runhi, 70, -1, 7);

  int pastrun=0;
  // Fill Pixel and FEB level graphs with rates
  for (int i = 0; i < n_entries; ++i)
  {
    ch.GetEntry(i);
    if(Nt->run!= pastrun && Nt->run % 5 == 0){
      std::cout << "RUN:  " << Nt->run << std::endl;
      pastrun=Nt->run;
    }
    bool dothispix=0;
    double RUN = (double)(Nt->run)+((double)(Nt->subrun)/64.0);

    //    if(Nt->db == 1 && Nt->dcm == 3 && Nt->feb == 49) dothispix=1;
    // if(Nt->db == 1 && Nt->dcm == 3 && Nt->feb == 40) dothispix=1;
    // if(Nt->db == 1 && Nt->dcm == 3 && Nt->feb == 20) dothispix=1;
    // if(Nt->db == 1 && Nt->dcm == 10 && Nt->feb == 1) dothispix=1;
    // if(Nt->db == 1 && Nt->dcm == 10 && Nt->feb == 6) dothispix=1;
    if(Nt->db == 1 && Nt->dcm == 10 && Nt->feb == 11) dothispix=1;
    // if(Nt->db == 2 && Nt->dcm == 10 && Nt->feb == 31) dothispix=1;
    // if(Nt->db == 2 && Nt->dcm == 10 && Nt->feb == 25) dothispix=1;
    // if(Nt->db == 2 && Nt->dcm == 10 && Nt->feb == 41) dothispix=1;
    // if(Nt->db == 2 && Nt->dcm == 10 && Nt->feb == 23) dothispix=1;
    // if(Nt->db == 2 && Nt->dcm == 10 && Nt->feb == 62) dothispix=1;

    if(dothispix==1){
      if(PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
	char histoname[30];
	sprintf(histoname, "RatePIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
	PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Hit Rate in Pixels;run;rate (Hz)", 
								runhitbins, runlo, runhi); 
      }
      PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->rate);
      if(PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
	char histoname[30];
	sprintf(histoname, "MinADCPIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
	PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Min ADC in Pixels;run;ADC", 
							      runhitbins, runlo, runhi); 
      }
      PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->minADC);
    }
  }

  //  Open output file for this subrun
  char outfilename[100];
  sprintf(outfilename, "Onmon_ana-HistosPIXMin-run%.2u-%.2u.root", runlo, runhi-1);
  TFile* out_file = new TFile(outfilename, "RECREATE");
  out_file->mkdir("PIXPlots");
  std::cout << "Writting " << outfilename << std::endl;

  out_file->cd("PIXPlots");
  for(unsigned int i = 1; i < DB_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      for(unsigned int k = 0; k < FEB_size; ++k) {
   	for(int l = 0; l < PIX_size; ++l) {
   	  if( PIXRates[i][j][k][l] !=0) {
  	    PIXRates[i][j][k][l]->SetLineColor(colors[l]);
   	    PIXRates[i][j][k][l]->SetMarkerStyle(34);
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXRates[i][j][k][l]->SetAxisRange(0.01,1000000,"Y");
  	    PIXRates[i][j][k][l]->Write();
  	    PIXMin[i][j][k][l]->SetLineStyle(2);
  	    PIXMin[i][j][k][l]->SetLineColor(colors[l]);
   	    PIXMin[i][j][k][l]->SetMarkerStyle(34);
  	    PIXMin[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXMin[i][j][k][l]->SetAxisRange(0,200,"Y");
  	    PIXMin[i][j][k][l]->Write();
   	  }
   	}
      }
    }
  }

  out_file->Close();

  for(int i = 1; i < DCM_size; ++i) {
    for(int j = 1; j < DCM_size; ++j) {
      for(int k = 0; k < FEB_size; ++k) {
	bool hascanvas=0;
  	for(int l = 0; l < PIX_size; ++l) {
  	  if( PIXRates[i][j][k][l] !=0) {
	    if(hascanvas==0){
	      char name[30];
	      sprintf(name, "PIXratesFEB_%.2u_%.2u_%.2u", i, j, k);
	      TCanvas* CanvasP = new TCanvas(name,name,1167,335);  
	      CanvasP->SetLogy();
	      hascanvas=1;
	    }
  	    PIXRates[i][j][k][l]->SetMarkerStyle(34);
  	    PIXRates[i][j][k][l]->SetAxisRange(0.01,1000000,"Y");
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXRates[i][j][k][l]->Draw("same, P");
	    PIXRates[i][j][k][l]->GetYaxis()->SetTitleOffset(0.5);
	    PIXRates[i][j][k][l]->SetStats(0);
	    PIXRates[i][j][k][l]->SetTitleColor(kWhite);
	    TPaveText *pt = new TPaveText(0.12, 0.78, 0.25, 0.89, "NDC");
	    pt->AddText("Hit Rate in Pixels");
	    char address[30];
	    sprintf(address, "DB%.2u DCM%.2u FEB%.2u", i, j, k);
	    pt->AddText(address);
	    pt->SetLineColor(kWhite);
	    pt->SetShadowColor(kWhite);
	    pt->SetFillColor(kWhite);
	    pt->Draw();
	    pt->SetTextFont(42);
  	  }
  	}
      }
    }
  }

  for(int i = 1; i < DCM_size; ++i) {
    for(int j = 1; j < DCM_size; ++j) {
      for(int k = 0; k < FEB_size; ++k) {
	bool hascanvas=0;
  	for(int l = 0; l < PIX_size; ++l) {
  	  if( PIXMin[i][j][k][l] !=0) {
	    if(hascanvas==0){
	      char name[30];
	      sprintf(name, "PIXMINratesFEB_%.2u_%.2u_%.2u", i, j, k);
	      TCanvas* CanvasM = new TCanvas(name,name,1167,195);  
	      hascanvas=1;
	    }
  	    PIXMin[i][j][k][l]->SetLineColor(colors[l]);
  	    PIXMin[i][j][k][l]->SetLineStyle(2);
   	    PIXMin[i][j][k][l]->SetMarkerStyle(34);
  	    PIXMin[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXMin[i][j][k][l]->SetAxisRange(1,200,"Y");
  	    PIXMin[i][j][k][l]->Draw("same, P");
	    PIXMin[i][j][k][l]->SetStats(0);
	    PIXMin[i][j][k][l]->SetTitleColor(kWhite);
	    PIXMin[i][j][k][l]->GetYaxis()->SetNdivisions(504, kTRUE);
	    PIXMin[i][j][k][l]->GetYaxis()->SetLabelSize(0.05);
	    PIXMin[i][j][k][l]->GetYaxis()->SetTitleSize(0.067);
	    PIXMin[i][j][k][l]->GetYaxis()->SetTitleOffset(0.25);
	    PIXMin[i][j][k][l]->GetXaxis()->SetLabelSize(0.067);
	    PIXMin[i][j][k][l]->GetXaxis()->SetTitleSize(0.067);
	    PIXMin[i][j][k][l]->GetXaxis()->SetTitleOffset(0.7);
	    TPaveText *pt = new TPaveText(0.12, 0.7, 0.25, 0.89, "NDC");
	    pt->AddText("Min ADC in Pixels");
	    char address[30];
	    sprintf(address, "DB%.2u DCM%.2u FEB%.2u", i, j, k);
	    pt->AddText(address);
	    pt->SetLineColor(kWhite);
	    pt->SetShadowColor(kWhite);
	    pt->SetFillColor(kWhite);
	    pt->Draw();
	    pt->SetTextFont(42);
  	  }
  	}
      }
    }
  }

}

