#include <TFile.h>
#include <TTree.h>
#include <TLine.h>
#include <TText.h>
#include <TArrow.h>
#include <TChain.h>
#include <TMarker.h>
#include <TCanvas.h>
#include <TNtuple.h>
#include <TLegend.h>
#include <TH1.h>
#include <TH2.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TGraph.h>
#include <TH1D.h>
#include <TProfile.h>
#include <TColor.h>
#include <TMultiGraph.h> 
#include <TPaveText.h>
#include "TStyle.h"
#include <iostream>
#include <fstream>
#include <vector>
#include "/home/novasoft/onmon_testrel_CR13-07-29/OnlineMonitoring/ana/RateSummaryNt.h"

// do     gSystem->Load("libOnlineMonitoringAna.so");
// load   .L OnMonAnaTestStandHistos.C+
// then   MakeTestStandPlots()

void MakeTestStandPlots_AllRuns()
{
  std::cout << "\n\nMaking plots for all runs...\n\n";

  /*   Plot Color Options
   */
  int colors[33];
  colors[0]=632;//red
  colors[1]=920+2;//gray
  colors[2]=632-7;//red
  colors[3]=900+5;//pink
  colors[4]=400-6;//yellow
  colors[5]=616;//mgnt
  colors[6]=616+2;//mgnt
  colors[7]=616-6;//mgnt
  colors[8]=800+10;//orange
  colors[9]=880-7;//violet
  colors[10]=600;//blue
  colors[11]=600+2;//blue
  colors[12]=600-3;//blue
  colors[13]=600+3;//blue
  colors[14]=600-6;//blue
  colors[15]=860+7;//azure
  colors[16]=432+1;//cyan
  colors[17]=880-5;//violet
  colors[18]=432-6;//cyan
  colors[19]=416;//green
  colors[20]=416+3;//green
  colors[21]=416-6;//green
  colors[22]=416+2;//green
  colors[23]=820-5;//spring
  colors[24]=820-7;//spring
  colors[25]=820+4;//spring
  colors[26]=400+2;//yellow
  colors[27]=400+3;//yellow
  colors[28]=800+7;//orange
  colors[29]=632+2;//red
  colors[30]=860+7;//azure
  colors[31]=900+6;//pink
  colors[32]=800+4;//orange

  const Int_t NRGBs = 3;	
  const Int_t NCont = 255;
  Double_t stops[NRGBs] = { 0.00, 0.34, 1.00};
  Double_t red[NRGBs]   = { 1.00, 1.00, 0.60};
  Double_t green[NRGBs] = { 0.97, 0.40, 0.00};
  Double_t blue[NRGBs]  = { 0.99, 0.70, 0.00};

  TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
  gStyle->SetNumberContours(NCont);
  //...............
  /*   Get run info from file
   */

  std::ifstream in;
  in.open("/home/novasoft/onmon_testrel_CR13-07-29/OnMon_teststand_results/current_runs.txt");

  int run, sr, hw, hc, newData, Nbins;
  std::vector<int> runList;
  std::vector<int> srList;
  std::vector<int> hwList;
  std::vector<int> hcList;
  std::vector<int> newDataList;
  std::vector<int> startBin;

  // read information in from text file on disk
  while(in.good()) {
    in >> run;
    in >> sr;
    in >> hw;
    in >> hc;
    in >> newData;

    runList.push_back(run);
    srList .push_back(sr);
    hwList .push_back(hw);
    hcList .push_back(hc);
    newDataList.push_back(newData);
  }

  // sort lists by run number ?
  // make startBin list
  int bin = 0;
  for(unsigned int i = 0; i < runList.size(); i++) {
    startBin.push_back(bin);
    bin += srList[i];
  }
  Nbins = startBin[runList.size()-1] + srList[runList.size()-1];

  /*
  for(unsigned int i = 0; i < runList.size(); i++){
    std::cout << "\nrun = " << runList[i] << "\tsubrun = " << srList[i]
	      << "\tnew hardware? = " << hwList[i] << "\tstartBin = " << startBin[i];
  }
  std::cout << "\nBINS = " << Nbins;
  */


  /*   Chain Files
   */
  TChain ch("OnMonSummary");
  
  for(unsigned int i = 0; i < runList.size(); i++) {
    char anafilename[200];
    //    sprintf(anafilename, "/nova/app/users/psihas/MyTestRel/results/TestStandAna/Summary-onmon_ana-subrun-%.3u-*.root", runList[i]);
    sprintf(anafilename, "/home/novasoft/onmon_testrel_CR13-07-29/OnMon_teststand_results/onmon_ana_files/Summary-onmon_ana-subrun-%.3u-*.root", runList[i]);
    ch.Add(anafilename);
    std::cout << " Chained  ana file : " << anafilename << std::endl;
  }
  //...............

  /*   Initializing
   */

  om::RateSummaryNt* Nt = new om::RateSummaryNt();
  ch.SetBranchAddress("fRateSummaryNt", &Nt);

  unsigned int DB_size  = 16;
  unsigned int DCM_size = 13;
  unsigned int FEB_size = 31;
  unsigned int PIX_size = 33;
  TH1D* PIXRates[DB_size][DCM_size][FEB_size][PIX_size];
  TH1D* PIXMin[DB_size][DCM_size][FEB_size][PIX_size];
  TH2D* HighRatesFEBs[DB_size][DCM_size];
  TH2D* AllTheFEBs[DB_size][DCM_size];

  TH2D* EmptyPIXs[DB_size][DCM_size];
  TH2D* MissingData[DB_size][DCM_size];

  std::cout << "\n\nInitializing histos... " << std::endl;
  for(unsigned int i = 0; i < DB_size; ++i) {
    for(unsigned int j = 0; j < DCM_size; ++j) {
      HighRatesFEBs[i][j] = 0;
      AllTheFEBs[i][j]    = 0;
      EmptyPIXs[i][j]     = 0;
      MissingData[i][j]   = 0;
      for(unsigned int k = 0; k < FEB_size; ++k) {
      	for(unsigned int l = 0; l < PIX_size; ++l) {
      	  PIXRates[i][j][k][l]  = 0;
      	  PIXMin[i][j][k][l]  = 0;
      	}
      }
    }
  }
  
  int n_entries = ch.GetEntries();
  std::cout << "\nLooping over " << n_entries << " entries..." <<std::endl;

  /*   Fill Pixel and FEB level graphs with rates
   */
  int pastrun=0;
  for (int i = 0; i < n_entries; ++i)
  {
    ch.GetEntry(i);
    if(Nt->run!= pastrun){
      std::cout << "Processing run " << Nt->run << std::endl;
      pastrun=Nt->run;
    }
    // Calculate temp run number for plotting
    int RUN=-1;
    for(unsigned int s = 0; s < runList.size(); s++) {
      if(Nt->run == runList[s]){
	RUN= Nt->subrun + startBin[s];
	s=runList.size()+1;
      }
    }

    if(PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
      char histoname[30];
      sprintf(histoname, "RatePIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
      PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Hit Rate in Pixels;run;rate (Hz)", 
      							      Nbins, 0, Nbins); 
    }
    PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->rate);
    if(PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
      char histoname[30];
      sprintf(histoname, "MinADCPIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
      PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Min ADC in Pixels;run;ADC", 
      							      Nbins, 0, Nbins); 
    }
    PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->minADC);
    
    if(AllTheFEBs[Nt->db][Nt->dcm] == 0){
      char ratehistoname[50];
      sprintf(ratehistoname, "AllTheFEBs_%.2u_%.2u", Nt->db, Nt->dcm);
      char ratehistotitle[100];
      sprintf(ratehistotitle, "All rates on FEBs for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      AllTheFEBs[Nt->db][Nt->dcm]     = new TH2D(ratehistoname, ratehistotitle, Nbins, 0, Nbins, 31, 0, 31);
    }
    AllTheFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    if(Nt->rate > 500){
      if(HighRatesFEBs[Nt->db][Nt->dcm] == 0){
	char hratehistoname[50];
	sprintf(hratehistoname, "HighRateFEB_%.2u_%.2u", Nt->db, Nt->dcm);
	char ratehistotitle[100];
	sprintf(ratehistotitle, "Fraction of noise incidents per FEB (DB%.2u DCM%.2u);run;FEB", Nt->db, Nt->dcm);
	HighRatesFEBs[Nt->db][Nt->dcm]  = new TH2D(hratehistoname, ratehistotitle, Nbins, 0, Nbins, 31, 0, 31);
      }
      HighRatesFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    }

    if(EmptyPIXs[Nt->db][Nt->dcm] == 0){
      char emptyhistoname[50];
      sprintf(emptyhistoname, "EmptyPIXs_%.2u_%.2u", Nt->db, Nt->dcm);
      char emptyhistotitle[100];
      sprintf(emptyhistotitle, "Empty PIXs for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      EmptyPIXs[Nt->db][Nt->dcm]     = new TH2D(emptyhistoname, emptyhistotitle, Nbins, 0, Nbins, 31, 0, 31);

      char mdhistoname[50];
      sprintf(mdhistoname, "MissingData_%.2u_%.2u", Nt->db, Nt->dcm);
      char mdhistotitle[100];
      sprintf(mdhistotitle, "Missing Data for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      MissingData[Nt->db][Nt->dcm]     = new TH2D(mdhistoname, mdhistotitle, Nbins, 0, Nbins, 31, 0, 31);
    }
    if(Nt->rate == 0.0)
      EmptyPIXs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);

  }

  /*    Compute Ratios
  */
  for(unsigned int i = 1; i < DB_size+1; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( AllTheFEBs[i][j] !=0 && HighRatesFEBs[i][j]!=0) HighRatesFEBs[i][j]->Divide(AllTheFEBs[i][j]);
    }
  }



  //
  // Make missing data plot
  //
  for(unsigned int i = 1; i < DB_size+1; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( EmptyPIXs[i][j] !=0 ) {

	int NXbins = EmptyPIXs[i][j]->GetNbinsX();
	int NYbins = EmptyPIXs[i][j]->GetNbinsY();
	
	for(int a = 1; a <= NXbins; ++a) {
	  for(int b = 1; b <= NYbins; ++b) {

	    double bin1 = AllTheFEBs[i][j]->GetBinContent(AllTheFEBs[i][j]->GetBin(a,b));
	    double bin2 = EmptyPIXs[i][j]->GetBinContent(EmptyPIXs[i][j]->GetBin(a,b));

	    if( bin1 == 0.0 || bin2 == 32.0 )
	      MissingData[i][j]->SetBinContent(a,b,1.0);

	  }
	}

      }
    }
  }


  /*    Display Pots
  */
  TCanvas* cAllDCMsDB1 = new TCanvas("AllDCMsDB1","AllDCMsDB1",1000,500);
  cAllDCMsDB1->Divide(2);
  cAllDCMsDB1->cd(1);
  AllTheFEBs[1][1]->Draw("colz");
  AllTheFEBs[1][1]->SetStats(0);
  cAllDCMsDB1->cd(2);
  HighRatesFEBs[1][1]->Draw("colz");
  HighRatesFEBs[1][1]->SetStats(0);

  std::cout << "\nWriting AllRuns_AllDCMsDB1.png to disk.\n";
  cAllDCMsDB1->Print("AllRuns_AllDCMsDB1.png");



  TCanvas* cHighRatesTestStandFEBsRatio = new TCanvas("HighRatesTestStandFEBs","cHighRatesTestStandFEBs",880,800); 
  MissingData[1][1]->SetTitle("Fraction of noise incidents per FEB (Test Stand)");
  MissingData[1][1]->GetYaxis()->SetNdivisions(31);
  MissingData[1][1]->GetYaxis()->SetLabelSize(0.015);
  MissingData[1][1]->GetXaxis()->SetTickLength(0);
  MissingData[1][1]->GetXaxis()->SetLabelColor(0);
  MissingData[1][1]->GetXaxis()->SetLabelSize(0.02);
  MissingData[1][1]->GetXaxis()->SetTitleSize(0.025);
  MissingData[1][1]->GetYaxis()->SetTitleSize(0.025);
  MissingData[1][1]->GetXaxis()->SetNdivisions(1,122, kFALSE);
  MissingData[1][1]->SetMaximum(0.501);
  MissingData[1][1]->SetFillColor(kGray);
  MissingData[1][1]->Draw("box");
  MissingData[1][1]->SetStats(0);

  HighRatesFEBs[1][1]->SetTitle("Fraction of noise incidents per FEB (Test Stand)");
  HighRatesFEBs[1][1]->GetYaxis()->SetNdivisions(31);
  HighRatesFEBs[1][1]->GetYaxis()->SetLabelSize(0.015);
  HighRatesFEBs[1][1]->GetXaxis()->SetTickLength(0);
  HighRatesFEBs[1][1]->GetXaxis()->SetLabelColor(0);
  HighRatesFEBs[1][1]->GetXaxis()->SetLabelSize(0.02);
  HighRatesFEBs[1][1]->GetXaxis()->SetTitleSize(0.025);
  HighRatesFEBs[1][1]->GetYaxis()->SetTitleSize(0.025);
  HighRatesFEBs[1][1]->GetXaxis()->SetNdivisions(1,122, kFALSE);
  HighRatesFEBs[1][1]->SetMaximum(0.501);
  HighRatesFEBs[1][1]->Draw("samecolz");
  HighRatesFEBs[1][1]->SetStats(0);
  cHighRatesTestStandFEBsRatio->SetGridy();

  for(unsigned int i = 0; i < runList.size(); i++) {
    double y2 = 31;
    TLine *line = new TLine(startBin[i], 0, startBin[i], y2);
    line->SetLineStyle(2);
    if(hwList[i] == 1){
      line->SetLineWidth(2);
      line->SetLineStyle(9);
    }
    line->Draw();
    char buff[16];
    sprintf(buff,"%.3d",runList[i]);
    TText *t = new TText(startBin[i],-0.7,buff);
    t->SetTextAlign(22);
    t->SetTextFont(42);
    t->SetTextSize(0.016);
    t->SetTextAngle(90);
    t->Draw();
    TArrow *a = new TArrow(startBin[i],31.5,startBin[i]+1,31.5, 0.03, "|>");
    a->SetLineWidth(3);
    a->SetLineColor(kAzure-3);
    a->SetFillColor(kAzure-3);
    if(hcList[i] == 1){
      a->SetLineColor(kRed);
      a->SetFillColor(kRed);
    }
    a->SetArrowSize(0.01);
    a->Draw();
  }

  std::cout << "\nWriting AllRuns_TestStandFEBRates.png to disk.\n";
  cHighRatesTestStandFEBsRatio->Print("AllRuns_TestStandFEBRates.png");  


  
  DCM_size = 2;
  //.............. Show plots with both parts
  for(unsigned int i = 1; i < DCM_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {

      TCanvas* CanvasP[FEB_size];
      bool  goodCanvas[FEB_size];

      for(unsigned int k = 0; k < FEB_size; ++k) {
	// bool hascanvas=0, haspad=0;
	bool haspad=0;

	
	char name[40];
	sprintf(name, "AllRuns_2PIXratesTestStandFEB_%.2u",k);
	CanvasP[k] = new TCanvas(name,name,1167,600);
	CanvasP[k]->SetLogy();
	CanvasP[k]->cd(1);
	CanvasP[k]->SetBottomMargin(0.40);
	
	goodCanvas[k] = false;
	
  	for(unsigned int l = 0; l < PIX_size; ++l) {

  	  if( PIXRates[i][j][k][l] !=0) {
	    goodCanvas[k] = true;

  	    PIXRates[i][j][k][l]->SetLineColor(colors[l]);
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXRates[i][j][k][l]->SetMarkerStyle(34);
  	    PIXRates[i][j][k][l]->SetAxisRange(0.01,1000000,"Y");
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
	    PIXRates[i][j][k][l]->Draw("same, L");
	    PIXRates[i][j][k][l]->GetXaxis()->SetNdivisions(1,Nbins-1, kFALSE);
	    PIXRates[i][j][k][l]->GetXaxis()->SetTickLength(0);
	    PIXRates[i][j][k][l]->GetXaxis()->SetLabelColor(0);
	    PIXRates[i][j][k][l]->SetStats(0);
	    PIXRates[i][j][k][l]->SetTitle("");
	    TPaveText *pt = new TPaveText(0.12, 0.82, 0.25, 0.89, "NDC");
	    pt->AddText("Hit Rate in Pixels");
	    char address[30];
	    sprintf(address, "Test Stand FEB %.2u", k);
	    //title must include dcm & db if more than 1 !
	    pt->AddText(address);
	    pt->SetLineColor(kWhite);
	    pt->SetShadowColor(kWhite);
	    pt->SetFillColor(kWhite);
	    pt->Draw();
	    pt->SetTextFont(42);

	    TLine *rate500 = new TLine(0,500,startBin[runList.size()-1]+srList[runList.size()-1],500);
	    rate500->SetLineWidth(2);
	    rate500->SetLineStyle(2);
	    rate500->Draw();
	    
	    for(unsigned int s = 0; s < runList.size(); s++) {
	      double y2 = 1000000;
	      if((double)startBin[s] < (double)(Nbins/6) ) y2=50000;
	      TLine *line = new TLine(startBin[s], 0, startBin[s], y2);
	      line->SetLineStyle(2);
	      if(hwList[s] == 1){
		line->SetLineWidth(2);
		line->SetLineStyle(9);
	      }
	      line->Draw();
	      TArrow *a = new TArrow(startBin[s],2000000,startBin[s]+1,2000000, 0.03, "|>");
	      a->SetLineWidth(3);
	      a->SetLineColor(kAzure-3);
	      a->SetFillColor(kAzure-3);
	      if(hcList[s] == 1){
		a->SetLineColor(kRed);
		a->SetFillColor(kRed);
	      }
	      a->SetArrowSize(0.01);
	      a->Draw();
	    } // end for s

  	  } // end if(pixrate != 0)
  	}//for pix

	for(unsigned int l = 0; l < PIX_size; ++l) {
	  if( PIXMin[i][j][k][l] !=0) {
	    if(haspad==0 && goodCanvas[k]){
	      TPad* pad = new TPad("pad", "pad", 0., 0., 1., 1.);
	      pad->SetTopMargin(0.63);
	      pad->SetFillColor(0);
	      pad->SetFillStyle(0);
	      pad->Draw();
	      pad->cd(0);
	      haspad=1;
	    }
  	    PIXMin[i][j][k][l]->SetLineColor(colors[l]);
	    PIXMin[i][j][k][l]->SetTitle("");
  	    PIXMin[i][j][k][l]->SetLineStyle(2);
   	    PIXMin[i][j][k][l]->SetMarkerStyle(34);
  	    PIXMin[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXMin[i][j][k][l]->SetAxisRange(0,200,"Y");
  	    PIXMin[i][j][k][l]->Draw("same, L");
  	    PIXMin[i][j][k][l]->SetStats(0);
  	    PIXMin[i][j][k][l]->GetXaxis()->SetNdivisions(1,122, kFALSE);
	    PIXMin[i][j][k][l]->GetXaxis()->SetTickLength(0);
	    PIXMin[i][j][k][l]->GetXaxis()->SetLabelColor(0);
  	    PIXMin[i][j][k][l]->GetYaxis()->SetNdivisions(504, kTRUE);
  	    TPaveText *pt1 = new TPaveText(0.12, 0.3, 0.25, 0.36, "NDC");
  	    pt1->AddText("Min ADC in Pixels");
  	    char address[30];
  	    sprintf(address, "Test Stand FEB%.2u", k);
	    // title must include dcm & db if more than 1 !
  	    pt1->AddText(address);
  	    pt1->SetLineColor(kWhite);
  	    pt1->SetShadowColor(kWhite);
  	    pt1->SetFillColor(kWhite);
  	    pt1->Draw();
  	    pt1->SetTextFont(42);
	    for(unsigned int s = 0; s < runList.size(); s++) {
	      double y2 = 200;
	      if((double)startBin[s] < (double)(Nbins/6) ) y2=150;
	      TLine *line = new TLine(startBin[s], 0, startBin[s], y2);
	      line->SetLineStyle(2);
	      if(hwList[s] == 1){
		line->SetLineWidth(2);
		line->SetLineStyle(9);
	      }
	      line->Draw();
	      char buff[16];
	      sprintf(buff,"%.3d",runList[s]);
	      TText *t = new TText(startBin[s],-20,buff);
	      t->SetTextAlign(22);
	      t->SetTextFont(42);
	      t->SetTextSize(0.016);
	      t->SetTextAngle(90);
	      t->Draw();
	    }
	  }
	}//for pix

	char name2[40];
	sprintf(name2, "AllRuns_2PIXratesTestStandFEB_%.2u.png",k);
	if(goodCanvas[k]) {
	  std::cout << "Printing canvas: " << name2 << std::endl;
	  CanvasP[k]->Print(name2);
	}

      }//for feb (loop over k)
    }//for dcm (loop over j)
  }//for db (loop over i)
  

  /*
  //  Open output file for this subrun
  char outfilename[100];
  sprintf(outfilename, "/nova/app/users/psihas/MyTestRel/results/TestStandAna/Onmon_ana-HistosPIXMin-run%.2u-%.2u.root",  runList[0],  runList[runList.size()-1]);
  TFile* out_file = new TFile(outfilename, "RECREATE");
  std::cout << "Writting " << outfilename << std::endl;

  for(unsigned int i = 1; i < DB_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( AllTheFEBs[i][j] !=0 && HighRatesFEBs[i][j] !=0) {
	HighRatesFEBs[i][j]->Write();
      }
      for(unsigned int k = 0; k < FEB_size; ++k) {
   	for(int l = 0; l < PIX_size; ++l) {
   	  if( PIXRates[i][j][k][l] !=0) {
  	    PIXRates[i][j][k][l]->Write();
  	    PIXMin[i][j][k][l]->Write();
   	  }
   	}
      }
    }
  }

  out_file->Close();
  */


}




void MakeTestStandPlots_NewRuns()
{
  std::cout << "\n\nMaking plots for new runs only...\n\n";

  /*   Plot Color Options
   */
  int colors[33];
  colors[0]=632;//red
  colors[1]=920+2;//gray
  colors[2]=632-7;//red
  colors[3]=900+5;//pink
  colors[4]=400-6;//yellow
  colors[5]=616;//mgnt
  colors[6]=616+2;//mgnt
  colors[7]=616-6;//mgnt
  colors[8]=800+10;//orange
  colors[9]=880-7;//violet
  colors[10]=600;//blue
  colors[11]=600+2;//blue
  colors[12]=600-3;//blue
  colors[13]=600+3;//blue
  colors[14]=600-6;//blue
  colors[15]=860+7;//azure
  colors[16]=432+1;//cyan
  colors[17]=880-5;//violet
  colors[18]=432-6;//cyan
  colors[19]=416;//green
  colors[20]=416+3;//green
  colors[21]=416-6;//green
  colors[22]=416+2;//green
  colors[23]=820-5;//spring
  colors[24]=820-7;//spring
  colors[25]=820+4;//spring
  colors[26]=400+2;//yellow
  colors[27]=400+3;//yellow
  colors[28]=800+7;//orange
  colors[29]=632+2;//red
  colors[30]=860+7;//azure
  colors[31]=900+6;//pink
  colors[32]=800+4;//orange

  const Int_t NRGBs = 3;	
  const Int_t NCont = 255;
  Double_t stops[NRGBs] = { 0.00, 0.34, 1.00};
  Double_t red[NRGBs]   = { 1.00, 1.00, 0.60};
  Double_t green[NRGBs] = { 0.97, 0.40, 0.00};
  Double_t blue[NRGBs]  = { 0.99, 0.70, 0.00};

  TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
  gStyle->SetNumberContours(NCont);
  //...............
  /*   Get run info from file
   */

  std::ifstream in;
  in.open("/home/novasoft/onmon_testrel_CR13-07-29/OnMon_teststand_results/current_runs.txt");

  int run, sr, hw, hc, newData, Nbins;
  std::vector<int> runList;
  std::vector<int> srList;
  std::vector<int> hwList;
  std::vector<int> hcList;
  std::vector<int> newDataList;
  std::vector<int> startBin;

  // read information in from text file on disk
  while(in.good()) {
    in >> run;
    in >> sr;
    in >> hw;
    in >> hc;
    in >> newData;

    if(newData == 1) {
      runList.push_back(run);
      srList .push_back(sr);
      hwList .push_back(hw);
      hcList .push_back(hc);
      newDataList.push_back(newData);
    }
  }

  // sort lists by run number ?
  // make startBin list
  int bin = 0;
  for(unsigned int i = 0; i < runList.size(); i++) {
    startBin.push_back(bin);
    bin += srList[i];
  }
  Nbins = startBin[runList.size()-1] + srList[runList.size()-1];

  /*
  for(unsigned int i = 0; i < runList.size(); i++){
    std::cout << "\nrun = " << runList[i] << "\tsubrun = " << srList[i]
	      << "\tnew hardware? = " << hwList[i] << "\tstartBin = " << startBin[i];
  }
  std::cout << "\nBINS = " << Nbins;
  */


  /*   Chain Files
   */
  TChain ch("OnMonSummary");
  
  for(unsigned int i = 0; i < runList.size(); i++) {
    char anafilename[200];
    //    sprintf(anafilename, "/nova/app/users/psihas/MyTestRel/results/TestStandAna/Summary-onmon_ana-subrun-%.3u-*.root", runList[i]);
    sprintf(anafilename, "/home/novasoft/onmon_testrel_CR13-07-29/OnMon_teststand_results/onmon_ana_files/Summary-onmon_ana-subrun-%.3u-*.root", runList[i]);
    ch.Add(anafilename);
    std::cout << " Chained  ana file : " << anafilename << std::endl;
  }
  //...............



  /*   Initializing
   */

  om::RateSummaryNt* Nt = new om::RateSummaryNt();
  ch.SetBranchAddress("fRateSummaryNt", &Nt);

  unsigned int DB_size  = 16;
  unsigned int DCM_size = 13;
  unsigned int FEB_size = 31;
  unsigned int PIX_size = 33;
  TH1D* PIXRates[DB_size][DCM_size][FEB_size][PIX_size];
  TH1D* PIXMin[DB_size][DCM_size][FEB_size][PIX_size];
  TH2D* HighRatesFEBs[DB_size][DCM_size];
  TH2D* AllTheFEBs[DB_size][DCM_size];

  TH2D* EmptyPIXs[DB_size][DCM_size];
  TH2D* MissingData[DB_size][DCM_size];

  std::cout << "\n\nInitializing histos... " << std::endl;
  for(unsigned int i = 0; i < DB_size; ++i) {
    for(unsigned int j = 0; j < DCM_size; ++j) {
      HighRatesFEBs[i][j] = 0;
      AllTheFEBs[i][j]    = 0;
      EmptyPIXs[i][j]     = 0;
      MissingData[i][j]   = 0;
      for(unsigned int k = 0; k < FEB_size; ++k) {
      	for(unsigned int l = 0; l < PIX_size; ++l) {
      	  PIXRates[i][j][k][l]  = 0;
      	  PIXMin[i][j][k][l]  = 0;
      	}
      }
    }
  }
  
  int n_entries = ch.GetEntries();
  std::cout << "\nLooping over " << n_entries << " entries..." <<std::endl;

  /*   Fill Pixel and FEB level graphs with rates
   */
  int pastrun=0;
  for (int i = 0; i < n_entries; ++i)
  {
    ch.GetEntry(i);
    if(Nt->run!= pastrun){
      std::cout << "Processing run " << Nt->run << std::endl;
      pastrun=Nt->run;
    }
    // Calculate temp run number for plotting
    int RUN=-1;
    for(unsigned int s = 0; s < runList.size(); s++) {
      if(Nt->run == runList[s]){
	RUN= Nt->subrun + startBin[s];
	s=runList.size()+1;
      }
    }

    if(PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
      char histoname[30];
      sprintf(histoname, "RatePIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
      PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Hit Rate in Pixels;run;rate (Hz)", 
      							      Nbins, 0, Nbins); 
    }
    PIXRates[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->rate);
    if(PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix] == 0){
      char histoname[30];
      sprintf(histoname, "MinADCPIX_%.2u_%.2u_%.2u_%.2u", Nt->db, Nt->dcm, Nt->feb, Nt->pix);
      PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]  = new TH1D(histoname, "Min ADC in Pixels;run;ADC", 
      							      Nbins, 0, Nbins); 
    }
    PIXMin[Nt->db][Nt->dcm][Nt->feb][Nt->pix]->Fill(RUN, Nt->minADC);
    
    if(AllTheFEBs[Nt->db][Nt->dcm] == 0){
      char ratehistoname[50];
      sprintf(ratehistoname, "AllTheFEBs_%.2u_%.2u", Nt->db, Nt->dcm);
      char ratehistotitle[100];
      sprintf(ratehistotitle, "All rates on FEBs for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      AllTheFEBs[Nt->db][Nt->dcm]     = new TH2D(ratehistoname, ratehistotitle, Nbins, 0, Nbins, 31, 0, 31);
    }
    AllTheFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    if(Nt->rate > 500){
      if(HighRatesFEBs[Nt->db][Nt->dcm] == 0){
	char hratehistoname[50];
	sprintf(hratehistoname, "HighRateFEB_%.2u_%.2u", Nt->db, Nt->dcm);
	char ratehistotitle[100];
	sprintf(ratehistotitle, "Fraction of noise incidents per FEB (DB%.2u DCM%.2u);run;FEB", Nt->db, Nt->dcm);
	HighRatesFEBs[Nt->db][Nt->dcm]  = new TH2D(hratehistoname, ratehistotitle, Nbins, 0, Nbins, 31, 0, 31);
      }
      HighRatesFEBs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);
    }

    if(EmptyPIXs[Nt->db][Nt->dcm] == 0){
      char emptyhistoname[50];
      sprintf(emptyhistoname, "EmptyPIXs_%.2u_%.2u", Nt->db, Nt->dcm);
      char emptyhistotitle[100];
      sprintf(emptyhistotitle, "Empty PIXs for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      EmptyPIXs[Nt->db][Nt->dcm]     = new TH2D(emptyhistoname, emptyhistotitle, Nbins, 0, Nbins, 31, 0, 31);

      char mdhistoname[50];
      sprintf(mdhistoname, "MissingData_%.2u_%.2u", Nt->db, Nt->dcm);
      char mdhistotitle[100];
      sprintf(mdhistotitle, "Missing Data for DB%.2u DCM%.2u;run;FEB", Nt->db, Nt->dcm);
      MissingData[Nt->db][Nt->dcm]     = new TH2D(mdhistoname, mdhistotitle, Nbins, 0, Nbins, 31, 0, 31);
    }
    if(Nt->rate == 0.0)
      EmptyPIXs[Nt->db][Nt->dcm]->Fill(RUN,Nt->feb);

  }

  /*    Compute Ratios
  */
  for(unsigned int i = 1; i < DB_size+1; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( AllTheFEBs[i][j] !=0 && HighRatesFEBs[i][j]!=0) HighRatesFEBs[i][j]->Divide(AllTheFEBs[i][j]);
    }
  }



  //
  // Make missing data plot
  //
  for(unsigned int i = 1; i < DB_size+1; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( EmptyPIXs[i][j] !=0 ) {

	int NXbins = EmptyPIXs[i][j]->GetNbinsX();
	int NYbins = EmptyPIXs[i][j]->GetNbinsY();
	
	for(int a = 1; a <= NXbins; ++a) {
	  for(int b = 1; b <= NYbins; ++b) {

	    double bin1 = AllTheFEBs[i][j]->GetBinContent(AllTheFEBs[i][j]->GetBin(a,b));
	    double bin2 = EmptyPIXs[i][j]->GetBinContent(EmptyPIXs[i][j]->GetBin(a,b));

	    if( bin1 == 0.0 || bin2 == 32.0 )
	      MissingData[i][j]->SetBinContent(a,b,1.0);

	  }
	}

      }
    }
  }


  /*    Display Pots
  */
  TCanvas* cAllDCMsDB1 = new TCanvas("AllDCMsDB1","AllDCMsDB1",1000,500);
  cAllDCMsDB1->Divide(2);
  cAllDCMsDB1->cd(1);
  AllTheFEBs[1][1]->Draw("colz");
  AllTheFEBs[1][1]->SetStats(0);
  cAllDCMsDB1->cd(2);
  HighRatesFEBs[1][1]->Draw("colz");
  HighRatesFEBs[1][1]->SetStats(0);

  std::cout << "\nWriting NewRuns_AllDCMsDB1.png to disk.\n";
  cAllDCMsDB1->Print("NewRuns_AllDCMsDB1.png");



  TCanvas* cHighRatesTestStandFEBsRatio = new TCanvas("HighRatesTestStandFEBs","cHighRatesTestStandFEBs",880,800); 
  MissingData[1][1]->SetTitle("Fraction of noise incidents per FEB (Test Stand)");
  MissingData[1][1]->GetYaxis()->SetNdivisions(31);
  MissingData[1][1]->GetYaxis()->SetLabelSize(0.015);
  MissingData[1][1]->GetXaxis()->SetTickLength(0);
  MissingData[1][1]->GetXaxis()->SetLabelColor(0);
  MissingData[1][1]->GetXaxis()->SetLabelSize(0.02);
  MissingData[1][1]->GetXaxis()->SetTitleSize(0.025);
  MissingData[1][1]->GetYaxis()->SetTitleSize(0.025);
  MissingData[1][1]->GetXaxis()->SetNdivisions(1,122, kFALSE);
  MissingData[1][1]->SetMaximum(0.501);
  MissingData[1][1]->SetFillColor(kGray);
  MissingData[1][1]->Draw("box");
  MissingData[1][1]->SetStats(0);

  HighRatesFEBs[1][1]->SetTitle("Fraction of noise incidents per FEB (Test Stand)");
  HighRatesFEBs[1][1]->GetYaxis()->SetNdivisions(31);
  HighRatesFEBs[1][1]->GetYaxis()->SetLabelSize(0.015);
  HighRatesFEBs[1][1]->GetXaxis()->SetTickLength(0);
  HighRatesFEBs[1][1]->GetXaxis()->SetLabelColor(0);
  HighRatesFEBs[1][1]->GetXaxis()->SetLabelSize(0.02);
  HighRatesFEBs[1][1]->GetXaxis()->SetTitleSize(0.025);
  HighRatesFEBs[1][1]->GetYaxis()->SetTitleSize(0.025);
  HighRatesFEBs[1][1]->GetXaxis()->SetNdivisions(1,122, kFALSE);
  HighRatesFEBs[1][1]->SetMaximum(0.501);
  HighRatesFEBs[1][1]->Draw("samecolz");
  HighRatesFEBs[1][1]->SetStats(0);
  cHighRatesTestStandFEBsRatio->SetGridy();

  for(unsigned int i = 0; i < runList.size(); i++) {
    double y2 = 31;
    TLine *line = new TLine(startBin[i], 0, startBin[i], y2);
    line->SetLineStyle(2);
    if(hwList[i] == 1){
      line->SetLineWidth(2);
      line->SetLineStyle(9);
    }
    line->Draw();
    char buff[16];
    sprintf(buff,"%.3d",runList[i]);
    TText *t = new TText(startBin[i],-0.7,buff);
    t->SetTextAlign(22);
    t->SetTextFont(42);
    t->SetTextSize(0.016);
    t->SetTextAngle(90);
    t->Draw();
    TArrow *a = new TArrow(startBin[i],31.5,startBin[i]+1,31.5, 0.03, "|>");
    a->SetLineWidth(3);
    a->SetLineColor(kAzure-3);
    a->SetFillColor(kAzure-3);
    if(hcList[i] == 1){
      a->SetLineColor(kRed);
      a->SetFillColor(kRed);
    }
    a->SetArrowSize(0.01);
    a->Draw();
  }

  std::cout << "\nWriting NewRuns_TestStandFEBRates.png to disk.\n";
  cHighRatesTestStandFEBsRatio->Print("NewRuns_TestStandFEBRates.png");



  DCM_size = 2;
  //.............. Show plots with both parts
  for(unsigned int i = 1; i < DCM_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {

      TCanvas* CanvasP[FEB_size];
      bool  goodCanvas[FEB_size];

      for(unsigned int k = 0; k < FEB_size; ++k) {
      	// bool hascanvas=0, haspad=0;
	bool haspad=0;

	char name[40];
	sprintf(name, "NewRuns_2PIXratesTestStandFEB_%.2u",k);
	CanvasP[k] = new TCanvas(name,name,1167,600);
	CanvasP[k]->SetLogy();
	CanvasP[k]->cd(1);
	CanvasP[k]->SetBottomMargin(0.40);
	
	goodCanvas[k] = false;
	
  	for(unsigned int l = 0; l < PIX_size; ++l) {

  	  if( PIXRates[i][j][k][l] !=0) {
	    goodCanvas[k] = true;

  	    PIXRates[i][j][k][l]->SetLineColor(colors[l]);
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXRates[i][j][k][l]->SetMarkerStyle(34);
  	    PIXRates[i][j][k][l]->SetAxisRange(0.01,1000000,"Y");
  	    PIXRates[i][j][k][l]->SetMarkerColor(colors[l]);
	    PIXRates[i][j][k][l]->Draw("same, L");
	    PIXRates[i][j][k][l]->GetXaxis()->SetNdivisions(1,Nbins-1, kFALSE);
	    PIXRates[i][j][k][l]->GetXaxis()->SetTickLength(0);
	    PIXRates[i][j][k][l]->GetXaxis()->SetLabelColor(0);
	    PIXRates[i][j][k][l]->SetStats(0);
	    PIXRates[i][j][k][l]->SetTitle("");
	    TPaveText *pt = new TPaveText(0.12, 0.82, 0.25, 0.89, "NDC");
	    pt->AddText("Hit Rate in Pixels");
	    char address[30];
	    sprintf(address, "Test Stand FEB %.2u", k);
	    //title must include dcm & db if more than 1 !
	    pt->AddText(address);
	    pt->SetLineColor(kWhite);
	    pt->SetShadowColor(kWhite);
	    pt->SetFillColor(kWhite);
	    pt->Draw();
	    pt->SetTextFont(42);

	    TLine *rate500 = new TLine(0,500,startBin[runList.size()-1]+srList[runList.size()-1],500);
	    rate500->SetLineWidth(2);
	    rate500->SetLineStyle(2);
	    rate500->Draw();
	    
	    for(unsigned int s = 0; s < runList.size(); s++) {
	      double y2 = 1000000;
	      if((double)startBin[s] < (double)(Nbins/6) ) y2=50000;
	      TLine *line = new TLine(startBin[s], 0, startBin[s], y2);
	      line->SetLineStyle(2);
	      if(hwList[s] == 1){
		line->SetLineWidth(2);
		line->SetLineStyle(9);
	      }
	      line->Draw();
	      TArrow *a = new TArrow(startBin[s],2000000,startBin[s]+1,2000000, 0.03, "|>");
	      a->SetLineWidth(3);
	      a->SetLineColor(kAzure-3);
	      a->SetFillColor(kAzure-3);
	      if(hcList[s] == 1){
		a->SetLineColor(kRed);
		a->SetFillColor(kRed);
	      }
	      a->SetArrowSize(0.01);
	      a->Draw();
	    } // end for s

  	  } // end if(pixrate != 0)
  	}//for pix

	for(unsigned int l = 0; l < PIX_size; ++l) {
	  if( PIXMin[i][j][k][l] !=0) {
	    if(haspad==0 && goodCanvas[k]){
	      TPad* pad = new TPad("pad", "pad", 0., 0., 1., 1.);
	      pad->SetTopMargin(0.63);
	      pad->SetFillColor(0);
	      pad->SetFillStyle(0);
	      pad->Draw();
	      pad->cd(0);
	      haspad=1;
	    }
  	    PIXMin[i][j][k][l]->SetLineColor(colors[l]);
	    PIXMin[i][j][k][l]->SetTitle("");
  	    PIXMin[i][j][k][l]->SetLineStyle(2);
   	    PIXMin[i][j][k][l]->SetMarkerStyle(34);
  	    PIXMin[i][j][k][l]->SetMarkerColor(colors[l]);
  	    PIXMin[i][j][k][l]->SetAxisRange(0,200,"Y");
  	    PIXMin[i][j][k][l]->Draw("same, L");
  	    PIXMin[i][j][k][l]->SetStats(0);
  	    PIXMin[i][j][k][l]->GetXaxis()->SetNdivisions(1,122, kFALSE);
	    PIXMin[i][j][k][l]->GetXaxis()->SetTickLength(0);
	    PIXMin[i][j][k][l]->GetXaxis()->SetLabelColor(0);
  	    PIXMin[i][j][k][l]->GetYaxis()->SetNdivisions(504, kTRUE);
  	    TPaveText *pt1 = new TPaveText(0.12, 0.3, 0.25, 0.36, "NDC");
  	    pt1->AddText("Min ADC in Pixels");
  	    char address[30];
  	    sprintf(address, "Test Stand FEB%.2u", k);
	    // title must include dcm & db if more than 1 !
  	    pt1->AddText(address);
  	    pt1->SetLineColor(kWhite);
  	    pt1->SetShadowColor(kWhite);
  	    pt1->SetFillColor(kWhite);
  	    pt1->Draw();
  	    pt1->SetTextFont(42);
	    for(unsigned int s = 0; s < runList.size(); s++) {
	      double y2 = 200;
	      if((double)startBin[s] < (double)(Nbins/6) ) y2=150;
	      TLine *line = new TLine(startBin[s], 0, startBin[s], y2);
	      line->SetLineStyle(2);
	      if(hwList[s] == 1){
		line->SetLineWidth(2);
		line->SetLineStyle(9);
	      }
	      line->Draw();
	      char buff[16];
	      sprintf(buff,"%.3d",runList[s]);
	      TText *t = new TText(startBin[s],-20,buff);
	      t->SetTextAlign(22);
	      t->SetTextFont(42);
	      t->SetTextSize(0.016);
	      t->SetTextAngle(90);
	      t->Draw();
	    }
	  }
	}//for pix

	char name2[40];
	sprintf(name2, "NewRuns_2PIXratesTestStandFEB_%.2u.png",k);
	if(goodCanvas[k]) {
	  std::cout << "Printing canvas: " << name2 << std::endl;
	  CanvasP[k]->Print(name2);
	}

      }//for feb (loop over k)
    }//for dcm (loop over j)
  }//for db (loop over i)


  /*
  //  Open output file for this subrun
  char outfilename[100];
  sprintf(outfilename, "/nova/app/users/psihas/MyTestRel/results/TestStandAna/Onmon_ana-HistosPIXMin-run%.2u-%.2u.root",  runList[0],  runList[runList.size()-1]);
  TFile* out_file = new TFile(outfilename, "RECREATE");
  std::cout << "Writting " << outfilename << std::endl;

  for(unsigned int i = 1; i < DB_size; ++i) {
    for(unsigned int j = 1; j < DCM_size; ++j) {
      if( AllTheFEBs[i][j] !=0 && HighRatesFEBs[i][j] !=0) {
	HighRatesFEBs[i][j]->Write();
      }
      for(unsigned int k = 0; k < FEB_size; ++k) {
   	for(int l = 0; l < PIX_size; ++l) {
   	  if( PIXRates[i][j][k][l] !=0) {
  	    PIXRates[i][j][k][l]->Write();
  	    PIXMin[i][j][k][l]->Write();
   	  }
   	}
      }
    }
  }

  out_file->Close();
  */


}
