extern "C" {
#include <unistd.h>
}
#include <iostream>
#include <string>
#include "TFile.h"
#include "OnlineMonitoring/ana/FEBRateAnalysis.h"

static void usage() 
{
  //
  // Print help.
  //
  std::cout << "\nUsage: onmon_ana"
	    << "\n-b       - b does not take an argument"
	    << "\n-d       - d detector type"
	    << "\n[file1.root file2.root file3.root ...]"
	    << std::endl;
  exit(0);
}

//......................................................................
//
// Parse the run and subrun numbers out of a file in format:
//
// stuff_r1234_s56.root
//
// Where run is 1234 and 56 is the subrun
//
void get_run_numbers(const char* f, int* run, int* subrun) 
{
  std::string n(f);
  
  // size_t n3 = n.find_last_of('.');
  // size_t n2 = n.find_last_of('-',n3-1);
  // size_t n1 = n.find_last_of('-',n2-1);
  
  size_t nx = n.find_first_of('.');
  size_t ns = n.find_last_of('s',nx-1);
  size_t nr = n.find_last_of('r',ns-1);
  
  std::string r = n.substr(nr+1,8);
  std::string s = n.substr(ns+1,3);
  
  std::cout << "\n\n" << r << " " << s << "\n\n";

  *run    = atoi(r.c_str());
  *subrun = atoi(s.c_str());
}

//......................................................................

int main(int argc, char** argv) 
{
  //
  // Parse out command line options
  //
  std::string d = "none";
  bool        b =  false;
  int         c;

  while ((c = getopt(argc, argv, "d:b"))!=-1) {
    switch (c) {
    case 'd':
      d = optarg;
      break;
    case 'b':
      b = true;
      break;
    default:
      usage();
      break;
    }
  }
  std::cout << "d = " << d << std::endl;
  std::cout << "b = " << b << std::endl;

  //
  // Build an object which does analysis on a onmon histogram
  // file. This one analyzes FEB rates.
  //
  om::FEBRateAnalysis febana(d);
  
  for (int i=optind; i<argc; ++i) {
    std::cout << "Processing file " << argv[i] << std::endl;

    int  run    = -1;
    int  subrun = -1;
    get_run_numbers(argv[i], &run, &subrun);

    TFile f(argv[i]);
    febana.AnaFile(f, run, subrun);
  }

  return 0;
}

////////////////////////////////////////////////////////////////////////
