#include "OnlineMonitoring/producer/HitCounts.h"
#include <iostream>
#include "TH1F.h"
#include "TH2F.h"
#include "OnlineMonitoring/producer/HistoSet.h"
#include "OnlineMonitoring/producer/RawEventSummary.h"
using namespace om;

//......................................................................

HitCounts::HitCounts() : 
  SummarySubscriber(SummarySubscriber::kEvent)
{
  HistoSet& h = HistoSet::Instance();
  
  fNhitVsHour        = h.GetTH2F("NhitVsHour");
  fNfebsVsHour       = h.GetTH2F("NfebsVsHour");
  fNdcmsVsHour       = h.GetTH2F("NdcmsVsHour");
  fAveNmicroSlVsHour = h.GetTH2F("AveNmicroSlVsHour");
  fTotNmicroSlVsHour = h.GetTH2F("TotNmicroSlVsHour");
}

//......................................................................

void HitCounts::GetRawEventSummary(const RawEventSummary& r)
{

  fNdcmsVsHour->Fill(r.fHour, r.fDCMs.size());
  fNfebsVsHour->Fill(r.fHour, r.fFEBs.size());
  fNhitVsHour-> Fill(r.fHour, r.fNhit);
  if(r.fDCMs.size() != 0) {
    unsigned int num = r.fNmicroSl;
    unsigned int den = r.fDCMs.size();
    double ratio = (double)num/(double)den;
    fAveNmicroSlVsHour->Fill(r.fHour, ratio);
  }
  else
    fAveNmicroSlVsHour->Fill(r.fHour, 0);
  fTotNmicroSlVsHour->Fill(r.fHour, r.fNmicroSl);

  // This filling is done so that when the histogram is displayed, if the
  // current time is between 0 and 1, one can still see the information from
  // between 23 and 24 on the "previous day."
  if(r.fHour >= 23.0) {
    fNdcmsVsHour->Fill(r.fHour - 24.0, r.fDCMs.size());
    fNfebsVsHour->Fill(r.fHour - 24.0, r.fFEBs.size());
    fNhitVsHour-> Fill(r.fHour - 24.0, r.fNhit);
    if(r.fDCMs.size() != 0) {
      unsigned int num = r.fNmicroSl;
      unsigned int den = r.fDCMs.size();
      double ratio = (double)num/(double)den;
      fAveNmicroSlVsHour->Fill(r.fHour - 24.0, ratio);
    }
    else
      fAveNmicroSlVsHour->Fill(r.fHour - 24.0, 0);
    fTotNmicroSlVsHour->Fill(r.fHour - 24.0, r.fNmicroSl);
  }
}

////////////////////////////////////////////////////////////////////////
