#include <iostream>
#include <cmath>
#include "TH1F.h"
#include "TH2F.h"
#include "DAQChannelMap/HardwareDisplay.h"
#include "OnlineMonitoring/producer/HitMaps.h"
#include "OnlineMonitoring/producer/HistoSet.h"
#include "OnlineMonitoring/producer/RawEventSummary.h"
#include "OnlineMonitoring/producer/MicroSliceSummary.h"
#include "OnlineMonitoring/producer/NanoSliceSummary.h"
#include "OnlineMonitoring/util/Settings.h"
#include "OnlineMonitoring/util/HistoData.h"


using namespace om;
class RawEventSummary;
class DataBlockSummary;
class MicroSliceSummary;

static const unsigned int kADC_ERROR = 0xFFFFFFFF;

HitMaps::HitMaps() : 
  SummarySubscriber(SummarySubscriber::kNanoSlice),
  fHwMap(new daqchannelmap::HardwareDisplay)
{
  fHwMap->SetupDet(Settings::Instance().fDet);

  for(unsigned int i = 0; i < PixelsDCM_size; ++i) {
    for(unsigned int j = 0; j < PixelsDCM_size; ++j) {
      fPixelsRateDCM[i][j]    = 0;
      fLowADCPixelsRateDCM[i][j]  = 0;
      fMipADCPixelsRateDCM[i][j]  = 0;
      fHighADCPixelsRateDCM[i][j] = 0;
      fPixelsDCM[i][j]        = 0;
      fLowADCPixelsDCM[i][j]  = 0;
      fMipADCPixelsDCM[i][j]  = 0;
      fHighADCPixelsDCM[i][j] = 0;
      fMinADCPixelsDCM[i][j]  = 0;
      fMaxADCPixelsDCM[i][j]  = 0;
      fTotADCPixelsDCM[i][j]  = 0;
    } // end for j
  } // end for i

  HistoSet& h = HistoSet::Instance();
  
  fHitMapEvtCount       = h.GetTH1F("HitMapEvtCount");
  fRecordedTime         = h.GetTH1F("RecordedTime");
  fDCMHitRateMap        = h.GetTH2F("DCMHitRateMap");
  fDCMHitRateSpectrum   = h.GetTH1F("DCMHitRateSpectrum");
  fDCMHitRateMapLowADC  = h.GetTH2F("DCMHitRateMapLowADC");
  fDCMHitRateMapMipADC  = h.GetTH2F("DCMHitRateMapMipADC");
  fDCMHitRateMapHighADC = h.GetTH2F("DCMHitRateMapHighADC");
  fFEBHitRateMap        = h.GetTH2F("FEBHitRateMap");
  fFEBHitRateSpectrum   = h.GetTH1F("FEBHitRateSpectrum");
  fFEBHitRateMapLowADC  = h.GetTH2F("FEBHitRateMapLowADC");
  fFEBHitRateMapMipADC  = h.GetTH2F("FEBHitRateMapMipADC");
  fFEBHitRateMapHighADC = h.GetTH2F("FEBHitRateMapHighADC");

  fPixelHitMap          = h.GetTH2F("PixelHitMap");
  fPixelHitRateMap      = h.GetTH2F("PixelHitRateMap");
  fPixelHitRateSpectrum = h.GetTH1F("PixelHitRateSpectrum");

  fDCMHitMap        = h.GetTH2F("DCMHitMap");
  fDCMHitMapLowADC  = h.GetTH2F("DCMHitMapLowADC");
  fDCMHitMapMipADC  = h.GetTH2F("DCMHitMapMipADC");
  fDCMHitMapHighADC = h.GetTH2F("DCMHitMapHighADC");
  fFEBHitMap        = h.GetTH2F("FEBHitMap");
  fFEBHitMapLowADC  = h.GetTH2F("FEBHitMapLowADC");
  fFEBHitMapMipADC  = h.GetTH2F("FEBHitMapMipADC");
  fFEBHitMapHighADC = h.GetTH2F("FEBHitMapHighADC");
  fPlaneCellHitMapXviewAllHits     = h.GetTH2F("PlaneCellHitMapXviewAllHits");
  fPlaneCellHitMapYviewAllHits     = h.GetTH2F("PlaneCellHitMapYviewAllHits");
  fPlaneCellHitRateMapXviewAllHits = h.GetTH2F("PlaneCellHitRateMapXviewAllHits");
  fPlaneCellHitRateMapYviewAllHits = h.GetTH2F("PlaneCellHitRateMapYviewAllHits");

  if(Settings::Instance().fDet == kNDOS) {
    fPlaneCellHitMapXviewCalib = h.GetTH2F("PlaneCellHitMapXviewCalib");
    fPlaneCellHitMapYviewCalib = h.GetTH2F("PlaneCellHitMapYviewCalib");
    fPlaneCellHitMapXviewNuMI  = h.GetTH2F("PlaneCellHitMapXviewNuMI");
    fPlaneCellHitMapYviewNuMI  = h.GetTH2F("PlaneCellHitMapYviewNuMI");
    fPlaneCellHitMapXviewBNB   = h.GetTH2F("PlaneCellHitMapXviewBNB");
    fPlaneCellHitMapYviewBNB   = h.GetTH2F("PlaneCellHitMapYviewBNB");
  }
  if(Settings::Instance().fDet == kNEARDET) {
    fPlaneCellHitMapXviewCalib = h.GetTH2F("PlaneCellHitMapXviewCalib");
    fPlaneCellHitMapYviewCalib = h.GetTH2F("PlaneCellHitMapYviewCalib");
    fPlaneCellHitMapXviewNuMI  = h.GetTH2F("PlaneCellHitMapXviewNuMI");
    fPlaneCellHitMapYviewNuMI  = h.GetTH2F("PlaneCellHitMapYviewNuMI");
  }

  fTotADCFEBHitMap = h.GetTH2F("TotADCFEBHitMap");
  
}

//......................................................................

HitMaps::~HitMaps() {
  if(fHwMap) {delete fHwMap; fHwMap = 0;}
}

//......................................................................

void HitMaps::GetNanoSliceSummary(const RawEventSummary&   r,
				  const DataBlockSummary&  dbs,
				  const MicroSliceSummary& mss,
				  const NanoSliceSummary&  nss)
{
  HistoSet& h = HistoSet::Instance();

  //
  // fill hit map event counter //
  static unsigned int lastRun=0, lastEvent=0;
  static const double secPerRange = 5E-7;     // fRange is in units of 500nsec

  if(lastRun!=r.fRun || lastEvent!=r.fEvent) {
    double nEvents     = fHitMapEvtCount->Integral();
    double recordedSec = (fRecordedTime->Integral())*secPerRange;

    if((int)nEvents%100 == 0 && recordedSec != 0.0){

      // Compute rate plots.
      this->CopyHistContent(fDCMHitMap,fDCMHitRateMap);
      this->CopyHistContent(fDCMHitMapLowADC,fDCMHitRateMapLowADC);
      this->CopyHistContent(fDCMHitMapMipADC,fDCMHitRateMapMipADC);
      this->CopyHistContent(fDCMHitMapHighADC,fDCMHitRateMapHighADC);
      this->CopyHistContent(fFEBHitMap,fFEBHitRateMap);
      this->CopyHistContent(fPixelHitMap,fPixelHitRateMap);
      this->CopyHistContent(fFEBHitMapLowADC,fFEBHitRateMapLowADC);
      this->CopyHistContent(fFEBHitMapMipADC,fFEBHitRateMapMipADC);
      this->CopyHistContent(fFEBHitMapHighADC,fFEBHitRateMapHighADC);
      this->CopyHistContent(fPlaneCellHitMapXviewAllHits,fPlaneCellHitRateMapXviewAllHits);
      this->CopyHistContent(fPlaneCellHitMapYviewAllHits,fPlaneCellHitRateMapYviewAllHits);
      fDCMHitRateMap        ->Scale(1.0/recordedSec);
      fDCMHitRateMapLowADC  ->Scale(1.0/recordedSec);
      fDCMHitRateMapMipADC  ->Scale(1.0/recordedSec);
      fDCMHitRateMapHighADC ->Scale(1.0/recordedSec);
      fFEBHitRateMap        ->Scale(1.0/recordedSec);
      fPixelHitRateMap      ->Scale(1.0/recordedSec);
      fFEBHitRateMapLowADC  ->Scale(1.0/recordedSec);
      fFEBHitRateMapMipADC  ->Scale(1.0/recordedSec);
      fFEBHitRateMapHighADC ->Scale(1.0/recordedSec);
      fPlaneCellHitRateMapXviewAllHits->Scale(1.0/recordedSec);
      fPlaneCellHitRateMapYviewAllHits->Scale(1.0/recordedSec);
      
      // Compute rate spectra.
      fPixelHitRateSpectrum->Reset();
      fFEBHitRateSpectrum  ->Reset();
      fDCMHitRateSpectrum  ->Reset();
      for(int i = 1; i <= fPixelHitRateMap->GetNbinsX(); ++i) {
	for(int j = 1; j <= fPixelHitRateMap->GetNbinsY(); ++j) {
	  double rate = fPixelHitRateMap->GetBinContent(fPixelHitRateMap->GetBin(i,j));
	  if(rate > 0.0)
	    rate = log10(rate);
	  else
	    rate = -100.0;
	  fPixelHitRateSpectrum->Fill(rate);
	}
      }
      for(int i = 1; i <= fFEBHitRateMap->GetNbinsX(); ++i) {
	for(int j = 1; j <= fFEBHitRateMap->GetNbinsY(); ++j) {
	  double rate = fFEBHitRateMap->GetBinContent(fFEBHitRateMap->GetBin(i,j));
	  if(rate > 0.0)
	    rate = log10(rate);
	  else
	    rate = -100.0;
	  fFEBHitRateSpectrum->Fill(rate);
	}
      }
      for(int i = 1; i <= fDCMHitRateMap->GetNbinsX(); ++i) {
	for(int j = 1; j <= fDCMHitRateMap->GetNbinsY(); ++j) {
	  double rate = fDCMHitRateMap->GetBinContent(fDCMHitRateMap->GetBin(i,j));
	  if(rate > 0.0)
	    rate = log10(rate);
	  else
	    rate = -100.0;
	  fDCMHitRateSpectrum->Fill(rate);
	}
      }
      
      for(unsigned int i = 0; i < PixelsDCM_size; ++i) {
      	for(unsigned int j = 0; j < PixelsDCM_size; ++j) {
	  if(fPixelsDCM[i][j]!=0){
	    if(fPixelsRateDCM[i][j]==0){
	      char histoname[30];
	      sprintf(histoname, "PixelsRateDCM_%.2u_%.2u", i, j);
	      fPixelsRateDCM[i][j] = h.GetTH2F(histoname);  
	    }
	    this->CopyHistContent(fPixelsDCM[i][j],fPixelsRateDCM[i][j]);
	    fPixelsRateDCM[i][j]->Scale(1/recordedSec);
	  }
	  if(fLowADCPixelsDCM[i][j]!=0){
	    if(fLowADCPixelsRateDCM[i][j]==0){
	      char histoname[30];
	      sprintf(histoname, "LowADCPixelsRateDCM_%.2u_%.2u", i, j);
	      fLowADCPixelsRateDCM[i][j] = h.GetTH2F(histoname);  
	    }
	    this->CopyHistContent(fLowADCPixelsDCM[i][j],fLowADCPixelsRateDCM[i][j]);
	    fLowADCPixelsRateDCM[i][j]->Scale(1/recordedSec);
	  }
	  if(fMipADCPixelsDCM[i][j]!=0){
	    if(fMipADCPixelsRateDCM[i][j]==0){
	      char histoname[30];
	      sprintf(histoname, "MipADCPixelsRateDCM_%.2u_%.2u", i, j);
	      fMipADCPixelsRateDCM[i][j] = h.GetTH2F(histoname);  
	    }
	    this->CopyHistContent(fMipADCPixelsDCM[i][j],fMipADCPixelsRateDCM[i][j]);
	    fMipADCPixelsRateDCM[i][j]->Scale(1/recordedSec);
	  }
	  if(fHighADCPixelsDCM[i][j]!=0){
	    if(fHighADCPixelsRateDCM[i][j]==0){
	      char histoname[30];
	      sprintf(histoname, "HighADCPixelsRateDCM_%.2u_%.2u", i, j);
	      fHighADCPixelsRateDCM[i][j] = h.GetTH2F(histoname);  
	    }
	    this->CopyHistContent(fHighADCPixelsDCM[i][j],fHighADCPixelsRateDCM[i][j]);
	    fHighADCPixelsRateDCM[i][j]->Scale(1/recordedSec);
	  }
      	} // end for j
      } // end for i

    }

    fHitMapEvtCount->Fill(0);
    double range = (double)r.fRange;

    // Adjust the range due to the fact that we always get any micro blocks
    // that touch the edge of the trigger range.
    range = 100.0*floor(range/100.0)+100.0;

    fRecordedTime->Fill(0.0,range);

  }

  lastRun   = r.fRun;
  lastEvent = r.fEvent;

  //
  // fill hit maps for entire detector
  //
  fDCMHitMap  ->Fill(nss.fHdwX, nss.fHdwY);
  fFEBHitMap  ->Fill(nss.fHdwX, nss.fHdwY);
  fPixelHitMap->Fill(nss.fHdwX, nss.fHdwY);

  if(nss.fView == daqchannelmap::X_VIEW) {
    //
    // The plane/cell maps are made to match the hardware views.
    // So X and Z are reversed but Y is not.
    //
    double P = (-1.0)*(nss.fPlane);
    double C = (-1.0)*(nss.fCell);
    fPlaneCellHitMapXviewAllHits->Fill(P,C);
    if(Settings::Instance().fDet == kNDOS) {
      if(r.fTrigger == 0) fPlaneCellHitMapXviewNuMI ->Fill(P,C);
      if(r.fTrigger == 1) fPlaneCellHitMapXviewBNB  ->Fill(P,C);
      if(r.fTrigger == 2) fPlaneCellHitMapXviewCalib->Fill(P,C);
    }
    if(Settings::Instance().fDet == kNEARDET) {
      if(r.fTrigger == 0) fPlaneCellHitMapXviewNuMI ->Fill(P,C);
      if(r.fTrigger == 2) fPlaneCellHitMapXviewCalib->Fill(P,C);
    }
  }

  if(nss.fView == daqchannelmap::Y_VIEW) {
    //
    // The plane/cell maps are made to match the hardware views.
    // So X and Z are reversed but Y is not.
    //
    double P = (-1.0)*(nss.fPlane);
    double C = ( 1.0)*(nss.fCell);
    fPlaneCellHitMapYviewAllHits->Fill(P,C);
    if(Settings::Instance().fDet == kNDOS) {
      if(r.fTrigger == 0) fPlaneCellHitMapYviewNuMI ->Fill(P,C);
      if(r.fTrigger == 1) fPlaneCellHitMapYviewBNB  ->Fill(P,C);
      if(r.fTrigger == 2) fPlaneCellHitMapYviewCalib->Fill(P,C);
    }
    if(Settings::Instance().fDet == kNEARDET) {
      if(r.fTrigger == 0) fPlaneCellHitMapYviewNuMI ->Fill(P,C);
      if(r.fTrigger == 2) fPlaneCellHitMapYviewCalib->Fill(P,C);
    }
  }



  unsigned int lowADC  = 175;
  unsigned int highADC = 3200;
  
  if(nss.fADC < lowADC) {
    fDCMHitMapLowADC->Fill(nss.fHdwX, nss.fHdwY);
    fFEBHitMapLowADC->Fill(nss.fHdwX, nss.fHdwY);
  }
  if(nss.fADC > highADC) {
    fDCMHitMapHighADC->Fill(nss.fHdwX, nss.fHdwY);
    fFEBHitMapHighADC->Fill(nss.fHdwX, nss.fHdwY);
  }
  if(nss.fADC >= lowADC && nss.fADC <= highADC) {
    fDCMHitMapMipADC->Fill(nss.fHdwX, nss.fHdwY);
    fFEBHitMapMipADC->Fill(nss.fHdwX, nss.fHdwY);
  }



  //
  // fill hit maps for individual DCMs
  //

  if(fPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
    char histoname[15];
    sprintf(histoname, "PixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
    fPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
  }

  unsigned int x1, x2, y1, y2;
  fHwMap->FEBXY(nss.fFEB, &x1, &y1);
  fHwMap->PixXY(nss.fPix, &x2, &y2);
  fPixelsDCM[mss.fDiblock][mss.fDCM]->Fill(x1+x2, y1+y2);

  // this could later be put with the above ifs to consolidate
  if(nss.fADC < lowADC) {
    if(fLowADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
      char histoname[15];
      sprintf(histoname, "LowADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
      fLowADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
    }

    unsigned int x3, x4, y3, y4;
    fHwMap->FEBXY(nss.fFEB, &x3, &y3);
    fHwMap->PixXY(nss.fPix, &x4, &y4);
    fLowADCPixelsDCM[mss.fDiblock][mss.fDCM]->Fill(x3+x4, y3+y4);
  }

  if(nss.fADC > highADC) {
    if(fHighADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
      char histoname[15];
      sprintf(histoname, "HighADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
      fHighADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
    }

    unsigned int x3, x4, y3, y4;
    fHwMap->FEBXY(nss.fFEB, &x3, &y3);
    fHwMap->PixXY(nss.fPix, &x4, &y4);
    fHighADCPixelsDCM[mss.fDiblock][mss.fDCM]->Fill(x3+x4, y3+y4);
  }

  if(nss.fADC >= lowADC && nss.fADC <= highADC) {
    if(fMipADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
      char histoname[15];
      sprintf(histoname, "MipADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
      fMipADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
    }

    unsigned int x3, x4, y3, y4;
    fHwMap->FEBXY(nss.fFEB, &x3, &y3);
    fHwMap->PixXY(nss.fPix, &x4, &y4);
    fMipADCPixelsDCM[mss.fDiblock][mss.fDCM]->Fill(x3+x4, y3+y4);
  }



  //
  // Fill the Min ADC plots
  //
  if(fMinADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
    char histoname[15];
    sprintf(histoname, "MinADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
    fMinADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
  }
  
  unsigned int bin     = fMinADCPixelsDCM[mss.fDiblock][mss.fDCM]->FindBin(x1+x2,y1+y2);
  unsigned int content = fMinADCPixelsDCM[mss.fDiblock][mss.fDCM]->GetBinContent(bin);

  if(content == 0 || nss.fADC < content)
    fMinADCPixelsDCM[mss.fDiblock][mss.fDCM]->SetBinContent(bin,nss.fADC);



  //
  // Fill the Max ADC plots
  //
  if(fMaxADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
    char histoname[15];
    sprintf(histoname, "MaxADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
    fMaxADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
  }
  
  bin     = fMaxADCPixelsDCM[mss.fDiblock][mss.fDCM]->FindBin(x1+x2,y1+y2);
  content = fMaxADCPixelsDCM[mss.fDiblock][mss.fDCM]->GetBinContent(bin);

  if(nss.fADC > content)
    fMaxADCPixelsDCM[mss.fDiblock][mss.fDCM]->SetBinContent(bin,nss.fADC);



  //
  // Fill the Total ADC plots
  //
  if(fTotADCPixelsDCM[mss.fDiblock][mss.fDCM] == 0) {
    char histoname[15];
    sprintf(histoname, "TotADCPixelsDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
    fTotADCPixelsDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
  }

  fTotADCPixelsDCM[mss.fDiblock][mss.fDCM]->Fill(x1+x2,y1+y2,nss.fADC);
  fTotADCFEBHitMap->Fill(nss.fHdwX,nss.fHdwY,nss.fADC);

}
//......................................................................
void HitMaps::CopyHistContent(TH2F*& histFrom, 
			      TH2F*& histTo)
{
  // Copy histogram content only
  // *Assumes same binning for both histograms
  //
  int NbinsX = histFrom->GetNbinsX();
  int NbinsY = histFrom->GetNbinsY();
  for(int i=0;i<NbinsX+2;i++){
    for(int j=0;j<NbinsY+2;j++){
      double content = histFrom->GetBinContent(i, j);
      histTo->SetBinContent(i,j, content);
    }
  }  
}

////////////////////////////////////////////////////////////////////////
