#ifndef OM_NANOERRORS_H
#define OM_NANOERRORS_H
#include "OnlineMonitoring/producer/SummarySubscriber.h"

// This must be one larger than the number of Diblocks/DCMs for the FD
// so that it can be easily accessed as fDCM[Diblock][DCM] instead
// of fDCM[Diblock - 1][DCM - 1].
static const unsigned int DCM_size_nano = 16;

class TH2F;
namespace om            { class RawEventSummary;   }
namespace om            { class DataBlockSummary;  }
namespace om            { class MicroSliceSummary; }
namespace om            { class NanoSliceSummary;  }
namespace daqchannelmap { class HardwareDisplay;   }

namespace om {
  class NanoErrors : public SummarySubscriber {
  public:
    NanoErrors();
    ~NanoErrors();
    virtual void GetNanoSliceSummary(const RawEventSummary&   r,
				     const DataBlockSummary&  dbs,
				     const MicroSliceSummary& mss,
				     const NanoSliceSummary&  nss);
  private:
    daqchannelmap::HardwareDisplay* fHwMap;

    // Key for histogram abbreviations:
    //
    // DPE  = "Data Present Error"
    // LSE  = "Link Status Error"
    // DbM  = "DebugMode"
    // FEBF = "FEB Full"
    // FEBE = "FEB Empty"
    // EDAQ = "Enable DAQ"
    // ADCE = "ADC Error"
    // OFE  = "Overflow Error"
    // PE   = "Packet Error"
    // CE   = "Communications Error"
    // MC   = "Monte Carlo flag"
    // AE   = "All Errors"
    // AA   = "All Alerts"

    // histos displaying the entire detector

    TH2F* fDPEbyPixelDCM;
    TH2F* fLSEbyPixelDCM;
    TH2F* fDbMbyPixelDCM;
    TH2F* fFEBFbyPixelDCM;
    TH2F* fFEBEbyPixelDCM;
    TH2F* fEDAQbyPixelDCM;
    TH2F* fADCEbyPixelDCM;
    TH2F* fOFEbyPixelDCM;
    TH2F* fPEbyPixelDCM;
    TH2F* fCEbyPixelDCM;

    // histos displaying specific DCMs
    TH2F* fDPEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fLSEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fDbMperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fFEBFperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fFEBEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fEDAQperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fADCEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fOFEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fPEperDCM[DCM_size_nano][DCM_size_nano];
    TH2F* fCEperDCM[DCM_size_nano][DCM_size_nano];

    TH2F* fAEVsHour;
    TH2F* fAAVsHour;    

  };
}

#endif
////////////////////////////////////////////////////////////////////////
