///
/// \file    SummarySubscriber.h
/// \brief   Interface for classes that plot data from the event summaries
/// \version $Id: SummarySubscriber.h,v 1.2 2012-02-13 22:00:43 messier Exp $
/// \author  messier@indiana.edu
///
#ifndef OM_SUBSCRIBER_H
#define OM_SUBSCRIBER_H
#include <cstdlib>
namespace om { class RawEventSummary;   }
namespace om { class DataBlockSummary;  }
namespace om { class MicroSliceSummary; }
namespace om { class NanoSliceSummary; }

namespace om {
  class SummarySubscriber 
  {
  public:
    /// Which summaries does a class want to subscribe to?
    enum _which {
      kEvent      = 0x01,
      kDataBlock  = 0x02,
      kMicroSlice = 0x04,
      kNanoSlice  = 0x08
    };
    
  public:
    SummarySubscriber(unsigned int which);
    
  public:
    /// Interface to the event summary
    virtual void GetRawEventSummary(const RawEventSummary& res) { abort(); };
    
    /// Interface to the data block summary
    virtual void GetDataBlockSummary(const RawEventSummary&  res,
				     const DataBlockSummary& dbs) { abort(); }
    
    /// Interface to the microslice summary
    virtual void GetMicroSliceSummary(const RawEventSummary&   res,
				      const DataBlockSummary&  dbs,
				      const MicroSliceSummary& mss) { abort(); }
    
    /// Interface to the nanoslice summary
    virtual void GetNanoSliceSummary(const RawEventSummary&   res,
				     const DataBlockSummary&  dbs,
				     const MicroSliceSummary& mss,
				     const NanoSliceSummary&  nss) { abort(); }
  private:
    /// Default constructor not allowed. Use the one at the top of
    /// this file
    SummarySubscriber() { abort(); }
  };
}
#endif
////////////////////////////////////////////////////////////////////////
