#include "OnlineMonitoring/producer/TQPlots.h"
#include <iostream>
#include "TH1F.h"
#include "TH2F.h"
#include "OnlineMonitoring/producer/HistoSet.h"
#include "OnlineMonitoring/producer/RawEventSummary.h"
#include "OnlineMonitoring/producer/MicroSliceSummary.h"
#include "OnlineMonitoring/producer/NanoSliceSummary.h"

#include "OnlineMonitoring/util/HistoData.h"
#include "OnlineMonitoring/util/Settings.h"

using namespace om;
class RawEventSummary;
class DataBlockSummary;
class MicroSliceSummary;

static const double kUSEC_PER_TDC = (1.0E6/64.0E6);

TQPlots::TQPlots() : 
  SummarySubscriber(SummarySubscriber::kNanoSlice)
{
  for(unsigned int i = 0; i < TQDCM_size; ++i) {
    for(unsigned int j = 0; j < TQDCM_size; ++j) {
      fTQDCM[i][j] = 0;
      fTQDCMnumi[i][j] = 0;
    } // end for j
  } // end for i

  // create the initial histos
  HistoSet& h = HistoSet::Instance();
  fTPlotALL  = h.GetTH1F("TPlotALL");
  fTPlotZOOM = h.GetTH1F("TPlotZOOM");
  fQPlotALL  = h.GetTH1F("QPlotALL");

}

//......................................................................

TQPlots::~TQPlots() { }

//......................................................................

void TQPlots::GetNanoSliceSummary(const RawEventSummary&   r,
				  const DataBlockSummary&  dbs,
				  const MicroSliceSummary& mss,
				  const NanoSliceSummary&  nss)
{
  HistoSet& h = HistoSet::Instance();

  // NOTE: nss.fUsec and nss.fT0lo32Usec are both calcuated with only the
  // lower 32 bits of the time (in NOvA time units.)  This means that every
  // now and then, nss.fT0lo32Usec will "roll over" back to zero before 
  // nss.fUsec does resulting in a difference between the two numbers of 
  // about 67 sec.  Since this is rare and we are interested in
  // the overall performance of a specific DCM, this exception is ignored
  // in favor of not adding extra if statements.

  //
  // fill TQDCM plots for all triggers
  //
  if(fTQDCM[mss.fDiblock][mss.fDCM] == 0) {
    char histoname[15];
    sprintf(histoname, "TQDCM_%.2u_%.2u", mss.fDiblock, mss.fDCM);
    fTQDCM[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);   
  }
  fTQDCM[mss.fDiblock][mss.fDCM]->Fill(nss.fUsec-r.fT0lo32Usec, nss.fADC);

  fTPlotALL->Fill(nss.fUsec-r.fT0lo32Usec);

  // Only fill the Zoom plot if the trigger is NuMI.
  if(r.fTrigger == 0) fTPlotZOOM->Fill(nss.fUsec-r.fT0lo32Usec);

  fQPlotALL->Fill(nss.fADC);

  //
  // if detector is NearDet, fill TQDCM plots for just NuMI triggers
  //
  if(Settings::Instance().fDet == kNEARDET &&
     fTQDCMnumi[mss.fDiblock][mss.fDCM] == 0 &&
     r.fTrigger == 0) {
    char histoname[15];
    sprintf(histoname, "TQDCM_%.2u_%.2u_NuMI", mss.fDiblock, mss.fDCM);
    fTQDCMnumi[mss.fDiblock][mss.fDCM] = h.GetTH2F(histoname);
  }
  if(Settings::Instance().fDet == kNEARDET &&
     r.fTrigger == 0) {
  fTQDCMnumi[mss.fDiblock][mss.fDCM]->Fill(nss.fUsec-r.fT0lo32Usec, nss.fADC);
  }

}

////////////////////////////////////////////////////////////////////////
