#include "OnlineMonitoring/producer/TriggerPlots.h"
#include <iostream>
#include "TH1F.h"
#include "TH2F.h"
#include "OnlineMonitoring/producer/HistoSet.h"
#include "OnlineMonitoring/producer/RawEventSummary.h"
#include "OnlineMonitoring/util/HistoData.h"
#include "OnlineMonitoring/util/Settings.h"
using namespace om;

static const double kSEC_PER_TDC = (1.0/64.0E6);

TriggerPlots::TriggerPlots() : 
  SummarySubscriber(SummarySubscriber::kEvent)
{
  HistoSet& h = HistoSet::Instance();

  if(Settings::Instance().fDet == kNDOS) {
    fDtBNBTrig   = h.GetTH1F("DtBNBTrig");
    fDtNuMITrig  = h.GetTH1F("DtNuMITrig");
    fDtCalibTrig = h.GetTH1F("DtCalibTrig");
    fTriggerVsHour = h.GetTH2F("TriggerVsHour");
    fTriggerVsHourGeneral = h.GetTH2F("TriggerVsHourGeneral");
  }
  if(Settings::Instance().fDet == kNEARDET) {
    fDtBNBTrig   = h.GetTH1F("DtBNBTrig");
    fDtNuMITrig  = h.GetTH1F("DtNuMITrig");
    fDtCalibTrig = h.GetTH1F("DtCalibTrig");
    fTriggerVsHour = h.GetTH2F("TriggerVsHour");
    fTriggerVsHourGeneral = h.GetTH2F("TriggerVsHourGeneral");
  }
  if(Settings::Instance().fDet == kFARDET) {
    fDtNuMITrig  = h.GetTH1F("DtNuMITrig");
    fDtCalibTrig = h.GetTH1F("DtCalibTrig");
    fTriggerVsHour = h.GetTH2F("TriggerVsHour");
    fTriggerVsHourGeneral = h.GetTH2F("TriggerVsHourGeneral");
  }
  
  fEvtMinusPrevEvt = h.GetTH1F("EvtMinusPrevEvt");

  fT0prevNuMI  = 0;
  fT0prevBNB   = 0;
  fT0prevCalib = 0;

  fRunPrev    = 0;
  fEventPrev  = 0;
}

//......................................................................

TriggerPlots::~TriggerPlots() { }

//......................................................................

void TriggerPlots::GetRawEventSummary(const RawEventSummary& r)
{
  // NOTE:  Since the fT0prev* variables are initialized to zero, the very
  // first dt will be huge.  BUT, this will only happen the very first time
  // and that dt will go into the overflow bin of the histogram so I felt
  // it wasn't worth it to add an extra "if(fT0prev* != 0)" statement that
  // would only be necessary the very first time a trigger appears.
  if(r.fTrigger == 0) {
    double dt = (r.fT0 - fT0prevNuMI)*kSEC_PER_TDC;
    fDtNuMITrig->Fill(dt);
    fT0prevNuMI = r.fT0;
  }
  if(r.fTrigger == 1 && Settings::Instance().fDet != kFARDET) {
    double dt = (r.fT0 - fT0prevBNB)*kSEC_PER_TDC;
    fDtBNBTrig->Fill(dt);
    fT0prevBNB = r.fT0;
  }
  if(r.fTrigger == 2) {
    double dt = (r.fT0 - fT0prevCalib)*kSEC_PER_TDC;
    fDtCalibTrig->Fill(dt);
    fT0prevCalib = r.fT0;
  }

  // NOTE:  Bad trigger numbers will just go into the under/over-flow bins.
  fTriggerVsHour->Fill(r.fHour, r.fTrigger);
  fTriggerVsHourGeneral->Fill(r.fHour, r.fTrigger);
  if(r.fHour >= 23.0) {
    fTriggerVsHour->Fill(r.fHour - 24.0, r.fTrigger);
    fTriggerVsHourGeneral->Fill(r.fHour - 24.0, r.fTrigger);
  }

  if (r.fRun==fRunPrev) {
    fEvtMinusPrevEvt->Fill(r.fEvent - fEventPrev);
  }
  fRunPrev   = r.fRun;
  fEventPrev = r.fEvent;
}

////////////////////////////////////////////////////////////////////////
