#include "OnlineMonitoring/producer/WatchListManager.h"
#include "OnlineMonitoring/producer/HistoSet.h"

#include "TH1F.h"
#include "TH2F.h"
#include "TGString.h"

using namespace om;

//......................................................................

WatchListManager& WatchListManager::Instance()
{
  static WatchListManager freddiemercury;
  return freddiemercury;
}

//......................................................................

WatchListManager::WatchListManager() :
  fCurrentWL()
{ }

//......................................................................

WatchListManager::~WatchListManager()
{ }

//......................................................................

void WatchListManager::Add(int db, int dcm, int feb, int pix)
{

  ///\ todo: send some kind of message to the viewer that the max WL
  //         size has been exceeded???
  if(fCurrentWL.fDB.size() >= NMAX_WATCHLIST) return;

  // test to see if item is already on the list
  bool IsOnList = false;
  unsigned int id;
  for(unsigned int i = 0; i < fCurrentWL.fDB.size(); ++i) {
    if(db  == fCurrentWL.fDB[i]  && dcm == fCurrentWL.fDCM[i] &&
       feb == fCurrentWL.fFEB[i] && pix == fCurrentWL.fPIX[i]) {
      IsOnList = true;
      id = i;
      break;
    }
  }

  if(IsOnList && fCurrentWL.fActive[id] == 1) return;
  if(IsOnList && fCurrentWL.fActive[id] == 0)
    fCurrentWL.fActive[id] = 1;
  if(!IsOnList) {
    // book the new histograms
    HistoSet& h = HistoSet::Instance();

    TGString WLlabel;
    if(db < 0) WLlabel += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", db);
      WLlabel += temp;
    }
    WLlabel += "-";
    if(dcm < 0) WLlabel += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", dcm);
      WLlabel += temp;
    }
    WLlabel += "-";
    if(feb < 0) WLlabel += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", feb);
      WLlabel += temp;
    }
    WLlabel += "-";
    if(pix < 0) WLlabel += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", pix);
      WLlabel += temp;
    }

    TGString WLTQname = "WLTQPlot_";
    TGString WLTname  = "WLTPlot_";
    TGString WLQname  = "WLQPlot_";

    WLTQname += WLlabel;
    fWLTQ.push_back(h.GetTH2F(WLTQname));

    WLTname += WLlabel;
    fWLT.push_back(h.GetTH1F(WLTname));

    WLQname += WLlabel;
    fWLQ.push_back(h.GetTH1F(WLQname));

    // add to list
    fCurrentWL.fDB .push_back(db);
    fCurrentWL.fDCM.push_back(dcm);
    fCurrentWL.fFEB.push_back(feb);
    fCurrentWL.fPIX.push_back(pix);
    fCurrentWL.fActive.push_back(1);
  }

}

//......................................................................

void WatchListManager::Remove(int db, int dcm, int feb, int pix)
{
  // find location of item on the list
  bool IsOnList = false;
  unsigned int id;
  for(unsigned int i = 0; i < fCurrentWL.fDB.size(); ++i) {
    if(db  == fCurrentWL.fDB[i]  && dcm == fCurrentWL.fDCM[i] &&
       feb == fCurrentWL.fFEB[i] && pix == fCurrentWL.fPIX[i]) {
      IsOnList = true;
      id = i;
      break;
    }
  }

  if(IsOnList && fCurrentWL.fActive[id] == 0) return;
  if(IsOnList && fCurrentWL.fActive[id] == 1)
    fCurrentWL.fActive[id] = 0;

}

//......................................................................

void WatchListManager::GetList(std::vector<int>& db,
			       std::vector<int>& dcm,
			       std::vector<int>& feb,
			       std::vector<int>& pix)
{
  for(unsigned int i = 0; i < fCurrentWL.fDB.size(); ++i) {
    if(fCurrentWL.fActive[i] == 1) {
      db .push_back(fCurrentWL.fDB [i]);
      dcm.push_back(fCurrentWL.fDCM[i]);
      feb.push_back(fCurrentWL.fFEB[i]);
      pix.push_back(fCurrentWL.fPIX[i]);
    }
  }
}

//......................................................................

void WatchListManager::FillPlots(int db, int dcm, int feb, int pix,
				 double time, unsigned int adc)
{

  for(unsigned int i = 0; i < fCurrentWL.fDB.size(); ++i) {
    if(fCurrentWL.fActive[i] == 1) {
      if((db  == fCurrentWL.fDB[i]  || fCurrentWL.fDB[i]  == -1) &&
	 (dcm == fCurrentWL.fDCM[i] || fCurrentWL.fDCM[i] == -1) &&
	 (feb == fCurrentWL.fFEB[i] || fCurrentWL.fFEB[i] == -1) &&
	 (pix == fCurrentWL.fPIX[i] || fCurrentWL.fPIX[i] == -1)) {

	fWLTQ[i]->Fill(time, adc);
	fWLT [i]->Fill(time);
	fWLQ [i]->Fill(adc);

      } // end if match to CurrentWL
    } // end if fActive == 1
  } // end loop over i

}

//......................................................................

void WatchListManager::EndSubrunCleanUp()
{
  // Clean up watchlist items marked as inactive and delete the associated
  // histograms.
  HistoSet& h = HistoSet::Instance();

  std::vector< unsigned int > remove;

  for(unsigned int i = 0; i < fCurrentWL.fDB.size(); ++i) {
    if(fCurrentWL.fActive[i] == 0) remove.push_back(i);
  }

  for(unsigned int i = remove.size(); i > 0; i--) {
    fCurrentWL.fDB .erase(fCurrentWL.fDB .begin()+remove[i-1]);
    fCurrentWL.fDCM.erase(fCurrentWL.fDCM.begin()+remove[i-1]);
    fCurrentWL.fFEB.erase(fCurrentWL.fFEB.begin()+remove[i-1]);
    fCurrentWL.fPIX.erase(fCurrentWL.fPIX.begin()+remove[i-1]);
    fCurrentWL.fActive.erase(fCurrentWL.fActive.begin()+remove[i-1]);
    
    h.DeleteTH2F(fWLTQ[remove[i-1]]);
    fWLTQ.erase(fWLTQ.begin()+remove[i-1]);

    h.DeleteTH1F(fWLT[remove[i-1]]);
    fWLT.erase(fWLT.begin()+remove[i-1]);

    h.DeleteTH1F(fWLQ[remove[i-1]]);
    fWLQ.erase(fWLQ.begin()+remove[i-1]);
  }

}

////////////////////////////////////////////////////////////////////////
