#include "OnlineMonitoring/viewer/HwDCMLabel.h"
#include <cstdio>
#include <iostream>
#include "TH1.h"
#include "TText.h"
#include "TLine.h"
using namespace om;

static const unsigned int gsNfebx   = 16;
static const unsigned int gsNfeby   = 4;
static const unsigned int gsNpixelx = 4;
static const unsigned int gsNpixely = 8;

HwDCMLabel::HwDCMLabel() { }

//......................................................................

void HwDCMLabel::Config(const TH1* h) 
{
  this->Clear();
  this->MakeFEBLabels();
  this->MakeVerticalLines();
  this->MakeHorizontalLines();
  this->MakePixelKey();
}

//......................................................................

void HwDCMLabel::Clear()
{
  std::list<TText*>::const_iterator text   (fText.begin());
  std::list<TText*>::const_iterator textEnd(fText.end());
  for (;text!=textEnd; ++text) delete *text;
  fText.clear();

  std::list<TLine*>::const_iterator line   (fLine.begin());
  std::list<TLine*>::const_iterator lineEnd(fLine.end());
  for (;line!=lineEnd; ++line) delete *line;
  fLine.clear();
}

//......................................................................

HwDCMLabel::~HwDCMLabel() { this->Clear(); }

//......................................................................

void HwDCMLabel::Draw() 
{
  std::list<TText*>::const_iterator text   (fText.begin());
  std::list<TText*>::const_iterator textEnd(fText.end());
  for (;text!=textEnd; ++text) (*text)->Draw();
  
  std::list<TLine*>::const_iterator line   (fLine.begin());
  std::list<TLine*>::const_iterator lineEnd(fLine.end());
  for (;line!=lineEnd; ++line) (*line)->Draw();
}

//......................................................................

void HwDCMLabel::MakeFEBLabels() 
{
  unsigned int i;
  static const char* feblabel[gsNfeby] = {
    "FEB 48 - 63",
    "FEB 32 - 47",
    "FEB 16 - 31",
    "FEB 00 - 15"
  };
  
  TText* t;
  float x, y;
  for (i=0; i<gsNfeby; ++i) {
    
    x = -0.01*gsNfebx*gsNpixelx;
    y = ((float)i+0.95)*gsNpixely;
    
    t = new TText(x, y, feblabel[i]);

    t->SetTextSize(0.022);
    t->SetTextAlign(32);
    t->SetBit(kCannotPick);
    
    fText.push_back(t);
  }
}

//......................................................................

void HwDCMLabel::MakeVerticalLines() 
{
  unsigned int i;
  
  TLine* l;
  for (i=0; i<=gsNfebx*gsNpixelx; ++i) {
    l = new TLine(i, 0, 
		  i, gsNfeby*gsNpixely);

    l->SetLineColor(kBlack);
    if (i%4==0) l->SetLineWidth(3);
    else        l->SetLineWidth(1);

    l->SetBit(kCannotPick);
    
    fLine.push_back(l);
  }
}

//......................................................................

void HwDCMLabel::MakeHorizontalLines()
{
  unsigned int i;

  TLine* l;
  for (i=0; i<=gsNfeby*gsNpixely; ++i) {
    l = new TLine(0,                 i, 
		  gsNfebx*gsNpixelx, i);
    l->SetLineColor(kBlack);
    if (i%8==0) l->SetLineWidth(3);
    else        l->SetLineWidth(1);
    
    l->SetBit(kCannotPick);

    fLine.push_back(l);
  }
}

//......................................................................

void HwDCMLabel::MakePixelKey() 
{
  unsigned int i;
  TLine* l;
  TText* t;
  
  float sf = 1.15;
  float x0,  y0;  // Center of key
  float xll, yll; // Lower left corner
  float xur, yur; // Uppwer right corner
  
  x0 = -0.06*gsNfebx*gsNpixelx;
  y0 =  0.04*gsNfeby*gsNpixely;

  xll = x0-sf*(gsNpixelx/2);
  yll = y0-sf*(gsNpixely/2);

  xur = x0+sf*(gsNpixelx/2);
  yur = y0+sf*(gsNpixely/2);

  for (i=0; i<=gsNpixelx; ++i) {
    l = new TLine(xll+i*sf, yll, xll+i*sf, yur);
    l->SetLineColor(kBlack);
    if (i%4==0) l->SetLineWidth(2);
    else        l->SetLineWidth(1);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }

  for (i=0; i<=gsNpixely; ++i) {
    l = new TLine(xll, yll+i*sf, xur, yll+i*sf);
    l->SetLineColor(kBlack);
    if (i%8==0) l->SetLineWidth(2);
    else        l->SetLineWidth(1);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }

  t = new TText(xll-1.1*sf,y0,"APD pixels");
  t->SetTextSize(0.02);
  t->SetTextAngle(90);
  t->SetTextAlign(22);
  t->SetBit(kCannotPick);
  fText.push_back(t);

  static const char* pixel[32] = {
    "30","31","28","29","26","27","24","25",
    "22","23","20","21","18","19","17","16",
    "09","08","11","10","13","12","14","15",
    "01","00","03","02","05","04","07","06"
  };
  for (i=0; i<32; ++i) {
    unsigned int ii;
    unsigned int jj;
    ii = i/8;
    jj = i%8;
    t = new TText(xll + sf*(ii+0.5),
		  yll + sf*(jj+0.5),
		  pixel[i]);
    t->SetTextColor(kGray+2);
    t->SetTextAlign(22);
    t->SetTextSize(0.014);
    t->SetBit(kCannotPick);
    fText.push_back(t);
  }
}

////////////////////////////////////////////////////////////////////////
