#include "OnlineMonitoring/viewer/HwDetLabel.h"
#include <iostream>
#include "TH1.h"
#include "TLine.h"
#include "TText.h"
using namespace om;

static const int gsNfebX   = 16;
static const int gsNfebY   = 4;
static const int gsNpixelX = 4;
static const int gsNpixelY = 8;

HwDetLabel::HwDetLabel(bool show_feb_boundaries) : 
  fShowFEBBoundaries(show_feb_boundaries),
  fNdiblock(0), 
  fNdcm(0) 
{ }

//......................................................................

void HwDetLabel::Config(const TH1* h)
{
  int ndb  = h->GetXaxis()->GetXmax()/gsNpixelX/gsNfebX;
  int ndcm = h->GetYaxis()->GetXmax()/gsNpixelY/gsNfebY;
  
  if ( (ndb != fNdiblock) || (ndcm != fNdcm) ) {
    fNdiblock = ndb;
    fNdcm     = ndcm;
    this->Clear();
    this->MakeVerticalLines();
    this->MakeHorizontalLines();
    this->MakeDBLabels();
    this->MakeDCMLabels();
  }
}

//......................................................................

void HwDetLabel::Clear() 
{
  std::list<TLine*>::const_iterator line(fLine.begin());
  std::list<TLine*>::const_iterator lineend(fLine.end());
  for (; line!=lineend; ++line) delete *line;
  fLine.clear();
  
  std::list<TText*>::const_iterator text(fText.begin());
  std::list<TText*>::const_iterator textend(fText.end());
  for (; text!=textend; ++text) delete *text;
  fText.clear();
}

//......................................................................

HwDetLabel::~HwDetLabel() 
{
  this->Clear();
}

//......................................................................

void HwDetLabel::Draw()
{
  std::list<TLine*>::const_iterator line(fLine.begin());
  std::list<TLine*>::const_iterator lineend(fLine.end());
  for (; line!=lineend; ++line) (*line)->Draw();

  std::list<TText*>::const_iterator text(fText.begin());
  std::list<TText*>::const_iterator textend(fText.end());
  for (; text!=textend; ++text) (*text)->Draw();
}

//......................................................................

void HwDetLabel::MakeHorizontalLines()
{
  int i;
  TLine* l;
  for (i=0; i<=fNdcm*gsNfebY; ++i) {
    //
    // If the index is an FEB boundary and we've been asked not to
    // show the FEB boundaries, skip to the next iteration
    //
    if ((i%gsNfebY)!=0 && fShowFEBBoundaries==false) continue;

    //
    // Make the line. Make the DCM boundaries thicker
    //
    l = new TLine(0,                           i*gsNpixelY,
		  fNdiblock*gsNpixelX*gsNfebX, i*gsNpixelY);
    if ((i%gsNfebY)==0) l->SetLineWidth(3);
    else                l->SetLineWidth(1);

    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }
}

//......................................................................

void HwDetLabel::MakeVerticalLines() 
{
  int i;
  TLine* l;
  for (i=0; i<=fNdiblock*gsNfebX; ++i) {
    //
    // If we are on an FEB boundary and we've been asked not to draw
    // them, skip to the next iteration
    //
    if ((i%gsNfebX)==1 && fShowFEBBoundaries==false) continue;
    
    //
    // Draw the line. Make the DCM boundary thicker.
    //
    l = new TLine(i*gsNpixelX, 0,
		  i*gsNpixelX, fNdcm*gsNpixelY*gsNfebY);
    if ((i%gsNfebX)==0) l->SetLineWidth(3);
    else                l->SetLineWidth(1);
    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }
}

//......................................................................

void HwDetLabel::MakeDBLabels() 
{
  int i;
  TText* t;
  char buff[16];
  for (i=0; i<fNdiblock; ++i) {
    sprintf(buff,"DB%.2d",fNdiblock-i);
    t = new TText((i+0.5)*gsNpixelX*gsNfebX,
		  1.02*gsNpixelY*gsNfebY*fNdcm,
		  buff);
    t->SetTextAlign(22);
    t->SetTextSize(0.03);
    t->SetBit(kCannotPick);
    fText.push_back(t);
  }
}

//......................................................................

void HwDetLabel::MakeDCMLabels() 
{
  int i;
  TText* t;
  char buff[16];
  for (i=0; i<fNdcm; ++i) {
    sprintf(buff,"DCM%.2d",fNdcm-i);
    t = new TText(-0.005*gsNpixelX*gsNfebX*fNdiblock,
		  gsNpixelY*gsNfebY*(i+0.5),
		  buff);
    t->SetTextAlign(32);
    t->SetTextSize(0.03);
    // t->SetTextAngle(90);
    t->SetBit(kCannotPick);
    fText.push_back(t);
  }
}

////////////////////////////////////////////////////////////////////////
