#include "OnlineMonitoring/viewer/HwPCLabel.h"
#include <iostream>
#include "TH1.h"
#include "TLine.h"
#include "TPaveText.h"
using namespace om;

static const int gsNplPERdb  = 64;
static const int gsNclPERmod = 32;

HwPCLabel::HwPCLabel() : 
  fNdiblock(0), 
  fNmod(0) 
{
    fTextTop1  = new TPaveText(0.1, 0.905, 0.2, 0.925, "NDC");
    fTextTop2  = new TPaveText(0.3, 0.905, 0.7, 0.925, "NDC");
    fTextTop3  = new TPaveText(0.8, 0.905, 0.9, 0.925, "NDC");

    fTextSide1 = new TPaveText(0.04, 0.1, 0.08, 0.2, "NDC");
    fTextSide2 = new TPaveText(0.04, 0.46, 0.08, 0.49, "NDC");
    fTextSide3 = new TPaveText(0.04, 0.51, 0.08, 0.54, "NDC");
    fTextSide4 = new TPaveText(0.04, 0.8, 0.08, 0.9, "NDC");
}

//......................................................................

void HwPCLabel::Config(const TH1* h, bool isXview)
{ 
  fNdiblock = h->GetNbinsX()/gsNplPERdb;
  fNmod     = h->GetNbinsY()/gsNclPERmod;
  this->Clear();
  if(isXview) {
    this->MakeLabelsX();
    this->MakeHorizontalLinesX();
    this->MakeVerticalLinesX();
  }
  else {
    this->MakeLabelsY();
    this->MakeHorizontalLinesY();
    this->MakeVerticalLinesY();
  }
}

//......................................................................

void HwPCLabel::Clear() 
{
  std::list<TLine*>::const_iterator line(fLine.begin());
  std::list<TLine*>::const_iterator lineend(fLine.end());
  for (; line!=lineend; ++line) delete *line;
  fLine.clear();
  
  fTextTop1->Clear();
  fTextTop2->Clear();
  fTextTop3->Clear();
  fTextSide1->Clear();
  fTextSide2->Clear();
  fTextSide3->Clear();
  fTextSide4->Clear();
}

//......................................................................

HwPCLabel::~HwPCLabel() 
{
  this->Clear();
  if(fTextTop1)  { delete fTextTop1;  fTextTop1  = 0; }
  if(fTextTop2)  { delete fTextTop2;  fTextTop2  = 0; }
  if(fTextTop3)  { delete fTextTop3;  fTextTop3  = 0; }
  if(fTextSide1) { delete fTextSide1; fTextSide1 = 0; }
  if(fTextSide2) { delete fTextSide2; fTextSide2 = 0; }
  if(fTextSide3) { delete fTextSide3; fTextSide3 = 0; }
  if(fTextSide4) { delete fTextSide4; fTextSide4 = 0; }
}

//......................................................................

void HwPCLabel::Draw()
{
  std::list<TLine*>::const_iterator line(fLine.begin());
  std::list<TLine*>::const_iterator lineend(fLine.end());
  for (; line!=lineend; ++line) (*line)->Draw();

  fTextTop1->Draw();
  fTextTop2->Draw();
  fTextTop3->Draw();
  fTextSide1->Draw();
  fTextSide2->Draw();
  fTextSide3->Draw();
  fTextSide4->Draw();
}

//......................................................................

void HwPCLabel::MakeHorizontalLinesX()
{
  int i;
  TLine* l;
  for (i = 0; i <= fNmod; ++i) {
    l = new TLine(0.5,                         -1*i*gsNclPERmod+0.5,
		  -1*fNdiblock*gsNplPERdb+0.5, -1*i*gsNclPERmod+0.5);
    if(i ==0 || i == fNmod)    l->SetLineWidth(3);
    else { l->SetLineStyle(2); l->SetLineWidth(1); }
    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }
}

//......................................................................

void HwPCLabel::MakeHorizontalLinesY()
{
  int i;
  TLine* l;
  for (i = 0; i <= fNmod; ++i) {
    l = new TLine(0.5,                         i*gsNclPERmod-0.5,
		  -1*fNdiblock*gsNplPERdb+0.5, i*gsNclPERmod-0.5);
    if(i ==0 || i == fNmod)    l->SetLineWidth(3);
    else { l->SetLineStyle(2); l->SetLineWidth(1); }
    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
  }
}

//......................................................................

void HwPCLabel::MakeVerticalLinesX() 
{
  int i;
  TLine* l;
  for (i = 0; i <= fNdiblock; ++i) {
    l = new TLine(-1*i*gsNplPERdb+0.5, 0.5,
		  -1*i*gsNplPERdb+0.5, -1*fNmod*gsNclPERmod+0.5);
    l->SetLineWidth(3);
    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
    }
}

//......................................................................

void HwPCLabel::MakeVerticalLinesY() 
{
  int i;
  TLine* l;
  for (i = 0; i <= fNdiblock; ++i) {  
    l = new TLine(-1*i*gsNplPERdb+0.5, -0.5,
		  -1*i*gsNplPERdb+0.5, fNmod*gsNclPERmod-0.5);
    l->SetLineWidth(3);
    l->SetLineColor(kBlack);
    l->SetBit(kCannotPick);
    fLine.push_back(l);
    }
}

//......................................................................

void HwPCLabel::MakeLabelsX() 
{
  fTextTop1->AddText("#leftarrow North");
  fTextTop1->SetLineColor(0);
  fTextTop1->SetFillColor(0);
  fTextTop1->SetBorderSize(1);
  fTextTop1->SetMargin(0.0);
  fTextTop1->SetTextAlign(21);
  fTextTop1->SetTextSize(0.03);
  fTextTop1->SetBit(kCannotPick);

  fTextTop2->AddText("Planes / Diblocks");
  fTextTop2->SetLineColor(0);
  fTextTop2->SetFillColor(0);
  fTextTop2->SetBorderSize(1);
  fTextTop2->SetMargin(0.0);
  fTextTop2->SetTextAlign(21);
  fTextTop2->SetTextSize(0.03);
  fTextTop2->SetBit(kCannotPick);

  fTextTop3->AddText("South #rightarrow");
  fTextTop3->SetLineColor(0);
  fTextTop3->SetFillColor(0);
  fTextTop3->SetBorderSize(1);
  fTextTop3->SetMargin(0.0);
  fTextTop3->SetTextAlign(21);
  fTextTop3->SetTextSize(0.03);
  fTextTop3->SetBit(kCannotPick);

  fTextSide1->AddText("#downarrow West");
  fTextSide1->SetLineColor(0);
  fTextSide1->SetFillColor(0);
  fTextSide1->SetBorderSize(1);
  fTextSide1->SetMargin(0.0);
  fTextSide1->SetTextAlign(21);
  fTextSide1->SetTextSize(0.03);
  fTextSide1->SetBit(kCannotPick);

  fTextSide2->AddText("Modules");
  fTextSide2->SetLineColor(0);
  fTextSide2->SetFillColor(0);
  fTextSide2->SetBorderSize(1);
  fTextSide2->SetMargin(0.0);
  fTextSide2->SetTextAlign(21);
  fTextSide2->SetTextSize(0.03);
  fTextSide2->SetBit(kCannotPick);

  fTextSide3->AddText("Cells /");
  fTextSide3->SetLineColor(0);
  fTextSide3->SetFillColor(0);
  fTextSide3->SetBorderSize(1);
  fTextSide3->SetMargin(0.0);
  fTextSide3->SetTextAlign(21);
  fTextSide3->SetTextSize(0.03);
  fTextSide3->SetBit(kCannotPick);

  fTextSide4->AddText("#uparrow East");
  fTextSide4->SetLineColor(0);
  fTextSide4->SetFillColor(0);
  fTextSide4->SetBorderSize(1);
  fTextSide4->SetMargin(0.0);
  fTextSide4->SetTextAlign(21);
  fTextSide4->SetTextSize(0.03);
  fTextSide4->SetBit(kCannotPick);
}

//......................................................................

void HwPCLabel::MakeLabelsY() 
{
  fTextTop1->AddText("#leftarrow North");
  fTextTop1->SetLineColor(0);
  fTextTop1->SetFillColor(0);
  fTextTop1->SetBorderSize(1);
  fTextTop1->SetMargin(0.0);
  fTextTop1->SetTextAlign(21);
  fTextTop1->SetTextSize(0.03);
  fTextTop1->SetBit(kCannotPick);

  fTextTop2->AddText("Planes / Diblocks");
  fTextTop2->SetLineColor(0);
  fTextTop2->SetFillColor(0);
  fTextTop2->SetBorderSize(1);
  fTextTop2->SetMargin(0.0);
  fTextTop2->SetTextAlign(21);
  fTextTop2->SetTextSize(0.03);
  fTextTop2->SetBit(kCannotPick);

  fTextTop3->AddText("South #rightarrow");
  fTextTop3->SetLineColor(0);
  fTextTop3->SetFillColor(0);
  fTextTop3->SetBorderSize(1);
  fTextTop3->SetMargin(0.0);
  fTextTop3->SetTextAlign(21);
  fTextTop3->SetTextSize(0.03);
  fTextTop3->SetBit(kCannotPick);

  fTextSide1->AddText("#downarrow Bottom");
  fTextSide1->SetLineColor(0);
  fTextSide1->SetFillColor(0);
  fTextSide1->SetBorderSize(1);
  fTextSide1->SetMargin(0.0);
  fTextSide1->SetTextAlign(21);
  fTextSide1->SetTextSize(0.03);
  fTextSide1->SetBit(kCannotPick);

  fTextSide2->AddText("Modules");
  fTextSide2->SetLineColor(0);
  fTextSide2->SetFillColor(0);
  fTextSide2->SetBorderSize(1);
  fTextSide2->SetMargin(0.0);
  fTextSide2->SetTextAlign(21);
  fTextSide2->SetTextSize(0.03);
  fTextSide2->SetBit(kCannotPick);

  fTextSide3->AddText("Cells /");
  fTextSide3->SetLineColor(0);
  fTextSide3->SetFillColor(0);
  fTextSide3->SetBorderSize(1);
  fTextSide3->SetMargin(0.0);
  fTextSide3->SetTextAlign(21);
  fTextSide3->SetTextSize(0.03);
  fTextSide3->SetBit(kCannotPick);

  fTextSide4->AddText("#uparrow Top");
  fTextSide4->SetLineColor(0);
  fTextSide4->SetFillColor(0);
  fTextSide4->SetBorderSize(1);
  fTextSide4->SetMargin(0.0);
  fTextSide4->SetTextAlign(21);
  fTextSide4->SetTextSize(0.03);
  fTextSide4->SetBit(kCannotPick);
}

//......................................................................

void HwPCLabel::DrawNDOS(bool isXview) 
{
  // NDOS is an asymmetric little dude, that is different from the ND and FD
  // so I have to handle drawing his lines as a special case...
  // 
  //    :(
  //
  // I decided that I'm just going to skip it for now...
  //
  // Take that NDOS!!!  How's it feel to be different??!!!


  // I guess I can draw the text labels though...
  this->Clear();
  if(isXview) this->MakeLabelsX();
  else        this->MakeLabelsY();
}

////////////////////////////////////////////////////////////////////////
