#include "DAQChannelMap/HardwareDisplay.h"
#include "OnlineMonitoring/viewer/UTCLabel.h"
#include "OnlineMonitoring/viewer/GUIModel.h"
#include <cstdio>
#include <iostream>
#include "TText.h"
#include "TLine.h"
using namespace om;

UTCLabel::UTCLabel() :
  fHwMap(new daqchannelmap::HardwareDisplay)
{ }

//......................................................................

void UTCLabel::Clear()
{
  std::list<TText*>::const_iterator text   (fText.begin());
  std::list<TText*>::const_iterator textEnd(fText.end());
  for (;text!=textEnd; ++text) delete *text;
  fText.clear();

  std::list<TLine*>::const_iterator line   (fLine.begin());
  std::list<TLine*>::const_iterator lineEnd(fLine.end());
  for (;line!=lineEnd; ++line) delete *line;
  fLine.clear();
}

//......................................................................

UTCLabel::~UTCLabel() {
  this->Clear();
  if(fHwMap) {delete fHwMap; fHwMap = 0;}
}

//......................................................................

void UTCLabel::Draw() 
{
  std::list<TText*>::const_iterator text   (fText.begin());
  std::list<TText*>::const_iterator textEnd(fText.end());
  for (;text!=textEnd; ++text) (*text)->Draw();
  
  std::list<TLine*>::const_iterator line   (fLine.begin());
  std::list<TLine*>::const_iterator lineEnd(fLine.end());
  for (;line!=lineEnd; ++line) (*line)->Draw();
}

//......................................................................

void UTCLabel::FEBLabels() 
{
  this->Clear();

  fHwMap->SetupDet(GUIModel::Instance().Data().fDetector);

  unsigned int i;
  
  unsigned int Ndcm = fHwMap->fNdcmTop+fHwMap->fNdcmSide;

  TLine* l;
  for (i=0; i<=Ndcm; ++i) {
    l = new TLine(0, i*64, 24.0, i*64);
    l->SetLineColor(kBlack);
    l->SetLineWidth(1);
    
    l->SetBit(kCannotPick);
    
    fLine.push_back(l);
  }

  TText* t;
  char buff[16];

  for (i=0; i<Ndcm; ++i) {
    sprintf(buff,"DCM%.2d",i+1);
    t = new TText(-0.04*24,64*(i+0.5),buff);
    t->SetTextAlign(22);
    t->SetTextSize(0.03);
    t->SetBit(kCannotPick);
    fText.push_back(t);
  }
}

//......................................................................

void UTCLabel::DCMLabels() 
{
  this->Clear();

  fHwMap->SetupDet(GUIModel::Instance().Data().fDetector);

  unsigned int i;
  
  unsigned int Ndb = fHwMap->fNdiblocks;
  unsigned int NdcmPERdb = fHwMap->fNdcmTop+fHwMap->fNdcmSide;

  TLine* l;
  for (i=0; i<=Ndb; ++i) {
    l = new TLine(0, i*NdcmPERdb+1, 24.0, i*NdcmPERdb+1);
    l->SetLineColor(kBlack);
    l->SetLineWidth(1);
    
    l->SetBit(kCannotPick);
    
    fLine.push_back(l);
  }

  TText* t;
  char buff[16];

  for (i=0; i<Ndb; ++i) {
    sprintf(buff,"DB%.2d",i+1);
    t = new TText(-0.03*24,NdcmPERdb*(i+0.5),buff);
    t->SetTextAlign(32);
    t->SetTextSize(0.03);
    t->SetBit(kCannotPick);
    fText.push_back(t);
  }
}

////////////////////////////////////////////////////////////////////////
