#include "OnlineMonitoring/viewer/WatchListBox.h"
#include <iostream>
#include "TGButton.h"
#include "TGListBox.h"
#include "TGText.h"
#include "TGFrame.h"
#include "TGComboBox.h"
#include "TGLabel.h"
#include "TGString.h"
#include "TList.h"
#include "OnlineMonitoring/util/HistoData.h"
#include "OnlineMonitoring/util/HistoTable.h"
#include "OnlineMonitoring/util/IPC.h"
#include "OnlineMonitoring/viewer/GUIModel.h"
#include "OnlineMonitoring/viewer/GUIModelData.h"
#include "OnlineMonitoring/viewer/OStream.h"

using namespace om;

WatchListBox::WatchListBox(const TGWindow* win,
			     unsigned int w,
			     unsigned int h,
			     unsigned int opt) :
  TGCompositeFrame(win, w, h, opt),
  GUIModelSubscriber(kWatchListUpdateID|kDetectorID|kHistogramSourceID)
{

  TGLayoutHints* xx = new TGLayoutHints(kLHintsExpandX);

  fHardwareFrame = new TGGroupFrame(this, "Choose Hardware");
  this->LayoutHardwareFrame();
  this->AddFrame(fHardwareFrame, xx);

  fCurrentListFrame = new TGGroupFrame(this, "Current WatchList:");
  this->LayoutCurrentListFrame();
  this->AddFrame(fCurrentListFrame, xx);

  fRemoveFromList = new TGTextButton(this, "Remove from WatchList");
  fRemoveFromList->SetToolTipText("Remove the selected item from the WatchList");
  fRemoveFromList->Connect("Clicked()",
			   "om::WatchListBox",
			   this,
			   "HandleRemoveFromList()");
  this->AddFrame(fRemoveFromList, xx);

}



//......................................................................

void WatchListBox::LayoutHardwareFrame() 
{

  ///\todo: add detectorID republish into everything that would change
  //        when the detector changes...

  TGCompositeFrame* fHWboxes = new TGCompositeFrame(fHardwareFrame,
						    300, 20,
						    kHorizontalFrame);
  fDB  = new TGComboBox(fHWboxes);
  fDCM = new TGComboBox(fHWboxes);
  fFEB = new TGComboBox(fHWboxes);
  fPIX = new TGComboBox(fHWboxes);
 
  TGLayoutHints* yy = new TGLayoutHints(kLHintsNormal, 10, 0, 0, 0);
  TGLayoutHints* zz = new TGLayoutHints(kLHintsNormal, 2,  0, 0, 0);



  // These numbers will be properly set in the GUIModelDataIssue function
  // after the GUIModelData is properly initialized.
  int numDB  = 0;
  int numDCM = 0;

  if(om::GUIModel::Instance().Data().fDetector == kNDOS) {
    numDB  = 1;
    numDCM = 1;
  }
  else if(om::GUIModel::Instance().Data().fDetector == kNEARDET) {
    numDB  = 1;
    numDCM = 1;
  }
  else if(om::GUIModel::Instance().Data().fDetector == kFARDET) {
    numDB  = 1;
    numDCM = 1;
  }
  
  fDB->RemoveAll();
  fHWboxes->AddFrame(fDB, zz);
  fDB->AddEntry("ALL", 1);
  for(int i = 1; i <= numDB; ++i) {
    char entry[2];
    sprintf(entry, "%.2u", i);
    fDB->AddEntry(entry, i+1);
  }
  fDB->Resize(50, 20);

  fDCM->RemoveAll();
  fHWboxes->AddFrame(fDCM, zz);
  fDCM->AddEntry("ALL", 1);
  for(int i = 1; i <= numDCM; ++i) {
    char entry[2];
    sprintf(entry, "%.2u", i);
    fDCM->AddEntry(entry, i+1);
  }
  fDCM->Resize(50, 20);

  fFEB->RemoveAll();
  fHWboxes->AddFrame(fFEB, zz);
  fFEB->AddEntry("ALL", 1);
  for(int i = 0; i < 64; ++i) {
    char entry[2];
    sprintf(entry, "%.2u", i);
    fFEB->AddEntry(entry, i+2);
  }
  fFEB->Resize(50, 20);

  fPIX->RemoveAll();
  fHWboxes->AddFrame(fPIX, zz);
  fPIX->AddEntry("ALL", 1);
  for(int i = 0; i < 32; ++i) {
    char entry[2];
    sprintf(entry, "%.2u", i);
    fPIX->AddEntry(entry, i+2);
  }
  fPIX->Resize(50, 20);

  fAddToList = new TGTextButton(fHWboxes, "ADD");
  fAddToList->SetToolTipText("Add the chosen hardware to the WatchList");
  fAddToList->Connect("Clicked()",
		      "om::WatchListBox",
		      this,
		      "HandleAddToList()");
  fHWboxes->AddFrame(fAddToList, yy);

  fHWlabel = new TGLabel(fHardwareFrame,
			 "Diblock  -  DCM   -  FEB   -  Pixel");
  fHardwareFrame->AddFrame(fHWlabel, zz);
  fHardwareFrame->AddFrame(fHWboxes, zz);
  
}



//......................................................................

void WatchListBox::LayoutCurrentListFrame() 
{

  TGLayoutHints* zz = new TGLayoutHints(kLHintsExpandX);

  fCurrentList = new TGListBox(fCurrentListFrame, kSunkenFrame);
  fCurrentListFrame->AddFrame(fCurrentList, zz);
  fCurrentList->Resize(100,300);

  fCurrentList->SetMultipleSelections();

}



//......................................................................

void WatchListBox::HandleAddToList()
{

  // GetSelected() returns:
  //
  // -1 = nothing selected
  // 1 = first selection (for me this is ALL)
  // 2 = second selection (etc...)

  // check for legit entries & pass info to GUIModelData
  if(fDB ->GetSelected() < 1 || fDCM->GetSelected() < 1 ||
     fFEB->GetSelected() < 1 || fPIX->GetSelected() < 1) {
    om::cout << "You must make a selection in every hardware field!"
	     << "";
    return;
  }
  
  // We have good values to pass to GUIModel.  Note: the first item in the ComboBox is
  // "ALL" which is translated to -1 and the second item is 00, so we will subtract 2
  // from the ComboBox value to get the number right.  Diblocks and DCMs are different
  // since they start at 01 (honestly, who came up with this numbering system???)
  int DB = fDB->GetSelected();
  if(fDB->GetSelected() == 1) DB = -1;
  else DB = DB - 1;

  int DCM = fDCM->GetSelected();
  if(fDCM->GetSelected() == 1) DCM = -1;
  else DCM = DCM - 1;

  int FEB = fFEB->GetSelected()-2;
  int PIX = fPIX->GetSelected()-2;

  // check to see that histo source is SHM
  // if source is a root file, do nothing and return
  std::string s(GUIModel::Instance().Data().fHistogramSource.c_str());
  bool is_root = s.find(".root")<s.length();
  bool is_shm  = s.find(".shm")< s.length();

  if (is_root && is_shm) abort();
  if (is_root) {
    om::cout << "WatchList disabled (input source is not SHM.)";
    this->UpdateHWboxes();
    return;
  }

  IPC ipc(kIPC_CLIENT, GUIModel::Instance().Data().fHistogramSource.c_str());

  ipc.RequestAddToWatchList(DB, DCM, FEB, PIX);

  GUIModel::Instance().SendWatchListUpdate();

  return;
}



//......................................................................

void WatchListBox::HandleRemoveFromList()
{

  // check to see that histo source is SHM
  // if source is a root file, do nothing and return
  std::string s(GUIModel::Instance().Data().fHistogramSource.c_str());
  bool is_root = s.find(".root")<s.length();
  bool is_shm  = s.find(".shm")< s.length();
  
  if (is_root && is_shm) abort();
  if (is_root) return;
  
  IPC ipc(kIPC_CLIENT, GUIModel::Instance().Data().fHistogramSource.c_str());

  TList* selected = new TList();
  fCurrentList->GetSelectedEntries(selected);

  std::vector<unsigned int> rlist;

  for(int i = 0; i < selected->GetSize(); ++i) {
    for(int j = 0; j < fCurrentList->GetNumberOfEntries(); ++j) {
      if(fCurrentList->GetEntry(j) == selected->At(i)) rlist.push_back(j);
    }
  }
  
  for(unsigned int k = 0; k < rlist.size(); ++k) {   
    ipc.RequestRemoveFromWatchList(fCWLdb [rlist[k]], fCWLdcm[rlist[k]],
				   fCWLfeb[rlist[k]], fCWLpix[rlist[k]]);
  }
  
  GUIModel::Instance().SendWatchListUpdate();
  
  return;
}



//......................................................................

void WatchListBox::UpdateCurrentList()
{

  // check to see that histo source is SHM
  // if source is a root file, do nothing and return
  std::string s(GUIModel::Instance().Data().fHistogramSource.c_str());
  bool is_root = s.find(".root")<s.length();
  bool is_shm  = s.find(".shm")< s.length();

  if (is_root && is_shm) abort();
  if (is_root) return;

  IPC ipc(kIPC_CLIENT, GUIModel::Instance().Data().fHistogramSource.c_str());

  std::vector<int> db;
  std::vector<int> dcm;
  std::vector<int> feb;
  std::vector<int> pix;

  ipc.RequestCurrentWatchList(db, dcm, feb, pix);

  fCWLdb .clear();
  fCWLdcm.clear();
  fCWLfeb.clear();
  fCWLpix.clear();

  fCWLdb  = db;
  fCWLdcm = dcm;
  fCWLfeb = feb;
  fCWLpix = pix;

  // make the list of strings for displaying the current watchlist
  fCurrentList->RemoveAll();  
  for(unsigned int i = 0; i < db.size(); ++i) {

    TGString label;

    if(db[i] < 0) label += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", db[i]);
      label += temp;
    }
    label += " - ";

    if(dcm[i] < 0) label += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", dcm[i]);
      label += temp;
    }
    label += " - ";

    if(feb[i] < 0) label += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", feb[i]);
      label += temp;
    }
    label += " - ";

    if(pix[i] < 0) label += "XX";
    else {
      char temp[2];
      sprintf(temp, "%.2u", pix[i]);
      label += temp;
    }

    fCurrentList->AddEntry(label, i);
  }
  
  fCurrentList->Layout();
  
}



//......................................................................

void WatchListBox::UpdateHWboxes()
{
    int numDB  = 0;
    int numDCM = 0;
    
    if(om::GUIModel::Instance().Data().fDetector == kNDOS) {
      numDB  = 4;
      numDCM = 3;
    }
    else if(om::GUIModel::Instance().Data().fDetector == kNEARDET) {
      numDB  = 4;
      numDCM = 3;
    }
    else if(om::GUIModel::Instance().Data().fDetector == kFARDET) {
      numDB  = 14;
      numDCM = 12;
    }
    
    fDB->RemoveAll();
    fDB->AddEntry("ALL", 1);
    for(int i = 1; i <= numDB; ++i) {
      char entry[2];
      sprintf(entry, "%.2u", i);
      fDB->AddEntry(entry, i+1);
    }
    
    fDCM->RemoveAll();
    fDCM->AddEntry("ALL", 1);
    for(int i = 1; i <= numDCM; ++i) {
      char entry[2];
      sprintf(entry, "%.2u", i);
      fDCM->AddEntry(entry, i+1);
    }
    
    fFEB->RemoveAll();
    fFEB->AddEntry("ALL", 1);
    for(int i = 0; i < 64; ++i) {
      char entry[2];
      sprintf(entry, "%.2u", i);
      fFEB->AddEntry(entry, i+2);
    }
    
    fPIX->RemoveAll();
    fPIX->AddEntry("ALL", 1);
    for(int i = 0; i < 32; ++i) {
      char entry[2];
      sprintf(entry, "%.2u", i);
      fPIX->AddEntry(entry, i+2);
    }
}



//......................................................................

void WatchListBox::GUIModelDataIssue(const GUIModelData& m, 
				      unsigned int which) 
{
  if (this->GetLock()==false) return;
  
  if((which&kDetectorID) != 0 || (which&kWatchListUpdateID) != 0) {
    this->UpdateHWboxes();    
  } // end if (kGUIModelDataUpdateID or kWatchListUpdateID)

  if((which&kWatchListUpdateID) != 0 ||
     (which&kHistogramSourceID) != 0) {
    this->UpdateCurrentList();
  } // end if (kWatchListUpdateID or kHistogramSourceID)

  this->ReleaseLock();
}

//......................................................................



WatchListBox::~WatchListBox() 
{
  if (fHardwareFrame)    { delete fHardwareFrame;    fHardwareFrame    = 0; }
  if (fHWlabel)          { delete fHWlabel;          fHWlabel          = 0; }
  if (fHWboxes)          { delete fHWboxes;          fHWboxes          = 0; }
  if (fDB)               { delete fDB;               fDB               = 0; }
  if (fDCM)              { delete fDCM;              fDCM              = 0; }
  if (fFEB)              { delete fFEB;              fFEB              = 0; }
  if (fPIX)              { delete fPIX;              fPIX              = 0; }
  if (fAddToList)        { delete fAddToList;        fAddToList        = 0; }
  if (fCurrentListFrame) { delete fCurrentListFrame; fCurrentListFrame = 0; }
  if (fCurrentList)      { delete fCurrentList;      fCurrentList      = 0; }
  if (fRemoveFromList)   { delete fRemoveFromList;   fRemoveFromList   = 0; }
}

////////////////////////////////////////////////////////////////////////
