////////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  ksachdev@physics.umn.edu
////////////////////////////////////////////////////////////////////////

// Framework includes
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Core/FindManyP.h"
#include "art/Framework/Core/FindMany.h"
#include "art/Framework/Core/FindOneP.h"
#include "art/Framework/Core/FindOne.h"
// ROOT includes
#include "TTree.h"

// NOvASoft includes
#include "RecoJMShower/JMShower.h"
#include "RecoBase/WeightedHit.h"
#include "MCCheater/BackTracker.h"
#include "RecoBase/Shower.h"
#include "RecoBase/Vertex.h"
#include "ShowerLID/NuEEnergyAlg.h"
#include "LIDBuilder/ShowerLID.h"

namespace slid
{
  class ShowerLIDAna : public art::EDAnalyzer
  {
  public:
    explicit ShowerLIDAna(const fhicl::ParameterSet& pset);
    ~ShowerLIDAna();

    void analyze(const art::Event& evt);

    void reconfigure(const fhicl::ParameterSet& pset);

    void beginJob();

  protected:

    NuEEnergyAlg* fNuEEnergyAlg;
    fhicl::ParameterSet fNuEEnergyAlgPSet;

    std::string fJMLabel;
    std::string fVertexLabel;
    std::string fSliceLabel;
    std::string fProngLabel;
    std::string fAssn3D;
    std::string fAssn2D;
    std::string fShowerLabel;
    std::string fShowerLIDLabel;

    TTree *t;
    float hdnshowers;
    int hdncells;
    int hdncellx;
    int hdncelly;
    float hdtotpe;
    float hdtotgev;
    float hdmaxx;
    float hdmaxy;
    float hdmaxz;
    float hdminx;
    float hdminy;
    float hdminz;
    float hdmeanx;
    float hdmeany;
    float hdmeanz;
    float hdextentx;
    float hdextenty;
    float hdextentz;
    float hdtotwt;
    float hdeff;
    float hdpur;
    double hdshe;
    double hdshdepe;
    double hdhaddepe;
    double hdnuee;
    double hdvtxe;
    double hdpid;

    int ncells;
    int ncellx;
    int ncelly;
    float totpe;
    float totgev;
    float maxx;
    float maxy;
    float maxz;
    float minx;
    float miny;
    float minz;
    float meanx;
    float meany;
    float meanz;
    float extentx;
    float extenty;
    float extentz;
    float totwt;
    float eff;
    float pur;
    double she;
    double shdepe;
    double haddepe;
    double nuee;
    double vtxe;
    double pid;

    int event;
    int run;
    int srun;
    int prg;
  };
}


////////////////////////////////////////////////////////////////////////
namespace slid
{
  //.......................................................................
  ShowerLIDAna::ShowerLIDAna(const fhicl::ParameterSet& pset) 
    : EDAnalyzer(pset),
      fNuEEnergyAlg(0),
      fNuEEnergyAlgPSet(pset.get< fhicl::ParameterSet >("NuEEnergyAlgPSet") )
  {
    reconfigure(pset);
  }

  //......................................................................
  ShowerLIDAna::~ShowerLIDAna()
  {
    if( fNuEEnergyAlg ) 
      delete fNuEEnergyAlg;
  }

  //......................................................................
  void ShowerLIDAna::reconfigure(const fhicl::ParameterSet& pset)
  {
    fJMLabel    = pset.get<std::string>("JMLabel");
    fVertexLabel    = pset.get<std::string>("VertexLabel");
    fProngLabel = pset.get<std::string>("ProngLabel");
    fAssn3D     = pset.get<std::string>("Assn3D");
    fAssn2D     = pset.get<std::string>("Assn2D");
    fSliceLabel = pset.get<std::string>("SliceLabel");
    fShowerLabel= pset.get<std::string>("ShowerLabel");
    fShowerLIDLabel= pset.get<std::string>("ShowerLIDLabel");
  }

  //......................................................................
  void ShowerLIDAna::beginJob()
  {
    art::ServiceHandle<art::TFileService> tfs;
    t = tfs->make<TTree>("tree","tree");

    t->Branch("dnshowers", &hdnshowers);
    t->Branch("dncells", &hdncells);
    t->Branch("dncellx", &hdncellx);
    t->Branch("dncelly", &hdncelly);
    t->Branch("dtotwt", &hdtotwt);
    t->Branch("dtotpe", &hdtotpe);
    t->Branch("dtotgev", &hdtotgev);
    t->Branch("dmaxx", &hdmaxx);
    t->Branch("dmaxy", &hdmaxy);
    t->Branch("dmaxz", &hdmaxz);
    t->Branch("dminx", &hdminx);
    t->Branch("dminy", &hdminy);
    t->Branch("dminz", &hdminz);
    t->Branch("dmeanx", &hdmeanx);
    t->Branch("dmeany", &hdmeany);
    t->Branch("dmeanz", &hdmeanz);
    t->Branch("dextentx", &hdextentx);
    t->Branch("dextenty", &hdextenty);
    t->Branch("dextentz", &hdextentz);
    t->Branch("dshe", &hdshe);
    t->Branch("dshdepe", &hdshdepe);
    t->Branch("dhaddepe", &hdhaddepe);
    t->Branch("dnuee", &hdnuee);
    t->Branch("dvtxe", &hdvtxe);
    t->Branch("deff", &hdeff);
    t->Branch("dpur", &hdpur);
    t->Branch("dpid", &hdpid);

    t->Branch("ncells", &ncells);
    t->Branch("ncellx", &ncellx);
    t->Branch("ncelly", &ncelly);
    t->Branch("totwt", &totwt);
    t->Branch("totpe", &totpe);
    t->Branch("totgev", &totgev);
    t->Branch("maxx", &maxx);
    t->Branch("maxy", &maxy);
    t->Branch("maxz", &maxz);
    t->Branch("minx", &minx);
    t->Branch("miny", &miny);
    t->Branch("minz", &minz);
    t->Branch("meanx", &meanx);
    t->Branch("meany", &meany);
    t->Branch("meanz", &meanz);
    t->Branch("extentx", &extentx);
    t->Branch("extenty", &extenty);
    t->Branch("extentz", &extentz);
    t->Branch("she", &she);
    t->Branch("shdepe", &shdepe);
    t->Branch("haddepe", &haddepe);
    t->Branch("nuee", &nuee);
    t->Branch("vtxe", &vtxe);
    t->Branch("eff", &eff);
    t->Branch("pur", &pur);
    t->Branch("pid", &pid);

    t->Branch("evt", &event);
    t->Branch("run", &run);
    t->Branch("srun", &srun);
    t->Branch("prg", &prg);
  }

  //......................................................................
  void ShowerLIDAna::analyze(const art::Event& evt)
  {
 
    fNuEEnergyAlg = new NuEEnergyAlg( fNuEEnergyAlgPSet );
    event= evt.event();
    run= evt.run();
    srun= evt.subRun();
	
    art::Handle<std::vector<rb::Cluster> > slices;
    evt.getByLabel(fSliceLabel, slices);
    art::ServiceHandle<cheat::BackTracker> bt;

    art::FindManyP<rb::Prong>         fmprg3D(slices, evt, art::InputTag(fProngLabel, fAssn3D));
    art::FindManyP<rb::Prong>         fmprg2D(slices, evt, art::InputTag(fProngLabel, fAssn2D));
    art::FindManyP<rb::Vertex>        fmvtx(slices, evt, fVertexLabel);
    art::FindMany<rb::Shower>         fmrecs(slices, evt, fShowerLabel);
    art::FindMany<jmshower::JMShower> fmjms(slices, evt, fJMLabel);

    for(int iSli = 0; iSli < (int)slices->size(); ++iSli){
      art::Ptr<rb::Cluster> slice(slices, iSli);
      if( slice->IsNoise())
	continue;
      
      const std::vector<const rb::Shower*>         allreclusts  = fmrecs.at( iSli);
      const std::vector<const jmshower::JMShower*> alljmshowers = fmjms.at( iSli);
      const std::vector<art::Ptr< rb::Vertex> >    allvertices  = fmvtx.at( iSli);

      art::PtrVector<rb::CellHit> allHits = slice->AllCells();
      std::set<int> primETrackIDsSet;

      std::vector< cheat::NeutrinoEffPur > nuEffPur =
      	bt->SliceToNeutrinoInteractions(slice, allHits);

      if (nuEffPur.size() > 0 && nuEffPur[0].purity>0.6){
      	const simb::MCNeutrino& mcNu = nuEffPur[0].neutrinoInt->GetNeutrino();
      	if ( abs(mcNu.Nu().PdgCode()) == 12 && mcNu.CCNC()==simb::kCC){

      	  int motherId;

      	  const std::vector< const sim::Particle * > particles = bt->HitsToParticle(allHits);
      	  for( int p = 0; p < (int)particles.size(); p++)
      	    if( abs(particles[p]->PdgCode()) == 11 &&
      		(particles[p]->Process() == "Primary" ||
      		 particles[p]->Process() == "primary")){
      	      motherId = particles[p]->TrackId();
      	      primETrackIDsSet.insert(motherId);
      	    }

      	  for( int p = 0; p < (int)particles.size(); p++){
      	    if( particles[p]->Mother() == motherId )
      	      primETrackIDsSet.insert( particles[p]->TrackId() );
      	  }// end loop over particles associated with this MCTruth
	}// nu pdf is 12 and iscc
      }

      int nrecs = allreclusts.size();
      std::vector<art::Ptr< rb::Prong> > fuzzy = fmprg3D.at(iSli);
      int nfuzz = fuzzy.size();
      art::FindOneP<rb::Shower> fmre(fuzzy, evt, fShowerLabel);
      art::FindOneP<jmshower::JMShower> fmjm(fuzzy, evt, fJMLabel);
      if( !fmre.isValid() && !fmjm.isValid() )
	continue;

      if( !fmre.isValid() || !fmjm.isValid() ){
       	std::cerr<<" fmre or fmjm is not valid \n";
       	abort();
      }


      for( int ip = 0; ip < nfuzz; ++ip){
	if( fuzzy[ip]->Is2D() )
	  continue;

       	art::Ptr< rb::Shower> reclust = fmre.at(ip);
	std::vector< art::Ptr< rb::Shower> > vec;
	vec.push_back(reclust);
	art::FindOneP<slid::ShowerLID> fmlid(vec, evt, fShowerLIDLabel);
	art::Ptr< slid::ShowerLID> lid = fmlid.at(ip);

	art::Ptr<jmshower::JMShower> jmshowerptr = fmjm.at(ip);
	if(!jmshowerptr)
	  continue;
	jmshower::JMShower jmshower = *(jmshowerptr);

	std::vector<rb::WeightedHit> wHits;
	ncells = jmshower.NCell();

	for(int i = 0; i < ncells; i++){
	  art::PtrVector<rb::CellHit>::iterator it;
	  it = std::find( allHits.begin(), allHits.end(), jmshower.Cell(i));
	  if( it == allHits.end() )
	    continue;
	  double weight = jmshower.HitDeconvE(i)/jmshower.HitSumE(i);
	  rb::WeightedHit newHit( jmshower.Cell(i), weight );
	  jmshower.SetWeight( i, weight);
	  wHits.push_back(newHit);
	}
	

	int reidx;
	for( int ire = 0; ire < nrecs; ++ire){
	  if( allreclusts[ire]->Dir() == reclust->Dir())
	    reidx = ire;
	}
	
	hdncells = (reclust->NCell()) - (jmshower.NCell());
	hdncellx = (reclust->NXCell()) - (jmshower.NXCell());
	hdncelly = (reclust->NYCell()) - (jmshower.NYCell());
	hdtotpe  = (reclust->TotalPE()) - (jmshower.TotalPE());
	hdtotwt  = (reclust->TotalWeight()) - (jmshower.TotalWeight());
	hdtotgev = (reclust->TotalGeV()) - (jmshower.TotalGeV());
	hdmaxx   = (reclust->MaxX()) - (jmshower.MaxX());
	hdmaxy   = (reclust->MaxY()) - (jmshower.MaxY());
	hdmaxz   = (reclust->MaxZ()) - (jmshower.MaxZ());
	hdminx   = (reclust->MinX()) - (jmshower.MinX());
	hdminy   = (reclust->MinY()) - (jmshower.MinY());
	hdminz   = (reclust->MinZ()) - (jmshower.MinZ());
	hdmeanx  = (reclust->MeanX()) - (jmshower.MeanX());
	hdmeany  = (reclust->MeanY()) - (jmshower.MeanY());
	hdmeanz  = (reclust->MeanZ()) - (jmshower.MeanZ());
	hdextentx = (reclust->ExtentX()) - (jmshower.ExtentX());
	hdextenty = (reclust->ExtentY()) - (jmshower.ExtentY());
	hdextentz = (reclust->ExtentZ()) - (jmshower.ExtentZ());
	hdpid     = (lid->Value() )  - (jmshower.ANN(0));

	ncells = (jmshower.NCell());
	ncellx = (jmshower.NXCell());
	ncelly = (jmshower.NYCell());
	totpe  = (jmshower.TotalPE());
	totwt  = (jmshower.TotalWeight());
	totgev = (jmshower.TotalGeV());
	maxx   = (jmshower.MaxX());
	maxy   = (jmshower.MaxY());
	maxz   = (jmshower.MaxZ());
	minx   = (jmshower.MinX());
	miny   = (jmshower.MinY());
	minz   = (jmshower.MinZ());
	meanx  = (jmshower.MeanX());
	meany  = (jmshower.MeanY());
	meanz  = (jmshower.MeanZ());
	extentx = (jmshower.ExtentX());
	extenty = (jmshower.ExtentY());
	extentz = (jmshower.ExtentZ());
	pid     = (jmshower.ANN(0));

	std::cout<<"jmcells and recells "<<ncells<<" "<<reclust->NCell()<<"\n";
	
	prg = ip;
	
	bool energyEff = true;

	eff = bt->HitCollectionEfficiency( primETrackIDsSet, wHits, 
					   allHits,  geo::kXorY, NULL, energyEff);
	pur = bt->HitCollectionPurity( primETrackIDsSet, wHits, NULL, NULL, energyEff );
	float reeff = bt->HitCollectionEfficiency( primETrackIDsSet, reclust->WeightedHits(), 
						   allHits,  geo::kXorY, NULL, energyEff);
	float repur = bt->HitCollectionPurity( primETrackIDsSet, 
					       reclust->WeightedHits(), NULL, NULL, energyEff );
	

	hdeff = reeff - eff;
	hdpur = repur - pur;
	
	double reshe = fNuEEnergyAlg->ShowerEnergy( reclust.get(), allreclusts, evt);
	she   = jmshower.Energy();
	hdshe  = reshe - she;

	double reshdepe = fNuEEnergyAlg->ShowerDepEnergy( reclust.get(), allreclusts, evt);
	double shdepe   = jmshower.DepositEnergy();
	hdshdepe  = reshdepe - shdepe;

	double rehaddepe = fNuEEnergyAlg->HadronicDepEnergy( allreclusts, slice.get(), reidx, evt);
	haddepe = jmshower.ExclEnergy();
	hdhaddepe = rehaddepe - haddepe;

	double renuee = fNuEEnergyAlg->NuEEnergy( allreclusts, slice.get(), reidx, evt);
	double nuee = jmshower.NueEnergy();
	hdnuee = renuee - nuee;

	std::cout<<"slice number "<<iSli<<std::endl;

	double revtxe=0;
      if( allvertices.size() > 0)
	revtxe = fNuEEnergyAlg->VertexEnergy( reclust.get(), allreclusts, allvertices[0].get(), slice.get(), evt);
      vtxe = jmshower.VtxGeV();
      hdvtxe = revtxe - vtxe;

	
	std::cout<<"vertex   e : jm and recluster "<<vtxe<<" "<<revtxe<<std::endl;
	std::cout<<"shower   e : jm and recluster "<<she<<" "<<reshe<<std::endl;
	std::cout<<"haddep   e : jm and recluster "<<haddepe<<" "<<rehaddepe<<std::endl;
	std::cout<<"neutrino e : jm and recluster "<<nuee<<" "<<renuee<<std::endl;
	std::cout<<"pid        : jm and recluster "<<pid<<" "<<lid->Value()<<std::endl;
	t->Fill();	
	
      }//  End loop over prongs
      
    }// end loop over slices

  }// end analyze

} // end namespace tut

////////////////////////////////////////////////////////////////////////
namespace slid
{
  DEFINE_ART_MODULE(ShowerLIDAna);
}


