////////////////////////////////////////////////////////////////////////
// \file    SRBeam.cxx
// \brief   This class contains information about the neutrino production
// \version $Id: SRBeam.cxx,v 1.4 2012-12-07 06:37:29 gsdavies Exp $  
// \author  $Author: gsdavies $
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////

#include "StandardRecord/SRBeam.h"

#include <limits>

namespace
{
  const float kUninit = std::numeric_limits<float>::signaling_NaN();
}

namespace caf
{
  SRBeam::SRBeam() :
    tv(kUninit, kUninit, kUninit),
    tp(kUninit, kUninit, kUninit),
    runjob(0),
    potnum(0),
    tptype(0),
    nimpwt(kUninit),
    ndecay(0),
    v(kUninit, kUninit, kUninit),
    pdp(kUninit, kUninit, kUninit),
    ppdxdz(kUninit),
    ppdydz(kUninit),
    pppz(kUninit),
    ppenergy(kUninit),
    ppmedium(0),
    ptype(0),
    ppv(kUninit, kUninit, kUninit)
  {
  }

  SRBeam::~SRBeam() {}

  void SRBeam::setDefault()
  {
    tv       = TVector3(-5, -5, -5);
    tp       = TVector3(-5, -5, -5);
    tptype   = -5;
    nimpwt   = -5;
    ndecay   = -5;
    v        = TVector3(-5, -5, -5);
    pdp      = TVector3(-5, -5, -5);
    ppdxdz   = -5;
    ppdydz   = -5;
    pppz     = -5;
    ppenergy = -5;
    ppmedium = -5;
    ptype    = -5;
    ppv      = TVector3(-5, -5, -5);
  }
}
