////////////////////////////////////////////////////////////////////////
// \file    SRContain.h
// \brief   Class to store containment related variables
// \author  $Author: bays $
// \date    $Date: 2014-09-22 14:11:00 $
////////////////////////////////////////////////////////////////////////
#ifndef SRCONTAIN_H
#define SRCONTAIN_H

namespace caf
{
  /// This is a container for containment variables
  class SRContain
    {
    public:
      ///  Default constructor.
      SRContain();

      /// Default destuctor.
      virtual ~SRContain();

      bool numucontain;        ///< is this contained by Numu Standards? (can use as general containment)
      bool nuecontain;         ///< is this contained by Nue Standards? (NOT general)

      float missE;             ///< sum of energy of particles that leave detector. example cut: missE/trueE > 0.01 is truly uncontained

      float cosfwddist;        ///< cosmictrack projected distance (cm) from end point forwards to det edge
      int   cosfwdcell;        ///< cosmictrack projected # cells from end point forwards to det edge
      float cosfwdair;         ///< for cosmictrack projected distance forwards how much is through air (ND only, NYI)
      float cosfwdsteel;       ///< for cosmictrack projected distance forwards, how much is through steel (ND only, currently is just distance in muon catcher, cells and all)
      float cosbakdist;        ///< cosmictrack projected distance (cm) from start point backwards to det edge
      int   cosbakcell;        ///< cosmictrack projected # cells from start point backwards to det edge
      float cosbakair;         ///< for cosmictrack projected distance backwards how much is through air (ND only, NYI)
      float cosbaksteel;       ///< for cosmictrack projected distance backwards, how much is through steel (ND only, currently is just distance in muon catcher, cells and all)
      float kalfwddist;        ///< Kalmantrack projected distance (cm) from end point forwards to det edge
      int   kalfwdcell;        ///< Kalmantrack projected # cells from end point forwards to det edge
      float kalfwdair;         ///< for Kalmantrack projected distance forwards how much is through air (ND only, NYI)
      float kalfwdsteel;       ///< for Kalmantrack projected distance forwards, how much is through steel (ND only, currently is just distance in muon catcher, cells and all)
      float kalbakdist;        ///< Kalmantrack projected distance (cm) from start point backwards to det edge
      int   kalbakcell;        ///< Kalmantrack projected # cells from start point backwards to det edge
      float kalbakair;         ///< for Kalmantrack projected distance backwards how much is through air (ND only, NYI)
      float kalbaksteel;       ///< for Kalmantrack projected distance backwards, how much is through steel (ND only, currently is just distance in muon catcher, cells and all)
      float cosyposattrans;    ///< Y position of cosmic track at transition (ND only, use to check if went through air gap)
      float kalyposattrans;    ///< Y position of Kalman track and transition (ND only, use to check if went through air gap)

      float xmin2;             ///< x position of hit second closest to edge in -X dimension (slc.boxmin.fX gives first closest)
      float ymin2;             ///< y position of hit second closest to edge in -Y dimension (slc.boxmin.fY gives first closest)
      float zmin2;             ///< Z position of hit second closest to edge in -Z dimension (slc.boxmin.fZ gives first closest)
      float xmax2;             ///< x position of hit second closest to edge in +X dimension (slc.boxmax.fX gives first closest)
      float ymax2;             ///< y position of hit second closest to edge in +Y dimension (slc.boxmax.fY gives first closest)
      float zmax2;             ///< z position of hit second closest to edge in +Z dimension (slc.boxmax.fZ gives first closest)

      float xmint;             ///< x position of hit closest to edge in -X dimension that has > 100 photoelectons (TODO: use pecorr)
      float ymint;             ///< y position of hit closest to edge in -Y dimension that has > 100 photoelectons (TODO: use pecorr)
      float zmint;             ///< z position of hit closest to edge in -Z dimension that has > 100 photoelectons (TODO: use pecorr)
      float xmaxt;             ///< x position of hit closest to edge in +X dimension that has > 100 photoelectons (TODO: use pecorr)
      float ymaxt;             ///< y position of hit closest to edge in +Y dimension that has > 100 photoelectons (TODO: use pecorr)
      float zmaxt;             ///< z position of hit closest to edge in +Z dimension that has > 100 photoelectons (TODO: use pecorr)

      virtual void setDefault();

    };

} // end namespace

#endif // SRCONTAIN_H
//////////////////////////////////////////////////////////////////////////////
