

#ifndef SRNUECOSREJ_H
#define SRNUECOSREJ_H


namespace caf
{
    /// Class for storing output from Cosmic Rejection (Nuecosrej) module
    class SRNueCosRej{
    public:
      ///  Default constructor
      SRNueCosRej();

      /// Default destuctor
      ~SRNueCosRej();
      void setDefault();

      float hitplane;     ///< Number of slice hits per shower planes. See \ref cosrej::NueCosRej::HitsPerPlane
      float pngptp;       ///< Event transverse momentum fraction, based on prongs. See \ref cosrej::NueCosRej::ProngTransMom
      float partptp;      ///< Event transverse momentum fraction, based on showers. See \ref cosrej::NueCosRej::ParticleShowerTransMom
      float photptp;      ///< Event transverse momentum fraction, based on showers. See \ref cosrej::NueCosRej::PhotonShowerTransMom
      float starttop;     ///< Leading shower start to detector top. See \ref cosrej::NueCosRej::StartDistToTop
      float startbottom;  ///< Leading shower start to detector bottom. See \ref cosrej::NueCosRej::StartDistToBottom
      float startfront;   ///< Leading shower start to detector front. See \ref cosrej::NueCosRej::StartDistToFront
      float startback;    ///< Leading shower start to detector back. See \ref cosrej::NueCosRej::StartDistToBack
      float startwest;    ///< Leading shower start to detector west. See \ref cosrej::NueCosRej::StartDistToWest
      float starteast;    ///< Leading shower start to detector east. See \ref cosrej::NueCosRej::StartDistToEast
      float stoptop;      ///< Leading shower start to detector top. See \ref cosrej::NueCosRej::StartDistToTop
      float stopbottom;   ///< Leading shower start to detector bottom. See \ref cosrej::NueCosRej::StartDistToBottom
      float stopfront;    ///< Leading shower start to detector front. See \ref cosrej::NueCosRej::StartDistToFront
      float stopback;     ///< Leading shower start to detector back. See \ref cosrej::NueCosRej::StartDistToBack
      float stopwest;     ///< Leading shower start to detector west. See \ref cosrej::NueCosRej::StartDistToWest
      float stopeast;     ///< Leading shower start to detector east. See \ref cosrej::NueCosRej::StartDistToEast
      float dang;         ///< Angle between first and second leading showers. See \ref cosrej::NueCosRej::AngleToNextShower
      float vtxdoca;      ///< Distance of closest approach of leading shower to vertex. See \ref cosrej::NueCosRej::ShowerDistToVtx
      float showermaxx;   ///< Leading shower max X between start and stop. See \ref cosrej::NueCosRej::ShowerMaxX
      float showermaxy;   ///< Leading shower max X between start and stop. See \ref cosrej::NueCosRej::ShowerMaxY
      float showermaxz;   ///< Leading shower max X between start and stop. See \ref cosrej::NueCosRej::ShowerMaxZ
    };

} // end namespace

#endif // SRNUECOSREJ_H
//////////////////////////////////////////////////////////////////////////////
