#include "RandHisto.h"

#include "TMath.h"

RandHisto::RandHisto(CLHEP::HepRandomEngine& engine) : fFlat(engine)
{}

void RandHisto::GetRandom(TH1* histo, double& x, double& y, double& z)
{
  int dim = histo->GetDimension();
  int nbins = histo->GetNbinsX();
  if (dim > 1) nbins *= histo->GetNbinsY();
  if (dim > 2) nbins *= histo->GetNbinsZ();
  
  int ibinX(0), ibinY(0), ibinZ(0);
  int ibinG = TMath::BinarySearch(nbins, histo->GetIntegral(), fFlat.fire());
  histo->GetBinXYZ(ibinG, ibinX, ibinY, ibinZ);

  //x = histo->GetXaxis()->GetBinCenter(ibinX);
  double xCenter = histo->GetXaxis()->GetBinCenter(ibinX);
  double xWidth = histo->GetXaxis()->GetBinWidth(ibinX);
  x = (xCenter-0.5*xWidth) + xWidth*fFlat.fire();

  if (dim > 1)
    {
      //y = histo->GetYaxis()->GetBinCenter(ibinY);
      double yCenter = histo->GetYaxis()->GetBinCenter(ibinY);
      double yWidth = histo->GetYaxis()->GetBinWidth(ibinY);
      y = (yCenter-0.5*yWidth) + yWidth*fFlat.fire();
    }

  if (dim > 2)
    {
      //z = histo->GetZaxis()->GetBinCenter(ibinZ);
      double zCenter = histo->GetZaxis()->GetBinCenter(ibinZ);
      double zWidth = histo->GetZaxis()->GetBinWidth(ibinZ);
      z = (zCenter-0.5*zWidth) + zWidth*fFlat.fire();
    }

  return;
}



