// \file ShowerAna.h
//
// Shower Analysis 
// 
//
// \author $Author: Jianming Bian $
//
////////////////////////////////////////////////////////////////////////
#include <string>
#include "art/Framework/Core/EDAnalyzer.h"	
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h" 
#include "art/Framework/Principal/Handle.h" 
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "TVector3.h"
#include "TGraph2DErrors.h"

#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "RawData/DAQHeader.h"

#include <cmath>
#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>

#include "TFile.h"
#include "TMath.h"
#include "TH1D.h"
#include "TNtuple.h"
#include "TFitter.h"
#include "TMinuit.h"
#include "TVector3.h"
#include "TGraph2DErrors.h"

#include "RecoBase/FilterList.h"
#include "RecoBase/Track.h"
#include "RecoBase/Vertex.h"
#include "RecoBase/RecoHit.h"
#include "RecoBase/CellHit.h"
#include "SimulationBase/MCFlux.h"
#include "SimulationBase/MCNeutrino.h"
#include "SimulationBase/MCTruth.h"
#include "SimulationBase/MCTrajectory.h"
#include "SimulationBase/MCParticle.h"
#include "SimulationBase/GTruth.h"
//#include "SimulationBase/simbase.h"
#include "Simulation/FLSHit.h"
//#include "Simulation/FLSHitList.h"
//#include "Simulation/PhotonSignal.h"
#include "SimulationBase/MCTruth.h"
#include "Calibrator/Calibrator.h"
#include "RecoJMShower/JMShower.h"
#include "MCCheater/BackTracker.h"

// ART includes
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Core/FindManyP.h"


#include "fhiclcpp/ParameterSet.h"  
#include "messagefacility/MessageLogger/MessageLogger.h" 

#include "OscLib/services/OscCalcService.h"
#include "OscLib/services/OscCalcGenService.h"

#include "SummaryData/POTSum.h"
#include "TRandom.h"
#include "TMatrixD.h"

#include "Utilities/func/MathUtil.h"
#include "CalibrationUtils/CalibUtil.h"

class TH1D;
class TNtuple;
class TTree;
class TGraph2DErrors;

namespace showerana {

  class PCluster : public rb::Cluster
  {
  public:
    PCluster();
    virtual ~PCluster();
    void setPlane(unsigned ip) {m_p = ip;}
    unsigned int  Plane() {return m_p;}

  private:
    unsigned int m_p;//Plane 
  };


  class ShowerAna : public art::EDAnalyzer
    {
    public:
      explicit ShowerAna(fhicl::ParameterSet const& pset); 
      ~ShowerAna();                        
      void analyze(const art::Event& evt);
      void beginJob();
      void beginSubRun(const art::SubRun& sr);
      void reconfigure(const fhicl::ParameterSet& pset);

      typedef std::vector<TLorentzVector> Vp4;
      typedef std::vector<int> Vint;
      typedef std::vector<double> Vdouble;

      int  ededxplane[200];
      double  ededxmean[200];
      double  ededxrms[200];

      int  gdedxplane[200];
      double  gdedxmean[200];
      double  gdedxrms[200];

      int  mudedxplane[200];
      double  mudedxmean[200];
      double  mudedxrms[200];

      int  pi0dedxplane[200];
      double  pi0dedxmean[200];
      double  pi0dedxrms[200];

      int  eradiusplane[200];
      double  eradiusmean[200];
      double  eradiusrms[200];

      int  gradiusplane[200];
      double  gradiusmean[200];
      double  gradiusrms[200];

      int  muradiusplane[200];
      double  muradiusmean[200];
      double  muradiusrms[200];

      int  pi0radiusplane[200];
      double  pi0radiusmean[200];      
      double  pi0radiusrms[200];

      TGraph2DErrors *dtE[200];
      TGraph2DErrors *dtG[200];
      TGraph2DErrors *dtMu[200];
      TGraph2DErrors *dtPi0[200];

      TH1D *ht_nue_rw;
      TH1D *ht_bkg_nue_rw;
      TH1D *ht_bkg_numu_rw;

//McTruth
    std::vector<int> getFaveIds(std::vector<cheat::NeutrinoWithIndex>& allNus,  
                                std::vector< std::vector<cheat::NeutrinoEffPur> >& sliceEffPurs);

//Geometry
      TVector3 GetTrkHitPos(unsigned int plane, unsigned int cell, art::Ptr<rb::Prong> trk);
      TVector3 GetTrkHitPos(unsigned int plane, unsigned int cell, TVector3 trkdir, TVector3 trkstart, TVector3 trkstop);
      TVector3 GetTrkPlanePos(unsigned int plane, rb::Prong trk);
      unsigned int  GetTrkPlaneCell(unsigned int plane, art::Ptr<rb::Prong> trk);
      unsigned int  GetTrkPlaneCell(unsigned int plane, rb::Prong trk);


      double GetPointDoca(TVector3 vtx, TVector3 start, TVector3 stop);
      TVector3 GetTrkPlaneDistToEdge(unsigned int plane, art::Ptr<rb::Prong> trk);
      TVector3 GetTrkPlaneDistToEdge(unsigned int plane, rb::Prong trk);
      bool IsFiducial(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double GetTrkHitPath(unsigned int plane, unsigned int cell, rb::Prong shower);

//Basic variables
      double DepositEnergy(rb::Cluster cluster, int i,  std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double Energy(rb::Cluster cluster, int i,  std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double ECorrMC(double gev);
      double Radius(rb::Prong shower, int i,  std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double Radius(rb::Cluster cluster, rb::Prong shower, int i,  std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double Gap(rb::Prong shower, TVector3 vertex);


//Dedx
      double GetPlaneDedx(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, unsigned int plane);
      double DedxChisq(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, int type);
      double GetExpPlaneDedx(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, unsigned int plane,  int type); 
      double GetExpPlaneDedxRMS(rb::Prong shower, unsigned int plane, int type);

      double GetPlaneDedxProb(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, int igev, unsigned int plane, double dedx, int type);
      double GetInterPlaneDedxProb(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, unsigned int plane, double dedx, int type);
      double GetDedxLongLL(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, int type);

      double GetCellTransDedxProb(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, int igev, unsigned int cell, double dedx, int type);
      double GetInterCellTransDedxProb(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, unsigned int cell, double dedx, int type);
      double GetDedxTransLL(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, int type);

      unsigned int NMIPPlane(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap);
      double PrePlaneDedx(rb::Prong shower, int i, std::map<geo::OfflineChan, Vdouble> hiteweightmap, unsigned int plane);
//Cell-to-cell nodes
      TVector3 GetCellNodePos(unsigned int plane1, unsigned int cell1, unsigned int plane2, unsigned int cell2);

      double GetCellDistToTrk(unsigned int plane, unsigned int cell, TVector3 start, TVector3 stop);

      std::string pSliceDir;
      std::string pTrackDir;
      std::string pInstLabel;
      std::string pVertexLabel;
      double pCellHitPHThresh;
      static const int GAMMAID;
      static const int PIZEROID;
      static const int PIPLUSID;
      static const int PIMINUSID;
      static const int NUEID;
      static const int NUMUID;
      static const int NUTAUID;

    private:
      TH1D*         fHistoCosTheta;
      TH1D*         fHistoInvMass;
      TH1D*         fHistoDist;
      TH1D*         fHistoTracks;
      TH1D*         fHistoXCoord;
      TH1D*         fHistoYCoord;
      TH1D*         fHistoZCoord;
//      TNtuple*      fShower;
//      TNtuple*      fXCell;
//      TNtuple*      fYCell;
      TNtuple*      fXPlane;
      TNtuple*      fYPlane;

      TTree*      fEvent;
      int    m_evtTrueNuCCNC;
      int    m_evtTrueNuMode;
      int    m_evtTrueNuPdg;
      double m_evtTrueNuEnergy;
      int    m_evtTrueNuIsFid;
      double m_evtTrueNuP4[4];
      double m_evtTrueNuVtx[3];

      double m_evtTruePF;
      double m_evtTruePNu;
      double m_evtTruePHad;
      double m_evtTrueCosTheta;
      double m_evtTruePt;
      double m_evtTruePl;


      int m_evtOsTag;
      double m_evtOsProb[20];
      double m_evtOsRand;
      double m_evtRwProb[20];
      double m_evtRwRand[20];
      int    m_evtTruePdg[20];
      int    m_evtTrueMother[20];
      double m_evtTrueEnergy[20];
      int    m_evtTrueNTrk;
      double m_evtTrueVtx[3];
      double m_evtTrueDir[3];
      double m_evtTrueElecEnergy[20];
      double m_evtTrueXElecEnergy[20];
      double m_evtTrueYElecEnergy[20];
      int m_evtTrueElecNXHit[20];
      int m_evtTrueElecNYHit[20];


      int  m_evtRun;
      int  m_evtSRun;
      int  m_evtEvent;
      int  m_evtSubEvent;
      bool  m_evtFTag;
      bool  m_evtIsNoise;
      int m_evtNSlice;
      int m_evtNSigSlice;
      int m_evtNTrk;
      double m_evtVtx[3];
      int m_evtNSh;
      double m_evtANN;
      double m_evtTotalRecoGeV;
      double m_evtTotalRecoNCell;
      double m_evtPEtot;
      double m_evtTotalRecoGeV0;
      double m_evtTotalRecoNCell0;
      double m_evtPEtot0;


      double m_evtPi0Mgg0;
      double m_evtPi0Mgg;
      double m_evtPi0Eg1;
      double m_evtPi0Eg2;
      double m_evtPi0StartDist;
      double m_evtPi0LineDist;
      double m_evtPi0Vtx[3];


      double m_evtPi0TrueDirMgg0;
      double m_evtPi0TrueDirMgg;

      double m_evtEtot0;
      double m_evtEtot;
      double m_evtTrueEtot;
      double m_evtTrueElecPhotons;


      double m_evtSh1Energy; 
      double m_evtSh1TNS; 
      double m_evtSh1Energy0; 
      double m_evtSh1Length; 
      double m_evtSh1TrkGeV; 
      double m_evtSh1SliceGeV; 
      double m_evtSh1SliceEdge10GeV; 
      double m_evtSh1SliceEdge5GeV; 
      double m_evtSh1SliceEdge2GeV; 
      double m_evtSh1Gap; 
      int m_evtSh1NTrk; 
      double m_evtSh1IsFid; 
      double m_evtSh1DistToEdge; 
      double m_evtSh1P4[4];

      double m_evtSh1Dir[3];
      double m_evtSh1Start[3];
      double m_evtSh1Stop[3];

      double m_evtSh1MinPlane;
      double m_evtSh1MaxPlane;

      double m_evtSh1DedxLLL[20];
      double m_evtSh1DedxLLT[20];
      double m_evtSh1DedxANN;
      double m_evtSh1Moment12;
      double m_evtSh1Asym;
      double m_evtSh1Ixx;
      double m_evtSh1Iyy;
      double m_evtSh1DedxANN1;
      double m_evtSh1NPlane;
      double m_evtSh1NZeroPlane;
      double m_evtSh1ExtentX;
      double m_evtSh1ExtentY; 
      double m_evtSh1NXCell; 
      double m_evtSh1NYCell; 

      double m_evtSh1NMIPPlane;
      double m_evtSh1VtxGeV;
      double m_evtSh1Pi0Mgg;

      double m_evtSh1ContStartPlane;

      double m_evtSh1TrueDang;
      double m_evtSh1TrueEDang;
      int  m_evtSh1TruePdg;
      double  m_evtSh1TrueT;
      int m_evtSh1TrueId;
      double m_evtSh1TrueVtx[3];
      double m_evtSh1TrueP4[4];

      int m_evtSh1TrueMotherPdg;
      int m_evtSh1TrueMotherId;
      double m_evtSh1TrueMotherP4[4];

      int m_evtSh1TrueNavMotherPdg;
      int m_evtSh1TrueNavMotherId;
      double m_evtSh1TrueNavMotherP4[4];


      double m_evtSh1TrueElecPhotons;
      double m_evtSh1TrueTotPhotons;
      double m_evtSh1TrueTrkElecPhotons;
      double m_evtSh1TrueTrkTotPhotons;
      double m_evtSh1TrueRWElecPhotons;
      double m_evtSh1TrueRWTotPhotons;
      double m_evtSh1TrueTrkRWElecPhotons;
      double m_evtSh1TrueTrkRWTotPhotons;




      double m_evtSh1Ee;
      double m_evtSh1CosTheta;
      double m_evtSh1Mh;
      double m_evtSh1MhTrue;
      double m_evtSh1Meff;
      double m_evtSh1TrueSumHadM;
      double m_evtSh1TrueSumHadNtrk;
      double m_evtSh1Enue;
      double m_evtSh1TruePt;
      double m_evtSh1TruePl;
      double m_evtSh1TrueSumHadPt;
      double m_evtSh1TrueSumHadPl;
      double m_evtSh1TrueSumHadEnue;
      double m_evtSh1CosTrueTheta;
      double m_evtSh1CosTrueHadTheta;
      double m_evtSh1TrueHadEnue; 
      double m_evtSh1TrueEnue;

      double m_evtSh1SumHadP;
      double m_evtSh1SumPt;
      double m_evtSh1SumP;
      double m_evtSh1P;


      double m_evtSh2SliceGeV;
      double m_evtSh2Gap;
      double m_evtSh2Pi0Mgg;
      double m_evtSh2Energy; 
      double m_evtSh2TNS; 
      double m_evtSh2TrkGeV; 
      double m_evtSh2VtxGeV;
      double m_evtSh2IsFid; 
      double m_evtSh2P4[4];
      double m_evtSh2Dir[3];
      double m_evtSh2Start[3];
      double m_evtSh2Stop[3];
      double m_evtSh2DedxLLL[20];
      double m_evtSh2DedxLLT[20];
      double m_evtSh2DedxANN;
      double m_evtSh2DedxANN1;
      double m_evtSh2NPlane;
      double m_evtSh2NMIPPlane;
      double m_evtSh2ContStartPlane;

      double m_evtSh2TrueDang;
      int m_evtSh2TruePdg;
      double m_evtSh2TrueT;
      int m_evtSh2TrueId;
      double m_evtSh2TrueVtx[3];
      double m_evtSh2TrueP4[4];

      int m_evtSh2TrueMotherPdg;
      int m_evtSh2TrueMotherId;
      double m_evtSh2TrueMotherP4[4];



      int m_evtSh2TrueNavMotherPdg;
      int m_evtSh2TrueNavMotherId;
      double m_evtSh2TrueNavMotherP4[4];

      double m_evtSh3SliceGeV;
      double m_evtSh3Gap;
      double m_evtSh3Pi0Mgg;
      double m_evtSh3Energy; 
      double m_evtSh3TNS; 
      double m_evtSh3TrkGeV; 
      double m_evtSh3VtxGeV;
      double m_evtSh3IsFid; 
      double m_evtSh3P4[4];
      double m_evtSh3Dir[3];
      double m_evtSh3Start[3];
      double m_evtSh3Stop[3];
      double m_evtSh3DedxLLL[20];
      double m_evtSh3DedxLLT[20];
      double m_evtSh3DedxANN;
      double m_evtSh3DedxANN1;
      double m_evtSh3NPlane;
      double m_evtSh3NMIPPlane;
      double m_evtSh3ContStartPlane;

      double m_evtSh3TrueDang;
      int m_evtSh3TruePdg;
      double m_evtSh3TrueT;
      int m_evtSh3TrueId;
      double m_evtSh3TrueVtx[3];
      double m_evtSh3TrueP4[4];

      int m_evtSh3TrueMotherPdg;
      int m_evtSh3TrueMotherId;
      double m_evtSh3TrueMotherP4[4];


      double m_evtLen1DedxANN; 
      double m_evtLen1DedxANN1; 
      double m_evtLen1Energy; 
      double m_evtLen1TNS; 
      double m_evtLen1TrkGeV; 
      double m_evtLen1IsFid; 
      double m_evtLen1P4[4];
      double m_evtLen1Start[3];
      double m_evtLen1Stop[3];
      double m_evtLen1DedxLLL[20];
      double m_evtLen1DedxLLT[20];
      double m_evtLen1Energy0; 
      double m_evtLen1VtxGeV;
      double m_evtLen1Pi0Mgg;
      double m_evtLen1SliceGeV;
      double m_evtLen1Gap;

      double m_evtLen1NPlane;
      double m_evtLen1NMIPPlane;
      double m_evtLen1ContStartPlane;


      double m_evtLen1TrueDang;
      double m_evtLen1TrueEDang;
      int m_evtLen1TrueId;
      int m_evtLen1TruePdg;
      double m_evtLen1TrueT;
      double m_evtLen1TrueVtx[3];
      double m_evtLen1TrueP4[4];

      int m_evtLen1TrueMotherPdg;
      int m_evtLen1TrueMotherId;
      double m_evtLen1TrueMotherP4[4];
      double m_evtLen1TrueElecPhotons;
      double m_evtLen1TrueTotPhotons;
      double m_evtLen1TrueRWElecPhotons;
      double m_evtLen1TrueRWTotPhotons;



      double m_evtSheEnergy; 
      double m_evtSheTNS; 
      double m_evtSheTrkGeV; 
      double m_evtSheGap; 
      double m_evtSheVtxGeV;
      double m_evtSheIsFid; 
      double m_evtSheP4[4];
      double m_evtSheDir[3];
      double m_evtSheStart[3];
      double m_evtSheStop[3];
      double m_evtSheDedxLLL[20];
      double m_evtSheDedxLLT[20];
      double m_evtSheDedxANN;
      double m_evtSheDedxANN1;
      double m_evtSheNPlane;
      double m_evtSheNMIPPlane;
      double m_evtSheContStartPlane;

      double m_evtSheTrueDang;
      double m_evtSheTrueEDang;
      int m_evtSheTruePdg;
      double m_evtSheTrueT;
      int m_evtSheTrueId;
      double m_evtSheTrueVtx[3];
      double m_evtSheTrueP4[4];

      double m_evtSheTrueElecPhotons;
      double m_evtSheTrueTotPhotons;
      double m_evtSheTrueTrkElecPhotons;
      double m_evtSheTrueTrkTotPhotons;

      double m_evtSheTrueRWElecPhotons;
      double m_evtSheTrueRWTotPhotons;
      double m_evtSheTrueTrkRWElecPhotons;
      double m_evtSheTrueTrkRWTotPhotons;


      double m_evtShExclPE; 
      double m_evtShExclRadius; 
      double m_evtShExclCentroid[3]; 


      double m_evtNXTrk; 
      double m_evtNYTrk;
      double m_evtXSh1GeV; 
      double m_evtYSh1GeV;
      double m_evtXSh1Plane;
      double m_evtYSh1Plane;

/*
      double m_evtXSheEnergy;
      double m_evtXSheDang; 
      double m_evtXSheNCell; 
      double m_evtXSheNPlane; 
      double m_evtXSheMinPlane; 
      double m_evtXSheMaxPlane; 
      double m_evtXSheDir[3];
      double m_evtYSheEnergy; 
      double m_evtYSheDang;
      double m_evtYSheNCell;
      double m_evtYSheNPlane;
      double m_evtYSheMinPlane;
      double m_evtYSheMaxPlane;
      double m_evtYSheDir[3];
*/


      double m_evtNXCluster;
      double m_evtNYCluster;
      double m_evtXClusterGeV;
      double m_evtYClusterGeV;
      double m_evtXClusterMaxNCell;
      double m_evtYClusterMaxNCell;
      double m_evtXClusterNCell;
      double m_evtYClusterNCell;


 
      TTree* fCell;
      double m_cellRun;
      double m_cellSubRun;
      double m_cellEvent;
      int m_cellSubEvent;
      double m_cellPlaneId;
      double m_cellView;
      double m_cellId;
      double m_cellXYZ[3];
      double m_cellSlice;
      int    m_cellNTrk;
      int    m_cellNSlice;
      double m_cellADC;
      double m_cellTDC;
      double m_cellTNS;
      double m_cellPE;
      double m_cellPECorr;
      double m_cellEnergy;
      double m_cellEnergyM1;
      double m_cellEnergyM2;
      double m_cellEnergyP1;
      double m_cellEnergyP2;
      double m_cellTrueEnergy;
      double m_cellTrueEnergyM1;
      double m_cellTrueEnergyM2;
      double m_cellTrueEnergyP1;
      double m_cellTrueEnergyP2;
      double m_cellTrueTime;
      double m_cellTrueHitXYZ[3];
      double m_cellHitXYZ[3];
      double m_cellPath;

      double m_cellTrueNuCCNC;
      double m_cellTrueNuMode;
      int m_cellTrueNuPdg;
      double m_cellTrueNuEnergy;
      double m_cellTrueNuIsFid;
      double m_cellTrueNuP4[4];
      double m_cellTrueNuVtx[3];
      double m_cellTrueDang;
      int m_cellTruePdg;
      double m_cellTrueT;
      double m_cellTrueMother;
      double m_cellTrueVtx[3];
      double m_cellTrueP4[4];

      double m_cellTruePartElec;

      double m_cellSh1Dir[3];
      double m_cellSh2Dir[3];
      double m_cellSheDir[3];
      int m_cellSheNPlane;
      double m_cellSheNCell;
      double m_cellSheLength;

      double m_cellSheEnergy; 
      double m_cellSheTrkGeV; 
      double m_cellSheVtxGeV;
      double m_cellSheIsFid; 
      double m_cellSheP4[4];
      double m_cellSheStart[3];
      double m_cellSheStop[3];
      double m_cellSheDedxLLL[20];
      double m_cellSheDedxLLT[20];
      double m_cellSheDedxANN;
      double m_cellSheDedxANN1;
      double m_cellSheNMIPPlane;
      double m_cellSheContStartPlane;

      double m_cellSheTrueDang;
      double m_cellSheTrueEDang;
      int m_cellSheTruePdg;
      double m_cellSheTrueT;
      int m_cellSheTrueId;
      double m_cellSheTrueVtx[3];
      double m_cellSheTrueP4[4];

      double m_cellSheTrueElecPhotons;
      double m_cellSheTrueTotPhotons;
      double m_cellSheTrueTrkElecPhotons;
      double m_cellSheTrueTrkTotPhotons;

      double m_cellSheTrueRWElecPhotons;
      double m_cellSheTrueRWTotPhotons;
      double m_cellSheTrueTrkRWElecPhotons;
      double m_cellSheTrueTrkRWTotPhotons;

      double m_cellToSh1Dist;
      double m_cellToSh1Plane;
      double m_cellToSheDist;
      double m_cellToShePlane;
      double m_cellToSh2Dist;
      double m_cellToSh2Plane;


      TTree* fNode;

      double m_nodeRun;
      double m_nodeSubRun;
      double m_nodeEvent;
      int m_nodeSubEvent;
      double m_nodeShId;
      double m_nodeXYZ[3];
      int m_nodeShowerNPlane;
      double m_nodeShowerLength;
      double m_nodeShowerEnergy;
      double m_nodeShowerNCell;
      double m_nodeShowerDir[3];
      double m_nodeShowerStart[3];
      double m_nodeShowerStop[3];
      double m_nodeE;
      int    m_nodeTrueNuCCNC;
      int    m_nodeTrueNuMode;
      int    m_nodeTrueNuPdg;
      double m_nodeTrueNuEnergy;
      int    m_nodeTrueNuIsFid;
      double m_nodeTrueNuP4[4];
      double m_nodeTrueNuVtx[3];

      double m_nodeTrueDang;
      int m_nodeTruePdg;
      double  m_nodeTrueId;
      int  m_nodeTrueMotherPdg;
      double  m_nodeTrueNavMotherPdg;
      double  m_nodeTrueNavMotherId;
      double  m_nodeTrueNavMotherP4[4];

      double m_nodeTrueElecPhotons;
      double m_nodeTrueTotPhotons;



      TTree*        fShower;
      double m_showerRun;
      double m_showerSubRun;
      double m_showerEvent;
      int m_showerSubEvent;
      bool m_showerFTag;
      double m_showerId;


      int    m_showerTrueNuCCNC;
      int    m_showerTrueNuMode;
      int    m_showerTrueNuPdg;
      double m_showerTrueNuEnergy;
      int    m_showerTrueNuIsFid;
      double m_showerTrueNuP4[4];
      double m_showerTrueNuVtx[3];
      int    m_showerEvtTruePdg[20];
      double m_showerEvtTrueEnergy[20];
      double m_showerEvtTrueElecPhotons;


      double m_showerTrueDang;
      double m_showerTrueEDang;
      double m_showerTrueVtxEDang;
      int m_showerTruePdg;
      double m_showerTrueT;
      bool m_showerTrueEtag;
      int m_showerTrueId;
      double m_showerTrueVtx[3];
      double m_showerTrueP4[4];
      double m_showerTruePartHit[10];
      double m_showerTruePartEnergy[10];
      double m_showerTrueDepEnergy;
      double m_showerTrueElecPhotons;
      double m_showerTrueTotPhotons;
      double m_showerTrueTrkElecPhotons;
      double m_showerTrueTrkTotPhotons;

      double m_showerTrueRWElecPhotons;
      double m_showerTrueRWTotPhotons;
      double m_showerTrueTrkRWElecPhotons;
      double m_showerTrueTrkRWTotPhotons;

      double m_showerTrueG1Ratio;
      double m_showerTrueG2Ratio;
      double m_showerTrueG1Photons;
      double m_showerTrueG2Photons;
      double m_showerTrueRWG1Ratio;
      double m_showerTrueRWG2Ratio;
      double m_showerTrueRWG1Photons;
      double m_showerTrueRWG2Photons;

      double m_showerTruePi0Dang;
      double m_showerTruePi0P4[4];


      int m_showerTrueMotherPdg;
      int m_showerTrueMotherId;
      double m_showerTrueMotherP4[4];

      int m_showerTrueNavMotherPdg;
      int m_showerTrueNavMotherId;
      double m_showerTrueNavMotherP4[4];


      int m_showerNPlane;
      int m_showerNZeroPlane;
      int m_showerNTrk;
      double m_showerLength;
      double m_showerVtx[3];
      double m_showerVtxDoca;
      double m_showerVtxToStart;
      double m_showerVtxToStop;
      double m_showerVtxIntToStart;
      double m_showerVtxIntToStop;
      double m_showerVtxTrackLength;
      bool m_showerVtxBackTag;
      double m_showerVtxTrueDoca;
      double m_showerVtxTrueDang;
      double m_showerVtxStartTrueDoca;
      double m_showerVtxStopTrueDoca;


      double m_showerNXcell;
      double m_showerNYcell;
      double m_showerNMIPPlane;
      double m_showerContStartPlane;
      double m_showerDir[3];
      double m_showerStart[3];
      double m_showerStop[3];
      int m_showerIReg;
      int m_showerIE;
      double m_showerPlaneDedx[200];
      double m_showerPlaneDeltaDedx[200];
      double m_showerTransCellDedx[20];
      double m_showerTransCellProbE[20];
      double m_showerTransCellProbPi0[20];

/*
      double m_showerPlaneDedxprobE[200];
      double m_showerPlaneDedxprobG[200];
      double m_showerPlaneDedxprobMu[200];
      double m_showerPlaneDedxprobPi0[200];
      double m_showerPlaneDedxprobHad[200];
      double m_showerPlaneDedxprobProton[200];
      double m_showerPlaneDedxprobNeutron[200];
      double m_showerPlaneDedxprobPion[200];
      double m_showerPlaneDedxprobEqe[200];
      double m_showerPlaneDedxprobEres[200];
      double m_showerPlaneDedxprobEdis[200];
      double m_showerPlaneDedxprobEcoh[200];
*/

      double m_showerDedxChisq[20];
      double m_showerRadiusChisq[20];
      double m_showerDedxNdof[20];
      double m_showerDedxProb[20];
      double m_showerTransDedxProb[20];
      double m_showerDedxLLL[20];
      double m_showerDedxLLL1[20];
      double m_showerDedxLLL2[20];
      double m_showerDedxLLL3[20];
      double m_showerDedxLLL4[20];
      double m_showerDedxLLL5[20];
      double m_showerDedxLLL6[20];
      double m_showerDedxLLT[20];
      double m_showerDedxANN;
      double m_showerDedxANN1;
      double m_showerMoment12;
      double m_showerAsym;
      double m_showerIxx;
      double m_showerIyy;
      double m_showerPlaneRadius[200];

      double m_showerPlaneHitX[200];
      double m_showerPlaneHitY[200];
      double m_showerPlaneHitZ[200];

      int m_showerPlaneHitCell[200];
      int m_showerPlaneE1Cell[200];
      int m_showerPlaneE2Cell[200];
      int m_showerPlaneCentroidCell[200];
      double  m_showerPlaneHitCellGeV[200];
      double  m_showerPlaneE1CellGeV[200];
      double  m_showerPlaneE2CellGeV[200];
      double  m_showerPlaneCentroidCellGeV[200];

      int m_showerPlaneHitIReg[200];

      double m_showerPlaneEnergy[200];
      double m_showerTruePlaneEnergy[200];
      double m_showerTransCellEnergy[20];
      double m_showerTrueTransCellEnergy[20];


      double m_showerADC;
      double m_showerEnergy0;
      double m_showerEnergy;
      double m_showerTNS;
      double m_showerTrkGeV;
      double m_showerSliceGeV;
      double m_showerSliceEdge10GeV;
      double m_showerSliceEdge5GeV;
      double m_showerSliceEdge2GeV;
      int m_showerTotalRecoNCell;
      double m_showerEnergySvc;
      double m_showerNCell;
      double m_showerRadius;
      double m_showerVtxGeV;
      double m_showerGap;
      double m_showerIsFid;
      double m_showerDistToEdge;
      double m_showerRminmax;

      double m_showerEe;
      double m_showerCosTheta;
      double m_showerCosTrueTheta;
      double m_showerCosTrueHadTheta;
      double m_showerMh;
      double m_showerMhTrue;
      double m_showerMeff;
      double m_showerEnue;
      double m_showerTrueHadEnue;
      double m_showerTrueEnue;
      double m_showerTrueSumHadM;
      int m_showerTrueSumHadNtrk;
      double m_showerTrueSumHadEnue;
      double m_showerTruePt;
      double m_showerTruePl;
      double m_showerTrueSumHadP;
      double m_showerTrueSumHadPt;
      double m_showerTrueSumHadPl;

      double m_showerSumHadP;
      double m_showerSumHadPCor;
      double m_showerSumPt;
      double m_showerSumP;
      double m_showerP;


      double m_trackADC;
      double m_trackEnergy;
      double m_trackTNS;
      double m_trackNCell;
      double m_trackRadius;


      unsigned int  fMinTrack;
      unsigned int  fMaxTrack;
      double        fXThresh;
      double        fYThresh;
      double        fZMin;
      double        fZMax;
      std::string   fPotLabel;           ///< Module that produced the POTSum object


    };

}

namespace showerana{
  //______________________________________________________________________________

  const int ShowerAna::GAMMAID = 22;
  const int ShowerAna::PIZEROID = 111;
  const int ShowerAna::PIPLUSID = 211;
  const int ShowerAna::PIMINUSID = -211;
  const int ShowerAna::NUEID = 12;
  const int ShowerAna::NUMUID = 14;
  const int ShowerAna::NUTAUID = 16;
  ShowerAna::ShowerAna(fhicl::ParameterSet const& pset):
    EDAnalyzer(pset),
    pSliceDir("./Reco"),
    pTrackDir("./Reco"),
    pCellHitPHThresh(50.)
  {
   std::cout<<"ShowerAna::ShowerAna(fhicl::ParameterSet const& pset)"<<std::endl;
    reconfigure(pset);
  }


  PCluster::PCluster() : m_p(9999)
  {
  }

  PCluster::~PCluster()
  {
  }

 
  //......................................................................
  
  void ShowerAna::reconfigure(const fhicl::ParameterSet& pset)
  {
    pSliceDir       = pset.get< std::string >("SliceDir");
    pTrackDir       = pset.get< std::string >("TrackDir");
    pInstLabel       = pset.get< std::string >("InstLabel");
    pVertexLabel    = pset.get<std::string>("VertexLabel");
    pCellHitPHThresh = pset.get< double      >("CellHitPHThresh"      );
    fMinTrack       =pset.get< int  >("MinTrack"      );
    fMaxTrack       =pset.get< int  >("MaxTrack"      );
    fXThresh        =pset.get< double        >("XThresh"       );
    fYThresh        =pset.get< double        >("YThresh"       );
    fZMin           =pset.get< double        >("ZMin"          );
    fZMax           =pset.get< double        >("ZMax"          );
    fPotLabel       =pset.get< std::string>("PotLabel");
  }
  
  //......................................................................
  
  ShowerAna::~ShowerAna()
  {
  }
  
  //......................................................................
  
  void ShowerAna::beginJob()
  {
    art::ServiceHandle<art::TFileService> tfs;
    fEvent = tfs->make<TTree>("fEvent","fEvent");
//True neutrino information
      fEvent->Branch("evtTrueNuCCNC", &m_evtTrueNuCCNC);
      fEvent->Branch("evtTrueNuMode", &m_evtTrueNuMode);
      fEvent->Branch("evtTrueNuPdg", &m_evtTrueNuPdg);
      fEvent->Branch("evtTrueNuEnergy", &m_evtTrueNuEnergy);
      fEvent->Branch("evtTrueNuIsFid", &m_evtTrueNuIsFid);
      fEvent->Branch("evtTrueNuP4[4]", m_evtTrueNuP4);
      fEvent->Branch("evtTrueNuVtx[3]", m_evtTrueNuVtx);
      fEvent->Branch("evtOsTag", &m_evtOsTag);
      fEvent->Branch("evtOsProb[20]", m_evtOsProb);
      fEvent->Branch("evtOsRand", &m_evtOsRand);
      fEvent->Branch("evtRwRand[20]", m_evtRwRand);
      fEvent->Branch("evtRwProb[20]", m_evtRwProb);
      fEvent->Branch("evtTrueEnergy[20]", m_evtTrueEnergy);
      fEvent->Branch("evtTrueElecEnergy[20]", m_evtTrueElecEnergy);
      fEvent->Branch("evtTrueXElecEnergy[20]", m_evtTrueXElecEnergy);
      fEvent->Branch("evtTrueYElecEnergy[20]", m_evtTrueYElecEnergy);
      fEvent->Branch("evtTrueElecNXHit[20]", m_evtTrueElecNXHit);
      fEvent->Branch("evtTrueElecNYHit[20]", m_evtTrueElecNYHit);


      fEvent->Branch("evtTruePdg[20]", m_evtTruePdg);
      fEvent->Branch("evtTrueMother[20]", m_evtTrueMother);
      fEvent->Branch("evtTrueDir[3]", m_evtTrueDir);
      fEvent->Branch("evtTrueVtx[3]", m_evtTrueVtx);
      fEvent->Branch("evtTrueNTrk", &m_evtTrueNTrk);

      fEvent->Branch("evtTruePF",&m_evtTruePF);
      fEvent->Branch("evtTruePNu",&m_evtTruePNu);
      fEvent->Branch("evtTruePHad",&m_evtTruePHad);
      fEvent->Branch("evtTrueCosTheta",&m_evtTrueCosTheta);
      fEvent->Branch("evtTruePt",&m_evtTruePt);
      fEvent->Branch("evtTruePl",&m_evtTruePl);


//Reconstructed information
      fEvent->Branch("evtEvent", &m_evtEvent);
      fEvent->Branch("evtSubEvent", &m_evtSubEvent);
      fEvent->Branch("evtRun", &m_evtRun);
      fEvent->Branch("evtSRun", &m_evtSRun);
      fEvent->Branch("evtNSlice", &m_evtNSlice);
      fEvent->Branch("evtFTag", &m_evtFTag);
      fEvent->Branch("evtNSigSlice", &m_evtNSigSlice);
      fEvent->Branch("evtNTrk", &m_evtNTrk);
      fEvent->Branch("evtNXTrk", &m_evtNXTrk);
      fEvent->Branch("evtNYTrk", &m_evtNYTrk);
      fEvent->Branch("evtVtx[3]", m_evtVtx);
      fEvent->Branch("evtXSh1GeV", &m_evtXSh1GeV);
      fEvent->Branch("evtYSh1GeV", &m_evtYSh1GeV);
      fEvent->Branch("evtXSh1Plane", &m_evtXSh1Plane);
      fEvent->Branch("evtYSh1Plane", &m_evtYSh1Plane);
/*
      fEvent->Branch("evtXSheEnergy", &m_evtXSheEnergy);
      fEvent->Branch("evtXSheDang", &m_evtXSheDang);
      fEvent->Branch("evtXSheNCell", &m_evtXSheNCell);
      fEvent->Branch("evtXSheNPlane", &m_evtXSheNPlane);
      fEvent->Branch("evtXSheMinPlane", &m_evtXSheMinPlane);
      fEvent->Branch("evtXSheMaxPlane", &m_evtXSheMaxPlane);
      fEvent->Branch("evtXSheDir", m_evtXSheDir);

      fEvent->Branch("evtYSheEnergy", &m_evtYSheEnergy);
      fEvent->Branch("evtYSheDang", &m_evtYSheDang);
      fEvent->Branch("evtYSheNCell", &m_evtYSheNCell);
      fEvent->Branch("evtYSheNPlane", &m_evtYSheNPlane);
      fEvent->Branch("evtYSheMinPlane", &m_evtYSheMinPlane);
      fEvent->Branch("evtYSheMaxPlane", &m_evtYSheMaxPlane);
      fEvent->Branch("evtYSheDir", m_evtYSheDir);
*/

      fEvent->Branch("evtNXCluster", &m_evtNXCluster);
      fEvent->Branch("evtNYCluster", &m_evtNYCluster);
      fEvent->Branch("evtXClusterGeV", &m_evtXClusterGeV);
      fEvent->Branch("evtYClusterGeV", &m_evtYClusterGeV);
      fEvent->Branch("evtXClusterMaxNCell", &m_evtXClusterMaxNCell);
      fEvent->Branch("evtYClusterMaxNCell", &m_evtYClusterMaxNCell);
      fEvent->Branch("evtXClusterNCell", &m_evtXClusterNCell);
      fEvent->Branch("evtYClusterNCell", &m_evtYClusterNCell);


      fEvent->Branch("evtNSh", &m_evtNSh);
      fEvent->Branch("evtANN", &m_evtANN);
      fEvent->Branch("evtTotalRecoGeV", &m_evtTotalRecoGeV);
      fEvent->Branch("evtTotalRecoNCell", &m_evtTotalRecoNCell);
      fEvent->Branch("evtPEtot", &m_evtPEtot);
      fEvent->Branch("evtTotalRecoGeV0", &m_evtTotalRecoGeV0);
      fEvent->Branch("evtTotalRecoNCell0", &m_evtTotalRecoNCell0);
      fEvent->Branch("evtPEtot0", &m_evtPEtot0);



      fEvent->Branch("evtPi0Mgg0", &m_evtPi0Mgg0);
      fEvent->Branch("evtPi0Mgg",  &m_evtPi0Mgg);
      fEvent->Branch("evtPi0Eg1",  &m_evtPi0Eg1);
      fEvent->Branch("evtPi0Eg2",  &m_evtPi0Eg2);
      fEvent->Branch("evtPi0StartDist", &m_evtPi0StartDist);
      fEvent->Branch("evtPi0LineDist", &m_evtPi0LineDist); 
      fEvent->Branch("evtPi0Vtx[3]", m_evtPi0Vtx); 
      fEvent->Branch("evtPi0TrueDirMgg0", &m_evtPi0TrueDirMgg0);
      fEvent->Branch("evtPi0TrueDirMgg", &m_evtPi0TrueDirMgg);

      fEvent->Branch("evtEtot0", &m_evtEtot0);
      fEvent->Branch("evtEtot", &m_evtEtot);
      fEvent->Branch("evtTrueEtot", &m_evtTrueEtot);
      fEvent->Branch("evtTrueElecPhotons", &m_evtTrueElecPhotons);


      fEvent->Branch("evtSh1Energy", &m_evtSh1Energy);
      fEvent->Branch("evtSh1TNS", &m_evtSh1TNS);
      fEvent->Branch("evtSh1Energy0", &m_evtSh1Energy0);
      fEvent->Branch("evtSh1Length", &m_evtSh1Length);
      fEvent->Branch("evtSh1TrkGeV", &m_evtSh1TrkGeV);
      fEvent->Branch("evtSh1SliceGeV", &m_evtSh1SliceGeV);
      fEvent->Branch("evtSh1SliceEdge10GeV", &m_evtSh1SliceEdge10GeV);
      fEvent->Branch("evtSh1SliceEdge5GeV", &m_evtSh1SliceEdge5GeV);
      fEvent->Branch("evtSh1SliceEdge2GeV", &m_evtSh1SliceEdge2GeV);
      fEvent->Branch("evtSh1Gap", &m_evtSh1Gap);
      fEvent->Branch("evtSh1NTrk", &m_evtSh1NTrk);
      fEvent->Branch("evtSh1IsFid", &m_evtSh1IsFid);
      fEvent->Branch("evtSh1DistToEdge", &m_evtSh1DistToEdge);
      fEvent->Branch("evtSh1P4[4]",m_evtSh1P4);
      fEvent->Branch("evtSh1Dir[3]",m_evtSh1Dir);
      fEvent->Branch("evtSh1Start[3]",m_evtSh1Start);
      fEvent->Branch("evtSh1Stop[3]",m_evtSh1Stop);

      fEvent->Branch("evtSh1MinPlane",&m_evtSh1MinPlane);
      fEvent->Branch("evtSh1MaxPlane",&m_evtSh1MaxPlane);

      fEvent->Branch("evtSh1DedxLLL[20]", m_evtSh1DedxLLL);
      fEvent->Branch("evtSh1DedxLLT[20]", m_evtSh1DedxLLT);
      fEvent->Branch("evtSh1DedxANN", &m_evtSh1DedxANN);
      fEvent->Branch("evtSh1DedxANN1", &m_evtSh1DedxANN1);
      fEvent->Branch("evtSh1Moment12", &m_evtSh1Moment12);
      fEvent->Branch("evtSh1Asym", &m_evtSh1Asym);
      fEvent->Branch("evtSh1Ixx", &m_evtSh1Ixx);
      fEvent->Branch("evtSh1Iyy", &m_evtSh1Iyy);
      fEvent->Branch("evtSh1NPlane", &m_evtSh1NPlane);
      fEvent->Branch("evtSh1NZeroPlane", &m_evtSh1NZeroPlane);
      fEvent->Branch("evtSh1NMIPPlane", &m_evtSh1NMIPPlane);
      fEvent->Branch("evtSh1ExtentX", &m_evtSh1ExtentX);
      fEvent->Branch("evtSh1ExtentY", &m_evtSh1ExtentY);
      fEvent->Branch("evtSh1NXCell", &m_evtSh1NXCell);
      fEvent->Branch("evtSh1NYCell", &m_evtSh1NYCell);

      fEvent->Branch("evtSh1VtxGeV", &m_evtSh1VtxGeV);
      fEvent->Branch("evtSh1Pi0Mgg", &m_evtSh1Pi0Mgg);
      fEvent->Branch("evtSh1ContStartPlane", &m_evtSh1ContStartPlane);
      fEvent->Branch("evtSh1TrueDang", &m_evtSh1TrueDang);
      fEvent->Branch("evtSh1TrueEDang", &m_evtSh1TrueEDang);
      fEvent->Branch("evtSh1TrueEDang", &m_evtSh1TrueEDang);
      fEvent->Branch("evtSh1TruePdg", &m_evtSh1TruePdg);
      fEvent->Branch("evtSh1TrueT", &m_evtSh1TrueT);
      fEvent->Branch("evtSh1TrueId", &m_evtSh1TrueId);
      fEvent->Branch("evtSh1TrueVtx[3]", m_evtSh1TrueVtx);
      fEvent->Branch("evtSh1TrueP4[4]", m_evtSh1TrueP4);

      fEvent->Branch("evtSh1TrueMotherPdg", &m_evtSh1TrueMotherPdg);
      fEvent->Branch("evtSh1TrueMotherId", &m_evtSh1TrueMotherId);
      fEvent->Branch("evtSh1TrueMotherP4[4]",  m_evtSh1TrueMotherP4);

      fEvent->Branch("evtSh1TrueNavMotherPdg", &m_evtSh1TrueNavMotherPdg);
      fEvent->Branch("evtSh1TrueNavMotherId", &m_evtSh1TrueNavMotherId);
      fEvent->Branch("evtSh1TrueNavMotherP4[4]",  m_evtSh1TrueNavMotherP4);

      fEvent->Branch("evtSh1TrueElecPhotons", &m_evtSh1TrueElecPhotons);
      fEvent->Branch("evtSh1TrueTotPhotons", &m_evtSh1TrueTotPhotons);
      fEvent->Branch("evtSh1TrueTrkElecPhotons", &m_evtSh1TrueTrkElecPhotons);
      fEvent->Branch("evtSh1TrueTrkTotPhotons", &m_evtSh1TrueTrkTotPhotons);

      fEvent->Branch("evtSh1TrueRWElecPhotons", &m_evtSh1TrueRWElecPhotons);
      fEvent->Branch("evtSh1TrueRWTotPhotons", &m_evtSh1TrueRWTotPhotons);
      fEvent->Branch("evtSh1TrueTrkRWElecPhotons", &m_evtSh1TrueTrkRWElecPhotons);
      fEvent->Branch("evtSh1TrueTrkRWTotPhotons", &m_evtSh1TrueTrkRWTotPhotons);




      fEvent->Branch("evtSh1Ee", &m_evtSh1Ee);
      fEvent->Branch("evtSh1CosTheta", &m_evtSh1CosTheta);
      fEvent->Branch("evtSh1CosTrueTheta", &m_evtSh1CosTrueTheta);
      fEvent->Branch("evtSh1Mh", &m_evtSh1Mh);
      fEvent->Branch("evtSh1MhTrue", &m_evtSh1MhTrue);
      fEvent->Branch("evtSh1Meff", &m_evtSh1Meff);
      fEvent->Branch("evtSh1Enue", &m_evtSh1Enue);
      fEvent->Branch("evtSh1TrueCosTheta", &m_evtSh1CosTrueTheta);
      fEvent->Branch("evtSh1TrueCosHadTheta", &m_evtSh1CosTrueHadTheta);
      fEvent->Branch("evtSh1TrueHadEnue", &m_evtSh1TrueHadEnue);
      fEvent->Branch("evtSh1TrueEnue", &m_evtSh1TrueEnue);
      fEvent->Branch("evtSh1TrueSumHadM", &m_evtSh1TrueSumHadM);
      fEvent->Branch("evtSh1TrueSumHadNtrk", &m_evtSh1TrueSumHadNtrk);
      fEvent->Branch("evtSh1TrueSumHadEnue", &m_evtSh1TrueSumHadEnue);
      fEvent->Branch("evtSh1TruePt",&m_evtSh1TruePt);
      fEvent->Branch("evtSh1TruePl",&m_evtSh1TruePl);
      fEvent->Branch("evtSh1TrueSumHadPt",&m_evtSh1TrueSumHadPt);
      fEvent->Branch("evtSh1TrueSumHadPl",&m_evtSh1TrueSumHadPl);

      fEvent->Branch("evtSh1SumHadP", &m_evtSh1SumHadP);
      fEvent->Branch("evtSh1SumPt", &m_evtSh1SumPt);
      fEvent->Branch("evtSh1SumP", &m_evtSh1SumP);
      fEvent->Branch("evtSh1P", &m_evtSh1P);


      fEvent->Branch("evtSh2Gap", &m_evtSh2Gap);
      fEvent->Branch("evtSh2Pi0Mgg", &m_evtSh2Pi0Mgg);
      fEvent->Branch("evtSh2SliceGeV", &m_evtSh2SliceGeV);

      fEvent->Branch("evtSh2VtxGeV", &m_evtSh2VtxGeV);
      fEvent->Branch("evtSh2Energy", &m_evtSh2Energy);
      fEvent->Branch("evtSh2TNS", &m_evtSh2TNS);
      fEvent->Branch("evtSh2TrkGeV", &m_evtSh2TrkGeV);
      fEvent->Branch("evtSh2IsFid", &m_evtSh2IsFid);
      fEvent->Branch("evtSh2P4[4]",m_evtSh2P4);
      fEvent->Branch("evtSh2Start[3]",m_evtSh2Start);
      fEvent->Branch("evtSh2Dir[3]",m_evtSh2Dir);
      fEvent->Branch("evtSh2Stop[3]",m_evtSh2Stop);
      fEvent->Branch("evtSh2DedxLLL[20]", m_evtSh2DedxLLL);
      fEvent->Branch("evtSh2DedxLLT[20]", m_evtSh2DedxLLT);
      fEvent->Branch("evtSh2DedxANN", &m_evtSh2DedxANN);
      fEvent->Branch("evtSh2DedxANN1", &m_evtSh2DedxANN1);
      fEvent->Branch("evtSh2NPlane", &m_evtSh2NPlane);
      fEvent->Branch("evtSh2NMIPPlane", &m_evtSh2NMIPPlane);
      fEvent->Branch("evtSh2ContStartPlane", &m_evtSh2ContStartPlane);
      fEvent->Branch("evtSh2TrueDang", &m_evtSh2TrueDang);
      fEvent->Branch("evtSh2TruePdg", &m_evtSh2TruePdg);
      fEvent->Branch("evtSh2TrueT", &m_evtSh2TrueT);
      fEvent->Branch("evtSh2TrueId", &m_evtSh2TrueId);
      fEvent->Branch("evtSh2TrueVtx[3]", m_evtSh2TrueVtx);
      fEvent->Branch("evtSh2TrueP4[4]", m_evtSh2TrueP4);
      fEvent->Branch("evtSh2TrueMotherPdg", &m_evtSh2TrueMotherPdg);
      fEvent->Branch("evtSh2TrueMotherId", &m_evtSh2TrueMotherId);
      fEvent->Branch("evtSh2TrueMotherP4[4]",  m_evtSh2TrueMotherP4);


      fEvent->Branch("evtSh2TrueNavMotherPdg", &m_evtSh2TrueNavMotherPdg);
      fEvent->Branch("evtSh2TrueNavMotherId", &m_evtSh2TrueNavMotherId);
      fEvent->Branch("evtSh2TrueNavMotherP4[4]",  m_evtSh2TrueNavMotherP4);

      fEvent->Branch("evtSh3Gap", &m_evtSh3Gap);
      fEvent->Branch("evtSh3Pi0Mgg", &m_evtSh3Pi0Mgg);
      fEvent->Branch("evtSh3SliceGeV", &m_evtSh3SliceGeV);
      fEvent->Branch("evtSh3VtxGeV", &m_evtSh3VtxGeV);
      fEvent->Branch("evtSh3Energy", &m_evtSh3Energy);
      fEvent->Branch("evtSh3TNS", &m_evtSh3TNS);
      fEvent->Branch("evtSh3TrkGeV", &m_evtSh3TrkGeV);
      fEvent->Branch("evtSh3IsFid", &m_evtSh3IsFid);
      fEvent->Branch("evtSh3P4[4]",m_evtSh3P4);
      fEvent->Branch("evtSh3Start[3]",m_evtSh3Start);
      fEvent->Branch("evtSh3Dir[3]",m_evtSh3Dir);
      fEvent->Branch("evtSh3Stop[3]",m_evtSh3Stop);
      fEvent->Branch("evtSh3DedxLLL[20]", m_evtSh3DedxLLL);
      fEvent->Branch("evtSh3DedxLLT[20]", m_evtSh3DedxLLT);
      fEvent->Branch("evtSh3DedxANN", &m_evtSh3DedxANN);
      fEvent->Branch("evtSh3DedxANN1", &m_evtSh3DedxANN1);
      fEvent->Branch("evtSh3NPlane", &m_evtSh3NPlane);
      fEvent->Branch("evtSh3NMIPPlane", &m_evtSh3NMIPPlane);
      fEvent->Branch("evtSh3ContStartPlane", &m_evtSh3ContStartPlane);
      fEvent->Branch("evtSh3TrueDang", &m_evtSh3TrueDang);
      fEvent->Branch("evtSh3TruePdg", &m_evtSh3TruePdg);
      fEvent->Branch("evtSh3TrueT", &m_evtSh3TrueT);
      fEvent->Branch("evtSh3TrueId", &m_evtSh3TrueId);
      fEvent->Branch("evtSh3TrueVtx[3]", m_evtSh3TrueVtx);
      fEvent->Branch("evtSh3TrueP4[4]", m_evtSh3TrueP4);
      fEvent->Branch("evtSh3TrueMotherPdg", &m_evtSh3TrueMotherPdg);
      fEvent->Branch("evtSh3TrueMotherId", &m_evtSh3TrueMotherId);
      fEvent->Branch("evtSh3TrueMotherP4[4]",  m_evtSh3TrueMotherP4);




      fEvent->Branch("evtSheVtxGeV", &m_evtSheVtxGeV);
      fEvent->Branch("evtSheEnergy", &m_evtSheEnergy);
      fEvent->Branch("evtSheTNS", &m_evtSheTNS);
      fEvent->Branch("evtSheTrkGeV", &m_evtSheTrkGeV);
      fEvent->Branch("evtSheGap", &m_evtSheGap);
      fEvent->Branch("evtSheIsFid", &m_evtSheIsFid);
      fEvent->Branch("evtSheP4[4]",m_evtSheP4);
      fEvent->Branch("evtSheDir[3]",m_evtSheDir);
      fEvent->Branch("evtSheStart[3]",m_evtSheStart);
      fEvent->Branch("evtSheStop[3]",m_evtSheStop);
      fEvent->Branch("evtSheDedxLLL[20]", m_evtSheDedxLLL);
      fEvent->Branch("evtSheDedxLLT[20]", m_evtSheDedxLLT);
      fEvent->Branch("evtSheDedxANN", &m_evtSheDedxANN);
      fEvent->Branch("evtSheDedxANN1", &m_evtSheDedxANN1);
      fEvent->Branch("evtSheNPlane", &m_evtSheNPlane);
      fEvent->Branch("evtSheNMIPPlane", &m_evtSheNMIPPlane);
      fEvent->Branch("evtSheContStartPlane", &m_evtSheContStartPlane);
      fEvent->Branch("evtSheTrueDang", &m_evtSheTrueDang);
      fEvent->Branch("evtSheTrueEDang", &m_evtSheTrueEDang);
      fEvent->Branch("evtSheTruePdg", &m_evtSheTruePdg);
      fEvent->Branch("evtSheTrueT", &m_evtSheTrueT);
      fEvent->Branch("evtSheTrueId", &m_evtSheTrueId);
      fEvent->Branch("evtSheTrueVtx[3]", m_evtSheTrueVtx);
      fEvent->Branch("evtSheTrueP4[4]", m_evtSheTrueP4);
      fEvent->Branch("evtSheTrueElecPhotons", &m_evtSheTrueElecPhotons);
      fEvent->Branch("evtSheTrueTotPhotons", &m_evtSheTrueTotPhotons);
      fEvent->Branch("evtSheTrueTrkElecPhotons", &m_evtSheTrueTrkElecPhotons);
      fEvent->Branch("evtSheTrueTrkTotPhotons", &m_evtSheTrueTrkTotPhotons);

      fEvent->Branch("evtSheTrueRWElecPhotons", &m_evtSheTrueRWElecPhotons);
      fEvent->Branch("evtSheTrueRWTotPhotons", &m_evtSheTrueRWTotPhotons);
      fEvent->Branch("evtSheTrueTrkRWElecPhotons", &m_evtSheTrueTrkRWElecPhotons);
      fEvent->Branch("evtSheTrueTrkRWTotPhotons", &m_evtSheTrueTrkRWTotPhotons);



      fEvent->Branch("evtLen1Energy", &m_evtLen1Energy);
      fEvent->Branch("evtLen1TNS", &m_evtLen1TNS);
      fEvent->Branch("evtLen1DedxANN", &m_evtLen1DedxANN);
      fEvent->Branch("evtLen1DedxANN1", &m_evtLen1DedxANN1);
      fEvent->Branch("evtLen1TrkGeV", &m_evtLen1TrkGeV);
      fEvent->Branch("evtLen1IsFid", &m_evtLen1IsFid);
      fEvent->Branch("evtLen1P4[4]",m_evtLen1P4);
      fEvent->Branch("evtLen1Start[3]",m_evtLen1Start);
      fEvent->Branch("evtLen1Stop[3]",m_evtLen1Stop);
      fEvent->Branch("evtLen1DedxLLL[20]", m_evtLen1DedxLLL);
      fEvent->Branch("evtLen1DedxLLT[20]", m_evtLen1DedxLLT);
      fEvent->Branch("evtLen1Energy0", &m_evtLen1Energy0); 
      fEvent->Branch("evtLen1VtxGeV", &m_evtLen1VtxGeV);
      fEvent->Branch("evtLen1Pi0Mgg", &m_evtLen1Pi0Mgg);
      fEvent->Branch("evtLen1SliceGeV", &m_evtLen1SliceGeV); 
      fEvent->Branch("evtLen1Gap", &m_evtLen1Gap);



      fEvent->Branch("evtLen1NPlane", &m_evtLen1NPlane);
      fEvent->Branch("evtLen1NMIPPlane", &m_evtLen1NMIPPlane);
      fEvent->Branch("evtLen1ContStartPlane", &m_evtLen1ContStartPlane);
      fEvent->Branch("evtLen1TrueDang",&m_evtLen1TrueDang); 
      fEvent->Branch("evtLen1TrueEDang",&m_evtLen1TrueEDang);
      fEvent->Branch("evtLen1TrueId",&m_evtLen1TrueId);
      fEvent->Branch("evtLen1TruePdg",&m_evtLen1TruePdg);
      fEvent->Branch("evtLen1TrueT",&m_evtLen1TrueT);
      fEvent->Branch("evtLen1TrueVtx[3]",m_evtLen1TrueVtx);
      fEvent->Branch("evtLen1TrueP4[4]",m_evtLen1TrueP4);

      fEvent->Branch("evtLen1TrueMotherPdg",&m_evtLen1TrueMotherPdg);
      fEvent->Branch("evtLen1TrueMotherId",&m_evtLen1TrueMotherId);
      fEvent->Branch("evtLen1TrueMotherP4[4]",m_evtLen1TrueMotherP4);
      fEvent->Branch("evtLen1TrueElecPhotons",&m_evtLen1TrueElecPhotons);
      fEvent->Branch("evtLen1TrueTotPhotons",&m_evtLen1TrueTotPhotons);
      fEvent->Branch("evtLen1TrueRWElecPhotons",&m_evtLen1TrueRWElecPhotons);
      fEvent->Branch("evtLen1TrueRWTotPhotons",&m_evtLen1TrueRWTotPhotons);


      fEvent->Branch("evtShExclPE", &m_evtShExclPE);
      fEvent->Branch("evtShExclCentroid[3]",m_evtShExclCentroid);
      fEvent->Branch("evtShExclRadius",&m_evtShExclRadius);

    fShower = tfs->make<TTree>("fShower","fShower");
      fShower->Branch("showerRun",&m_showerRun);
      fShower->Branch("showerSubRun",&m_showerSubRun);
      fShower->Branch("showerEvent",&m_showerEvent);
      fShower->Branch("showerSubEvent",&m_showerSubEvent);
      fShower->Branch("showerTrueNuCCNC", &m_showerTrueNuCCNC);
      fShower->Branch("showerTrueNuMode", &m_showerTrueNuMode);
      fShower->Branch("showerTrueNuPdg", &m_showerTrueNuPdg);
      fShower->Branch("showerTrueNuEnergy", &m_showerTrueNuEnergy);
      fShower->Branch("showerTrueNuIsFid", &m_showerTrueNuIsFid);
      fShower->Branch("showerTrueNuP4[4]", m_showerTrueNuP4);
      fShower->Branch("showerTrueNuVtx[3]", m_showerTrueNuVtx);
      fShower->Branch("showerTruePi0Dang", &m_showerTruePi0Dang); 
      fShower->Branch("showerTruePi0P4[4]", m_showerTruePi0P4);


      fShower->Branch("showerEvtTrueEnergy[20]", m_showerEvtTrueEnergy);
      fShower->Branch("showerEvtTruePdg[20]", m_showerEvtTruePdg);
      fShower->Branch("showerEvtTrueElecPhotons",&m_showerEvtTrueElecPhotons);

      fShower->Branch("showerId",&m_showerId);
      fShower->Branch("showerNPlane", &m_showerNPlane); 
      fShower->Branch("showerNZeroPlane", &m_showerNZeroPlane); 
      fShower->Branch("showerLength", &m_showerLength); 
      fShower->Branch("showerNTrk", &m_showerNTrk); 
      fShower->Branch("showerVtx[3]", m_showerVtx);
      fShower->Branch("showerVtxDoca", &m_showerVtxDoca);
      fShower->Branch("showerVtxToStart", &m_showerVtxToStart); 
      fShower->Branch("showerVtxToStop", &m_showerVtxToStop);
      fShower->Branch("showerVtxIntToStart", &m_showerVtxIntToStart);
      fShower->Branch("showerVtxIntToStop", &m_showerVtxIntToStop);
      fShower->Branch("showerVtxTrackLength", &m_showerVtxTrackLength);
      fShower->Branch("showerVtxBackTag", &m_showerVtxBackTag);
      fShower->Branch("showerVtxTrueDoca", &m_showerVtxTrueDoca); 
      fShower->Branch("showerVtxTrueDang", &m_showerVtxTrueDang); 
      fShower->Branch("showerVtxStartTrueDoca", &m_showerVtxStartTrueDoca);
      fShower->Branch("showerVtxStopTrueDoca", &m_showerVtxStopTrueDoca);


      fShower->Branch("showerNXcell", &m_showerNXcell); 
      fShower->Branch("showerNYcell", &m_showerNYcell); 
      fShower->Branch("showerDir[3]", m_showerDir);
      fShower->Branch("showerStart[3]", m_showerStart);
      fShower->Branch("showerStop[3]", m_showerStop);
      fShower->Branch("showerIReg", &m_showerIReg);
      fShower->Branch("showerIE", &m_showerIE);
      fShower->Branch("showerADC", &m_showerADC);
      fShower->Branch("showerEnergy0", &m_showerEnergy0);
      fShower->Branch("showerEnergy", &m_showerEnergy);
      fShower->Branch("showerFTag", &m_showerFTag);
      fShower->Branch("showerTNS", &m_showerTNS);
      fShower->Branch("showerTrkGeV", &m_showerTrkGeV);
      fShower->Branch("showerSliceGeV", &m_showerSliceGeV);
      fShower->Branch("showerSliceEdge10GeV", &m_showerSliceEdge10GeV);
      fShower->Branch("showerSliceEdge5GeV", &m_showerSliceEdge5GeV);
      fShower->Branch("showerSliceEdge2GeV", &m_showerSliceEdge2GeV);
      fShower->Branch("showerTotalRecoNCell", &m_showerTotalRecoNCell);
      fShower->Branch("showerNCell", &m_showerNCell);
      fShower->Branch("showerRadius", &m_showerRadius);
      fShower->Branch("showerVtxGeV", &m_showerVtxGeV);
      fShower->Branch("showerGap", &m_showerGap);
      fShower->Branch("showerIsFid", &m_showerIsFid);
      fShower->Branch("showerDistToEdge", &m_showerDistToEdge);
      fShower->Branch("showerNMIPPlane", &m_showerNMIPPlane);
      fShower->Branch("showerContStartPlane", &m_showerContStartPlane);
      fShower->Branch("showerRminmax", &m_showerRminmax);
      fShower->Branch("trackADC", &m_trackADC);
      fShower->Branch("trackEnergy", &m_trackEnergy);
      fShower->Branch("trackTNS", &m_trackTNS);
      fShower->Branch("trackNCell", &m_trackNCell);
      fShower->Branch("trackRadius", &m_trackRadius);
      fShower->Branch("showerPlaneEnergy[200]", m_showerPlaneEnergy);
      fShower->Branch("showerPlaneDedx[200]", m_showerPlaneDedx);
      fShower->Branch("showerPlaneHitX[200]", m_showerPlaneHitX);
      fShower->Branch("showerPlaneHitY[200]", m_showerPlaneHitY);
      fShower->Branch("showerPlaneHitZ[200]", m_showerPlaneHitZ);

      fShower->Branch("showerPlaneHitCell[200]",m_showerPlaneHitCell);
      fShower->Branch("showerPlaneE1Cell[200]",m_showerPlaneE1Cell);
      fShower->Branch("showerPlaneE2Cell[200]",m_showerPlaneE2Cell);
      fShower->Branch("showerPlaneCentroidCell[200]",m_showerPlaneCentroidCell);
      fShower->Branch("showerPlaneHitCellGeV[200]",m_showerPlaneHitCellGeV);
      fShower->Branch("showerPlaneE1CellGeV[200]",m_showerPlaneE1CellGeV);
      fShower->Branch("showerPlaneE2CellGeV[200]",m_showerPlaneE2CellGeV);
      fShower->Branch("showerPlaneCentroidCellGeV[200]",m_showerPlaneCentroidCellGeV);



      fShower->Branch("showerPlaneHitIReg", m_showerPlaneHitIReg);

      fShower->Branch("showerPlaneDeltaDedx[200]", m_showerPlaneDeltaDedx);
      fShower->Branch("showerDedxChisq[20]", m_showerDedxChisq);
      fShower->Branch("showerRadiusChisq[20]", m_showerRadiusChisq);
      fShower->Branch("showerDedxNdof[20]", m_showerDedxNdof);
      fShower->Branch("showerDedxProb[20]", m_showerDedxProb);
      fShower->Branch("showerTransDedxProb[20]", m_showerTransDedxProb);
      fShower->Branch("showerDedxLLL[20]", m_showerDedxLLL);
      fShower->Branch("showerDedxLLL1[20]", m_showerDedxLLL1);
      fShower->Branch("showerDedxLLL2[20]", m_showerDedxLLL2);
      fShower->Branch("showerDedxLLL3[20]", m_showerDedxLLL3);
      fShower->Branch("showerDedxLLL4[20]", m_showerDedxLLL4);
      fShower->Branch("showerDedxLLL5[20]", m_showerDedxLLL5);
      fShower->Branch("showerDedxLLL6[20]", m_showerDedxLLL6);
      fShower->Branch("showerDedxLLT[20]", m_showerDedxLLT);

      fShower->Branch("showerPlaneEnergy[200]", m_showerPlaneEnergy);
      fShower->Branch("showerTruePlaneEnergy[200]", m_showerTruePlaneEnergy);
      fShower->Branch("showerTransCellEnergy[20]", m_showerTransCellEnergy);
      fShower->Branch("showerTrueTransCellEnergy[20]", m_showerTrueTransCellEnergy);


      fShower->Branch("showerDedxANN", &m_showerDedxANN);
      fShower->Branch("showerDedxANN1", &m_showerDedxANN1);
      fShower->Branch("showerMoment12", &m_showerMoment12);
      fShower->Branch("showerAsym", &m_showerAsym);
      fShower->Branch("showerIxx", &m_showerIxx);
      fShower->Branch("showerIyy", &m_showerIyy);
      fShower->Branch("showerPlaneRadius[200]", m_showerPlaneRadius);
/*
      fShower->Branch("showerPlaneDedxprobE[200]", m_showerPlaneDedxprobE);
      fShower->Branch("showerPlaneDedxprobG[200]", m_showerPlaneDedxprobG);
      fShower->Branch("showerPlaneDedxprobMu[200]", m_showerPlaneDedxprobMu);
      fShower->Branch("showerPlaneDedxprobPi0[200]", m_showerPlaneDedxprobPi0);
      fShower->Branch("showerPlaneDedxprobHad[200]", m_showerPlaneDedxprobHad);
      fShower->Branch("showerPlaneDedxprobProton[200]", m_showerPlaneDedxprobProton);
      fShower->Branch("showerPlaneDedxprobNeutron[200]", m_showerPlaneDedxprobNeutron);
      fShower->Branch("showerPlaneDedxprobPion[200]", m_showerPlaneDedxprobPion);
      fShower->Branch("showerPlaneDedxprobEqe[200]", m_showerPlaneDedxprobEqe);
      fShower->Branch("showerPlaneDedxprobEres[200]", m_showerPlaneDedxprobEres);
      fShower->Branch("showerPlaneDedxprobEdis[200]", m_showerPlaneDedxprobEdis);
      fShower->Branch("showerPlaneDedxprobEcoh[200]", m_showerPlaneDedxprobEcoh);
*/
 
     fShower->Branch("showerTransCellDedx[20]", m_showerTransCellDedx);
      fShower->Branch("showerTransCellProbE[20]", m_showerTransCellProbE);
      fShower->Branch("showerTransCellProbPi0[20]", m_showerTransCellProbPi0);
      fShower->Branch("showerTrueDang", &m_showerTrueDang);
      fShower->Branch("showerTrueEDang", &m_showerTrueEDang);
      fShower->Branch("showerTrueVtxEDang", &m_showerTrueVtxEDang);
      fShower->Branch("showerTruePdg", &m_showerTruePdg);
      fShower->Branch("showerTrueT", &m_showerTrueT);
      fShower->Branch("showerTrueEtag", &m_showerTrueEtag);
      fShower->Branch("showerTrueId", &m_showerTrueId);
      fShower->Branch("showerTrueVtx[3]", m_showerTrueVtx);
      fShower->Branch("showerTrueP4[4]", m_showerTrueP4);
      fShower->Branch("showerTruePartHit[10]", m_showerTruePartHit);
      fShower->Branch("showerTruePartEnergy[10]", m_showerTruePartEnergy);
      fShower->Branch("showerTrueDepEnergy", &m_showerTrueDepEnergy);
      fShower->Branch("showerTrueElecPhotons", &m_showerTrueElecPhotons);
      fShower->Branch("showerTrueTotPhotons", &m_showerTrueTotPhotons);
      fShower->Branch("showerTrueTrkElecPhotons", &m_showerTrueTrkElecPhotons);
      fShower->Branch("showerTrueTrkTotPhotons", &m_showerTrueTrkTotPhotons);
      fShower->Branch("showerTrueRWElecPhotons", &m_showerTrueRWElecPhotons);
      fShower->Branch("showerTrueRWTotPhotons", &m_showerTrueRWTotPhotons);
      fShower->Branch("showerTrueTrkRWElecPhotons", &m_showerTrueTrkRWElecPhotons);
      fShower->Branch("showerTrueTrkRWTotPhotons", &m_showerTrueTrkRWTotPhotons);

      fShower->Branch("showerTrueG1Ratio", &m_showerTrueG1Ratio);
      fShower->Branch("showerTrueG2Ratio", &m_showerTrueG2Ratio);
      fShower->Branch("showerTrueG1Photons", &m_showerTrueG1Photons);
      fShower->Branch("showerTrueG2Photons", &m_showerTrueG2Photons);

      fShower->Branch("showerTrueRWG1Ratio", &m_showerTrueRWG1Ratio);
      fShower->Branch("showerTrueRWG2Ratio", &m_showerTrueRWG2Ratio);
      fShower->Branch("showerTrueRWG1Photons", &m_showerTrueRWG1Photons);
      fShower->Branch("showerTrueRWG2Photons", &m_showerTrueRWG2Photons);


      fShower->Branch("showerTrueMotherPdg", &m_showerTrueMotherPdg);
      fShower->Branch("showerTrueMotherId", &m_showerTrueMotherId);
      fShower->Branch("showerTrueMotherP4[4]",  m_showerTrueMotherP4);

      fShower->Branch("showerTrueNavMotherPdg", &m_showerTrueNavMotherPdg);
      fShower->Branch("showerTrueNavMotherId", &m_showerTrueNavMotherId);
      fShower->Branch("showerTrueNavMotherP4[4]",  m_showerTrueNavMotherP4);


      fShower->Branch("showerEe", &m_showerEe);
      fShower->Branch("showerCosTheta", &m_showerCosTheta);
      fShower->Branch("showerCosTrueTheta", &m_showerCosTrueTheta);
      fShower->Branch("showerMh", &m_showerMh);
      fShower->Branch("showerMhTrue", &m_showerMhTrue);
      fShower->Branch("showerMeff", &m_showerMeff);
      fShower->Branch("showerEnue", &m_showerEnue);
      fShower->Branch("showerTrueCosTheta", &m_showerCosTrueTheta);
      fShower->Branch("showerTrueCosHadTheta", &m_showerCosTrueHadTheta);
      fShower->Branch("showerTrueHadEnue", &m_showerTrueHadEnue);
      fShower->Branch("showerTrueEnue", &m_showerTrueEnue);
      fShower->Branch("showerTrueSumHadM", &m_showerTrueSumHadM);
      fShower->Branch("showerTrueSumHadNtrk", &m_showerTrueSumHadNtrk);
      fShower->Branch("showerTrueSumHadEnue", &m_showerTrueSumHadEnue);
      fShower->Branch("showerTruePt",&m_showerTruePt);
      fShower->Branch("showerTruePl",&m_showerTruePl);
      fShower->Branch("showerTrueSumHadP",&m_showerTrueSumHadP);
      fShower->Branch("showerTrueSumHadPt",&m_showerTrueSumHadPt);
      fShower->Branch("showerTrueSumHadPl",&m_showerTrueSumHadPl);

      fShower->Branch("showerSumHadP",&m_showerSumHadP);
      fShower->Branch("showerSumHadPCor",&m_showerSumHadPCor);
      fShower->Branch("showerSumPt",&m_showerSumPt);
      fShower->Branch("showerSumP",&m_showerSumP);
      fShower->Branch("showerP",&m_showerP);

    fNode = tfs->make<TTree>("fNode","fNode");
      fNode->Branch("nodeRun",&m_nodeRun);
      fNode->Branch("nodeSubRun",&m_nodeSubRun);
      fNode->Branch("nodeEvent",&m_nodeEvent);
      fNode->Branch("nodeSubEvent",&m_nodeSubEvent);
      fNode->Branch("nodeShId",&m_nodeShId);
      fNode->Branch("nodeXYZ", m_nodeXYZ,"nodeXYZ[3]/D");
      fNode->Branch("nodeE", &m_nodeE);
      fNode->Branch("nodeShowerDir[3]", m_nodeShowerDir);
      fNode->Branch("nodeShowerStart[3]", m_nodeShowerStart);
      fNode->Branch("nodeShowerStop[3]", m_nodeShowerStop);
      fNode->Branch("nodeShowerEnergy", &m_nodeShowerEnergy);
      fNode->Branch("nodeShowerNPlane", &m_nodeShowerNPlane);
      fNode->Branch("nodeShowerLength", &m_nodeShowerLength);
      fNode->Branch("nodeShowerNCell", &m_nodeShowerNCell);

      fNode->Branch("nodeTrueNuCCNC", &m_nodeTrueNuCCNC);
      fNode->Branch("nodeTrueNuMode", &m_nodeTrueNuMode);
      fNode->Branch("nodeTrueNuPdg", &m_nodeTrueNuPdg);
      fNode->Branch("nodeTrueNuEnergy", &m_nodeTrueNuEnergy);
      fNode->Branch("nodeTrueNuIsFid", &m_nodeTrueNuIsFid);
      fNode->Branch("nodeTrueNuP4[4]", m_nodeTrueNuP4);
      fNode->Branch("nodeTrueNuVtx[3]", m_nodeTrueNuVtx);
      fNode->Branch("nodeTrueDang", &m_nodeTrueDang);
      fNode->Branch("nodeTruePdg", &m_nodeTruePdg);
      fNode->Branch("nodeTrueMotherPdg", &m_nodeTrueMotherPdg);
      fNode->Branch("nodeTrueNavMotherPdg", &m_nodeTrueNavMotherPdg);
      fNode->Branch("nodeTrueNavMotherId", &m_nodeTrueNavMotherId);
      fNode->Branch("nodeTrueNavMotherP4[4]",  m_nodeTrueNavMotherP4);
      fNode->Branch("nodeTrueElecPhotons", &m_nodeTrueElecPhotons);
      fNode->Branch("nodeTrueTotPhotons", &m_nodeTrueTotPhotons);

      

    fCell = tfs->make<TTree>("fCell","fCell");
      fCell->Branch("cellRun",&m_cellRun);
      fCell->Branch("cellSubRun",&m_cellSubRun);
      fCell->Branch("cellEvent",&m_cellEvent);
      fCell->Branch("cellSubEvent",&m_cellSubEvent);
      fCell->Branch("cellPlaneId",&m_cellPlaneId);
      fCell->Branch("cellView",&m_cellView);
      fCell->Branch("cellId",&m_cellId);
      fCell->Branch("cellSlice",&m_cellSlice);
      fCell->Branch("cellXYZ", m_cellXYZ,"cellXYZ[3]/D");
      fCell->Branch("cellNTrk",&m_cellNTrk,"cellNTrk/I");
      fCell->Branch("cellNSlice",&m_cellNSlice,"cellNSlice/I");
      fCell->Branch("cellADC",&m_cellADC);
      fCell->Branch("cellTDC",&m_cellTDC);
      fCell->Branch("cellTNS",&m_cellTNS);
      fCell->Branch("cellPE",&m_cellPE);
      fCell->Branch("cellPECorr",&m_cellPECorr);
      fCell->Branch("cellEnergy",&m_cellEnergy);
      fCell->Branch("cellEnergyM1",&m_cellEnergyM1);
      fCell->Branch("cellEnergyM2",&m_cellEnergyM2);
      fCell->Branch("cellEnergyP1",&m_cellEnergyP1);
      fCell->Branch("cellEnergyP2",&m_cellEnergyP2);
      fCell->Branch("cellTrueEnergy",&m_cellTrueEnergy);
      fCell->Branch("cellTrueEnergyM1",&m_cellTrueEnergyM1);
      fCell->Branch("cellTrueEnergyM2",&m_cellTrueEnergyM2);
      fCell->Branch("cellTrueEnergyP1",&m_cellTrueEnergyP1);
      fCell->Branch("cellTrueEnergyP2",&m_cellTrueEnergyP2);
      fCell->Branch("cellTrueTime",&m_cellTrueTime);
      fCell->Branch("cellTrueHitXYZ[3]",m_cellTrueHitXYZ);
      fCell->Branch("cellHitXYZ[3]",m_cellHitXYZ);
      fCell->Branch("cellPath",&m_cellPath);

      fCell->Branch("cellTrueNuCCNC", &m_cellTrueNuCCNC);
      fCell->Branch("cellTrueNuMode", &m_cellTrueNuMode);
      fCell->Branch("cellTrueNuPdg", &m_cellTrueNuPdg);
      fCell->Branch("cellTrueNuEnergy", &m_cellTrueNuEnergy);
      fCell->Branch("cellTrueNuIsFid", &m_cellTrueNuIsFid);
      fCell->Branch("cellTrueNuP4[4]", m_cellTrueNuP4);
      fCell->Branch("cellTrueNuVtx[3]", m_cellTrueNuVtx);
      fCell->Branch("cellTrueDang", &m_cellTrueDang);
      fCell->Branch("cellTruePdg", &m_cellTruePdg);
      fCell->Branch("cellTrueT", &m_cellTrueT);
      fCell->Branch("cellTrueMother", &m_cellTrueMother);
      fCell->Branch("cellTrueVtx[3]", m_cellTrueVtx);
      fCell->Branch("cellTrueP4[4]", m_cellTrueP4);
      fCell->Branch("cellTruePartElec",&m_cellTruePartElec);
      fCell->Branch("cellToSh1Dist", &m_cellToSh1Dist);
      fCell->Branch("cellToSh1Plane", &m_cellToSh1Plane);
      fCell->Branch("cellToSheDist", &m_cellToSheDist);
      fCell->Branch("cellToShePlane", &m_cellToShePlane);
      fCell->Branch("cellToSh2Dist", &m_cellToSh2Dist);
      fCell->Branch("cellToSh2Plane", &m_cellToSh2Plane);
      fCell->Branch("cellSh1Dir[3]",m_cellSh1Dir);
      fCell->Branch("cellSh2Dir[3]",m_cellSh2Dir);
      fCell->Branch("cellSheDir[3]",m_cellSheDir);
      fCell->Branch("cellSheNCell", &m_cellSheNCell);
      fCell->Branch("cellSheNPlane", &m_cellSheNPlane);
      fCell->Branch("cellSheLength", &m_cellSheLength);

      fCell->Branch("cellSheVtxGeV", &m_cellSheVtxGeV);
      fCell->Branch("cellSheEnergy", &m_cellSheEnergy);
      fCell->Branch("cellSheTrkGeV", &m_cellSheTrkGeV);
      fCell->Branch("cellSheIsFid", &m_cellSheIsFid);
      fCell->Branch("cellSheP4[4]",m_cellSheP4);
      fCell->Branch("cellSheDir[3]",m_cellSheDir);
      fCell->Branch("cellSheStart[3]",m_cellSheStart);
      fCell->Branch("cellSheStop[3]",m_cellSheStop);
      fCell->Branch("cellSheDedxLLL[20]", m_cellSheDedxLLL);
      fCell->Branch("cellSheDedxLLT[20]", m_cellSheDedxLLT);
      fCell->Branch("cellSheDedxANN", &m_cellSheDedxANN);
      fCell->Branch("cellSheNMIPPlane", &m_cellSheNMIPPlane);
      fCell->Branch("cellSheContStartPlane", &m_cellSheContStartPlane);
      fCell->Branch("cellSheTrueDang", &m_cellSheTrueDang);
      fCell->Branch("cellSheTrueEDang", &m_cellSheTrueEDang);
      fCell->Branch("cellSheTruePdg", &m_cellSheTruePdg);
      fCell->Branch("cellSheTrueT", &m_cellSheTrueT);
      fCell->Branch("cellSheTrueId", &m_cellSheTrueId);
      fCell->Branch("cellSheTrueVtx[3]", m_cellSheTrueVtx);
      fCell->Branch("cellSheTrueP4[4]", m_cellSheTrueP4);
      fCell->Branch("cellSheTrueElecPhotons", &m_cellSheTrueElecPhotons);
      fCell->Branch("cellSheTrueTotPhotons", &m_cellSheTrueTotPhotons);
      fCell->Branch("cellSheTrueTrkElecPhotons", &m_cellSheTrueTrkElecPhotons);
      fCell->Branch("cellSheTrueTrkTotPhotons", &m_cellSheTrueTrkTotPhotons);

      fCell->Branch("cellSheTrueRWElecPhotons", &m_cellSheTrueRWElecPhotons);
      fCell->Branch("cellSheTrueRWTotPhotons", &m_cellSheTrueRWTotPhotons);
      fCell->Branch("cellSheTrueTrkRWElecPhotons", &m_cellSheTrueTrkRWElecPhotons);
      fCell->Branch("cellSheTrueTrkRWTotPhotons", &m_cellSheTrueTrkRWTotPhotons);

      TFile hfileExtra("/nova/data/users/bianjm/share/development/RecoJMShower/histograms/extrahist_fd.root");
      ht_nue_rw = (TH1D*)hfileExtra.Get("ht_nue_rw");
      ht_bkg_nue_rw = (TH1D*)hfileExtra.Get("ht_bkg_nue_rw");
      ht_bkg_numu_rw = (TH1D*)hfileExtra.Get("ht_bkg_numu_rw");

  }

  void ShowerAna::beginSubRun(art::SubRun const& sr) {
/*
    art::Handle< sumdata::POTSum > p;
    sr.getByLabel(fPotLabel,p);
    std::cout<<"pot = "<<p->totgoodpot<<std::endl;
*/
  } 
  //......................................................................
  void ShowerAna::analyze(const art::Event& evt)
  {
//----------------------------------------------------
//  Run, subrun and event information
//----------------------------------------------------

    art::ServiceHandle<cheat::BackTracker> bt;

    int run   = evt.run();
    int srun  = evt.subRun();
    int event = evt.id().event();
    std::cout<<"Event "<<event<<" ==============================================="<<std::endl;
    art::Handle<std::vector<rb::Cluster> > slicecol;
    evt.getByLabel(pSliceDir, slicecol);
    art::PtrVector<rb::Cluster> slicelist;
    std::cout<<"n Slice "<<slicecol->size()<<std::endl;
    for(unsigned int i = 0; i < slicecol->size(); ++i){
      slicelist.push_back(art::Ptr<rb::Cluster>(slicecol, i));
    }

/* 
   art::Handle< std::vector<rb::Prong> > trackhandle;
    std::vector<art::Ptr<rb::Prong> > track;
//    evt.getByLabel(pTrackDir, pInstLabel, trackhandle);
    evt.getByLabel(pTrackDir, trackhandle);
    for(unsigned int i = 0; i < trackhandle->size(); i++){
      art::Ptr<rb::Prong> p(trackhandle, i);
      track.push_back(p);
    }
*/
    int mcstudy=1;

    const sim::ParticleNavigator & pnav = bt->ParticleNavigator();
//    int nPrimaries = pnav.NumberOfPrimaries();

    //Get all of the hits from calhits
    art::Handle< std::vector<rb::CellHit> > hitsHandle;
    evt.getByLabel("calhit", hitsHandle);
    art::PtrVector<rb::CellHit> allHits;
    for(unsigned int i = 0; i < hitsHandle->size(); ++i){
      art::Ptr<rb::CellHit> hit(hitsHandle, i);
      allHits.push_back(hit);
    }
    // We're gonna get a vector of NeutrinoEffPurs for allHits
    std::vector<cheat::NeutrinoWithIndex> allNus = bt->allNeutrinoInteractions();
    
    std::vector< std::vector<cheat::NeutrinoEffPur> > sliceEffPurs;
    sliceEffPurs.resize(slicelist.size());
    
    // Then we're gonna get a vector of vectors with one vector of neutrinoEffPurs for each slice. 

    for(unsigned int iSubvt = 0; iSubvt < slicelist.size(); ++iSubvt)
    {
//std::cout<<"step 2.1 "<<std::endl;    
//      sliceEffPurs[iSubvt] = bt->SliceToNeutrinoInteractions(slicelist[iSubvt]->AllCells(), allHits);//slicer
      sliceEffPurs[iSubvt] = bt->SliceToMCTruth(slicelist[iSubvt]->AllCells(), allHits);//slicer4d
      
    }

    std::vector<int> faveIds = getFaveIds(allNus, sliceEffPurs);


/*
   art::Handle< std::vector<jmshower::JMShower> > showerhandle;
   evt.getByLabel("recojmshower", showerhandle);
   std::vector<art::Ptr<jmshower::JMShower> >  evtsvcshowercol;

  for(unsigned int i = 0; i < showerhandle->size(); i++){
     art::Ptr<jmshower::JMShower> p(showerhandle, i);
     evtsvcshowercol.push_back(p);
   }
*/

for(unsigned int sliceIdx = 0; sliceIdx < slicecol->size(); ++sliceIdx){//Reconstruct EID slice by slice
  std::cout<<"slice idx "<<sliceIdx<<std::endl;
  bool ftag = false;
//  if(rb::IsFiltered(evt,slicecol,sliceIdx)) continue;
  const rb::Cluster& slice = (*slicecol)[sliceIdx];
  if(slice.IsNoise()) continue;
  bool slicemctag = true;
  if (faveIds[sliceIdx] == -1) slicemctag=false;
  std::cout<<"i slice, faveIds "<<sliceIdx<<" "<<faveIds[sliceIdx]<<std::endl;
    m_evtRun = run;
    m_evtSRun = srun;
    m_evtEvent = event;
    m_evtSubEvent = (int)sliceIdx;

    m_evtTrueNuEnergy = 0;
    m_evtTrueNuP4[0]  = 0;
    m_evtTrueNuP4[1]  = 0;
    m_evtTrueNuP4[2]  = 0;
    m_evtTrueNuP4[3]  = 0;
    m_evtTrueNuVtx[0] = 0;
    m_evtTrueNuVtx[1] = 0;
    m_evtTrueNuVtx[2] = 0;


    
    m_evtOsTag = -1;
    for(int i=0;i<20;i++){
      m_evtOsProb[i] = -1;
    }
    m_evtOsRand = -1;

    for(int i=0;i<20;i++){
      m_evtRwRand[i] = -1;
      m_evtRwProb[i] = -1;
    }


    art::ServiceHandle<geo::Geometry> geom;
    art::ServiceHandle<calib::Calibrator> cal;

    std::cout<<"Total N plane"<<geom->NPlanes()<<std::endl;

//-----------------------------------------------------------------------------------------------
//  MC true information
//-----------------------------------------------------------------------------------------------
//Neutrino MC truth
    TLorentzVector evtTrueNuP4(0,0,0,0);
    TVector3 evtTrueNuVtx(0,0,-100);
    int evtTrueNuCCNC = -1;
    int evtTrueNuMode = -1;
    int evtTrueNuPdg = 0;

/*
    art::Handle< std::vector<sim::Particle> > plisthandle;
    evt.getByLabel("geantgen", plisthandle);
    std::vector<art::Ptr<sim::Particle> >  plistevt;
    for(unsigned int i = 0; i < plisthandle->size(); i++){
       art::Ptr<sim::Particle> plst(plisthandle, i);
       plistevt.push_back(plst);
    }
    std::cout<<"plistevt "<<plistevt.size()<<" "<<std::endl;
*/

    art::Ptr<simb::MCTruth> mc;
    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true) mc = sliceEffPurs[sliceIdx][0].neutrinoInt;
    std::vector<const sim::Particle*> plist;
    plist.clear();
    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true)plist = bt->MCTruthToParticles(mc);

    std::cout<<"plistslice "<<plist.size()<<std::endl;
    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
      if (mc->NeutrinoSet()) {
        const simb::MCNeutrino& gen = mc->GetNeutrino();
        const simb::MCParticle& nu  = gen.Nu();
        evtTrueNuP4 = nu.Momentum();
        evtTrueNuVtx.SetXYZ(nu.EndX(),nu.EndY(),nu.EndZ());
        evtTrueNuCCNC = gen.CCNC() ;
        evtTrueNuMode = gen.Mode() ;
        evtTrueNuPdg = nu.PdgCode() ;
        std::cout<<"Nuetrino, Energy "<<nu.PdgCode()<<" "<<evtTrueNuP4.E()<<std::endl;
        std::cout<<"CCNC, Mode "<<gen.CCNC()<<" "<<gen.Mode()<<std::endl;
        enum simb::int_type_ outMode = (enum simb::int_type_) gen.Mode();
//      enum simb::int_type_ outMode1 = (enum simb::int_type_) InteractionType();
        std::cout<<"Mode enum  "<<outMode<<std::endl;
//        std::cout<<"Type enum  "<<outMode1<<std::endl;
      }
    }


//      if(evtTrueNuPdg!=14) return;
//      if(evtTrueNuCCNC!=1) return;
//      if(evtTrueNuMode!=3) return;
/*
    art::ServiceHandle<osc::OscCalcService> prob;
    prob->SetTh13(0.15878);//sin^2(2theta_13) = 0.0975
    double osprobMuE = prob->P(14, 12, evtTrueNuP4.E());
    double osprobEE = prob->P(12, 12, evtTrueNuP4.E());
    double osprobMuMu = prob->P(14, 14, evtTrueNuP4.E());
    double osprobEMu = prob->P(12, 14, evtTrueNuP4.E());
    double osprobMuTau = prob->P(14, 16, evtTrueNuP4.E());
    double osprobETau = prob->P(12, 16, evtTrueNuP4.E());

    double osprobBeamE = prob->P(evtTrueNuPdg, 12, evtTrueNuP4.E());
    double osprobBeamMu = prob->P(evtTrueNuPdg, 14, evtTrueNuP4.E());
    TRandom rand(run*1000000000+srun*1000000+event);
    double randnum = rand.Uniform(0, 1);
    int ostag = -1;
    if(randnum<=osprobMuE*15)ostag = 1;
    if(randnum>osprobMuE*15&&randnum<1)ostag = 0;
    std::cout<<"rand, prob, ostag "<<randnum<<" "<<osprobMuE*15<<" "<<ostag<<std::endl; 
    std::cout<<"Prob="<<prob->P(evtTrueNuPdg, 12, evtTrueNuP4.E())<<std::endl;
//    if(outtag==0){
//      for(int i=0;i<50;i++){
//      double nuenergy = i*0.1;
//         std::cout<<"Energy, Prob="<<nuenergy<<" "<<prob->P(14, 12, nuenergy)<<std::endl;
//      }
//    }
//    bool evtTrueNuIsFid=geom->isInsideFiducialVolume(evtTrueNuVtx[0],evtTrueNuVtx[1],evtTrueNuVtx[2]); 
//    m_evtTrueNuIsFid = int(evtTrueNuIsFid);
    
    m_evtOsTag = ostag; 
    m_evtOsProb[0] = osprobMuE; 
    m_evtOsProb[1] = osprobEE; 
    m_evtOsProb[2] = osprobMuMu; 
    m_evtOsProb[3] = osprobEMu; 
    m_evtOsProb[4] = osprobMuTau;
    m_evtOsProb[5] = osprobETau;
    m_evtOsProb[6] = osprobBeamE; 
    m_evtOsProb[7] = osprobBeamMu; 

    m_evtOsRand = randnum;
*/


    m_evtTrueNuCCNC = evtTrueNuCCNC;
    m_evtTrueNuMode = evtTrueNuMode;
    m_evtTrueNuPdg = evtTrueNuPdg;


    m_evtTrueNuEnergy = evtTrueNuP4.E();
    m_evtTrueNuP4[0] = evtTrueNuP4.Px();
    m_evtTrueNuP4[1] = evtTrueNuP4.Py();
    m_evtTrueNuP4[2] = evtTrueNuP4.Pz();
    m_evtTrueNuP4[3] = evtTrueNuP4.E();
    m_evtTrueNuVtx[0] = evtTrueNuVtx[0];
    m_evtTrueNuVtx[1] = evtTrueNuVtx[1];
    m_evtTrueNuVtx[2] = evtTrueNuVtx[2];
    

    TRandom rand0(run*1100000000+srun*1000000+event);
    double randnum0 = rand0.Uniform(0, 1);
    TRandom rand1(run*1200000000+srun*100000+event);
    double randnum1 = rand1.Uniform(0, 1);
    TRandom rand2(run*1300000000+srun*1000000+event);
    double randnum2 = rand2.Uniform(0, 1);
 
    double p_nue_rw       = ht_nue_rw->Interpolate(evtTrueNuP4.E());
    double p_bkg_nue_rw   = 1;//ht_bkg_nue_rw->Interpolate(evtTrueNuP4.E());
    double p_bkg_numu_rw  = 1;//ht_bkg_numu_rw->Interpolate(evtTrueNuP4.E());
 
 
    m_evtRwRand[0] = randnum0;
    m_evtRwRand[1] = randnum1;
    m_evtRwRand[2] = randnum2;
    m_evtRwProb[0] = p_nue_rw;
    m_evtRwProb[1] = p_bkg_nue_rw;
    m_evtRwProb[2] = p_bkg_numu_rw;

    if(rb::IsFiltered(evt,slicecol,sliceIdx)){ 
//       fEvent->Fill();
//       continue;//
       ftag = true;
    }


    std::map<geo::OfflineChan, double> trueEnergy;
    std::map<geo::OfflineChan, double> trueDepositEnergy;
    std::map<geo::OfflineChan, double> trueDepositTime;
    std::map<geo::OfflineChan, double> trueDepositEnergyTmp;
    std::map<geo::OfflineChan, double> truePath;
    std::map<geo::OfflineChan, double> trueEndX;
    std::map<geo::OfflineChan, double> trueEndY;
    std::map<geo::OfflineChan, double> trueEndZ;
    std::map<geo::OfflineChan, double> trueAvgX;
    std::map<geo::OfflineChan, double> trueAvgY;
    std::map<geo::OfflineChan, double> trueAvgZ;
    std::map<geo::OfflineChan, Vint> truePartPdg;
    std::map<geo::OfflineChan, Vdouble> truePartEnergy;
    std::map<geo::OfflineChan, bool> truePartElec;



    trueEnergy.clear();
    trueDepositEnergy.clear();
    trueDepositTime.clear();
    trueDepositEnergyTmp.clear();
    truePath.clear();
    trueEndX.clear();
    trueEndY.clear();
    trueEndZ.clear();
    trueAvgX.clear();
    trueAvgY.clear();
    trueAvgZ.clear();
    truePartPdg.clear();
    truePartEnergy.clear();
    truePartElec.clear();

    std::vector<TLorentzVector> evtntpcol;
    std::vector<TVector3> evtntvtxcol;
    std::vector<TVector3> evtntpxzcol;
    std::vector<TVector3> evtntpyzcol;
    std::vector<int> evtntpdgcol;
    std::vector<int> evtntmothercol;

    evtntpcol.clear(); 
    evtntvtxcol.clear();
    evtntpxzcol.clear();
    evtntpyzcol.clear();
    evtntpdgcol.clear();
    evtntmothercol.clear();

    std::vector<TLorentzVector> evt0ntpcol;
    std::vector<TVector3> evt0ntvtxcol;
    std::vector<TVector3> evt0ntpxzcol;
    std::vector<TVector3> evt0ntpyzcol;
    std::vector<int> evt0ntpdgcol;
    std::vector<int> evt0ntmothercol;

    evt0ntpcol.clear();
    evt0ntvtxcol.clear();
    evt0ntpxzcol.clear();
    evt0ntpyzcol.clear();
    evt0ntpdgcol.clear();
    evt0ntmothercol.clear();



    double trueTotalDepositEnergy = 0;
    double evtTrueElecEnergy[20];
    double evtTrueXElecEnergy[20];
    double evtTrueYElecEnergy[20];
    int evtTrueElecNXHit[20];
    int evtTrueElecNYHit[20];
    for(int i=0;i<20;i++){
      evtTrueElecEnergy[i]=0;
      evtTrueXElecEnergy[i]=0;
      evtTrueYElecEnergy[i]=0;
      evtTrueElecNXHit[i]=0;
      evtTrueElecNYHit[i]=0;
    }
    int evtTrueNTrk = 0;

    int mcstudy = 1; 




    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
      for (unsigned int kk=0;kk<plist.size();kk++){    
        TLorentzVector ntp(0,0,0,0);                   
        TVector3 ntvtx(0,0,0);                         
        TVector3 ntpxz(0,0,0);                         
        TVector3 ntpyz(0,0,0);                         
        int ntpdg =0;                                  
        int ntmother = -1;                             
//        if(plist[kk]->Mother()!=0)continue;          
        ntpdg = plist[kk]->PdgCode();                  
        ntp = plist[kk]->Momentum();                   
        ntpxz.SetXYZ(ntp.Px(),0,ntp.Pz());             
        ntpyz.SetXYZ(0,ntp.Py(),ntp.Pz());             
        ntmother = plist[kk]->Mother();                
        ntvtx.SetXYZ(plist[kk]->Vx(),plist[kk]->Vy(),  plist[kk]->Vz());
        evtntpcol.push_back(ntp);                      
        evtntpxzcol.push_back(ntpxz);                  
        evtntpyzcol.push_back(ntpyz);                  
        evtntvtxcol.push_back(ntvtx);
        evtntpdgcol.push_back(ntpdg); 
        evtntmothercol.push_back(ntmother);
        if(plist[kk]->Mother()!=0)continue;
        if(abs(ntpdg)==12||abs(ntpdg)==14||abs(ntpdg)==16)continue;
        evt0ntpcol.push_back(ntp);
        evt0ntpxzcol.push_back(ntpxz);
        evt0ntpyzcol.push_back(ntpyz);
        evt0ntvtxcol.push_back(ntvtx);
        evt0ntpdgcol.push_back(ntpdg); 
        evt0ntmothercol.push_back(ntmother);
      }
    }

    TLorentzVector evt0ntp4tot(0,0,0,0);
    TLorentzVector evt0nthadp4tot(0,0,0,0);
    for(unsigned int i=0;i<evt0ntpcol.size();i++){
//      std::cout<<"pdg, p4 "<<evt0ntpdgcol[i]<<" "<<evt0ntpcol[i].E()<<std::endl;
      evt0ntp4tot+=evt0ntpcol[i];
      if(abs(evt0ntpdgcol[i])!=11)evt0nthadp4tot+=evt0ntpcol[i];
    }



    m_evtTruePF = 0; 
    m_evtTruePNu = 0; 
    m_evtTrueCosTheta = 0; 
    m_evtTruePt = 0; 
    m_evtTruePl = 0; 
    
    m_evtTruePF = evt0ntp4tot.Vect().Mag(); 
    m_evtTruePHad = evt0nthadp4tot.Vect().Mag(); 
    m_evtTruePNu = (evt0ntp4tot-evtTrueNuP4).Vect().Mag(); 
    m_evtTrueCosTheta = cos(evt0ntp4tot.Vect().Angle(evtTrueNuP4.Vect())); 
    m_evtTruePt = evt0ntp4tot.Vect().Mag()*sin(evt0ntp4tot.Vect().Angle(evtTrueNuP4.Vect()));
    m_evtTruePl = evt0ntp4tot.Vect().Mag()*cos(evt0ntp4tot.Vect().Angle(evtTrueNuP4.Vect()));

//    std::cout<<"ntp4tot "<<ntp4tot.Vect().Mag()<<std::endl;

    m_evtNSlice = 0;
    m_evtNSigSlice = 0;
    m_evtNTrk = 0;
    m_evtFTag = false;
    m_evtVtx[0] = 0;
    m_evtVtx[1] = 0;
    m_evtVtx[2] = 0;

    m_evtNXTrk = 0;
    m_evtNYTrk = 0;
    m_evtXSh1GeV = 0;
    m_evtYSh1GeV = 0;
    m_evtXSh1Plane = 0;
    m_evtYSh1Plane = 0;
/*
    m_evtXSheEnergy = 0;
    m_evtXSheDang = 0;
    m_evtXSheNCell = 0;
    m_evtXSheNPlane = 0;
    m_evtXSheMinPlane = 0;
    m_evtXSheMaxPlane = 0;
    m_evtXSheDir[0] = 0;
    m_evtXSheDir[1] = 0;
    m_evtXSheDir[2] = 0;

    m_evtYSheEnergy = 0;
    m_evtYSheDang = 0;
    m_evtYSheNCell = 0;
    m_evtYSheNPlane = 0;
    m_evtYSheMinPlane = 0;
    m_evtYSheMaxPlane = 0;
    m_evtYSheDir[0] = 0;
    m_evtYSheDir[1] = 0;
    m_evtYSheDir[2] = 0;
*/

    m_evtNXCluster = 0; 
    m_evtNYCluster = 0; 
    m_evtXClusterGeV = 0; 
    m_evtYClusterGeV = 0; 
    m_evtXClusterMaxNCell = 0;
    m_evtYClusterMaxNCell = 0;
    m_evtXClusterNCell = 0;
    m_evtYClusterNCell = 0;




    m_evtNSh = 0;
    m_evtTotalRecoGeV = 0; 
    m_evtTotalRecoNCell = 0; 
    m_evtPEtot = 0;
    m_evtTotalRecoGeV0 = 0;
    m_evtTotalRecoNCell0 = 0;
    m_evtPEtot0 = 0;



    m_evtANN = 0;
    m_evtEtot = 0;
    m_evtEtot0 = 0;
    m_evtTrueEtot = 0;
    m_evtTrueElecPhotons = 0;
    m_evtTrueNTrk = 0;

    for(int i=0;i<20;i++){
      m_evtTrueEnergy[i] = 0;
      m_evtTrueMother[i] = 0;
      m_evtTrueElecEnergy[i]=0;
      m_evtTrueXElecEnergy[i]=0;
      m_evtTrueYElecEnergy[i]=0;
      m_evtTrueElecNXHit[i]=0;
      m_evtTrueElecNYHit[i]=0;

    }

    m_evtTrueVtx[0] = 0;
    m_evtTrueVtx[1] = 0;
    m_evtTrueVtx[2] = 0;

    m_evtTrueDir[0] = 0;
    m_evtTrueDir[1] = 0;
    m_evtTrueDir[2] = 0;


    m_evtPi0Mgg0 = 0;
    m_evtPi0Mgg  = 0;
    m_evtPi0TrueDirMgg0 = 0;
    m_evtPi0TrueDirMgg = 0;
    m_evtPi0Eg1 = 0;
    m_evtPi0Eg2 = 0;
    m_evtPi0StartDist = 9999;
    m_evtPi0LineDist = 9999;
    m_evtPi0Vtx[0] = -1;
    m_evtPi0Vtx[1] = -1;
    m_evtPi0Vtx[2] = -1;


    m_evtSh1Energy = 0;
    m_evtSh1Energy0 = 0;
    m_evtSh1Length = 0;
    m_evtSh1TrkGeV = 0;
    m_evtSh1SliceGeV = 0;
    m_evtSh1SliceEdge10GeV = 0;
    m_evtSh1SliceEdge5GeV = 0;
    m_evtSh1SliceEdge2GeV = 0;
    m_evtSh1Gap = 0;
    m_evtSh1NTrk = 0;
    m_evtSh1IsFid = -2;
    m_evtSh1DistToEdge =0;
    m_evtSh1P4[0] = 0;
    m_evtSh1P4[1] = 0;
    m_evtSh1P4[2] = 0;
    m_evtSh1P4[3] = 0;

    m_evtSh1Dir[0] = 0;
    m_evtSh1Dir[1] = 0;
    m_evtSh1Dir[2] = 0;


    m_evtSh1Start[0] = 0;
    m_evtSh1Start[1] = 0;
    m_evtSh1Start[2] = 0;

    m_evtSh1Stop[0] = 0;
    m_evtSh1Stop[1] = 0;
    m_evtSh1Stop[2] = 0;

    m_evtSh1MinPlane = -1;
    m_evtSh1MaxPlane = -1;

    m_evtSh1TrueDang = 0;
    m_evtSh1TrueEDang = 0;
    m_evtSh1TruePdg = 0;
    m_evtSh1TrueT = 0;
    m_evtSh1TrueId = -1;
    m_evtSh1TrueVtx[0] = -1;
    m_evtSh1TrueVtx[1] = -1;
    m_evtSh1TrueVtx[2] = -1;
    m_evtSh1TrueP4[0] = 0;
    m_evtSh1TrueP4[1] = 0;
    m_evtSh1TrueP4[2] = 0;
    m_evtSh1TrueP4[3] = 0;

    m_evtSh1TrueMotherPdg = 0;
    m_evtSh1TrueMotherId = -1;
    m_evtSh1TrueMotherP4[0] = 0;
    m_evtSh1TrueMotherP4[1] = 0;
    m_evtSh1TrueMotherP4[2] = 0;
    m_evtSh1TrueMotherP4[3] = 0;

    m_evtSh1TrueNavMotherPdg = 0;
    m_evtSh1TrueNavMotherId = -1;
    m_evtSh1TrueNavMotherP4[0] = 0;
    m_evtSh1TrueNavMotherP4[1] = 0;
    m_evtSh1TrueNavMotherP4[2] = 0;
    m_evtSh1TrueNavMotherP4[3] = 0;


    m_evtSh1TrueElecPhotons = 0;
    m_evtSh1TrueTotPhotons = 0;
    m_evtSh1TrueTrkElecPhotons = 0;
    m_evtSh1TrueTrkTotPhotons = 0;
    m_evtSh1TrueRWElecPhotons = 0;
    m_evtSh1TrueRWTotPhotons = 0;
    m_evtSh1TrueTrkRWElecPhotons = 0;
    m_evtSh1TrueTrkRWTotPhotons = 0;



    for(int itype=0;itype<20;itype++){
      m_evtSh1DedxLLL[itype] = -9999;
      m_evtSh1DedxLLT[itype] = -9999;
    }
    m_evtSh1DedxANN = 0;
    m_evtSh1DedxANN1 = 0;
    m_evtSh1Moment12 = 0;
    m_evtSh1Asym = 0;
    m_evtSh1Ixx = 0;
    m_evtSh1Iyy = 0;
    m_evtSh1NPlane = 0;
    m_evtSh1NZeroPlane = 0;

    m_evtSh1ExtentX = 0; 
    m_evtSh1ExtentY = 0;



    m_evtSh1NXCell = 0;
    m_evtSh1NYCell = 0;

    m_evtSh1NMIPPlane = 0;
    m_evtSh1VtxGeV = 0;
    m_evtSh1Pi0Mgg = 0;

    m_evtSh1Ee = 0;
    m_evtSh1CosTheta = 0;
    m_evtSh1Mh = 0;
    m_evtSh1MhTrue = 0;
    m_evtSh1Meff = 0;
    m_evtSh1TrueSumHadM = 0;
    m_evtSh1TrueSumHadNtrk = 0;
    m_evtSh1Enue = 0;
    m_evtSh1TruePt = 0;
    m_evtSh1TruePl = 0;
    m_evtSh1TrueSumHadPt = 0;
    m_evtSh1TrueSumHadPl = 0;
    m_evtSh1TrueSumHadEnue = 0;
    m_evtSh1CosTrueTheta = 0;
    m_evtSh1CosTrueHadTheta = 0;
    m_evtSh1TrueHadEnue = 0;
    m_evtSh1TrueEnue = 0;
    m_evtSh1SumHadP=0;
    m_evtSh1SumPt=0;
    m_evtSh1SumP=0;
    m_evtSh1P =0;


    m_evtSh2Gap = 0;
    m_evtSh2Pi0Mgg = 0;
    m_evtSh2SliceGeV = 0;
    m_evtSh2VtxGeV = 0;
    m_evtSh2Energy = 0;
    m_evtSh2TrkGeV = 0;
    m_evtSh2IsFid = -2;
    m_evtSh2P4[0] = 0;
    m_evtSh2P4[1] = 0;
    m_evtSh2P4[2] = 0;
    m_evtSh2P4[3] = 0;
    m_evtSh2Dir[0] = 0;
    m_evtSh2Dir[1] = 0;
    m_evtSh2Dir[2] = 0;
    m_evtSh2Start[0] = 0;
    m_evtSh2Start[1] = 0;
    m_evtSh2Start[2] = 0;
    m_evtSh2Stop[0] = 0;
    m_evtSh2Stop[1] = 0;
    m_evtSh2Stop[2] = 0;
    m_evtSh2TrueDang = 0;
    m_evtSh2TruePdg = 0;
    m_evtSh2TrueT = 0;
    m_evtSh2TrueId = 0;
    m_evtSh2TrueVtx[0] = -1;
    m_evtSh2TrueVtx[1] = -1;
    m_evtSh2TrueVtx[2] = -1;
    m_evtSh2TrueP4[0] = 0;
    m_evtSh2TrueP4[1] = 0;
    m_evtSh2TrueP4[2] = 0;
    m_evtSh2TrueP4[3] = 0;
    m_evtSh2TrueMotherPdg = 0;
    m_evtSh2TrueMotherId = -1;
    m_evtSh2TrueMotherP4[0] = 0;
    m_evtSh2TrueMotherP4[1] = 0;
    m_evtSh2TrueMotherP4[2] = 0;
    m_evtSh2TrueMotherP4[3] = 0;

    m_evtSh2TrueNavMotherPdg = 0;
    m_evtSh2TrueNavMotherId = -1;
    m_evtSh2TrueNavMotherP4[0] = 0;
    m_evtSh2TrueNavMotherP4[1] = 0;
    m_evtSh2TrueNavMotherP4[2] = 0;
    m_evtSh2TrueNavMotherP4[3] = 0;

    for(int itype=0;itype<20;itype++){
      m_evtSh2DedxLLL[itype] = -9999;
      m_evtSh2DedxLLT[itype] = -9999;
    }
    m_evtSh2DedxANN = 0;
    m_evtSh2DedxANN1 = 0;
    m_evtSh2NPlane = 0;
    m_evtSh2NMIPPlane = 0;


    m_evtSh3Gap = 0;
    m_evtSh3Pi0Mgg = 0;
    m_evtSh3SliceGeV = 0;
    m_evtSh3VtxGeV = 0;
    m_evtSh3Energy = 0;
    m_evtSh3TrkGeV = 0;
    m_evtSh3IsFid = -2;
    m_evtSh3P4[0] = 0;
    m_evtSh3P4[1] = 0;
    m_evtSh3P4[2] = 0;
    m_evtSh3P4[3] = 0;
    m_evtSh3Dir[0] = 0;
    m_evtSh3Dir[1] = 0;
    m_evtSh3Dir[2] = 0;
    m_evtSh3Start[0] = 0;
    m_evtSh3Start[1] = 0;
    m_evtSh3Start[2] = 0;
    m_evtSh3Stop[0] = 0;
    m_evtSh3Stop[1] = 0;
    m_evtSh3Stop[2] = 0;
    m_evtSh3TrueDang = 0;
    m_evtSh3TruePdg = 0;
    m_evtSh3TrueT = 0;
    m_evtSh3TrueId = 0;
    m_evtSh3TrueVtx[0] = -1;
    m_evtSh3TrueVtx[1] = -1;
    m_evtSh3TrueVtx[2] = -1;
    m_evtSh3TrueP4[0] = 0;
    m_evtSh3TrueP4[1] = 0;
    m_evtSh3TrueP4[2] = 0;
    m_evtSh3TrueP4[3] = 0;
    m_evtSh3TrueMotherPdg = 0;
    m_evtSh3TrueMotherId = -1;
    m_evtSh3TrueMotherP4[0] = 0;
    m_evtSh3TrueMotherP4[1] = 0;
    m_evtSh3TrueMotherP4[2] = 0;
    m_evtSh3TrueMotherP4[3] = 0;
    for(int itype=0;itype<20;itype++){
      m_evtSh3DedxLLL[itype] = -9999;
      m_evtSh3DedxLLT[itype] = -9999;
    }
    m_evtSh3DedxANN = 0;
    m_evtSh3DedxANN1 = 0;
    m_evtSh3NPlane = 0;
    m_evtSh3NMIPPlane = 0;



    m_evtLen1DedxANN = 0;
    m_evtLen1DedxANN1 = 0;
    m_evtLen1Energy = 0;
    m_evtLen1TrkGeV = 0;
    m_evtLen1IsFid = -2;
    m_evtLen1P4[0] = 0;
    m_evtLen1P4[1] = 0;
    m_evtLen1P4[2] = 0;
    m_evtLen1P4[3] = 0;
    m_evtLen1Start[0] = 0;
    m_evtLen1Start[1] = 0;
    m_evtLen1Start[2] = 0;
    m_evtLen1Stop[0] = 0;
    m_evtLen1Stop[1] = 0;
    m_evtLen1Stop[2] = 0;

    m_evtLen1TrueDang = 0; 
    m_evtLen1TrueEDang = 0;
    m_evtLen1TrueId = 0;
    m_evtLen1TruePdg = 0;
    m_evtLen1TrueT = 0;
    m_evtLen1TrueVtx[0] = 0; 
    m_evtLen1TrueVtx[1] = 0;
    m_evtLen1TrueVtx[2] = 0;
    m_evtLen1TrueP4[0] = 0;
    m_evtLen1TrueP4[1] = 0;
    m_evtLen1TrueP4[2] = 0;
    m_evtLen1TrueP4[3] = 0;

    m_evtLen1TrueMotherPdg = 0; 
    m_evtLen1TrueMotherId = 0;
    m_evtLen1TrueMotherP4[0] = 0;
    m_evtLen1TrueMotherP4[1] = 0;
    m_evtLen1TrueMotherP4[2] = 0;
    m_evtLen1TrueMotherP4[3] = 0;
    m_evtLen1TrueElecPhotons = 0;
    m_evtLen1TrueTotPhotons = 0;
    m_evtLen1TrueRWElecPhotons = 0;
    m_evtLen1TrueRWTotPhotons = 0;



    for(int itype=0;itype<20;itype++){
      m_evtLen1DedxLLL[itype] = -9999;
      m_evtLen1DedxLLT[itype] = -9999;
    }

    m_evtLen1Energy0 = 0; 
    m_evtLen1VtxGeV = 0;
    m_evtLen1Pi0Mgg = 0;
    m_evtLen1SliceGeV = 0;
    m_evtLen1Gap = 0;


    m_evtLen1NPlane = 0;
    m_evtLen1NMIPPlane = 0;

    m_evtSheDedxANN = 0;
    m_evtSheDedxANN1 = 0;
    m_evtSheVtxGeV = 0;
    m_evtSheEnergy = 0;
    m_evtSheTrkGeV = 0;
    m_evtSheGap = 0;
    m_evtSheIsFid = -2;
    m_evtSheP4[0] = 0;
    m_evtSheP4[1] = 0;
    m_evtSheP4[2] = 0;
    m_evtSheP4[3] = 0;

    m_evtSheDir[0] = 0;
    m_evtSheDir[1] = 0;
    m_evtSheDir[2] = 0;

    m_evtSheStart[0] = 0;
    m_evtSheStart[1] = 0;
    m_evtSheStart[2] = 0;
    m_evtSheStop[0] = 0;
    m_evtSheStop[1] = 0;
    m_evtSheStop[2] = 0;

    m_evtSheTrueEDang = 0;
    m_evtSheTruePdg = 0;
    m_evtSheTrueT = 0;
    m_evtSheTrueId = 0;
    m_evtSheTrueVtx[0] = -1;
    m_evtSheTrueVtx[1] = -1;
    m_evtSheTrueVtx[2] = -1;
    m_evtSheTrueP4[0] = 0;
    m_evtSheTrueP4[1] = 0;
    m_evtSheTrueP4[2] = 0;
    m_evtSheTrueP4[3] = 0;
    m_evtSheTrueElecPhotons = 0;
    m_evtSheTrueTotPhotons = 0;
    m_evtSheTrueTrkElecPhotons = 0;
    m_evtSheTrueTrkTotPhotons = 0;

    m_evtSheTrueRWElecPhotons = 0;
    m_evtSheTrueRWTotPhotons = 0;
    m_evtSheTrueTrkRWElecPhotons = 0;
    m_evtSheTrueTrkRWTotPhotons = 0;



    int intE = -1;
    for(int i = 0; i < (int)evtntpdgcol.size(); i++){
      if(i>=20)break;
//      std::cout<<"i, pdg, "<<i<<" "<<evtntpdgcol[i]<<std::endl;
      if(fabs(evtntpdgcol[i])!=11)continue;
      if(evtntpcol[i].Vect().Mag()==0)continue;

      intE = i;
      m_evtTruePdg[0] = evtntpdgcol[i];
      m_evtTrueEnergy[0] = evtntpcol[i].Vect().Mag();
      m_evtTrueMother[0] = evtntmothercol[i];

      m_evtTrueVtx[0] = evtntvtxcol[i][0];
      m_evtTrueVtx[1] = evtntvtxcol[i][1];
      m_evtTrueVtx[2] = evtntvtxcol[i][2];
      m_evtTrueDir[0] = evtntpcol[i].Px()/evtntpcol[i].Vect().Mag();
      m_evtTrueDir[1] = evtntpcol[i].Py()/evtntpcol[i].Vect().Mag();
      m_evtTrueDir[2] = evtntpcol[i].Pz()/evtntpcol[i].Vect().Mag();
      m_evtTrueElecEnergy[0] = evtTrueElecEnergy[0];
      m_evtTrueXElecEnergy[0] = evtTrueXElecEnergy[0];
      m_evtTrueYElecEnergy[0] = evtTrueYElecEnergy[0];
      m_evtTrueElecNXHit[0] = evtTrueElecNXHit[0];
      m_evtTrueElecNYHit[0] = evtTrueElecNYHit[0];
      break;
    }


    m_evtTrueEtot = trueTotalDepositEnergy;



//-----------------------------------------------------
//  Read  slice, hit and track;
//-----------------------------------------------------

    std::map<geo::OfflineChan, int> hitslicemap;

    std::set<art::Ptr<rb::CellHit> > cellhitlist;
    std::vector<const rb::Cluster* >   cluster;


    double totalRecoGeV = 0;
    double totalRecoGeV0 = 0;
    int totalRecoNCell = 0;
    int totalRecoNCell0 = 0;



    int electruetrkid = -1;
    TLorentzVector electruep4(0,0,0,0);
    std::vector<int> pi0truetrkidcol;
    pi0truetrkidcol.clear();
    std::vector<int> pi0g1photonscol;
    pi0g1photonscol.clear();
    std::vector<int> pi0g2photonscol;
    pi0g2photonscol.clear();
    std::vector<int> pi0g1truetrkidcol;
    pi0g1truetrkidcol.clear();
    std::vector<int> pi0g2truetrkidcol;
    pi0g2truetrkidcol.clear();



    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){ 
      for( int i = 0; i < (int)plist.size(); i++ ){
        const sim::Particle* part = plist[i];
        TVector3 pvtx(part->Vx(),part->Vy(),part->Vz());
        double pvtxdist = (pvtx-evtTrueNuVtx).Mag();
//        int motherID = pnav.EveId(part->TrackId());
//        std::cout<<"dst to vtx, pdg, trkID, mother, navmother, energy "<<pvtxdist<<" "<<part->PdgCode()<<" "<<part->TrackId()<<" "<<part->Mother()<<" "<<motherID<<" "<<part->Momentum().E()<<" "<<std::endl;
        if(pvtxdist<0.001){
          if(abs(part->PdgCode()) == 11 && part->Mother()==0 ) evtTrueNTrk++;
          if(abs(part->PdgCode()) == 13 && part->Mother()==0) evtTrueNTrk++;
          if(abs(part->PdgCode()) == 211 ) evtTrueNTrk++;
          if(abs(part->PdgCode()) == 321 ) evtTrueNTrk++;
          if(abs(part->PdgCode()) == 2212 ) evtTrueNTrk++;
          if(abs(part->PdgCode()) == 111  ) evtTrueNTrk = evtTrueNTrk+2;
        }
        if(abs(part->PdgCode()) == 11 && part->Mother()==0){
//          std::cout<<"pnav elec. pdg, trkid, motherid "<<part->PdgCode()<<" "<<part->TrackId()<<" "<<part->Mother()<<std::endl;
          electruetrkid = part->TrackId();
          electruep4 = part->Momentum();
        }
        if(abs(part->PdgCode()) == 111 && part->Mother()==0){
//          std::cout<<"pnav pi0. pdg, trkid, motherid "<<part->PdgCode()<<" "<<part->TrackId()<<" "<<part->Mother()<<std::endl;
          pi0truetrkidcol.push_back(part->TrackId());
          pi0g1photonscol.push_back(0);
          pi0g2photonscol.push_back(0);
          pi0g1truetrkidcol.push_back(-1);
          pi0g2truetrkidcol.push_back(-1); 
        }
      }
   
      for( int i = 0; i < (int)plist.size(); i++ ){
        const sim::Particle* part = plist[i];
        if(part->PdgCode()!=22)continue;
        int motherID = pnav.EveId(part->TrackId());
        TVector3 pvtx(part->Vx(),part->Vy(),part->Vz());
//        double pvtxdist = (pvtx-evtTrueNuVtx).Mag();
//        if(pvtxdist>0.001)continue;
        for( int j =0; j< (int)pi0truetrkidcol.size();j++){
          if( motherID==pi0truetrkidcol[j] && pi0g1truetrkidcol[j]==-1){
            pi0g1truetrkidcol[j] = part->TrackId();
          }
        }
      }

      for( int i = 0; i < (int)plist.size(); i++ ){
        const sim::Particle* part = plist[i];
        if(part->PdgCode()!=22&&fabs(part->PdgCode())!=11)continue;
        int motherID = pnav.EveId(part->TrackId());
        TVector3 pvtx(part->Vx(),part->Vy(),part->Vz());
//        double pvtxdist = (pvtx-evtTrueNuVtx).Mag();
//        if(pvtxdist>0.001)continue;
        for(int j =0; j<(int)pi0truetrkidcol.size();j++){
          if(motherID==pi0truetrkidcol[j]&&(int)part->TrackId()!=pi0g1truetrkidcol[j] && pi0g2truetrkidcol[j]==-1)          {
            pi0g2truetrkidcol[j] = part->TrackId();
          }
        }
      }
    }

    std::cout<<"we have true multiplicity "<<evtTrueNTrk<<std::endl;
    m_evtTrueNTrk = evtTrueNTrk;
    std::cout<<"n pi0 "<<pi0truetrkidcol.size()<<std::endl;



    int evtTrueElecPhotons = 0;

    art::Ptr<rb::Cluster>slicelist(slicecol,sliceIdx);
    int nSlice = slicecol->size();
    double evtpetot = 0;
    double evtpetot0 = 0;

    TVector3 evtVtx(-1,-1,-1); 

    int nNoiseSlice =0;
      art::Ptr<rb::Cluster> clust = slicelist;
      std::cout<<"slice ic, n cell "<<sliceIdx<<" "<<clust->NCell()<<std::endl;

      //get verticies associated with each slice
    art::FindManyP<rb::Vertex> fmv(slicecol, evt, "elasticarmshs");
    //NOTE: if there is no vertex, then there are no tracks, cannot do EID, so return
    //This is a form of "preselection"
    if ((fmv.isValid())){ 
      for(unsigned int ic = 0; ic < slicecol->size(); ++ic){
        //if there was nue preselection, skip slice if it failed
        art::Ptr<rb::Cluster> clust(slicecol,ic);
        std::vector<art::Ptr<rb::Vertex> >  v = fmv.at(ic);
        //loop over verticies associated with slice
        for (unsigned int j=0; j<v.size(); ++j) {
          TVector3 evtVtx(0,0,0);
//          unsigned int vtxPlane = 0;
          //store vertex coordinates
          evtVtx.SetX(v[j]->GetX());
          evtVtx.SetY(v[j]->GetY());
          evtVtx.SetZ(v[j]->GetZ());
          double dmin = 9999999;
          //find plane number closest to vertex
          for(unsigned int pp = 0; pp < geom->NPlanes();pp++){
            Double_t cellXYZ[3]={0,0,0};
            geom->Plane(pp)->Cell(0)->GetCenter(cellXYZ);
            double mind = fabs(cellXYZ[2]-evtVtx[2]);
            if(mind<=dmin){
              dmin=mind;
//              vtxPlane = pp;
            }
          }
        }
      }
    }
/*
      std::vector<art::Ptr<rb::Vertex> >  v = fmv.at(ic);
      std::cout<<"slice ic, n vtx "<<ic<<" "<<v.size()<<std::endl;

      for (int j=0; j<v.size(); ++j) {
//        std::cout<<"vtx x, y , z "<<v[j]->GetX()<<" "<<v[j]->GetY()<<" "<<v[j]->GetZ()<<std::endl;
        evtVtx.SetX(v[j]->GetX());
        evtVtx.SetY(v[j]->GetY());
        evtVtx.SetZ(v[j]->GetZ());
      }
*/
      const double wx = (clust->NXCell() > 0) ? clust->W(clust->XCell(0).get()) : 0;
      const double wy = (clust->NYCell() > 0) ? clust->W(clust->YCell(0).get()) : 0;

      for(unsigned int i=0;i<clust->NXCell();i++){
        geo::OfflineChan key(clust->XCell(i)->Plane(),clust->XCell(i)->Cell());
        const art::Ptr<rb::CellHit>& chit = clust->XCell(i);
        if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
        const std::vector< sim::PhotonSignal> pSignal = bt->CellToPhotonSignal(chit->Plane(), chit->Cell());
        truePartElec[key]=false;
            for( int p = 0; p < (int)pSignal.size(); p++){
              int partID = pSignal[p].TrackId();
              int motherID = pnav.EveId(partID);
              if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
                truePartElec[key]=true;
                evtTrueElecPhotons += pSignal[p].NPhoton();
              }
              for(int j =0; j<(int)pi0truetrkidcol.size();j++){
                if(motherID==pi0truetrkidcol[j]){
                  if(partID==pi0g1truetrkidcol[j]){
                    pi0g1photonscol[j] += pSignal[p].NPhoton();
                  }else{
                    pi0g2photonscol[j] += pSignal[p].NPhoton();
                  }
                }
              }
            }
        }

        const rb::RecoHit rhit(cal->MakeRecoHit(*chit, wx ));

        if(!rhit.IsCalibrated()){
          std::cout<<"slice, cell  "<<sliceIdx<<" "<<i<<"  Not calibrated?!" << std::endl;
          double adc = double(chit->ADC());
          double Att = 1;
          geo::View_t view = geom->Plane(chit->Plane())->View();
          if(view == geo::kX){
                  Att =   2.99569e+04 //121116
                          -6.15844e+01*wx
                          +6.19731e-02*pow(wx,2)
                          -3.04255e-05*pow(wx,3)
                          +5.99059e-09*pow(wx,4);

          }else if(view == geo::kY){
                  Att =    2.97612e+04//121116 
                          -5.94945e+01*wy
                          +5.64589e-02*pow(wy,2)
                          -2.52809e-05*pow(wy,3)
                          +4.43062e-09*pow(wy,4);
          }
          double gev = adc/Att;
          totalRecoGeV0 += gev;
          if((clust->XCell(i)->PE())>pCellHitPHThresh ){
            totalRecoGeV += gev;
            totalRecoNCell++;
            hitslicemap[key] = sliceIdx;
            cellhitlist.insert(clust->XCell(i));
            evtpetot+=clust->XCell(i)->PE();
          }
        }
        else{
          totalRecoGeV0 += rhit.GeV();
          if((clust->XCell(i)->PE())>pCellHitPHThresh ){
            totalRecoGeV += rhit.GeV();
            totalRecoNCell++;
            hitslicemap[key] = sliceIdx;
            cellhitlist.insert(clust->XCell(i));
            evtpetot+=clust->XCell(i)->PE();
          }
        }
        totalRecoNCell0++;
        evtpetot0+=clust->XCell(i)->PE();
      }
      
      for(unsigned int i=0;i<clust->NYCell();i++){
        geo::OfflineChan key(clust->YCell(i)->Plane(),clust->YCell(i)->Cell());
        const art::Ptr<rb::CellHit>& chit = clust->YCell(i);

        if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
            const std::vector< sim::PhotonSignal> pSignal = bt->CellToPhotonSignal(chit->Plane(), chit->Cell());
            truePartElec[key]=false;
            for( int p = 0; p < (int)pSignal.size(); p++){
              int partID = pSignal[p].TrackId();
              int motherID = pnav.EveId(partID);
              if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
                truePartElec[key]=true;
                evtTrueElecPhotons += pSignal[p].NPhoton();
              }
              for(int j =0; j<(int)pi0truetrkidcol.size();j++){
                if(motherID==pi0truetrkidcol[j]){
                  if(partID==pi0g1truetrkidcol[j]){
                    pi0g1photonscol[j] += pSignal[p].NPhoton();
                  }else{
                    pi0g2photonscol[j] += pSignal[p].NPhoton();
                  }
                }
              }
            }
        }

        const rb::RecoHit rhit(cal->MakeRecoHit(*chit, wy));

        if(!rhit.IsCalibrated()){
          std::cout<<"slice, cell  "<<sliceIdx<<" "<<i<<"  Not calibrated?!" << std::endl;
          double adc = double(chit->ADC());
          double Att = 1;
          geo::View_t view = geom->Plane(chit->Plane())->View();
          if(view == geo::kX){
                  Att =   2.99569e+04 //121116
                          -6.15844e+01*wx
                          +6.19731e-02*pow(wx,2)
                          -3.04255e-05*pow(wx,3)
                          +5.99059e-09*pow(wx,4);

          }else if(view == geo::kY){
                  Att =    2.97612e+04//121116 
                          -5.94945e+01*wy
                          +5.64589e-02*pow(wy,2)
                          -2.52809e-05*pow(wy,3)
                          +4.43062e-09*pow(wy,4);
          }
          double gev = adc/Att;
          totalRecoGeV0 += gev;

          if((clust->YCell(i)->PE())>pCellHitPHThresh ){
            totalRecoGeV += gev;
            totalRecoNCell++;
            hitslicemap[key] = sliceIdx;
            cellhitlist.insert(clust->YCell(i));
            evtpetot+=clust->YCell(i)->PE();
          }
        }else{
           if((clust->YCell(i)->PE())>pCellHitPHThresh ){
             totalRecoGeV += rhit.GeV();
             totalRecoNCell++;
             hitslicemap[key] = sliceIdx;
             cellhitlist.insert(clust->YCell(i));
             evtpetot+=clust->YCell(i)->PE();
           }
           totalRecoGeV0 += rhit.GeV();
        }
        totalRecoNCell0++;
        evtpetot0+=clust->YCell(i)->PE();
      }



      m_evtTrueElecPhotons = evtTrueElecPhotons;



    if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
      for(std::set<art::Ptr<rb::CellHit>>::iterator dItr = cellhitlist.begin(); dItr!=cellhitlist.end(); dItr++){
        std::vector<sim::FLSHit> tHits = bt->HitToFLSHit((*dItr));
        for(unsigned int n = 0; n < tHits.size(); n++){
          sim::FLSHit fls = tHits[n];
          // Replaced += with =, on the theory that there's only one FLS hit per
          // cell, or at least that it's confusing otherwise
          geo::OfflineChan key(fls.GetPlaneID(), fls.GetCellID());
          geo::View_t view = geom->Plane(fls.GetPlaneID())->View();


  //        trueDepositEnergyTmp[key] = fls.GetEntryEnergy()-fls.GetExitEnergy();
          trueDepositEnergyTmp[key] += fls.GetEdep(); 
  //        trueDepositEnergy[key] = fls.GetEntryEnergy()-fls.GetExitEnergy();
          trueDepositEnergy[key] += fls.GetEdep(); 
          trueDepositTime[key] += fls.GetEntryT()*fls.GetEdep(); 
//          trueDepositTime[key] += fls.GetT()*fls.GetEdep(); 
  //        trueTotalDepositEnergy += fls.GetEntryEnergy()-fls.GetExitEnergy();
          trueTotalDepositEnergy += fls.GetEdep();
  //  
          trueEnergy[key] = trueTotalDepositEnergy;//(fls.GetEntryEnergy()+fls.GetExitEnergy())/2;
          truePath[key] = util::pythag(fls.GetExitX()-fls.GetEntryX(),
                                       fls.GetExitY()-fls.GetEntryY(),
                                       fls.GetExitZ()-fls.GetEntryZ());
          trueAvgX[key] += (fls.GetEntryX()+fls.GetExitX())*(fls.GetEdep())/2;
          trueAvgY[key] += (fls.GetEntryY()+fls.GetExitY())*(fls.GetEdep())/2;
          trueAvgZ[key] += (fls.GetEntryZ()+fls.GetExitZ())*(fls.GetEdep())/2;
  //        if(fls.GetPDG() != 11) continue;
//          std::cout<<"N FLS, PDG, DepositE, fls.GetEdp,  Plane, CellID "<<n<<" "<<fls.GetPDG()<<" "<<fls.GetEntryEnergy()-fls.GetExitEnergy()<<" "<<fls.GetEdep()<<" "<<fls.GetPlaneID()<<" "<<fls.GetCellID()<<std::endl;
          int flssavetag = 1;
//          for(int ifl = 0; ifl<truePartPdg[key].size(); ifl++){
//            if(fls.GetPDG()==truePartPdg[key][ifl])flssavetag=0;
//          }
          if(flssavetag==1){
            truePartPdg[key].push_back(fls.GetPDG());
            truePartEnergy[key].push_back(fls.GetEdep());
            if(fls.GetPDG()==11){
              evtTrueElecEnergy[0]+=fls.GetEdep();
              if(view == geo::kX){
                 evtTrueXElecEnergy[0]+=fls.GetEdep();
                 evtTrueElecNXHit[0]++;
              }else if(view == geo::kY){
                 evtTrueYElecEnergy[0]+=fls.GetEdep();
                 evtTrueElecNYHit[0]++;
              }

            }
          }
  //  
  //        // TODO - probably gets the wrong view...
  //        fdEdxTrue[fls.GetPlaneID()%2]->Fill(1000*trueEnergy[key],
  //                                        1000*trueDepositEnergy[key]/truePath[key]);
          if(fls.GetTrackID() < mc->NParticles()&&fls.GetTrackID()>0){
            const simb::MCParticle& part = mc->GetParticle(fls.GetTrackID());
//            std::cout<<"trk PDG, cell PDG "<<part.PdgCode()<<" "<<fls.GetPDG()<<std::endl; 
            const int endIdx = part.NumberTrajectoryPoints()-1;
            if(endIdx >= 0){
              trueEndX[key] = part.Vx(endIdx);
              trueEndY[key] = part.Vy(endIdx);
              trueEndZ[key] = part.Vz(endIdx);
            }
          }
        }
      }
    }    

/*      
      for(unsigned int ii=0;ii<clust->NCell();ii++){
       unsigned int keym1Cell = clust->Cell(ii)->Cell();
       unsigned int keym2Cell = clust->Cell(ii)->Cell();
       unsigned int keyp1Cell = clust->Cell(ii)->Cell();
       unsigned int keyp2Cell = clust->Cell(ii)->Cell();
       if(clust->Cell(ii)->Cell()-1>=0)
         keym1Cell = clust->Cell(ii)->Cell() - 1; 
       if(clust->Cell(ii)->Cell()-2>=0)
         keym2Cell = clust->Cell(ii)->Cell() - 2;
       if(clust->Cell(ii)->Cell()+1<geom->Plane(clust->Cell(ii)->Plane())->Ncells())
         keyp1Cell = clust->Cell(ii)->Cell() + 1;
       if(clust->Cell(ii)->Cell()+2<geom->Plane(clust->Cell(ii)->Plane())->Ncells())
         keyp2Cell = clust->Cell(ii)->Cell() + 2;
       geo::OfflineChan key(clust->Cell(ii)->Plane(), clust->Cell(ii)->Cell());
       geo::OfflineChan keym1(clust->Cell(ii)->Plane(), keym1Cell);
       geo::OfflineChan keym2(clust->Cell(ii)->Plane(), keym2Cell);
       geo::OfflineChan keyp1(clust->Cell(ii)->Plane(), keyp1Cell);
       geo::OfflineChan keyp2(clust->Cell(ii)->Plane(), keyp2Cell);


       double cellXYZ[3]; 
       geom->Plane(clust->Cell(ii)->Plane())->Cell(clust->Cell(ii)->Cell())->GetCenter(cellXYZ);
       double gev = 0;
       geo::View_t view = geom->Plane(clust->Cell(ii)->Plane())->View();
       if(view == geo::kX){
         m_cellView = 0;
       }else if(view == geo::kY){
         m_cellView = 1;
       }
       m_cellRun = run;
       m_cellSubRun = srun;
       m_cellEvent = event;
       m_cellSubEvent = (int)sliceIdx;
       m_cellPlaneId = clust->Cell(ii)->Plane();
       m_cellId = clust->Cell(ii)->Cell();
       m_cellXYZ[0] = cellXYZ[0];
       m_cellXYZ[1] = cellXYZ[1];
       m_cellXYZ[2] = cellXYZ[2];
       m_cellADC  = clust->Cell(ii)->ADC();
       m_cellTDC  = clust->Cell(ii)->TDC();
       m_cellTNS  = clust->Cell(ii)->TNS();
       m_cellTrueEnergy  = trueDepositEnergy[key];
       m_cellEnergy = clust->Cell(ii)->PE();
       m_cellNSlice = nSlice;
       m_cellNTrk = 1;//svcshowercol.size(); 
       m_cellSlice = hitslicemap[key]; 

       m_cellTrueTime = 0; 
       m_cellTrueHitXYZ[0] = 0; 
       m_cellTrueHitXYZ[1] = 0; 
       m_cellTrueHitXYZ[2] = 0; 

       if(trueDepositEnergy[key]>0.000001){
         m_cellTrueTime = trueDepositTime[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[0] = trueAvgX[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[1] = trueAvgY[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[2] = trueAvgZ[key]/trueDepositEnergy[key];
       }
       fCell->Fill();
      }
*/
    m_evtVtx[0]=evtVtx[0];
    m_evtVtx[1]=evtVtx[1];
    m_evtVtx[2]=evtVtx[2];

    m_evtPEtot = evtpetot;
    m_evtTotalRecoGeV = totalRecoGeV;
    m_evtTotalRecoNCell = totalRecoNCell;

    m_evtPEtot0 = evtpetot0;
    m_evtTotalRecoGeV0 = totalRecoGeV0;
    m_evtTotalRecoNCell0 = totalRecoNCell0;


    m_evtFTag = ftag;
    m_evtNSlice = nSlice;
    m_evtNSigSlice = nSlice - nNoiseSlice;

//    return;
//   art::FindManyP<rb::Prong> fkt(slicecol, evt, "recojmshower");
//   std::vector<art::Ptr<rb::Prong> > track = fkt.at(sliceIdx);


//After read slice, hit and tracks.

/*
//    art::Handle< std::vector<rb::Cluster> > mcclusthandle;
//    evt.getByLabel("makeclusterss", mcclusthandle);
    art::Handle< std::vector<rb::Prong> > mcclusthandle;
    evt.getByLabel("kalmantrack", mcclusthandle);
    std::vector<art::Ptr<rb::Cluster> >  mccluster;
    std::vector<art::Ptr<rb::Cluster> >  mcxzcluster;
    std::vector<art::Ptr<rb::Cluster> >  mcyzcluster;

    for(unsigned int i = 0; i < mcclusthandle->size(); i++){
      art::Ptr<rb::Cluster> p(mcclusthandle, i);
      mccluster.push_back(p);
    }
    double xtotgev = 0;
    double ytotgev = 0;
    int xtotnhit = 0;
    int ytotnhit = 0;
    int xmaxnhit = 0;
    int ymaxnhit = 0;

    for(unsigned int i=0;i<mccluster.size();i++){
      if (mccluster[i]->NXCell() > 0){
        mcxzcluster.push_back(mccluster[i]);
        xtotgev+=mccluster[i]->TotalGeV();
        xtotnhit+=mccluster[i]->NXCell();
        if(xmaxnhit<mccluster[i]->NCell()){
          xmaxnhit = mccluster[i]->NCell();
        }
      }
      if (mccluster[i]->NYCell() > 0){
        mcyzcluster.push_back(mccluster[i]);
        ytotgev+=mccluster[i]->TotalGeV();
        ytotnhit+=mccluster[i]->NYCell();
        if(ymaxnhit<mccluster[i]->NCell()){
          ymaxnhit = mccluster[i]->NCell();
        }
      }
    }

    m_evtNXCluster = mcxzcluster.size();
    m_evtNYCluster = mcyzcluster.size();
    m_evtXClusterGeV = xtotgev;
    m_evtYClusterGeV = ytotgev;
    m_evtXClusterMaxNCell = xmaxnhit;
    m_evtYClusterMaxNCell = ymaxnhit;
    m_evtXClusterNCell = xtotnhit;
    m_evtYClusterNCell = ytotnhit;
*/

/*
    art::Handle< std::vector<rb::Prong> > cmtrackhandle;
    evt.getByLabel("clustermerge",cmtrackhandle);
//    evt.getByLabel("kalmantrack",cmtrackhandle);
//    evt.getByLabel(pTrackDir, cmtrackhandle);
    std::vector<art::Ptr<rb::Prong> >  cmtrack;
    std::vector<art::Ptr<rb::Prong> >  cmxzTrack;
    std::vector<art::Ptr<rb::Prong> >  cmyzTrack;
    for(unsigned int i = 0; i < cmtrackhandle->size(); i++){
      art::Ptr<rb::Prong> p(cmtrackhandle, i);
      cmtrack.push_back(p);
    }
    int ixsh1 = -1;
    int iysh1 = -1;
    double xsh1gev = 0;
    double ysh1gev = 0;
    unsigned int  xsh1plane = 0;
    unsigned int  ysh1plane = 0;

    int ixshe = -1; 
    int iyshe = -1; 
    double ntdangxz=200;
    double ntdangyz=200;
    for(unsigned int i=0;i<cmtrack.size();i++){
      TVector3 trkdir=cmtrack[i]->Dir();
      if (cmtrack[i]->NXCell() > 0){
        cmxzTrack.push_back(cmtrack[i]);
        if(xsh1gev<cmtrack[i]->TotalGeV()){
          ixsh1 = i;
          xsh1gev=cmtrack[i]->TotalGeV();
          xsh1plane=cmtrack[i]->ExtentPlane();
        }

        if(intE!=-1){
          double ntangdxz = trkdir.Angle(evtntpxzcol[intE]);
          ntangdxz=fmod(ntangdxz, 2*3.14159265);
          if(ntangdxz < ntdangxz){
            ntdangxz = ntangdxz;
            ixshe = i;
          }
        }
      }
      if (cmtrack[i]->NYCell() > 0){
        cmyzTrack.push_back(cmtrack[i]);
        if(ysh1gev<cmtrack[i]->TotalGeV()){
          iysh1 = i;
          ysh1gev=cmtrack[i]->TotalGeV();
          ysh1plane=cmtrack[i]->ExtentPlane();
        }
        if(intE!=-1){
          double ntangdyz = trkdir.Angle(evtntpyzcol[intE]);
          ntangdyz=fmod(ntangdyz, 2*3.14159265);
          if(ntangdyz < ntdangyz){
            ntdangyz = ntangdyz;
            iyshe = i;
          }
        }

      }
    }

    m_evtNXTrk = cmxzTrack.size();
    m_evtNYTrk = cmyzTrack.size();
    m_evtXSh1GeV = xsh1gev;
    m_evtYSh1GeV = ysh1gev;
    m_evtXSh1Plane = xsh1plane;
    m_evtYSh1Plane = ysh1plane;

    if(ixshe!=-1){
      m_evtXSheEnergy = cmtrack[ixshe]->TotalGeV();
      m_evtXSheDang = ntdangxz*180/3.14159265;
      m_evtXSheNCell = cmtrack[ixshe]->NCell();
      m_evtXSheNPlane = cmtrack[ixshe]->ExtentPlane();
      m_evtXSheMinPlane = cmtrack[ixshe]->MinPlane();
      m_evtXSheMaxPlane = cmtrack[ixshe]->MaxPlane();
      m_evtXSheDir[0] = cmtrack[ixshe]->Dir()[0];
      m_evtXSheDir[1] = cmtrack[ixshe]->Dir()[1];
      m_evtXSheDir[2] = cmtrack[ixshe]->Dir()[2];
    }

    if(iyshe!=-1){
      m_evtYSheEnergy = cmtrack[iyshe]->TotalGeV();
      m_evtYSheDang = ntdangyz*180/3.14159265;
      m_evtYSheNCell = cmtrack[iyshe]->NCell();
      m_evtYSheNPlane = cmtrack[iyshe]->ExtentPlane();
      m_evtYSheMinPlane = cmtrack[iyshe]->MinPlane();
      m_evtYSheMaxPlane = cmtrack[iyshe]->MaxPlane();
      m_evtYSheDir[0] = cmtrack[iyshe]->Dir()[0];
      m_evtYSheDir[1] = cmtrack[iyshe]->Dir()[1];
      m_evtYSheDir[2] = cmtrack[iyshe]->Dir()[2];
    }
*/
//    fEvent->Fill();
//    return;
// Determination of start point, end point and direction
//  Fill true information of showers
   std::vector<double> ntdangcol;
   std::vector<TLorentzVector> ntpcol;
   std::vector<TVector3> ntvtxcol;
   std::vector<int> ntidcol;
   std::vector<int> ntpdgcol;
   std::vector<double> nttcol;
   std::vector<TLorentzVector> ntmotherpcol;
   std::vector<int> ntmotherpdgcol;
   std::vector<int> ntmotheridcol;
   std::vector<TLorentzVector> ntnavmotherpcol;
   std::vector<int> ntnavmotherpdgcol;
   std::vector<int> ntnavmotheridcol;

   std::vector<int>  ntelecphotonscol;
   std::vector<int>  nttotphotonscol;
   std::vector<int>  nttrkelecphotonscol;
   std::vector<int>  nttrktotphotonscol;

   std::vector<int>  ntrwelecphotonscol;
   std::vector<int>  ntrwtotphotonscol;
   std::vector<int>  nttrkrwelecphotonscol;
   std::vector<int>  nttrkrwtotphotonscol;

   std::vector<double>  ntg1ratiocol;
   std::vector<double>  ntg2ratiocol;
   std::vector<int>  ntg1photonscol;
   std::vector<int>  ntg2photonscol;

   std::vector<double>  ntrwg1ratiocol;
   std::vector<double>  ntrwg2ratiocol;
   std::vector<int>  ntrwg1photonscol;
   std::vector<int>  ntrwg2photonscol;


   ntdangcol.clear();
   ntpcol.clear();
   ntvtxcol.clear();
   ntpdgcol.clear();
   nttcol.clear();
   ntidcol.clear();
   ntmotheridcol.clear();
   ntmotherpdgcol.clear();
   ntmotherpcol.clear();
   ntnavmotheridcol.clear();
   ntnavmotherpdgcol.clear();
   ntnavmotherpcol.clear();

   ntelecphotonscol.clear();
   nttotphotonscol.clear();
   nttrkelecphotonscol.clear();
   nttrktotphotonscol.clear();

   ntrwelecphotonscol.clear();
   ntrwtotphotonscol.clear();
   nttrkrwelecphotonscol.clear();
   nttrkrwtotphotonscol.clear();

   ntg1ratiocol.clear();
   ntg2ratiocol.clear();
   ntg1photonscol.clear();
   ntg2photonscol.clear();

   ntrwg1ratiocol.clear();
   ntrwg2ratiocol.clear();
   ntrwg1photonscol.clear();
   ntrwg2photonscol.clear();

   std::vector<art::Ptr<jmshower::JMShower> > svcshowercol;
   svcshowercol.clear();

   art::FindManyP<jmshower::JMShower> jms(slicecol, evt, "recojmshower");
   if(slicecol.isValid()){
     svcshowercol = jms.at(sliceIdx);
   }

   std::cout<<"slice nsh, evt nsh "<<svcshowercol.size()<<" "<<std::endl;

    m_evtNTrk = svcshowercol.size();

/*
    if ((svcshowercol.size() < fMinTrack) || (svcshowercol.size() > fMaxTrack)){
      fEvent->Fill();
      return;
    }
*/

   int evtNSh = 0;
   for(unsigned int i = 0; i<svcshowercol.size(); i++){
     if(svcshowercol[i]->Energy()<0.3)continue;
     evtNSh++;
   }
//   m_evtNSh=evtsvcshowercol.size(); 
   m_evtNSh=evtNSh; 

   std::vector<std::vector<double>> showerPlaneEnergy;
   std::vector<std::vector<double>> showerTruePlaneEnergy;
   std::vector<std::vector<double>> showerTransCellEnergy;
   std::vector<std::vector<double>> showerTrueTransCellEnergy;
   showerPlaneEnergy.clear();
   showerTruePlaneEnergy.clear();
   showerTransCellEnergy.clear();
   showerTrueTransCellEnergy.clear();

   std::vector<double>  showersumhadpcol;
   std::vector<double>  showersumptcol;
   std::vector<double>  showersumpcol;
   std::vector<double>  showerpcol;
   std::vector<double>  showerMomentCol;
   std::vector<int>  showerNZeroPlaneCol;

   showersumhadpcol.clear();
   showersumptcol.clear();
   showersumpcol.clear();
   showerpcol.clear();
   showerMomentCol.clear();
   showerNZeroPlaneCol.clear();

 
   for(unsigned int i = 0; i < svcshowercol.size(); i++){
     std::vector<double> PlaneEnergy;
     std::vector<double> TruePlaneEnergy;
     std::vector<double> TransCellEnergy;
     std::vector<double> TrueTransCellEnergy;
     PlaneEnergy.clear();
     TruePlaneEnergy.clear();
     TransCellEnergy.clear();
     TrueTransCellEnergy.clear();

     for(unsigned int pp=0; pp<svcshowercol[i]->NPlane();pp++){
       PlaneEnergy.push_back(0);
       TruePlaneEnergy.push_back(0);
     }
     for(unsigned int tt=0; tt<=20;tt++){
       TransCellEnergy.push_back(0);
       TrueTransCellEnergy.push_back(0);
     }
     showerPlaneEnergy.push_back(PlaneEnergy);
     showerTruePlaneEnergy.push_back(TruePlaneEnergy);
     showerTransCellEnergy.push_back(TransCellEnergy);
     showerTrueTransCellEnergy.push_back(TrueTransCellEnergy);
   }

   int ishe = -1;
   int shephotons = 0;
//   double sheedang = 200;

   std::vector<std::map<geo::OfflineChan, double>>showerCellEnergyCol;
   showerCellEnergyCol.clear();

   for(unsigned int i = 0; i < svcshowercol.size(); i++){
//For cell energy check and mc truth 
     std::map<geo::OfflineChan, double> showerCellEnergy;
     showerCellEnergy.clear();
//     double showerRadiusA    = svcshowercol[i]->Radius();
//     double showerEnergyA    = svcshowercol[i]->Energy();
//     double showerEnergy0A    = svcshowercol[i]->DepositEnergy();

      TVector3 trkdir=svcshowercol[i]->Dir();
      TVector3 trkstart=svcshowercol[i]->Start();
      TVector3 trkstop=svcshowercol[i]->Stop();

      double ntdang= 200.0;
      TLorentzVector ntp(0,0,0,0);
      TVector3 ntvtx(0,0,0);
//      double ntdvtx=1000;
      int ntpdg =0;
      double ntt = 0;
      int ntid = -1;
      int ntmotherpdg = 0;
      int ntmotherid = -1;
      TLorentzVector ntmotherp(0,0,0,0);

      int ntnavmotherpdg = 0;
      int ntnavmotherid = -1;
      TLorentzVector ntnavmotherp(0,0,0,0);


      int showerElecPhotons = 0;
      int showerTotPhotons = 0;
      int trackElecPhotons = 0;
      int trackTotPhotons = 0;

      int showerRWElecPhotons = 0;
      int showerRWTotPhotons = 0;
      int trackRWElecPhotons = 0;
      int trackRWTotPhotons = 0;

      double g1ratio = 0;
      double g2ratio = 0;
      int g1photons = 0;
      int g2photons = 0;

      double rwg1ratio = 0;
      double rwg2ratio = 0;
      int rwg1photons = 0;
      int rwg2photons = 0;



      if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
        std::vector<int> shpi0g1photonscol;
        shpi0g1photonscol.clear();
        std::vector<int> shpi0g2photonscol;
        shpi0g2photonscol.clear();

        std::vector<int> shrwpi0g1photonscol;
        shrwpi0g1photonscol.clear();
        std::vector<int> shrwpi0g2photonscol;
        shrwpi0g2photonscol.clear();


        for(int j =0; j<(int)pi0truetrkidcol.size();j++){
          shpi0g1photonscol.push_back(0);
          shpi0g2photonscol.push_back(0);
          shrwpi0g1photonscol.push_back(0);
          shrwpi0g2photonscol.push_back(0);

        }

        for (unsigned int ii=0;ii<svcshowercol[i]->NCell();ii++) {
          const std::vector< sim::PhotonSignal> pSignal = bt->CellToPhotonSignal(svcshowercol[i]->Cell(ii)->Plane(), svcshowercol[i]->Cell(ii)->Cell());
          double rw = 0;
          if(svcshowercol[i]->HitSumE(ii)>0) rw = svcshowercol[i]->HitDeconvE(ii)/svcshowercol[i]->HitSumE(ii);
//          std::cout<<"HitDeconvE, HitSumE, rw "<<svcshowercol[i]->HitDeconvE(ii)<<" "<<svcshowercol[i]->HitSumE(ii)<<" "<<rw<<std::endl;
          for( int p = 0; p < (int)pSignal.size(); p++){
            int partID = pSignal[p].TrackId();
            int motherID = pnav.EveId(partID);
            showerTotPhotons += pSignal[p].NPhoton();
            showerRWTotPhotons += pSignal[p].NPhoton()*rw;
            if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
              showerElecPhotons += pSignal[p].NPhoton();
              showerRWElecPhotons += pSignal[p].NPhoton()*rw;
            }
          }
        }
/*
        if(track.size()==svcshowercol.size()){
          for (unsigned int ii=0;ii<track[i]->NCell();ii++) {
            const std::vector< sim::PhotonSignal> pSignal = bt->CellToPhotonSignal(track[i]->Cell(ii)->Plane(), track[i]->Cell(ii)->Cell());
          double rw = 0;
          if(svcshowercol[i]->HitSumE(ii)>0) rw = svcshowercol[i]->HitDeconvE(ii)/svcshowercol[i]->HitSumE(ii);
            for( int p = 0; p < (int)pSignal.size(); p++){
              int partID = pSignal[p].TrackId();
              int motherID = pnav.EveId(partID);
              trackTotPhotons += pSignal[p].NPhoton();
              trackRWTotPhotons += pSignal[p].NPhoton()*rw;
              if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
                trackElecPhotons += pSignal[p].NPhoton();
                trackRWElecPhotons += pSignal[p].NPhoton()*rw;
              }

              for(int j =0; j<(int)pi0truetrkidcol.size();j++){
                if(motherID==pi0truetrkidcol[j]){
                  if(partID==pi0g1truetrkidcol[j]){
                    shpi0g1photonscol[j] += pSignal[p].NPhoton();
                    shrwpi0g1photonscol[j] += pSignal[p].NPhoton();
                  }else{
                    shpi0g2photonscol[j] += pSignal[p].NPhoton();
                    shrwpi0g2photonscol[j] += pSignal[p].NPhoton()*rw;
                  }
                }
              }

            }
          }


          int pi0g1photonsmax=0;
          int pi0g2photonsmax=0;
          int ig1max = -1;
          int ig2max = -1;

          for(int j =0; j<(int)pi0truetrkidcol.size();j++){
            if(shpi0g1photonscol[j]>pi0g1photonsmax){
              pi0g1photonsmax = shpi0g1photonscol[j];
              ig1max = j;
            } 
            if(shpi0g2photonscol[j]>pi0g2photonsmax){
              pi0g2photonsmax = shpi0g2photonscol[j];
              ig2max = j;
            } 
          }
          
          if(pi0g1photonsmax>=pi0g2photonsmax){
            if(ig1max!=-1){
              if(pi0g1photonscol[ig1max]>0){
                g1ratio = double(shpi0g1photonscol[ig1max])/double(pi0g1photonscol[ig1max]);
                rwg1ratio = double(shrwpi0g1photonscol[ig1max])/double(pi0g1photonscol[ig1max]);
              } 
              if(pi0g2photonscol[ig1max]>0){
                g2ratio = double(shpi0g2photonscol[ig1max])/double(pi0g2photonscol[ig1max]);
                rwg2ratio = double(shrwpi0g2photonscol[ig1max])/double(pi0g2photonscol[ig1max]);
              } 
              g1photons = shpi0g1photonscol[ig1max]; 
              g2photons = shpi0g2photonscol[ig1max];
              rwg1photons = shrwpi0g1photonscol[ig1max];
              rwg2photons = shrwpi0g2photonscol[ig1max];
            }
          }else{
            if(ig2max!=-1){
              if(pi0g2photonscol[ig2max]>0){
                g1ratio = double(shpi0g2photonscol[ig2max])/double(pi0g2photonscol[ig2max]);
                rwg1ratio = double(shrwpi0g2photonscol[ig2max])/double(pi0g2photonscol[ig2max]);
              }
              if(pi0g1photonscol[ig2max]>0){
                g2ratio = double(shpi0g1photonscol[ig2max])/double(pi0g1photonscol[ig2max]);
                rwg2ratio = double(shrwpi0g1photonscol[ig2max])/double(pi0g1photonscol[ig2max]);
              }
              g1photons = shpi0g2photonscol[ig2max];
              g2photons = shpi0g1photonscol[ig2max];
              rwg1photons = shrwpi0g2photonscol[ig2max];
              rwg2photons = shrwpi0g1photonscol[ig2max];
            }
          }
        }
*/
/*
          for (unsigned int kk=0;kk<plist.size();kk++){
            std::cout<<"pdgslice, evt "<<plist[kk]->PdgCode()<<" "<<plistevt[kk]->PdgCode()<<std::endl;
          }
*/
          for (unsigned int kk=0;kk<plist.size();kk++){
//            int abspdg = abs(plist[kk]->PdgCode());
            int id = plist[kk]->TrackId();
            int motherpdg = 0; 
            int motherid = -1; 
            TLorentzVector motherp(0,0,0,0);
            for (unsigned int mm=0;mm<plist.size();mm++){
              int id1 = plist[mm]->TrackId();
              if(id==id1)continue;
              for(int dd=0;dd<plist[mm]->NumberDaughters();dd++){
                if(plist[mm]->Daughter(dd)==id){
                  motherpdg = plist[mm]->PdgCode();
                  motherid = plist[mm]->TrackId();
                  motherp = plist[mm]->Momentum();
                }
              }
            }

/*
            if(!((abspdg == 22)||(abspdg == 11) ||(abspdg == 13) ||(abspdg == 211) ||(abspdg == 321) ||
                (abspdg == 2212) ||
                (abspdg == 2112) ||
                (abspdg == 111)))continue;
*/
            TVector3 pvtx(plist[kk]->Vx(), plist[kk]->Vy(), plist[kk]->Vz());
//            double pvtxdist = (pvtx-evtTrueNuVtx).Mag();
 //           if(pvtxdist>0.001)continue;

            TLorentzVector true_4mom = plist[kk]->Momentum();
            if(true_4mom.Vect().Mag()<0.000001)continue;
            double ntangd = trkdir.Angle(true_4mom.Vect());
            ntangd=fmod(ntangd, 2*3.14159265);
//           std::cout<<"looping ntpdg, ntp, ntdang  "<<abspdg<<" "<<true_4mom.E()<<" "<<ntangd<<std::endl;
            if(ntangd<ntdang){
              ntdang = ntangd;
              ntpdg = plist[kk]->PdgCode();
              ntt = plist[kk]->T();
              ntp = true_4mom;
              ntid = id; 
              ntmotherpdg = motherpdg; 
              ntmotherid = motherid; 
              ntmotherp = motherp; 
              int navmotherid = pnav.EveId(plist[kk]->TrackId());
              for (unsigned int ll=0;ll<plist.size();ll++){
                if(plist[ll]->TrackId()==navmotherid){
                  ntnavmotherpdg = plist[ll]->PdgCode();
                  ntnavmotherid = plist[ll]->TrackId(); 
                  ntnavmotherp = plist[ll]->Momentum();
                  break;
                }
              }
              ntvtx.SetXYZ(plist[kk]->Vx(),plist[kk]->Vy(),plist[kk]->Vz());
//              ntdvtx = (ntvtx - trkstart).Mag();
            }
          }
      }
      ntdang = ntdang*180/3.14159265;
      ntdangcol.push_back(ntdang);
      ntidcol.push_back(ntid);
      ntpcol.push_back(ntp);
      ntvtxcol.push_back(ntvtx);
      ntpdgcol.push_back(ntpdg);
      nttcol.push_back(ntt);
      ntmotherpdgcol.push_back(ntmotherpdg);
      ntmotheridcol.push_back(ntmotherid);
      ntmotherpcol.push_back(ntmotherp);
      ntnavmotherpdgcol.push_back(ntnavmotherpdg);
      ntnavmotheridcol.push_back(ntnavmotherid);
      ntnavmotherpcol.push_back(ntnavmotherp);

      ntelecphotonscol.push_back(showerElecPhotons);
      nttotphotonscol.push_back(showerTotPhotons);
      nttrkelecphotonscol.push_back(trackElecPhotons);
      nttrktotphotonscol.push_back(trackTotPhotons);

      ntrwelecphotonscol.push_back(showerRWElecPhotons);
      ntrwtotphotonscol.push_back(showerRWTotPhotons);
      nttrkrwelecphotonscol.push_back(trackRWElecPhotons);
      nttrkrwtotphotonscol.push_back(trackRWTotPhotons);

      ntg1ratiocol.push_back(g1ratio);
      ntg2ratiocol.push_back(g2ratio);
      ntg1photonscol.push_back(g1photons);
      ntg2photonscol.push_back(g2photons);

      ntrwg1ratiocol.push_back(rwg1ratio);
      ntrwg2ratiocol.push_back(rwg2ratio);
      ntrwg1photonscol.push_back(rwg1photons);
      ntrwg2photonscol.push_back(rwg2photons);


     for (unsigned int ii=0;ii<svcshowercol[i]->NCell();ii++) {
       geo::OfflineChan key(svcshowercol[i]->Cell(ii)->Plane(), svcshowercol[i]->Cell(ii)->Cell());
       showerCellEnergy[key] = svcshowercol[i]->HitDeconvE(ii);
       unsigned int plane = abs((int)svcshowercol[i]->Cell(ii)->Plane()-(int)svcshowercol[i]->StartPlane());
       unsigned transcellId = svcshowercol[i]->Cell(ii)->Cell();
       unsigned int planecell = ShowerAna::GetTrkPlaneCell(svcshowercol[i]->Cell(ii)->Plane(), svcshowercol[i]);
       int itp = fabs(int(planecell) - int(transcellId));

       if(plane<=200){ 
         showerPlaneEnergy[i][plane]+=svcshowercol[i]->HitSumE(ii);
         showerTruePlaneEnergy[i][plane]+=trueDepositEnergy[key];
       }
       if(itp<=20){
         showerTransCellEnergy[i][itp]+=svcshowercol[i]->HitSumE(ii);
         showerTrueTransCellEnergy[i][itp]+=trueDepositEnergy[key];
       }
     }
     showerCellEnergyCol.push_back(showerCellEnergy);


      std::vector<geo::OfflineChan> extras;
      std::map<geo::OfflineChan, double> extrasTruthMap;

      //Calculate path quality:
      std::vector<calib::EPathQuality> quals;
      const rb::Prong* pathtrack = svcshowercol[i].get();
      rb::Track* temptrk = (rb::Track*)pathtrack;
      const std::vector<double> paths = BestPathEstimates(temptrk->PlaneCellPairs(), temptrk->Dir(), quals, extras);
//      std::cout<<"n path, cell "<<paths.size()<<" "<<svcshowercol[i]->NCell()<<std::endl;
//      if(paths.size()!=svcshowercol[i]->NCell())std::cout<<"bad path "<<std::endl; 
     for (unsigned int ii=0;ii<svcshowercol[i]->NCell();ii++) { //for cell calibration
       geo::View_t view = geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->View();
       double cellXYZ[3] = {0,0,0};
       geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Cell(svcshowercol[i]->Cell(ii)->Cell())->GetCenter(cellXYZ);

       double trkx = 0;
       double trky = 0;
       trkx = svcshowercol[i]->Start().x()+(svcshowercol[i]->Dir().x()/svcshowercol[i]->Dir().z())*(cellXYZ[2]-svcshowercol[i]->Start().z());
       trky = svcshowercol[i]->Start().y()+(svcshowercol[i]->Dir().y()/svcshowercol[i]->Dir().z())*(cellXYZ[2]-svcshowercol[i]->Start().z());
       double trkdtr = 0;
       if(view == geo::kX){
//         trkdtr = geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Cell(svcshowercol[i]->Cell(ii)->Cell())->DistToReadOut(trky);
         trkdtr = geom->DistToElectronics(trky, *geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Cell(svcshowercol[i]->Cell(ii)->Cell()));

       }else if(view == geo::kY){
//         trkdtr = geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Cell(svcshowercol[i]->Cell(ii)->Cell())->DistToReadOut(trkx);
         trkdtr = geom->DistToElectronics(trkx, *geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Cell(svcshowercol[i]->Cell(ii)->Cell()));
       }

       
       double wiSh = svcshowercol[i]->W(svcshowercol[i]->Cell(ii).get());

       const double cellPECorr = cal->GetPECorr(*(svcshowercol[i]->Cell(ii)), wiSh);


       unsigned int keym1Cell = svcshowercol[i]->Cell(ii)->Cell();
       unsigned int keym2Cell = svcshowercol[i]->Cell(ii)->Cell();
       unsigned int keyp1Cell = svcshowercol[i]->Cell(ii)->Cell();
       unsigned int keyp2Cell = svcshowercol[i]->Cell(ii)->Cell();
       if(svcshowercol[i]->Cell(ii)->Cell()-1>=0)
         keym1Cell = svcshowercol[i]->Cell(ii)->Cell() - 1; 
       if(svcshowercol[i]->Cell(ii)->Cell()-2>=0)
         keym2Cell = svcshowercol[i]->Cell(ii)->Cell() - 2;
       if(svcshowercol[i]->Cell(ii)->Cell()+1<(int)geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Ncells())
         keyp1Cell = svcshowercol[i]->Cell(ii)->Cell() + 1;
       if(svcshowercol[i]->Cell(ii)->Cell()+2<(int)geom->Plane(svcshowercol[i]->Cell(ii)->Plane())->Ncells())
         keyp2Cell = svcshowercol[i]->Cell(ii)->Cell() + 2;
       geo::OfflineChan key(svcshowercol[i]->Cell(ii)->Plane(), svcshowercol[i]->Cell(ii)->Cell());
       geo::OfflineChan keym1(svcshowercol[i]->Cell(ii)->Plane(), keym1Cell);
       geo::OfflineChan keym2(svcshowercol[i]->Cell(ii)->Plane(), keym2Cell);
       geo::OfflineChan keyp1(svcshowercol[i]->Cell(ii)->Plane(), keyp1Cell);
       geo::OfflineChan keyp2(svcshowercol[i]->Cell(ii)->Plane(), keyp2Cell);


//       double gev = 0;
       if(view == geo::kX){
         m_cellView = 0;
       }else if(view == geo::kY){
         m_cellView = 1;
       }
//
//       if(paths.size()==svcshowercol[i]->NCell()&&paths.size()>0)std::cout<<"path ii "<<paths[ii]<<std::endl; 

       m_cellRun = run;
       m_cellSubRun = srun;
       m_cellEvent = event;
       m_cellSheDir[0] = svcshowercol[i]->Dir()[0];         
       m_cellSheDir[1] = svcshowercol[i]->Dir()[1];         
       m_cellSheDir[2] = svcshowercol[i]->Dir()[2];
       m_cellSheNCell = svcshowercol[i]->NCell();
       m_cellSheNPlane = svcshowercol[i]->NPlane();
       m_cellSheLength = svcshowercol[i]->TotalLength();
       m_cellPath = 9999.0;
       if(paths.size()==svcshowercol[i]->NCell()&&paths.size()>0) m_cellPath = paths[ii];   

       m_cellToSheDist = ShowerAna::GetCellDistToTrk(svcshowercol[i]->Cell(ii)->Plane(), svcshowercol[i]->Cell(ii)->Cell(), svcshowercol[i]->Start(), svcshowercol[i]->Stop()); 
       m_cellToShePlane = int(svcshowercol[i]->Cell(ii)->Plane()) - int(svcshowercol[i]->MinPlane());  
       m_cellSheEnergy = svcshowercol[i]->Energy();
       m_cellSheTrkGeV = svcshowercol[i]->TrkGeV();
       m_cellSheIsFid = svcshowercol[i]->IsFiducial();
       m_cellSheStart[0] = svcshowercol[i]->Start()[0];
       m_cellSheStart[1] = svcshowercol[i]->Start()[1];
       m_cellSheStart[2] = svcshowercol[i]->Start()[2];
       m_cellSheStop[0] = svcshowercol[i]->Stop()[0];
       m_cellSheStop[1] = svcshowercol[i]->Stop()[1];
       m_cellSheStop[2] = svcshowercol[i]->Stop()[2];

       m_cellPlaneId = svcshowercol[i]->Cell(ii)->Plane();
       m_cellId = svcshowercol[i]->Cell(ii)->Cell();
       m_cellXYZ[0] = cellXYZ[0];
       m_cellXYZ[1] = cellXYZ[1];
       m_cellXYZ[2] = cellXYZ[2];
       m_cellADC  = svcshowercol[i]->Cell(ii)->ADC();
       m_cellTDC  = svcshowercol[i]->Cell(ii)->TDC();
       m_cellTNS  = svcshowercol[i]->Cell(ii)->TNS();
       m_cellPE   = svcshowercol[i]->Cell(ii)->PE();
       m_cellPECorr   = cellPECorr;
       m_cellTrueEnergy  = trueDepositEnergy[key];
       m_cellTrueTime = trueDepositTime[key]/trueDepositEnergy[key];
       m_cellEnergy = showerCellEnergy[key];

       m_cellEnergyM1 = 0;
       m_cellEnergyM2 = 0;
       m_cellEnergyP1 = 0;
       m_cellEnergyP2 = 0;
       m_cellTrueEnergyM1 = 0;
       m_cellTrueEnergyM2 = 0;
       m_cellTrueEnergyP1 = 0;
       m_cellTrueEnergyP2 = 0;

       if(keym1.Cell()!=key.Cell()){
         m_cellEnergyM1 = showerCellEnergy[keym1]; 
         m_cellTrueEnergyM1 = trueDepositEnergy[keym1];
       }
       if(keym2.Cell()!=key.Cell()){
         m_cellEnergyM2 = showerCellEnergy[keym2]; 
         m_cellTrueEnergyM2 = trueDepositEnergy[keym2];
       }
       if(keyp1.Cell()!=key.Cell()){
         m_cellEnergyP1 = showerCellEnergy[keyp1]; 
         m_cellTrueEnergyP1 = trueDepositEnergy[keyp1];
       }
       if(keyp2.Cell()!=key.Cell()){
         m_cellEnergyP2 = showerCellEnergy[keyp2]; 
         m_cellTrueEnergyP2 = trueDepositEnergy[keyp2];
       }

       m_cellNSlice = nSlice;
       m_cellNTrk = svcshowercol.size(); 
       m_cellSlice = hitslicemap[key]; 
       m_cellTrueHitXYZ[0] = trueAvgX[key]/trueDepositEnergy[key];
       m_cellTrueHitXYZ[1] = trueAvgY[key]/trueDepositEnergy[key];
       m_cellTrueHitXYZ[2] = trueAvgZ[key]/trueDepositEnergy[key];


       m_cellHitXYZ[0] = 0; 
       m_cellHitXYZ[1] = 0; 
       m_cellHitXYZ[2] = trkdtr; 

       m_cellTrueNuCCNC = evtTrueNuCCNC;
       m_cellTrueNuMode = evtTrueNuMode;
       m_cellTrueNuPdg = evtTrueNuPdg;
       m_cellTrueNuEnergy = evtTrueNuP4.E();
       m_cellTrueNuP4[0] = evtTrueNuP4.Px();
       m_cellTrueNuP4[1] = evtTrueNuP4.Py();
       m_cellTrueNuP4[2] = evtTrueNuP4.Pz();
       m_cellTrueNuP4[3] = evtTrueNuP4.E();
       m_cellTrueNuVtx[0] = evtTrueNuVtx[0];
       m_cellTrueNuVtx[1] = evtTrueNuVtx[1];
       m_cellTrueNuVtx[2] = evtTrueNuVtx[2];
       m_cellTrueDang =ntdang;
       m_cellTruePdg = ntpdg;
       m_cellTrueT = ntt;
       m_cellTrueMother = ntmotherpdg;
       m_cellTrueVtx[0] = ntvtx[0];
       m_cellTrueVtx[1] = ntvtx[1];
       m_cellTrueVtx[2] = ntvtx[2];
       m_cellTrueP4[0] = ntp.Px();
       m_cellTrueP4[1] = ntp.Py();
       m_cellTrueP4[2] = ntp.Pz();
       m_cellTrueP4[3] = ntp.E();
       fCell->Fill();
     }    
   }
//   return;//cell only
//**********************************************************
//         Fill showers information
//**********************************************************

   Vp4 showerpGam, trackpGam;
   Vp4 ntdirshowerpGam, ntdirtrackpGam; 
   showerpGam.clear();
   trackpGam.clear();
   ntdirshowerpGam.clear();
   ntdirtrackpGam.clear();
  
   double showeretot = 0;
//   double tracketot = 0;


   int ish1 = -1;
   int ish2 = -1;
   int ish3 = -1;
   int ilen1 = -1;

//   double len1nplane = 0;
   double sha1ann = -9999;
   double sh1gev = 0;
//   double sh2gev = 0;
   double sh3gev = 0;
//   double len1ell = -9999;
//   double sh1isfid = 0;
//   double shexclpe = 0;
   TLorentzVector sh1p4(0,0,0,0);
   TVector3 sh1start(0,0,0);
   TLorentzVector trackpTot(0,0,0,0);

   double elower[11]= {0.00, 0.25, 0.75, 1.25, 1.75, 2.25, 2.75, 3.25, 3.75, 4.25, 4.75};
   double eupper[11]= {0.25, 0.75, 1.25, 1.75, 2.25, 2.75, 3.25, 3.75, 4.25, 4.75, 5.25};


   for(unsigned int i = 0; i < svcshowercol.size(); i++){
/*
     if(svcshowercol[i]->NPlane() > len1nplane){
       ilen1 = i;  
       len1nplane = svcshowercol[i]->NPlane();
     }
*/
/*
      double ellmax = -9999;
      double elllmax = -9999;
      double elltmax = -9999;

      for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
        if(itype == int(jmshower::kElectron)||itype == int(jmshower::kElectronQE)||itype == int(jmshower::kElectronRES)||itype == int(jmshower::kElectronDIS)||itype == int(jmshower::kElectronCOH)){
          if(ellmax<svcshowercol[i]->DedxLongLL(itype)+svcshowercol[i]->DedxTransLL(itype)){
            ellmax = svcshowercol[i]->DedxLongLL(itype)+svcshowercol[i]->DedxTransLL(itype);
            elllmax = svcshowercol[i]->DedxLongLL(itype);
            elltmax = svcshowercol[i]->DedxTransLL(itype);
          }
        }
      }
     if(ellmax > len1ell){
       ilen1 = i;  
       len1ell = ellmax; 
     }
*/
     if(svcshowercol[i]->Energy() > sh1gev){
       ish1 = i;
       sh1gev = svcshowercol[i]->Energy(); 
     }

/*
     if(showerTrueEDang<181){
       double minedang = std::min(showerTrueEDang, fabs(180-showerTrueEDang));
       if(minedang < sheedang){
         ishe = i;
         sheedang = minedang;
       }
     }
*/

/*
     if(ntelecphotonscol[i] > shephotons){
       ishe = i;
       shephotons = ntelecphotonscol[i];
     }
*/
     if(nttrkelecphotonscol[i] > shephotons){
       ishe = i;
       shephotons = nttrkelecphotonscol[i];
     }
/*
     if(fabs(ntpdgcol[i]) == 11){
       ishe = i;
       std::cout<<"Elec i "<<i<<std::endl;
     }
*/
  }
   if(ish1!=-1){
     for(unsigned i=0;i<svcshowercol.size(); i++){
       std::cout<<"svcshowercol[i]->ID(),ish1,ish2,i "<<svcshowercol[i]->ID()<<" "<<ish1<<" "<<ish2<<std::endl;
       std::cout<<"Pi0G2ID "<<svcshowercol[ish1]->Pi0G2ID()<<std::endl;
       if(svcshowercol[i]->ID()==svcshowercol[ish1]->Pi0G2ID())ish2 = i;
     }
   }
//   if(ish1!=-1)ish2 = svcshowercol[ish1]->Pi0G2ID();
/*
   for(unsigned int i = 0; i < svcshowercol.size(); i++){
     if(i==ish1)continue;
     if(svcshowercol[i]->Energy() > sh2gev){
       ish2 = i;
       sh2gev = svcshowercol[i]->Energy();
     }
   }
*/

   for(unsigned int i = 0; i < svcshowercol.size(); i++){
     if((int)i==ish1)continue;
     if((int)i==ish2)continue;
     if(svcshowercol[i]->Energy() > sh3gev){
       ish3 = i;
       sh3gev = svcshowercol[i]->Energy();
     }
   }


   for(unsigned int i = 0; i < svcshowercol.size(); i++){
/*
     if(i==ish1)continue;
     double ellmax = -9999;
     double elllmax = -9999;
     double elltmax = -9999;
     for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
       if(itype == int(jmshower::kElectron)||itype == int(jmshower::kElectronQE)||itype == int(jmshower::kElectronRES)||itype == int(jmshower::kElectronDIS)||itype == int(jmshower::kElectronCOH)){
         if(ellmax<svcshowercol[i]->DedxLongLL(itype)+svcshowercol[i]->DedxTransLL(itype)){
           ellmax = svcshowercol[i]->DedxLongLL(itype)+svcshowercol[i]->DedxTransLL(itype);
           elllmax = svcshowercol[i]->DedxLongLL(itype);
           elltmax = svcshowercol[i]->DedxTransLL(itype);
         }
       }
     }
     if(ellmax > len1ell){
       ilen1 = i;  
       len1ell = ellmax; 
     }
*/
     if(svcshowercol[i]->ANN(1) > sha1ann){
       ilen1 = i;
       sha1ann = svcshowercol[i]->ANN(1);
     } 
   }
   TVector3 nuDir(-0.845e-3, 52.552e-3, 0.998618);
   for(unsigned int i = 0; i < svcshowercol.size(); i++){
// Set 4-Vector momentum
     double showerEnergy    = svcshowercol[i]->Energy();//;shsvc->Energy(svcshowercol[i]);
     double showerTrkGeV    = svcshowercol[i]->TrkGeV();//;shsvc->Energy(svcshowercol[i]);
     double showereraw = showerEnergy;
     TLorentzVector showerptrk(svcshowercol[i]->Dir()[0]*showereraw,svcshowercol[i]->Dir()[1]*showereraw,svcshowercol[i]->Dir()[2]*showereraw,showereraw);
     TLorentzVector showerptrkgev(svcshowercol[i]->Dir()[0]*showerTrkGeV,svcshowercol[i]->Dir()[1]*showerTrkGeV,svcshowercol[i]->Dir()[2]*showerTrkGeV,showerTrkGeV);
     TVector3 showerstart(svcshowercol[i]->Start());
     TVector3 nttrkp = ntpcol[i].Vect();
     double ntmag = sqrt(nttrkp[0]*nttrkp[0]+nttrkp[1]*nttrkp[1]+nttrkp[2]*nttrkp[2]);
     TVector3 nttrkdir;
     nttrkdir.SetX(nttrkp[0]/ntmag);
     nttrkdir.SetY(nttrkp[1]/ntmag);
     nttrkdir.SetZ(nttrkp[2]/ntmag);
     TLorentzVector ntdirshowerptrk(nttrkdir[0]*showereraw,nttrkdir[1]*showereraw,nttrkdir[2]*showereraw,showereraw);
     showerpGam.push_back(showerptrk);
     trackpGam.push_back(showerptrkgev);
     if(ntdangcol[i]<5){
       ntdirshowerpGam.push_back(ntdirshowerptrk);
     }
     showeretot+= showerEnergy;
     trackpTot+=trackpGam[i];
   }

   for(unsigned int i = 0; i < svcshowercol.size(); i++){
     double showerTruePartEnergy[10];
     double showerTrueDepEnergy = 0;
     double showerTrueEDang = 200;
     int showerTruePartHit[10];
     for(int ii = 0;ii<10;ii++){
       showerTruePartEnergy[ii]=0;
       showerTruePartHit[ii]=0;
     } 
     for(int ii=0;ii<10;ii++){
       m_showerTruePartEnergy[ii] = 0;
       m_showerTruePartHit[ii] = 0;
     }
     m_showerTrueDepEnergy = 0;
     m_showerTrueElecPhotons = 0;
     m_showerTrueG1Ratio = 0;
     m_showerTrueG2Ratio = 0;
     m_showerTrueG1Photons = 0;
     m_showerTrueG2Photons = 0;

     m_showerTruePi0Dang = 0; 
     m_showerTruePi0P4[0] = 0;
     m_showerTruePi0P4[1] = 0;
     m_showerTruePi0P4[2] = 0;
     m_showerTruePi0P4[3] = 0;


     m_showerTrueTotPhotons = 0;
     m_showerTrueTrkElecPhotons = 0;
     m_showerTrueTrkTotPhotons = 0;
     m_showerTrueRWElecPhotons = 0;
     m_showerTrueRWTotPhotons = 0;
     m_showerTrueTrkRWElecPhotons = 0;
     m_showerTrueTrkRWTotPhotons = 0;


     m_showerEvtTrueElecPhotons = 0;
     m_showerDedxANN = 0;
     m_showerDedxANN1 = 0;
     if(bt->HaveTruthInfo() &&mcstudy==1&&slicemctag==true){
       for(unsigned int isc = 0; isc<svcshowercol[i]->NCell(); isc++){
         geo::OfflineChan key(svcshowercol[i]->Cell(isc)->Plane(),svcshowercol[i]->Cell(isc)->Cell());
         for(int ifl = 0; ifl<(int)truePartPdg[key].size(); ifl++){
           showerTrueDepEnergy+=truePartEnergy[key][ifl];
           if(fabs(truePartPdg[key][ifl])==11){
             showerTruePartEnergy[0] += truePartEnergy[key][ifl];
             showerTruePartHit[0]++;
           } 
           if(fabs(truePartPdg[key][ifl])==22){
             showerTruePartEnergy[1] += truePartEnergy[key][ifl];
             showerTruePartHit[1]++;
           } 
           if(fabs(truePartPdg[key][ifl])==13){
             showerTruePartEnergy[2] += truePartEnergy[key][ifl];
             showerTruePartHit[2]++;
           } 
           if(fabs(truePartPdg[key][ifl])==2212){
             showerTruePartEnergy[3] += truePartEnergy[key][ifl];
             showerTruePartHit[3]++;
           } 
           if(fabs(truePartPdg[key][ifl])==2112){
             showerTruePartEnergy[4] += truePartEnergy[key][ifl];
             showerTruePartHit[4]++;
           } 
           if(fabs(truePartPdg[key][ifl])==211){
             showerTruePartEnergy[5] += truePartEnergy[key][ifl];
             showerTruePartHit[5]++;
           } 
         }
       }
       m_showerTrueDepEnergy = showerTrueDepEnergy;

       m_showerEvtTrueElecPhotons = evtTrueElecPhotons;


       for(int ii=0;ii<10;ii++){
         m_showerTruePartEnergy[ii]=showerTruePartEnergy[ii];
         m_showerTruePartHit[ii]=showerTruePartHit[ii];
       } 

       TVector3 trkdir=svcshowercol[i]->Dir();
       if(intE!=-1){
         m_showerEvtTrueEnergy[0] = evtntpcol[intE].Vect().Mag();
         showerTrueEDang = trkdir.Angle(evtntpcol[intE].Vect())*180/3.14159;
         m_showerTrueEDang = showerTrueEDang; 

       }else{
         m_showerEvtTrueEnergy[0] = 0;
         m_showerTrueEDang = 0;
       }

       if(ntmotherpdgcol[i]==111){
         m_showerTruePi0Dang = trkdir.Angle(ntmotherpcol[i].Vect())*180/3.14159;
         m_showerTruePi0P4[0] = ntmotherpcol[i].Px(); 
         m_showerTruePi0P4[1] = ntmotherpcol[i].Py(); 
         m_showerTruePi0P4[2] = ntmotherpcol[i].Pz(); 
         m_showerTruePi0P4[3] = ntmotherpcol[i].E(); 
       }else if(ntpdgcol[i]==111){
         m_showerTruePi0Dang = trkdir.Angle(ntpcol[i].Vect())*180/3.14159;
         m_showerTruePi0P4[0] = ntpcol[i].Px();
         m_showerTruePi0P4[1] = ntpcol[i].Py();
         m_showerTruePi0P4[2] = ntpcol[i].Pz();
         m_showerTruePi0P4[3] = ntpcol[i].E(); 
       }else{
         m_showerTruePi0Dang = 0; 
         m_showerTruePi0P4[0] = 0; 
         m_showerTruePi0P4[1] = 0;
         m_showerTruePi0P4[2] = 0;
         m_showerTruePi0P4[3] = 0;
       } 

/*
       for(int ii = 0; ii < evtntpdgcol.size(); ii++){
         if(ii>=20)break;
         m_showerEvtTruePdg[ii] = evtntpdgcol[ii];
         m_showerEvtTrueEnergy[ii] = evtntpcol[ii].Vect().Mag();
       }
*/
    }
// General information 
     double showerRadius    = svcshowercol[i]->Radius();
     double showerVtxGeV    = svcshowercol[i]->VtxGeV();
     double showerEnergy    = svcshowercol[i]->Energy();//;shsvc->Energy(svcshowercol[i]);
     double showerTNS    = svcshowercol[i]->MeanTNS();//;shsvc->Energy(svcshowercol[i]);
     double showerTrkGeV    = svcshowercol[i]->TrkGeV();//;shsvc->Energy(svcshowercol[i]);
     double showerCalTrkGeV = 0;
//     if(svcshowercol.size()==track.size()){std::cout<<"do sh cal "<<std::endl;showerCalTrkGeV = cal->GetShowerEnergy(track[i]->Start(), track[i]->Stop(), track[i]->TotalGeV());}
     std::cout<<"trkE, cal trkE "<<showerTrkGeV<<" "<<showerCalTrkGeV<<std::endl;
     double showerEnergy0    = svcshowercol[i]->DepositEnergy();
     double showerGap    = svcshowercol[i]->Gap(); 
     int  showerIsFid    = int(svcshowercol[i]->IsFiducial());
     if(showerEnergy>0.001)showerRadius = showerRadius/showerEnergy;
     int showerIE = -1;
     for(int ie=0; ie<11; ie++){
       if(showerEnergy>elower[ie]&&showerEnergy<eupper[ie])showerIE=ie;
     }

     m_showerIE = showerIE;
     m_showerTrueNuCCNC = evtTrueNuCCNC;
     m_showerTrueNuMode = evtTrueNuMode;
     m_showerTrueNuPdg = evtTrueNuPdg;
     m_showerTrueNuEnergy = evtTrueNuP4.E();
     m_showerTrueNuP4[0] = evtTrueNuP4.Px();
     m_showerTrueNuP4[1] = evtTrueNuP4.Py();
     m_showerTrueNuP4[2] = evtTrueNuP4.Pz();
     m_showerTrueNuP4[3] = evtTrueNuP4.E();
     m_showerTrueNuVtx[0] = evtTrueNuVtx[0];
     m_showerTrueNuVtx[1] = evtTrueNuVtx[1];
     m_showerTrueNuVtx[2] = evtTrueNuVtx[2];
     m_showerTrueDang = ntdangcol[i];
     m_showerTrueId = ntidcol[i];
     m_showerTruePdg = ntpdgcol[i];
     m_showerTrueT = nttcol[i];
     m_showerTrueEtag = 0; 
     if((int)i==ishe)m_showerTrueEtag=1;
     m_showerTrueVtx[0] = ntvtxcol[i][0];
     m_showerTrueVtx[1] = ntvtxcol[i][1];
     m_showerTrueVtx[2] = ntvtxcol[i][2];
     m_showerTrueP4[0] = ntpcol[i].Px();
     m_showerTrueP4[1] = ntpcol[i].Py();
     m_showerTrueP4[2] = ntpcol[i].Pz();
     m_showerTrueP4[3] = ntpcol[i].E();

     m_showerTrueMotherPdg = ntmotherpdgcol[i];
     m_showerTrueMotherId = ntmotheridcol[i];
     m_showerTrueMotherP4[0] = ntmotherpcol[i].Px();
     m_showerTrueMotherP4[1] = ntmotherpcol[i].Py();
     m_showerTrueMotherP4[2] = ntmotherpcol[i].Pz();
     m_showerTrueMotherP4[3] = ntmotherpcol[i].E();

     m_showerTrueNavMotherPdg = ntnavmotherpdgcol[i];
     m_showerTrueNavMotherId = ntnavmotheridcol[i];
     m_showerTrueNavMotherP4[0] = ntnavmotherpcol[i].Px();
     m_showerTrueNavMotherP4[1] = ntnavmotherpcol[i].Py();
     m_showerTrueNavMotherP4[2] = ntnavmotherpcol[i].Pz();
     m_showerTrueNavMotherP4[3] = ntnavmotherpcol[i].E();


     m_showerTrueElecPhotons = ntelecphotonscol[i]; 
     m_showerTrueTotPhotons =  nttotphotonscol[i];
     m_showerTrueTrkElecPhotons = nttrkelecphotonscol[i];
     m_showerTrueTrkTotPhotons =  nttrktotphotonscol[i];

     m_showerTrueRWElecPhotons = ntrwelecphotonscol[i];
     m_showerTrueRWTotPhotons =  ntrwtotphotonscol[i];
     m_showerTrueTrkRWElecPhotons = nttrkrwelecphotonscol[i];
     m_showerTrueTrkRWTotPhotons =  nttrkrwtotphotonscol[i];

     m_showerTrueG1Ratio = ntg1ratiocol[i];
     m_showerTrueG2Ratio = ntg2ratiocol[i];
     m_showerTrueG1Photons = ntg1photonscol[i];
     m_showerTrueG2Photons = ntg2photonscol[i];
      
     m_showerTrueRWG1Ratio = ntrwg1ratiocol[i];
     m_showerTrueRWG2Ratio = ntrwg2ratiocol[i];
     m_showerTrueRWG1Photons = ntrwg1photonscol[i];
     m_showerTrueRWG2Photons = ntrwg2photonscol[i];

     m_showerNCell = svcshowercol[i]->NCell();
     m_showerNPlane = svcshowercol[i]->NPlane();
     m_showerLength = svcshowercol[i]->TotalLength();
     m_showerADC = svcshowercol[i]->TotalADC ();
     m_showerEnergy = showerEnergy;
     m_showerFTag = ftag;
     m_showerTNS = showerTNS;
     m_showerTrkGeV = showerTrkGeV;
     m_showerSliceGeV = svcshowercol[i]->SliceGeV();
     m_showerSliceEdge10GeV = svcshowercol[i]->SliceEdge10GeV();
     m_showerSliceEdge5GeV = svcshowercol[i]->SliceEdge5GeV();
     m_showerSliceEdge2GeV = svcshowercol[i]->SliceEdge2GeV();
     m_showerTotalRecoNCell = totalRecoNCell;
     m_showerEnergy0 = showerEnergy0;
     m_showerDir[0] = (svcshowercol[i]->Dir())[0];
     m_showerDir[1] = (svcshowercol[i]->Dir())[1];
     m_showerDir[2] = (svcshowercol[i]->Dir())[2];
     m_showerStart[0]  = (svcshowercol[i]->Start())[0];
     m_showerStart[1]  = (svcshowercol[i]->Start())[1];
     m_showerStart[2]  = (svcshowercol[i]->Start())[2];
     m_showerStop[0]  = (svcshowercol[i]->Stop())[0];
     m_showerStop[1]  = (svcshowercol[i]->Stop())[1];
     m_showerStop[2]  = (svcshowercol[i]->Stop())[2];

     double showerMeanX = (svcshowercol[i]->Start()[0]+svcshowercol[i]->Stop()[0])/2.;
     double showerMeanY = (svcshowercol[i]->Start()[1]+svcshowercol[i]->Stop()[1])/2.;
     if(showerMeanX>0&&showerMeanY>0)m_showerIReg = 0;
     if(showerMeanX>0&&showerMeanY<0)m_showerIReg = 1;
     if(showerMeanX<0&&showerMeanY>0)m_showerIReg = 2;
     if(showerMeanX<0&&showerMeanY<0)m_showerIReg = 3;

    
     m_showerRadius = showerRadius;
     m_showerVtxGeV = showerVtxGeV;
     m_showerGap = showerGap;
     m_showerIsFid = showerIsFid;
     m_showerDistToEdge = svcshowercol[i]->DistToEdge();
     m_showerNMIPPlane = svcshowercol[i]->NMIPPlane();
//     m_showerContStartPlane = shsvc->ContStartPlane(svcshowercol[i], 0, 3);

// Fill longitudinal and transverse information
    double transcelldedx[20];
    for(int itd=0;itd<20;itd++){
      transcelldedx[itd] = 0;
    }

    for(unsigned int plane=0;plane<200;plane++){
      m_showerPlaneHitX[plane] = 0; 
      m_showerPlaneHitY[plane] = 0;
      m_showerPlaneHitZ[plane] = 0;

      m_showerPlaneHitCell[plane] = -1;
      m_showerPlaneE1Cell[plane] = -1;
      m_showerPlaneE2Cell[plane] = -1;
      m_showerPlaneCentroidCell[plane] = -1;
      m_showerPlaneHitCellGeV[plane] = 0;
      m_showerPlaneE1CellGeV[plane] = 0;
      m_showerPlaneE2CellGeV[plane] = 0;
      m_showerPlaneCentroidCellGeV[plane] = 0;

      m_showerPlaneHitIReg[plane] = -1;
      m_showerPlaneDedx[plane] = 0;
/*
      m_showerPlaneDedxprobE[plane] = 0; 
      m_showerPlaneDedxprobG[plane] = 0;
      m_showerPlaneDedxprobMu[plane] = 0; 
      m_showerPlaneDedxprobPi0[plane] = 0; 
      m_showerPlaneDedxprobHad[plane] = 0;
      m_showerPlaneDedxprobProton[plane] = 0; 
      m_showerPlaneDedxprobNeutron[plane] = 0; 
      m_showerPlaneDedxprobPion[plane] = 0; 
      m_showerPlaneDedxprobEqe[plane] = 0; 
      m_showerPlaneDedxprobEres[plane] = 0; 
      m_showerPlaneDedxprobEdis[plane] = 0;
      m_showerPlaneDedxprobEcoh[plane] = 0;
*/
      m_showerPlaneEnergy[plane] = 0; 
      m_showerTruePlaneEnergy[plane] = 0; 
    }

    double summoment12=0;
    double sume=0;
    int nzeroplane = 0;
    for(unsigned int plane=0;plane<svcshowercol[i]->NPlane();plane++){
      if(plane>=200)break;

//      double path = shsvc->GetTrkHitPath(shsvc->m_showerPClusterCol[i][plane].Plane(), 0, svcshowercol[i]);
//      double ep = shsvc->Energy(shsvc->m_showerPClusterCol[i][plane], i);
//      double dedx = ep/path;
//      double rad = shsvc->Radius(shsvc->m_showerPClusterCol[i][plane], svcshowercol[i]); 
//      double probE =  svcshowercol[i]->PlaneProb(plane, 0);
//      double probG =  svcshowercol[i]->PlaneProb(plane, 1);
//      double probMu=  svcshowercol[i]->PlaneProb(plane, 2);
//      double probPi0 =  svcshowercol[i]->PlaneProb(plane, 3);
//      double probHad =  svcshowercol[i]->PlaneProb(plane, 4);
//      double probProton =  svcshowercol[i]->PlaneProb(plane, 5);
//      double probNeutron =  svcshowercol[i]->PlaneProb(plane, 6);
//      double probPion =  svcshowercol[i]->PlaneProb(plane, 7);
//      double probEqe =  svcshowercol[i]->PlaneProb(plane, 8);
//      double probEres =  svcshowercol[i]->PlaneProb(plane, 9);
//      double probEdis =  svcshowercol[i]->PlaneProb(plane, 10);
//      double probEcoh =  svcshowercol[i]->PlaneProb(plane, 11);

//Longitudinal dE/dX
//      m_showerPlaneEnergy[plane] = ep;

      m_showerPlaneHitX[plane] = svcshowercol[i]->PlaneHitXYZ(plane)[0]; 
      m_showerPlaneHitY[plane] = svcshowercol[i]->PlaneHitXYZ(plane)[1]; 
      m_showerPlaneHitZ[plane] = svcshowercol[i]->PlaneHitXYZ(plane)[2]; 

      m_showerPlaneHitCell[plane] = (int)svcshowercol[i]->PlaneHitCell(plane);
      m_showerPlaneE1Cell[plane] = (int)svcshowercol[i]->PlaneE1Cell(plane);
      m_showerPlaneE2Cell[plane] = (int)svcshowercol[i]->PlaneE2Cell(plane);
      m_showerPlaneCentroidCell[plane] =  (int)svcshowercol[i]->PlaneCentroidCell(plane);

      unsigned absplane = svcshowercol[i]->StartPlane();
      if(svcshowercol[i]->StartPlane()<=svcshowercol[i]->StopPlane()){
        absplane=svcshowercol[i]->StartPlane()+plane;
      }else if(svcshowercol[i]->StartPlane()>=plane){
        absplane=svcshowercol[i]->StartPlane()-plane;
      }
      geo::OfflineChan keyplanehitcell(absplane, svcshowercol[i]->PlaneHitCell(plane));
      geo::OfflineChan keyplanee1cell(absplane, svcshowercol[i]->PlaneE1Cell(plane));
      geo::OfflineChan keyplanee2cell(absplane, svcshowercol[i]->PlaneE2Cell(plane));
      geo::OfflineChan keyplanecentroidcell(absplane, svcshowercol[i]->PlaneCentroidCell(plane));

      m_showerPlaneHitCellGeV[plane] = showerCellEnergyCol[i][keyplanehitcell];
      m_showerPlaneE1CellGeV[plane] = showerCellEnergyCol[i][keyplanee1cell];
      m_showerPlaneE2CellGeV[plane] = showerCellEnergyCol[i][keyplanee2cell];
      m_showerPlaneCentroidCellGeV[plane] = showerCellEnergyCol[i][keyplanecentroidcell];

     double e1 = 0;
     double r1 = 0;
     double r2 = 0;
     double cellW = 2*geom->Plane(plane)->Cell(0)->HalfW();
     if(svcshowercol[i]->PlaneE2Cell(plane)>=0){
       e1=(double)showerCellEnergyCol[i][keyplanee2cell];
       r1=(double)cellW*fabs(svcshowercol[i]->PlaneE1Cell(plane)-svcshowercol[i]->PlaneE2Cell(plane));
     }
     if(plane+1<svcshowercol[i]->NPlane()){
       if(svcshowercol[i]->PlaneE2Cell(plane+1)>=0){
         r2=(double)cellW*fabs(svcshowercol[i]->PlaneE1Cell(plane+1)-svcshowercol[i]->PlaneE2Cell(plane+1));
       }
     }
     summoment12+=e1*r1*r1+e1*r2*r2;
     sume+=e1;

      if(svcshowercol[i]->PlaneHitXYZ(plane)[0]>0&&svcshowercol[i]->PlaneHitXYZ(plane)[1]>0)m_showerPlaneHitIReg[plane]=0;
      if(svcshowercol[i]->PlaneHitXYZ(plane)[0]>0&&svcshowercol[i]->PlaneHitXYZ(plane)[1]<0)m_showerPlaneHitIReg[plane]=1;
      if(svcshowercol[i]->PlaneHitXYZ(plane)[0]<0&&svcshowercol[i]->PlaneHitXYZ(plane)[1]>0)m_showerPlaneHitIReg[plane]=2;
      if(svcshowercol[i]->PlaneHitXYZ(plane)[0]<0&&svcshowercol[i]->PlaneHitXYZ(plane)[1]<0)m_showerPlaneHitIReg[plane]=3;
      m_showerPlaneDedx[plane] = svcshowercol[i]->PlaneDedx(plane);
      if(svcshowercol[i]->PlaneDedx(plane)<0.000001)nzeroplane++;
/*
      m_showerPlaneDedxprobE[plane] = log(probE);
      m_showerPlaneDedxprobG[plane] = log(probG);
      m_showerPlaneDedxprobMu[plane] = log(probMu);
      m_showerPlaneDedxprobPi0[plane] = log(probPi0);
      m_showerPlaneDedxprobHad[plane] = log(probHad);
      m_showerPlaneDedxprobProton[plane] = log(probProton);
      m_showerPlaneDedxprobNeutron[plane] = log(probNeutron);
      m_showerPlaneDedxprobPion[plane] = log(probPion);
      m_showerPlaneDedxprobEqe[plane] = log(probEqe);
      m_showerPlaneDedxprobEres[plane] = log(probEres);
      m_showerPlaneDedxprobEdis[plane] = log(probEdis);
      m_showerPlaneDedxprobEcoh[plane] = log(probEcoh);
*/

      m_showerPlaneEnergy[plane] = showerPlaneEnergy[i][plane];
      m_showerTruePlaneEnergy[plane] = showerTruePlaneEnergy[i][plane];

//      m_showerPlaneRadius[plane] = rad;

//Transverse dE/dX
/*
      unsigned int planecell = shsvc->GetTrkPlaneCell(shsvc->m_showerPClusterCol[i][plane].Plane(), svcshowercol[i]);
      for (unsigned itp = 0;itp<shsvc->m_showerPClusterCol[i][plane].NCell();itp++) {
        unsigned transcellId = shsvc->m_showerPClusterCol[i][plane].Cell(itp)->Cell();
        geo::OfflineChan transcellkey(shsvc->m_showerPClusterCol[i][plane].Plane(),transcellId);
        int itc = fabs(int(planecell) - int(transcellId)); 
        if(itc>20)continue;
        transcelldedx[itc] += shsvc->fHitEWeightMap[transcellkey][i]/path;
      }
*/
    }
    m_showerNZeroPlane = nzeroplane;
    showerNZeroPlaneCol.push_back(nzeroplane);
    double moment12 = 0;
    if(sume>0.000001)moment12 = summoment12/sume;

    m_showerMoment12 = moment12;
//    std::cout<<"shower Asym, Ixx, Iyy "<<svcshowercol[i]->Asym()<<" "<<svcshowercol[i]->Ixyz()[0]<<" "<<svcshowercol[i]->Ixyz()[1]<<std::endl;
    m_showerAsym = svcshowercol[i]->Asym();
    m_showerIxx  = svcshowercol[i]->Ixyz()[0];
    m_showerIyy  = svcshowercol[i]->Ixyz()[1];
    showerMomentCol.push_back(moment12);
    for(unsigned itp = 0;itp<20;itp++){
//      if(shsvc->m_showerPClusterCol[i].size()>0)transcelldedx[itp]=double(transcelldedx[itp]/((svcshowercol[i]->Dir()[2])*double(shsvc->m_showerPClusterCol[i].size())));
      transcelldedx[itp] = svcshowercol[i]->CellTransDedx(itp);
    }
    for(int itd=0;itd<20;itd++){
       m_showerTransCellEnergy[itd] = 0; 
       m_showerTrueTransCellEnergy[itd] = 0; 
       m_showerTransCellDedx[itd] = 0; 
    }
    for(int itd=0;itd<20;itd++){
       m_showerTransCellEnergy[itd] = showerTransCellEnergy[i][itd];
       m_showerTrueTransCellEnergy[itd] = showerTrueTransCellEnergy[i][itd];
       m_showerTransCellDedx[itd] = transcelldedx[itd];
//     m_showerTransCellProbE[itd] = shsvc->GetInterCellTransDedxProb(svcshowercol[i], itd, transcelldedx[itd], 0);
//     m_showerTransCellProbPi0[itd] = shsvc->GetInterCellTransDedxProb(svcshowercol[i], itd, transcelldedx[itd], 3);
    }    

    for( int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){//itype: 0-electron, 1-photon, 2-muon, 3-pion-zero; 
      m_showerDedxNdof[itype]=svcshowercol[i]->NPlane();
      m_showerDedxProb[itype] = svcshowercol[i]->DedxLongLL(itype); 
      m_showerTransDedxProb[itype] = svcshowercol[i]->DedxTransLL(itype);
//      std::cout<<"itype LLL,LLT "<<itype<<" "<<svcshowercol[i]->DedxLongLL(itype)<<" "<<svcshowercol[i]->DedxTransLL(itype)<<std::endl;
      m_showerDedxLLL[itype] = svcshowercol[i]->DedxLongLL(itype);//, 1, 3);
      m_showerDedxLLT[itype] = svcshowercol[i]->DedxTransLL(itype);
    }

    m_showerDedxANN = svcshowercol[i]->ANN(0);; 
    m_showerDedxANN1 = svcshowercol[i]->ANN(1);; 
    m_showerRun= run; 
    m_showerSubRun= srun;
    m_showerEvent= event;
    m_showerSubEvent= (int)sliceIdx;
    m_showerId= i;

    int showerNTrk=0;
    for(unsigned int itk=0;itk<svcshowercol.size();itk++){
      if(svcshowercol[itk]->ISlice()!=svcshowercol[i]->ISlice())continue;
      showerNTrk++;
    }

    m_showerNTrk= showerNTrk;

    
    double vtxDoca = ShowerAna::GetPointDoca(evtVtx, svcshowercol[i]->Start(), svcshowercol[i]->Stop()); 
    double vtxToStart = (evtVtx-svcshowercol[i]->Start()).Mag(); 
    double vtxToStop = (evtVtx-svcshowercol[i]->Stop()).Mag(); 
    double vtxIntToStart = sqrt(vtxToStart*vtxToStart-vtxDoca*vtxDoca);
    double vtxIntToStop = sqrt(vtxToStop*vtxToStop-vtxDoca*vtxDoca);
    TVector3 vtxStart = svcshowercol[i]->Start();
    TVector3 vtxStop = svcshowercol[i]->Stop();
    double vtxTrkLength = (vtxStop-vtxStart).Mag(); 
    bool showerVtxBackTag = false;
    if(vtxIntToStart>vtxIntToStop) {
      showerVtxBackTag = true;
      vtxStart = svcshowercol[i]->Stop();
      vtxStop = svcshowercol[i]->Start();
      double tmp = vtxIntToStart; 
      vtxIntToStart = vtxIntToStop;
      vtxIntToStop = tmp;
    }

    if(vtxIntToStop<vtxTrkLength){
      vtxIntToStart = -vtxIntToStart;
    }

    TVector3 showerVtxVect = vtxStop-evtVtx; 
    TVector3 trueStart = ntvtxcol[i];
    TVector3 trueStop = ntvtxcol[i]+ntpcol[i].Vect();
    double showerVtxTrueDoca = ShowerAna::GetPointDoca(evtVtx, trueStart, trueStop);
    double showerVtxStartTrueDoca = ShowerAna::GetPointDoca(vtxStart, trueStart, trueStop);
    double showerVtxStopTrueDoca = ShowerAna::GetPointDoca(vtxStop, trueStart, trueStop);

    
    if(intE!=-1){
      m_showerTrueVtxEDang = showerVtxVect.Angle(evtntpcol[intE].Vect())*180/3.14159;
    }else{
      m_showerTrueVtxEDang = 0;
    }

    m_showerVtx[0] = evtVtx[0]; 
    m_showerVtx[1] = evtVtx[1];
    m_showerVtx[2] = evtVtx[2];

    m_showerVtxDoca = svcshowercol[i]->VtxDoca();
    m_showerVtxToStart = vtxToStart;
    m_showerVtxToStop = vtxToStop;
    m_showerVtxIntToStart = vtxIntToStart;
    m_showerVtxIntToStop = vtxIntToStop;
    m_showerVtxTrackLength = vtxTrkLength;

    m_showerVtxBackTag = showerVtxBackTag;
    m_showerVtxTrueDoca = showerVtxTrueDoca; 
    m_showerVtxStartTrueDoca = showerVtxStartTrueDoca; 
    m_showerVtxStopTrueDoca = showerVtxStopTrueDoca; 
    m_showerVtxTrueDang = showerVtxVect.Angle(ntpcol[i].Vect())*180/3.14159;

//  Play virtual Nodes

//    int nrminmax=0;
//    double sumrminmax=0;
//    double sumrad12=0;

/*
     std::vector<TVector3> nodetocorecol;
     std::vector<double> nodetocoreenergycol;
     nodetocorecol.clear();
     nodetocoreenergycol.clear();
     double rcntocoremax = 0.;
     double rcntocoremaxy = 0.;
//     double rcntocoremin = 9999.;
     TVector3 nodetocoremax(0,0,0);

     for(unsigned int plane=0;plane<shsvc->m_showerPClusterCol[i].size()-1;plane++){
       if(shsvc->m_showerPClusterCol[i][plane].NCell()==0)continue;
       unsigned int plane1 = shsvc->m_showerPClusterCol[i][plane].Plane();
       if(plane1==svcshowercol[i]->MaxPlane())continue;
       if(shsvc->m_showerPClusterCol[i][plane+1].NCell()==0)continue;
       unsigned int plane2 = shsvc->m_showerPClusterCol[i][plane+1].Plane();
       geo::View_t view1 = geom->Plane(plane1)->View();
       geo::View_t view2 = geom->Plane(plane2)->View();
       if(view1==view2)continue;
       TVector3 core = shsvc->GetTrkPlanePos(plane1, svcshowercol[i]);
       double rcnmax = 0.;
//       double rcnmaxy = 0.;
       TVector3 nodemax(0,0,0);
       std::vector<TVector3> planenodecol;
       std::vector<double> planenodeenergycol;
       planenodecol.clear();
       planenodeenergycol.clear();


       for (unsigned itp1 = 0;itp1<shsvc->m_showerPClusterCol[i][plane].NCell();itp1++) {
         unsigned int cell1 = shsvc->m_showerPClusterCol[i][plane].Cell(itp1)->Cell();
	 for(unsigned itp2 = 0;itp2<shsvc->m_showerPClusterCol[i][plane+1].NCell();itp2++){
           unsigned int cell2 = shsvc->m_showerPClusterCol[i][plane+1].Cell(itp2)->Cell();
           TVector3 node = ShowerAna::GetCellNodePos(plane1, cell1, plane2, cell2);
           TVector3 nodetocore = node - core;
           double rcn = nodetocore.Perp(); 
           geo::OfflineChan key1(plane1,cell1);
           geo::OfflineChan key2(plane2,cell2);
           double nodeenergy = shsvc->fHitEWeightMap[key1][i]+shsvc->fHitEWeightMap[key2][i]; 
           rcn = rcn*nodeenergy;

           if(rcn>rcnmax){
             nodemax = node; 
             rcnmax=rcn;
           }

           if(rcn>rcntocoremax){
             nodetocoremax = nodetocore;
             rcntocoremax=rcn;
           }

           planenodecol.push_back(node);
           planenodeenergycol.push_back(nodeenergy);

           nodetocorecol.push_back(nodetocore);
           nodetocoreenergycol.push_back(nodeenergy);
         }
       }
     }

     double rphi = nodetocoremax.Phi();
     TVector3 xvect(1,0,0);
     TVector3 rtest = nodetocoremax;
     rtest.RotateZ(-rphi);
     for(unsigned int inode=0;inode<nodetocorecol.size();inode++){
       TVector3 nodeinshower = nodetocorecol[inode];
       nodeinshower.RotateZ(-rphi);
       if(fabs(nodeinshower.y()*nodetocoreenergycol[inode])>rcntocoremaxy)rcntocoremaxy=fabs(nodeinshower.y()*nodetocoreenergycol[inode]);
     } 
     
     double rtocoreminmax = 0;
     if(rcntocoremax>0){
       rtocoreminmax = rcntocoremaxy/rcntocoremax;
     }
     m_showerRminmax = rtocoreminmax;
*/



//     if(ish1==-1) std::cout<<"showerEnergy > sh1gev "<<showerEnergy<<" "<<sh1gev<<std::endl;
//     if(svcshowercol[i]->ID()==999){
//       shexclpe = svcshowercol[i]->TotalPE();
//     }


     TLorentzVector ProtonP4(0, 0, 0, 0.938272);
     TLorentzVector showerTrueNuTrueLepHadP4 = evtTrueNuP4 + ProtonP4 - ntpcol[i];
     TLorentzVector showerTrueNuRecLepEHadP4 = evtTrueNuP4 + ProtonP4 - showerpGam[i];
     TLorentzVector showerTrueSumHadP4(0,0,0,0); 
     int showerTrueSumHadNtrk = 0;
     for(unsigned int ii=0;ii<ntpcol.size();ii++){
       if(ii == i)continue;
       if(fabs(ntpcol[ii].E()-ntpcol[i].E())<0.00001)continue;
       showerTrueSumHadP4+=ntnavmotherpcol[ii];
       showerTrueSumHadNtrk++;
     }

     TLorentzVector showerSumHadP4(0,0,0,0);
     for(unsigned int ii=0;ii<showerpGam.size();ii++){
       if(svcshowercol[ii]->ISlice()!=svcshowercol[i]->ISlice())continue;
       if(ii!=i){
         showerSumHadP4+=showerpGam[i];
       }
     }
     double showerP = showerpGam[i].Vect().Mag();
     double showerSumHadP=showerSumHadP4.Vect().Mag();
     double hadscale = (0.0677714+0.910447*showerSumHadP-0.0432529*showerSumHadP*showerSumHadP);
     if(showerSumHadP>10)hadscale=4.8469513;
     if(showerSumHadP<=0)hadscale=1.0;

     double showerSumHadPCor = showerSumHadP/hadscale;
     TLorentzVector showerSumP4 = showerpGam[i]+(showerSumHadP4*(1.0/hadscale));
     double showerSumTheta = showerSumP4.Angle(nuDir); 
     double showerSumPt = showerSumP4.Vect().Mag()*sin(showerSumTheta); 
     double showerSumP = showerSumP4.Vect().Mag(); 
//     std::cout<<"showerSumTheta, sinTheta, showerSumPt showerSumP "<<showerSumTheta<<" "<<sin(showerSumTheta)<<" "<<showerSumPt<<" "<<showerSumP<<std::endl;

     m_showerSumHadP = showerSumHadP;
     m_showerSumHadPCor = showerSumHadPCor;
     m_showerSumPt = showerSumPt;
     m_showerSumP = showerSumP;
     m_showerP = showerP;

     showersumhadpcol.push_back(showerSumHadP);     
     showersumptcol.push_back(showerSumPt);     
     showersumpcol.push_back(showerSumP);     
     showerpcol.push_back(showerP);     

     double Ee = showerpGam[i].E();
     double Ep = ProtonP4.E();
     double Theta = showerpGam[i].Angle(evtTrueNuP4.Vect());
     double TrueTheta = ntpcol[i].Angle(evtTrueNuP4.Vect());
     double TrueHadTheta = showerTrueSumHadP4.Angle(evtTrueNuP4.Vect());
     double Mh = showerTrueNuTrueLepHadP4.M();
     double MhTrue = Mh;
     if(evtTrueNuMode==0)MhTrue= 0.938272;
     if(evtTrueNuMode==1)MhTrue= 1.232;
     double Meff = sqrt(MhTrue*MhTrue+pow((evtTrueNuP4-ntpcol[i]).Vect().Mag(),2))-(evtTrueNuP4.E()-ntpcol[i].E());
     double showerEnue = (Mh*Mh+2*Ep*Ee)/(2*Ep-2*Ee*(1-cos(Theta)));
     double showerTrueHadEnue = (MhTrue*MhTrue+2*Ep*Ee)/(2*Ep-2*Ee*(1-cos(Theta))); 
     double showerTrueEnue = evtTrueNuP4.E(); 
     m_showerEe = Ee;
     m_showerCosTheta = cos(Theta);
     m_showerMh = Mh;
     m_showerMhTrue = MhTrue;
     m_showerMeff = Meff;
     m_showerTrueSumHadM = showerTrueSumHadP4.M();
     m_showerTrueSumHadNtrk = showerTrueSumHadNtrk;
     m_showerEnue = showerEnue;
     m_showerTruePt = (ntpcol[i]).Vect().Mag()*sin(TrueTheta);
     m_showerTruePl = (ntpcol[i]).Vect().Mag()*cos(TrueTheta);
     m_showerTrueSumHadP = evt0nthadp4tot.Vect().Mag(); 
     m_showerTrueSumHadPt = evt0nthadp4tot.Vect().Mag()*sin(TrueHadTheta);
     m_showerTrueSumHadPl = evt0nthadp4tot.Vect().Mag()*cos(TrueHadTheta);
     m_showerTrueSumHadEnue = (ntpcol[i]+showerTrueSumHadP4).Vect().Mag();
     m_showerCosTrueTheta = cos(TrueTheta);
     m_showerCosTrueHadTheta = cos(TrueHadTheta);
     m_showerTrueHadEnue = showerTrueHadEnue;
     m_showerTrueEnue = showerTrueEnue;
     fShower->Fill();
   }
//   int nGam = showerpGam.size();

   for(unsigned int i = 0; i < svcshowercol.size(); i++){
     for(int inode=0;inode<svcshowercol[i]->NNode();inode++){
//   std::cout<<"node xid "<<svcshowercol[i]->NodeXID(inode).Plane()<<" "<<svcshowercol[i]->NodeXID(inode).Cell()<<std::endl;
//   std::cout<<"node yid "<<svcshowercol[i]->NodeYID(inode).Plane()<<" "<<svcshowercol[i]->NodeYID(inode).Cell()<<std::endl;
//   std::cout<<"ncell "<<svcshowercol[i]->NCell()<<std::endl; 
       const std::vector< sim::PhotonSignal> xpSignal = bt->CellToPhotonSignal(svcshowercol[i]->NodeXID(inode).Plane(), svcshowercol[i]->NodeXID(inode).Cell());
//       const std::vector< sim::PhotonSignal> xpSignal = bt->CellToPhotonSignal(svcshowercol[i]->Cell(svcshowercol[i]->NodeXID(inode))->Plane(), svcshowercol[i]->Cell(svcshowercol[i]->NodeXID(inode))->Cell());
       const std::vector< sim::PhotonSignal> ypSignal = bt->CellToPhotonSignal(svcshowercol[i]->NodeYID(inode).Plane(), svcshowercol[i]->NodeYID(inode).Cell());
//       const std::vector< sim::PhotonSignal> ypSignal = bt->CellToPhotonSignal(svcshowercol[i]->Cell(svcshowercol[i]->NodeYID(inode))->Plane(), svcshowercol[i]->Cell(svcshowercol[i]->NodeYID(inode))->Cell());

       int nodeTotPhotons = 0;
       int nodeElecPhotons = 0;

       for( int p = 0; p < (int)xpSignal.size(); p++){
         int partID = xpSignal[p].TrackId();
         int motherID = pnav.EveId(partID);
         nodeTotPhotons += xpSignal[p].NPhoton();
         if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
           nodeElecPhotons += xpSignal[p].NPhoton();
         }
       }

       for( int p = 0; p < (int)ypSignal.size(); p++){
         int partID = ypSignal[p].TrackId();
         int motherID = pnav.EveId(partID);
         nodeTotPhotons += ypSignal[p].NPhoton();
         if( ( partID == electruetrkid ) || ( motherID == electruetrkid )){
           nodeElecPhotons += ypSignal[p].NPhoton();
         }
       }


       m_nodeTrueNuCCNC = evtTrueNuCCNC;
       m_nodeTrueNuMode = evtTrueNuMode;
       m_nodeTrueNuPdg = evtTrueNuPdg;
       m_nodeTrueNuEnergy = evtTrueNuP4.E();
       m_nodeTrueNuP4[0] = evtTrueNuP4.Px();
       m_nodeTrueNuP4[1] = evtTrueNuP4.Py();
       m_nodeTrueNuP4[2] = evtTrueNuP4.Pz();
       m_nodeTrueNuP4[3] = evtTrueNuP4.E();
       m_nodeTrueNuVtx[0] = evtTrueNuVtx[0];
       m_nodeTrueNuVtx[1] = evtTrueNuVtx[1];
       m_nodeTrueNuVtx[2] = evtTrueNuVtx[2];
       m_nodeTrueDang = ntdangcol[i];
       m_nodeTrueId = ntidcol[i];
       m_nodeTruePdg = ntpdgcol[i];
       m_nodeTrueMotherPdg = ntmotherpdgcol[i];
       m_nodeTrueNavMotherPdg = ntnavmotherpdgcol[i];
       m_nodeTrueNavMotherId = ntnavmotheridcol[i];
       m_nodeTrueNavMotherP4[0] = ntnavmotherpcol[i].Px();
       m_nodeTrueNavMotherP4[1] = ntnavmotherpcol[i].Py();
       m_nodeTrueNavMotherP4[2] = ntnavmotherpcol[i].Pz();
       m_nodeTrueNavMotherP4[3] = ntnavmotherpcol[i].E();

       m_nodeTrueTotPhotons = nodeTotPhotons;
       m_nodeTrueElecPhotons = nodeElecPhotons;


       m_nodeRun = run;
       m_nodeSubRun = srun;
       m_nodeEvent = event;
       m_nodeSubEvent = (int)sliceIdx;
       m_nodeShId = i;
       m_nodeXYZ[0] = svcshowercol[i]->NodePos(inode)[0];
       m_nodeXYZ[1] = svcshowercol[i]->NodePos(inode)[1];
       m_nodeXYZ[2] = svcshowercol[i]->NodePos(inode)[2];
       m_nodeE  = svcshowercol[i]->NodeE(inode); 
       m_nodeShowerNCell = svcshowercol[i]->NCell();
       m_nodeShowerNPlane = svcshowercol[i]->NPlane();
       m_nodeShowerLength = svcshowercol[i]->TotalLength();
       m_nodeShowerEnergy = svcshowercol[i]->Energy();
       m_nodeShowerDir[0] = (svcshowercol[i]->Dir())[0];
       m_nodeShowerDir[1] = (svcshowercol[i]->Dir())[1];
       m_nodeShowerDir[2] = (svcshowercol[i]->Dir())[2];
       m_nodeShowerStart[0]  = (svcshowercol[i]->Start())[0];
       m_nodeShowerStart[1]  = (svcshowercol[i]->Start())[1];
       m_nodeShowerStart[2]  = (svcshowercol[i]->Start())[2];
       m_nodeShowerStop[0]  = (svcshowercol[i]->Stop())[0];
       m_nodeShowerStop[1]  = (svcshowercol[i]->Stop())[1];
       m_nodeShowerStop[2]  = (svcshowercol[i]->Stop())[2];
       fNode->Fill();
     }
   }

/*
   if(nGam>=2){
     double minmpi0=9999.;
     double eg1 = 0;
     double eg2 = 0;
     int ig1 = -1;
     int ig2 = -1;
     for(int i = 0; i < nGam - 1; i++){ // fit pi0
       for(int j = i+1; j < nGam ; j++){
         TLorentzVector p2g = showerpGam[i] + showerpGam[j];
         double mgg = p2g.M();
         if(fabs(mgg - 0.135)<fabs(minmpi0-0.135)){
           minmpi0 = mgg;
           eg1 = showerpGam[i].E();
           eg2 = showerpGam[j].E();
           ig1 = i;
           ig2 = j;
         }
       }
     }
     if(ig1!=-1&&ig2!=-1){
       m_evtPi0Mgg = (showerpGam[ig1]+showerpGam[ig2]).M();
       m_evtPi0Eg1 = showerpGam[ig1].E();
       m_evtPi0Eg2 = showerpGam[ig2].E();
     }
  }
   if(ntdirshowerpGam.size()==2)
   { 
     m_evtPi0TrueDirMgg=(ntdirshowerpGam[0]+ntdirshowerpGam[1]).M();
   }

 */ 

/*
      int ish2 = -1;
      for(int i=0;i<svcshowercol.size();i++){
        if(ish1==i)continue;
//        if(showerpGam[i].E()<0.3)continue;
        TLorentzVector p2g = showerpGam[ish1] + showerpGam[i];
        double mgg = p2g.M();
        if( fabs(minSh1Pi0Mgg-0.135) > fabs(mgg-0.135)){
          minSh1Pi0Mgg = mgg;
          ish2 = i; 
        } 
      }
*/

/*
    for(unsigned int ic=0;ic<slicecol->size();ic++){// Cell information after known ish1, ishe, ishe2
      art::Ptr<rb::Cluster> clust = slicelist[ic];
      if(clust->IsNoise()){
        continue;
      }
      for(unsigned int ii=0;ii<clust->NCell();ii++){

       m_cellTruePartElec = 0;
       m_cellToSh1Dist = 0;
       m_cellToSh1Plane = -9999;
       m_cellToSheDist = 0;
       m_cellToShePlane = -9999;
       m_cellToSh2Dist = 0;
       m_cellToSh2Plane = -9999;

       m_cellSh1Dir[0] = 0; 
       m_cellSh1Dir[1] = 0; 
       m_cellSh1Dir[2] = 0; 

       m_cellSh2Dir[0] = 0; 
       m_cellSh2Dir[1] = 0; 
       m_cellSh2Dir[2] = 0; 

       m_cellSheDir[0] = 0; 
       m_cellSheDir[1] = 0; 
       m_cellSheDir[2] = 0; 

       m_cellSheTrueDang = 0; 
       m_cellSheTrueEDang = 0; 
       m_cellSheTruePdg = 0; 
       m_cellSheTrueT = 0; 
       m_cellSheTrueId = 0; 
       m_cellSheTrueVtx[0] = 0;
       m_cellSheTrueVtx[1] = 0;
       m_cellSheTrueVtx[2] = 0;
       m_cellSheTrueP4[0] = 0;
       m_cellSheTrueP4[1] = 0;
       m_cellSheTrueP4[2] = 0;
       m_cellSheTrueP4[3] = 0;
       m_cellSheTrueElecPhotons = 0; 
       m_cellSheTrueTotPhotons = 0; 
       m_cellSheTrueRWElecPhotons = 0; 
       m_cellSheTrueRWTotPhotons = 0; 



       unsigned int keym1Cell = clust->Cell(ii)->Cell();
       unsigned int keym2Cell = clust->Cell(ii)->Cell();
       unsigned int keyp1Cell = clust->Cell(ii)->Cell();
       unsigned int keyp2Cell = clust->Cell(ii)->Cell();
       if(clust->Cell(ii)->Cell()-1>=0)
         keym1Cell = clust->Cell(ii)->Cell() - 1; 
       if(clust->Cell(ii)->Cell()-2>=0)
         keym2Cell = clust->Cell(ii)->Cell() - 2;
       if(clust->Cell(ii)->Cell()+1<geom->Plane(clust->Cell(ii)->Plane())->Ncells())
         keyp1Cell = clust->Cell(ii)->Cell() + 1;
       if(clust->Cell(ii)->Cell()+2<geom->Plane(clust->Cell(ii)->Plane())->Ncells())
         keyp2Cell = clust->Cell(ii)->Cell() + 2;
       geo::OfflineChan key(clust->Cell(ii)->Plane(), clust->Cell(ii)->Cell());
       geo::OfflineChan keym1(clust->Cell(ii)->Plane(), keym1Cell);
       geo::OfflineChan keym2(clust->Cell(ii)->Plane(), keym2Cell);
       geo::OfflineChan keyp1(clust->Cell(ii)->Plane(), keyp1Cell);
       geo::OfflineChan keyp2(clust->Cell(ii)->Plane(), keyp2Cell);


       double cellXYZ[3]; 
       geom->Plane(clust->Cell(ii)->Plane())->Cell(clust->Cell(ii)->Cell())->GetCenter(cellXYZ);
       double gev = 0;
       geo::View_t view = geom->Plane(clust->Cell(ii)->Plane())->View();
       if(view == geo::kX){
         m_cellView = 0;
       }else if(view == geo::kY){
         m_cellView = 1;
       }

       m_cellRun = run;
       m_cellSubRun = srun;
       m_cellEvent = event;
       m_cellSubEvent = (int)sliceIdx;
       m_cellPlaneId = clust->Cell(ii)->Plane();
       m_cellId = clust->Cell(ii)->Cell();
       m_cellXYZ[0] = cellXYZ[0];
       m_cellXYZ[1] = cellXYZ[1];
       m_cellXYZ[2] = cellXYZ[2];
       m_cellADC  = clust->Cell(ii)->ADC();
       m_cellTDC  = clust->Cell(ii)->TDC();
       m_cellTNS  = clust->Cell(ii)->TNS();
       m_cellTrueEnergy  = trueDepositEnergy[key];
       m_cellEnergy = clust->Cell(ii)->PE();
       m_cellNSlice = nSlice;
       m_cellNTrk = 1;//svcshowercol.size(); 
       m_cellSlice = hitslicemap[key]; 

       m_cellTrueTime = 0; 
       m_cellTrueHitXYZ[0] = 0; 
       m_cellTrueHitXYZ[1] = 0; 
       m_cellTrueHitXYZ[2] = 0; 

       if(trueDepositEnergy[key]>0.000001){
         m_cellTrueTime = trueDepositTime[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[0] = trueAvgX[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[1] = trueAvgY[key]/trueDepositEnergy[key];
         m_cellTrueHitXYZ[2] = trueAvgZ[key]/trueDepositEnergy[key];
       }

       m_cellTrueNuCCNC = evtTrueNuCCNC;
       m_cellTrueNuMode = evtTrueNuMode;
       m_cellTrueNuPdg = evtTrueNuPdg;
       m_cellTrueNuEnergy = evtTrueNuP4.E();
       m_cellTrueNuP4[0] = evtTrueNuP4.Px();
       m_cellTrueNuP4[1] = evtTrueNuP4.Py();
       m_cellTrueNuP4[2] = evtTrueNuP4.Pz();
       m_cellTrueNuP4[3] = evtTrueNuP4.E();
       m_cellTrueNuVtx[0] = evtTrueNuVtx[0];
       m_cellTrueNuVtx[1] = evtTrueNuVtx[1];
       m_cellTrueNuVtx[2] = evtTrueNuVtx[2];

       if(intE!=-1){
         m_cellTrueP4[0] = evtntpcol[intE].Px();
         m_cellTrueP4[1] = evtntpcol[intE].Py();
         m_cellTrueP4[2] = evtntpcol[intE].Pz();
         m_cellTrueP4[3] = evtntpcol[intE].E();
       }

       m_cellTruePartElec = (int)truePartElec[key]; 
       if(ish1!=-1){
         m_cellSh1Dir[0] = svcshowercol[ish1]->Dir()[0];
         m_cellSh1Dir[1] = svcshowercol[ish1]->Dir()[1];
         m_cellSh1Dir[2] = svcshowercol[ish1]->Dir()[2];
         m_cellToSh1Dist = ShowerAna::GetCellDistToTrk(clust->Cell(ii)->Plane(), clust->Cell(ii)->Cell(), svcshowercol[ish1]->Start(), svcshowercol[ish1]->Stop()); 
         m_cellToSh1Plane = int(clust->Cell(ii)->Plane()) - int(svcshowercol[ish1]->MinPlane());  
       }
       if(ishe!=-1){

         m_cellSheDir[0] = svcshowercol[ishe]->Dir()[0];         
         m_cellSheDir[1] = svcshowercol[ishe]->Dir()[1];         
         m_cellSheDir[2] = svcshowercol[ishe]->Dir()[2];
         m_cellToSheDist = ShowerAna::GetCellDistToTrk(clust->Cell(ii)->Plane(), clust->Cell(ii)->Cell(), svcshowercol[ishe]->Start(), svcshowercol[ishe]->Stop()); 
         m_cellToShePlane = int(clust->Cell(ii)->Plane()) - int(svcshowercol[ishe]->MinPlane());  

         m_cellSheEnergy = svcshowercol[ishe]->Energy();
         m_cellSheTrkGeV = svcshowercol[ishe]->TrkGeV();
         m_cellSheIsFid = svcshowercol[ishe]->IsFiducial();
         m_cellSheP4[0] = showerpGam[ishe].Px();
         m_cellSheP4[1] = showerpGam[ishe].Py();
         m_cellSheP4[2] = showerpGam[ishe].Pz();
         m_cellSheP4[3] = showerpGam[ishe].E();
         m_cellSheStart[0] = svcshowercol[ishe]->Start()[0];
         m_cellSheStart[1] = svcshowercol[ishe]->Start()[1];
         m_cellSheStart[2] = svcshowercol[ishe]->Start()[2];
         m_cellSheStop[0] = svcshowercol[ishe]->Stop()[0];
         m_cellSheStop[1] = svcshowercol[ishe]->Stop()[1];
         m_cellSheStop[2] = svcshowercol[ishe]->Stop()[2];

         m_cellSheTrueDang = ntdangcol[ishe];
         m_cellSheTrueEDang = (showerpGam[ishe].Angle(electruep4.Vect()))*180/3.14159;
         m_cellSheTruePdg = ntpdgcol[ishe];
         m_cellSheTrueT = nttcol[ishe];
         m_cellSheTrueId = ntidcol[ishe];
         m_cellSheTrueVtx[0] = ntvtxcol[ishe][0];
         m_cellSheTrueVtx[1] = ntvtxcol[ishe][1];
         m_cellSheTrueVtx[2] = ntvtxcol[ishe][2];
         m_cellSheTrueP4[0] = ntpcol[ishe].Px();
         m_cellSheTrueP4[1] = ntpcol[ishe].Py();
         m_cellSheTrueP4[2] = ntpcol[ishe].Pz();
         m_cellSheTrueP4[3] = ntpcol[ishe].E();
         m_cellSheTrueElecPhotons = ntelecphotonscol[ishe];
         m_cellSheTrueTotPhotons = nttotphotonscol[ishe];
         m_cellSheTrueRWElecPhotons = ntrwelecphotonscol[ishe];
         m_cellSheTrueRWTotPhotons = ntrwtotphotonscol[ishe];
       }
       if(ish2!=-1){
         m_cellSh2Dir[0] = svcshowercol[ish2]->Dir()[0];                  
         m_cellSh2Dir[1] = svcshowercol[ish2]->Dir()[1];         
         m_cellSh2Dir[2] = svcshowercol[ish2]->Dir()[2];

         m_cellToSh2Dist = ShowerAna::GetCellDistToTrk(clust->Cell(ii)->Plane(), clust->Cell(ii)->Cell(), svcshowercol[ish2]->Start(), svcshowercol[ish2]->Stop()); 
         m_cellToSh2Plane = int(clust->Cell(ii)->Plane()) - int(svcshowercol[ish2]->MinPlane());  
       }
       fCell->Fill();
      }
    }
*/
//****************************************
// Fill event level information
//****************************************

    if(ish1!=-1){
      m_evtSh1Energy = svcshowercol[ish1]->Energy();
      m_evtSh1TNS = svcshowercol[ish1]->MeanTNS();
      m_evtSh1TrkGeV = svcshowercol[ish1]->TrkGeV();
      m_evtSh1IsFid = svcshowercol[ish1]->IsFiducial();
      m_evtSh1DistToEdge = svcshowercol[ish1]->DistToEdge();
      m_evtSh1P4[0] = showerpGam[ish1].Px();
      m_evtSh1P4[1] = showerpGam[ish1].Py();
      m_evtSh1P4[2] = showerpGam[ish1].Pz();
      m_evtSh1P4[3] = showerpGam[ish1].E();
      m_evtSh1Start[0] = svcshowercol[ish1]->Start()[0];
      m_evtSh1Start[1] = svcshowercol[ish1]->Start()[1];
      m_evtSh1Start[2] = svcshowercol[ish1]->Start()[2];
      m_evtSh1Stop[0] = svcshowercol[ish1]->Stop()[0];
      m_evtSh1Stop[1] = svcshowercol[ish1]->Stop()[1];
      m_evtSh1Stop[2] = svcshowercol[ish1]->Stop()[2];
      m_evtSh1Dir[0] = svcshowercol[ish1]->Dir()[0];
      m_evtSh1Dir[1] = svcshowercol[ish1]->Dir()[1];
      m_evtSh1Dir[2] = svcshowercol[ish1]->Dir()[2];

      m_evtSh1MinPlane = svcshowercol[ish1]->MinPlane();
      m_evtSh1MaxPlane = svcshowercol[ish1]->MaxPlane();


      m_evtSh1TrueDang = ntdangcol[ish1];
      m_evtSh1TrueEDang = (showerpGam[ish1].Angle(electruep4.Vect()))*180/3.14159;
      m_evtSh1TrueId = ntidcol[ish1];
      m_evtSh1TruePdg = ntpdgcol[ish1];
      m_evtSh1TrueT = nttcol[ish1];
      m_evtSh1TrueVtx[0] = ntvtxcol[ish1][0];
      m_evtSh1TrueVtx[1] = ntvtxcol[ish1][1];
      m_evtSh1TrueVtx[2] = ntvtxcol[ish1][2];
      m_evtSh1TrueP4[0] = ntpcol[ish1].Px();
      m_evtSh1TrueP4[1] = ntpcol[ish1].Py();
      m_evtSh1TrueP4[2] = ntpcol[ish1].Pz();
      m_evtSh1TrueP4[3] = ntpcol[ish1].E();

      m_evtSh1TrueMotherPdg = ntmotherpdgcol[ish1];
      m_evtSh1TrueMotherId = ntmotheridcol[ish1];
      m_evtSh1TrueMotherP4[0] = ntmotherpcol[ish1].Px();
      m_evtSh1TrueMotherP4[1] = ntmotherpcol[ish1].Py();
      m_evtSh1TrueMotherP4[2] = ntmotherpcol[ish1].Pz();
      m_evtSh1TrueMotherP4[3] = ntmotherpcol[ish1].E();
      m_evtSh1TrueNavMotherPdg = ntnavmotherpdgcol[ish1];
      m_evtSh1TrueNavMotherId = ntnavmotheridcol[ish1];
      m_evtSh1TrueNavMotherP4[0] = ntnavmotherpcol[ish1].Px();
      m_evtSh1TrueNavMotherP4[1] = ntnavmotherpcol[ish1].Py();
      m_evtSh1TrueNavMotherP4[2] = ntnavmotherpcol[ish1].Pz();
      m_evtSh1TrueNavMotherP4[3] = ntnavmotherpcol[ish1].E();

      m_evtSh1TrueElecPhotons = ntelecphotonscol[ish1];
      m_evtSh1TrueTotPhotons = nttotphotonscol[ish1];
      m_evtSh1TrueRWElecPhotons = ntrwelecphotonscol[ish1];
      m_evtSh1TrueRWTotPhotons = ntrwtotphotonscol[ish1];
/*
      if(track.size()==svcshowercol.size()){
        m_evtSh1TrueTrkElecPhotons = nttrkelecphotonscol[ish1];
        m_evtSh1TrueTrkTotPhotons = nttrktotphotonscol[ish1];
        m_evtSh1TrueTrkRWElecPhotons = nttrkrwelecphotonscol[ish1];
        m_evtSh1TrueTrkRWTotPhotons = nttrkrwtotphotonscol[ish1];

      }
*/
      double ellmax = -9999;
      double elllmax = -9999;
      double elltmax = -9999;

      for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
        m_evtSh1DedxLLL[itype] = svcshowercol[ish1]->DedxLongLL(itype);
        m_evtSh1DedxLLT[itype] = svcshowercol[ish1]->DedxTransLL(itype);
        if(itype == int(jmshower::kElectron)||itype == int(jmshower::kElectronQE)||itype == int(jmshower::kElectronRES)||itype == int(jmshower::kElectronDIS)||itype == int(jmshower::kElectronCOH)){
          if(ellmax<svcshowercol[ish1]->DedxLongLL(itype)+svcshowercol[ish1]->DedxTransLL(itype)){
            ellmax = svcshowercol[ish1]->DedxLongLL(itype)+svcshowercol[ish1]->DedxTransLL(itype);
            elllmax = svcshowercol[ish1]->DedxLongLL(itype);
            elltmax = svcshowercol[ish1]->DedxTransLL(itype);
          }
        }
      }

      m_evtSh1DedxLLL[0] = elllmax;
      m_evtSh1DedxLLT[0] = elltmax;

      m_evtSh1Energy0 = svcshowercol[ish1]->DepositEnergy();
      m_evtSh1Length = svcshowercol[ish1]->TotalLength();
      m_evtSh1VtxGeV = svcshowercol[ish1]->VtxGeV();

      m_evtSh1Pi0Mgg = svcshowercol[ish1]->Pi0Mgg();
      m_evtSh1SliceGeV = svcshowercol[ish1]->SliceGeV();
      m_evtSh1SliceEdge10GeV = svcshowercol[ish1]->SliceEdge10GeV();
      m_evtSh1SliceEdge5GeV = svcshowercol[ish1]->SliceEdge5GeV();
      m_evtSh1SliceEdge2GeV = svcshowercol[ish1]->SliceEdge2GeV();
      m_evtSh1Gap = svcshowercol[ish1]->Gap();

      int evtSh1NTrk = 0;
      for(unsigned int itk=0;itk<svcshowercol.size();itk++){
        if(svcshowercol[itk]->ISlice()!=svcshowercol[ish1]->ISlice())continue;
        evtSh1NTrk++;
      }

      m_evtSh1NTrk = evtSh1NTrk;



      m_evtSh1DedxANN = svcshowercol[ish1]->ANN(0);
      m_evtSh1DedxANN1 = svcshowercol[ish1]->ANN(1);
      m_evtANN = sha1ann; 

      m_evtSh1Moment12 = showerMomentCol[ish1];

      m_evtSh1Asym = svcshowercol[ish1]->Asym();
      m_evtSh1Ixx  = svcshowercol[ish1]->Ixyz()[0];
      m_evtSh1Iyy  = svcshowercol[ish1]->Ixyz()[1];

      m_evtSh1NPlane = svcshowercol[ish1]->NPlane();
      m_evtSh1NZeroPlane = showerNZeroPlaneCol[ish1];
      m_evtSh1ExtentX = svcshowercol[ish1]->ExtentX();
      m_evtSh1ExtentY = svcshowercol[ish1]->ExtentY();
      m_evtSh1NXCell = svcshowercol[ish1]->NXCell();
      m_evtSh1NYCell = svcshowercol[ish1]->NYCell();

      m_evtSh1NMIPPlane = svcshowercol[ish1]->NMIPPlane();


      TLorentzVector ProtonP4(0, 0, 0, 0.938272);
      TLorentzVector showerTrueNuTrueLepHadP4 = evtTrueNuP4 + ProtonP4 - ntpcol[ish1];
      TLorentzVector showerTrueNuRecLepEHadP4 = evtTrueNuP4 + ProtonP4 - showerpGam[ish1];
      TLorentzVector showerTrueSumHadP4(0,0,0,0); 
      int showerTrueSumHadNtrk = 0;
      for(unsigned int ii=0;ii<ntpcol.size();ii++){
        if((int)ii == ish1)continue;
        if(fabs(ntpcol[ii].E()-ntpcol[ish1].E())<0.00001)continue;
        showerTrueSumHadP4+=ntpcol[ii];
        showerTrueSumHadNtrk++;
      }

/*
      for( int ii = 0; ii < (int)plist.size(); ii++ ){//plist is wrong for cosmic
        const sim::Particle* part = plist[ii];
        if(part->Mother()!=0)continue;
        if(part->TrackId()==ntnavmotheridcol[ish1])continue;
        showerTrueSumHadP4+=ntnavmotherpcol[ii];
        showerTrueSumHadNtrk++;
      }
*/

      double Ee = showerpGam[ish1].E();
      double Ep = ProtonP4.E();
      double Theta = showerpGam[ish1].Angle(evtTrueNuP4.Vect());
      double TrueTheta = ntpcol[ish1].Angle(evtTrueNuP4.Vect());
      double TrueHadTheta = showerTrueSumHadP4.Angle(evtTrueNuP4.Vect());
      double Mh = showerTrueNuTrueLepHadP4.M();
      double MhTrue = Mh;
      if(evtTrueNuMode==0)MhTrue= 0.938272;
      if(evtTrueNuMode==1)MhTrue= 1.232;
      double Meff = sqrt(MhTrue*MhTrue+pow((evtTrueNuP4-ntpcol[ish1]).Vect().Mag(),2))-(evtTrueNuP4.E()-ntpcol[ish1].E());
      double showerEnue = (Mh*Mh+2*Ep*Ee)/(2*Ep-2*Ee*(1-cos(Theta)));
      double showerTrueHadEnue = (MhTrue*MhTrue+2*Ep*Ee)/(2*Ep-2*Ee*(1-cos(Theta))); 
      double showerTrueEnue = evtTrueNuP4.E(); 

      m_evtSh1Ee = Ee;
      m_evtSh1CosTheta = cos(Theta);
      m_evtSh1Mh = Mh;
      m_evtSh1MhTrue = MhTrue;
      m_evtSh1Meff = Meff;
      m_evtSh1TrueSumHadM = showerTrueSumHadP4.M();
      m_evtSh1TrueSumHadNtrk = showerTrueSumHadNtrk;
      m_evtSh1Enue = showerEnue;
      m_evtSh1TruePt = (ntnavmotherpcol[ish1]).Vect().Mag()*sin(TrueTheta);
      m_evtSh1TruePl = (ntnavmotherpcol[ish1]).Vect().Mag()*cos(TrueTheta);
      m_evtSh1TrueSumHadPt = (showerTrueSumHadP4).Vect().Mag()*sin(TrueHadTheta);
      m_evtSh1TrueSumHadPl = (showerTrueSumHadP4).Vect().Mag()*cos(TrueHadTheta);
      m_evtSh1TrueSumHadEnue = (ntpcol[ish1]+showerTrueSumHadP4).Vect().Mag();
      m_evtSh1CosTrueTheta = cos(TrueTheta);
      m_evtSh1CosTrueHadTheta = cos(TrueHadTheta);
      m_evtSh1TrueHadEnue = showerTrueHadEnue;
      m_evtSh1TrueEnue = showerTrueEnue;


      m_evtSh1SumHadP = showersumhadpcol[ish1];
      m_evtSh1SumPt = showersumptcol[ish1];    
      m_evtSh1SumP = showersumpcol[ish1];
      m_evtSh1P = showerpcol[ish1];   


 
      if(ish2!=-1){
        m_evtSh2Energy = svcshowercol[ish2]->Energy();
        m_evtSh2TNS = svcshowercol[ish2]->MeanTNS();
        m_evtSh2TrkGeV = svcshowercol[ish2]->TrkGeV();
        m_evtSh2IsFid = svcshowercol[ish2]->IsFiducial();
        m_evtSh2P4[0] = showerpGam[ish2].Px();
        m_evtSh2P4[1] = showerpGam[ish2].Py();
        m_evtSh2P4[2] = showerpGam[ish2].Pz();      
        m_evtSh2P4[3] = showerpGam[ish2].E();
        m_evtSh2Start[0] = svcshowercol[ish2]->Start()[0];
        m_evtSh2Start[1] = svcshowercol[ish2]->Start()[1];
        m_evtSh2Start[2] = svcshowercol[ish2]->Start()[2];
        m_evtSh2Stop[0] = svcshowercol[ish2]->Stop()[0];
        m_evtSh2Stop[1] = svcshowercol[ish2]->Stop()[1];
        m_evtSh2Stop[2] = svcshowercol[ish2]->Stop()[2];

        m_evtSh2TrueDang = ntdangcol[ish2];
        m_evtSh2TruePdg = ntpdgcol[ish2];
        m_evtSh2TrueT = nttcol[ish2];
        m_evtSh2TrueId = ntidcol[ish2];
        m_evtSh2TrueVtx[0] = ntvtxcol[ish2][0];
        m_evtSh2TrueVtx[1] = ntvtxcol[ish2][1];
        m_evtSh2TrueVtx[2] = ntvtxcol[ish2][2];
        m_evtSh2TrueP4[0] = ntpcol[ish2].Px();
        m_evtSh2TrueP4[1] = ntpcol[ish2].Py();
        m_evtSh2TrueP4[2] = ntpcol[ish2].Pz();
        m_evtSh2TrueP4[3] = ntpcol[ish2].E();

        m_evtSh2TrueMotherPdg = ntmotherpdgcol[ish2];
        m_evtSh2TrueMotherId = ntmotheridcol[ish2];
        m_evtSh2TrueMotherP4[0] = ntmotherpcol[ish2].Px();
        m_evtSh2TrueMotherP4[1] = ntmotherpcol[ish2].Py();
        m_evtSh2TrueMotherP4[2] = ntmotherpcol[ish2].Pz();
        m_evtSh2TrueMotherP4[3] = ntmotherpcol[ish2].E();
        m_evtSh2TrueNavMotherPdg = ntnavmotherpdgcol[ish2];
        m_evtSh2TrueNavMotherId = ntnavmotheridcol[ish2];
        m_evtSh2TrueNavMotherP4[0] = ntnavmotherpcol[ish2].Px();
        m_evtSh2TrueNavMotherP4[1] = ntnavmotherpcol[ish2].Py();
        m_evtSh2TrueNavMotherP4[2] = ntnavmotherpcol[ish2].Pz();
        m_evtSh2TrueNavMotherP4[3] = ntnavmotherpcol[ish2].E();


        for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
          m_evtSh2DedxLLL[itype] = svcshowercol[ish2]->DedxLongLL(itype);
          m_evtSh2DedxLLT[itype] = svcshowercol[ish2]->DedxTransLL(itype);
        }     

        m_evtSh2VtxGeV = svcshowercol[ish2]->VtxGeV();
        m_evtSh2Pi0Mgg = svcshowercol[ish2]->Pi0Mgg();
        m_evtSh2SliceGeV = svcshowercol[ish2]->SliceGeV();
        m_evtSh2Gap = svcshowercol[ish2]->Gap();
        m_evtSh2DedxANN = svcshowercol[ish2]->ANN(0);
        m_evtSh2DedxANN1 = svcshowercol[ish2]->ANN(1);



        m_evtPi0Eg1 = showerpGam[ish1].E();
        m_evtPi0Eg2 = showerpGam[ish2].E();
        m_evtPi0StartDist = (svcshowercol[ish1]->Start()-svcshowercol[ish2]->Start()).Mag();

        double x1 = svcshowercol[ish1]->Start()[0];
        double y1 = svcshowercol[ish1]->Start()[1];
        double z1 = svcshowercol[ish1]->Start()[2];
        double p1 = (svcshowercol[ish1]->Stop()[0] - svcshowercol[ish1]->Start()[0])/(svcshowercol[ish1]->Stop() - svcshowercol[ish1]->Start()).Mag();
        double q1 = (svcshowercol[ish1]->Stop()[1] - svcshowercol[ish1]->Start()[1])/(svcshowercol[ish1]->Stop() - svcshowercol[ish1]->Start()).Mag();
        double r1 = (svcshowercol[ish1]->Stop()[2] - svcshowercol[ish1]->Start()[2])/(svcshowercol[ish1]->Stop() - svcshowercol[ish1]->Start()).Mag();
        double x2 = svcshowercol[ish2]->Start()[0];
        double y2 = svcshowercol[ish2]->Start()[1];
        double z2 = svcshowercol[ish2]->Start()[2];
        double p2 = (svcshowercol[ish2]->Stop()[0] - svcshowercol[ish2]->Start()[0])/(svcshowercol[ish2]->Stop() - svcshowercol[ish2]->Start()).Mag();
        double q2 = (svcshowercol[ish2]->Stop()[1] - svcshowercol[ish2]->Start()[1])/(svcshowercol[ish2]->Stop() - svcshowercol[ish2]->Start()).Mag();
        double r2 = (svcshowercol[ish2]->Stop()[2] - svcshowercol[ish2]->Start()[2])/(svcshowercol[ish2]->Stop() - svcshowercol[ish2]->Start()).Mag();
        
        TMatrixD dNorm(3,3);
        dNorm[0][0] = x2-x1;
        dNorm[0][1] = y2-y1;
        dNorm[0][2] = z2-z1;
        dNorm[1][0] = p1;
        dNorm[1][1] = q1;
        dNorm[1][2] = r1;
        dNorm[2][0] = p2;
        dNorm[2][1] = q2;
        dNorm[2][2] = r2;

        double ma = x2-x1;
        double mb = y2-y1;
        double mc = z2-z1;
        double md = p1;
        double me = q1;
        double mf = r1;
        double mg = p2;
        double mh = q2;
        double mi = r2;

        
        TMatrixD dDeNorm1(2,2);
        TMatrixD dDeNorm2(2,2);
        TMatrixD dDeNorm3(2,2);
        dDeNorm1[0][0] = p1;
        dDeNorm1[0][1] = q1;
        dDeNorm1[1][0] = p2;
        dDeNorm1[1][1] = q2;
        double det1 = dDeNorm1[0][0]*dDeNorm1[1][1] - dDeNorm1[0][1]*dDeNorm1[1][0];
        dDeNorm2[0][0] = q1;
        dDeNorm2[0][1] = r1;
        dDeNorm2[1][0] = q2;
        dDeNorm2[1][1] = r2;
        double det2 = dDeNorm2[0][0]*dDeNorm2[1][1] - dDeNorm2[0][1]*dDeNorm2[1][0];
        dDeNorm3[0][0] = r1;
        dDeNorm3[0][1] = p1;
        dDeNorm3[1][0] = r2;
        dDeNorm3[1][1] = p2;
        double det3 = dDeNorm3[0][0]*dDeNorm3[1][1] - dDeNorm3[0][1]*dDeNorm3[1][0];
        double det0 = ma*me*mi + mb*mf*mg + mc*md*mh - mc*me*mg - mb*md*mi - ma*mf*mh;
        double pi0LineDist = fabs(det0)/sqrt(det1*det1+det2*det2+det3*det3); 
        m_evtPi0LineDist = pi0LineDist; 
      }

      if(ish3!=-1){
        m_evtSh3Energy = svcshowercol[ish3]->Energy();
        m_evtSh3TNS = svcshowercol[ish3]->MeanTNS();
        m_evtSh3TrkGeV = svcshowercol[ish3]->TrkGeV();
        m_evtSh3IsFid = svcshowercol[ish3]->IsFiducial();
        m_evtSh3P4[0] = showerpGam[ish3].Px();
        m_evtSh3P4[1] = showerpGam[ish3].Py();
        m_evtSh3P4[2] = showerpGam[ish3].Pz();      
        m_evtSh3P4[3] = showerpGam[ish3].E();
        m_evtSh3Start[0] = svcshowercol[ish3]->Start()[0];
        m_evtSh3Start[1] = svcshowercol[ish3]->Start()[1];
        m_evtSh3Start[2] = svcshowercol[ish3]->Start()[2];
        m_evtSh3Stop[0] = svcshowercol[ish3]->Stop()[0];
        m_evtSh3Stop[1] = svcshowercol[ish3]->Stop()[1];
        m_evtSh3Stop[2] = svcshowercol[ish3]->Stop()[2];

        m_evtSh3TrueDang = ntdangcol[ish3];
        m_evtSh3TruePdg = ntpdgcol[ish3];
        m_evtSh3TrueT = nttcol[ish3];
        m_evtSh3TrueId = ntidcol[ish3];
        m_evtSh3TrueVtx[0] = ntvtxcol[ish3][0];
        m_evtSh3TrueVtx[1] = ntvtxcol[ish3][1];
        m_evtSh3TrueVtx[2] = ntvtxcol[ish3][2];
        m_evtSh3TrueP4[0] = ntpcol[ish3].Px();
        m_evtSh3TrueP4[1] = ntpcol[ish3].Py();
        m_evtSh3TrueP4[2] = ntpcol[ish3].Pz();
        m_evtSh3TrueP4[3] = ntpcol[ish3].E();

        m_evtSh3TrueMotherPdg = ntmotherpdgcol[ish3];
        m_evtSh3TrueMotherId = ntmotheridcol[ish3];
        m_evtSh3TrueMotherP4[0] = ntmotherpcol[ish3].Px();
        m_evtSh3TrueMotherP4[1] = ntmotherpcol[ish3].Py();
        m_evtSh3TrueMotherP4[2] = ntmotherpcol[ish3].Pz();
        m_evtSh3TrueMotherP4[3] = ntmotherpcol[ish3].E();


        for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
          m_evtSh3DedxLLL[itype] = svcshowercol[ish3]->DedxLongLL(itype);
          m_evtSh3DedxLLT[itype] = svcshowercol[ish3]->DedxTransLL(itype);
        }     

        m_evtSh3Pi0Mgg = svcshowercol[ish3]->Pi0Mgg();
        m_evtSh3SliceGeV = svcshowercol[ish3]->SliceGeV();
        m_evtSh3Gap = svcshowercol[ish3]->Gap();
        m_evtSh3DedxANN = svcshowercol[ish3]->ANN(0);
        m_evtSh3DedxANN1 = svcshowercol[ish3]->ANN(1);
      }

    }


    if(ilen1!=-1){
      m_evtLen1DedxANN = svcshowercol[ilen1]->ANN(0);
      m_evtLen1DedxANN1 = svcshowercol[ilen1]->ANN(1);
      m_evtLen1Energy = svcshowercol[ilen1]->Energy();
      m_evtLen1TNS = svcshowercol[ilen1]->MeanTNS();
      m_evtLen1TrkGeV = svcshowercol[ilen1]->TrkGeV();
      m_evtLen1IsFid = svcshowercol[ilen1]->IsFiducial();
      m_evtLen1P4[0] = showerpGam[ilen1].Px();
      m_evtLen1P4[1] = showerpGam[ilen1].Py();
      m_evtLen1P4[2] = showerpGam[ilen1].Pz();
      m_evtLen1P4[3] = showerpGam[ilen1].E();
      m_evtLen1Start[0] = svcshowercol[ilen1]->Start()[0];
      m_evtLen1Start[1] = svcshowercol[ilen1]->Start()[1];
      m_evtLen1Start[2] = svcshowercol[ilen1]->Start()[2];
      m_evtLen1Stop[0] = svcshowercol[ilen1]->Stop()[0];
      m_evtLen1Stop[1] = svcshowercol[ilen1]->Stop()[1];
      m_evtLen1Stop[2] = svcshowercol[ilen1]->Stop()[2];


      m_evtLen1TrueDang = ntdangcol[ilen1];
      m_evtLen1TrueEDang = (showerpGam[ilen1].Angle(electruep4.Vect()))*180/3.14159;
      m_evtLen1TrueId = ntidcol[ilen1];
      m_evtLen1TruePdg = ntpdgcol[ilen1];
      m_evtLen1TrueT = nttcol[ilen1];
      m_evtLen1TrueVtx[0] = ntvtxcol[ilen1][0];
      m_evtLen1TrueVtx[1] = ntvtxcol[ilen1][1];
      m_evtLen1TrueVtx[2] = ntvtxcol[ilen1][2];
      m_evtLen1TrueP4[0] = ntpcol[ilen1].Px();
      m_evtLen1TrueP4[1] = ntpcol[ilen1].Py();
      m_evtLen1TrueP4[2] = ntpcol[ilen1].Pz();
      m_evtLen1TrueP4[3] = ntpcol[ilen1].E();

      m_evtLen1TrueMotherPdg = ntmotherpdgcol[ilen1];
      m_evtLen1TrueMotherId = ntmotheridcol[ilen1];
      m_evtLen1TrueMotherP4[0] = ntmotherpcol[ilen1].Px();
      m_evtLen1TrueMotherP4[1] = ntmotherpcol[ilen1].Py();
      m_evtLen1TrueMotherP4[2] = ntmotherpcol[ilen1].Pz();
      m_evtLen1TrueMotherP4[3] = ntmotherpcol[ilen1].E();
      m_evtLen1TrueElecPhotons = ntelecphotonscol[ilen1];
      m_evtLen1TrueTotPhotons = nttotphotonscol[ilen1];
      m_evtLen1TrueRWElecPhotons = ntrwelecphotonscol[ilen1];
      m_evtLen1TrueRWTotPhotons = ntrwtotphotonscol[ilen1];


      for( int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype)
      { //type: 0-electron, 1-photon, 2-muon, 3-pion-zero, 4-inc-had, 5-proton, 6-neutron, 7-charged-pion; 
        m_evtLen1DedxLLL[itype] = svcshowercol[ilen1]->DedxLongLL(itype);
        m_evtLen1DedxLLT[itype] = svcshowercol[ilen1]->DedxTransLL(itype);
      }

      double ellmax = -9999;
      double elllmax = -9999;
      double elltmax = -9999;

      for(int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype){
        m_evtLen1DedxLLL[itype] = svcshowercol[ilen1]->DedxLongLL(itype);
        m_evtLen1DedxLLT[itype] = svcshowercol[ilen1]->DedxTransLL(itype);
        if(itype == int(jmshower::kElectron)||itype == int(jmshower::kElectronQE)||itype == int(jmshower::kElectronRES)||itype == int(jmshower::kElectronDIS)||itype == int(jmshower::kElectronCOH)){
          if(ellmax<svcshowercol[ilen1]->DedxLongLL(itype)+svcshowercol[ilen1]->DedxTransLL(itype)){
            ellmax = svcshowercol[ilen1]->DedxLongLL(itype)+svcshowercol[ilen1]->DedxTransLL(itype);
            elllmax = svcshowercol[ilen1]->DedxLongLL(itype);
            elltmax = svcshowercol[ilen1]->DedxTransLL(itype);
          }
        }
      }
      m_evtLen1DedxLLL[0] = elllmax;
      m_evtLen1DedxLLT[0] = elltmax;

      m_evtLen1Energy0 = svcshowercol[ilen1]->DepositEnergy();
      m_evtLen1VtxGeV = svcshowercol[ilen1]->VtxGeV();
      m_evtLen1Pi0Mgg = svcshowercol[ilen1]->Pi0Mgg();
      m_evtLen1SliceGeV = svcshowercol[ilen1]->SliceGeV();
      m_evtLen1Gap = svcshowercol[ilen1]->Gap();


      m_evtLen1NPlane = svcshowercol[ilen1]->NPlane();
      m_evtLen1NMIPPlane = svcshowercol[ilen1]->NMIPPlane();
    }


    if(ishe!=-1){
      m_evtSheDedxANN = svcshowercol[ishe]->ANN(0);
      m_evtSheDedxANN1 = svcshowercol[ishe]->ANN(1);
      m_evtSheEnergy = svcshowercol[ishe]->Energy();
      m_evtSheTNS = svcshowercol[ishe]->MeanTNS();
      m_evtSheTrkGeV = svcshowercol[ishe]->TrkGeV();
      m_evtSheGap = svcshowercol[ishe]->Gap();
      m_evtSheIsFid = svcshowercol[ishe]->IsFiducial();
      m_evtSheP4[0] = showerpGam[ishe].Px();
      m_evtSheP4[1] = showerpGam[ishe].Py();
      m_evtSheP4[2] = showerpGam[ishe].Pz();
      m_evtSheP4[3] = showerpGam[ishe].E();
      m_evtSheStart[0] = svcshowercol[ishe]->Start()[0];
      m_evtSheStart[1] = svcshowercol[ishe]->Start()[1];
      m_evtSheStart[2] = svcshowercol[ishe]->Start()[2];
      m_evtSheStop[0] = svcshowercol[ishe]->Stop()[0];
      m_evtSheStop[1] = svcshowercol[ishe]->Stop()[1];
      m_evtSheStop[2] = svcshowercol[ishe]->Stop()[2];

      m_evtSheTrueDang = ntdangcol[ishe];
      m_evtSheTrueEDang = (showerpGam[ishe].Angle(electruep4.Vect()))*180/3.14159;
      m_evtSheTruePdg = ntpdgcol[ishe];
      m_evtSheTrueT = nttcol[ishe];
      m_evtSheTrueId = ntidcol[ishe];
      m_evtSheTrueVtx[0] = ntvtxcol[ishe][0];
      m_evtSheTrueVtx[1] = ntvtxcol[ishe][1];
      m_evtSheTrueVtx[2] = ntvtxcol[ishe][2];
      m_evtSheTrueP4[0] = ntpcol[ishe].Px();
      m_evtSheTrueP4[1] = ntpcol[ishe].Py();
      m_evtSheTrueP4[2] = ntpcol[ishe].Pz();
      m_evtSheTrueP4[3] = ntpcol[ishe].E();
      m_evtSheTrueElecPhotons = ntelecphotonscol[ishe];
      m_evtSheTrueTotPhotons = nttotphotonscol[ishe];
      m_evtSheTrueRWElecPhotons = ntrwelecphotonscol[ishe];
      m_evtSheTrueRWTotPhotons = ntrwtotphotonscol[ishe];

/*
      if(track.size()==svcshowercol.size()){
        m_evtSheTrueTrkElecPhotons = nttrkelecphotonscol[ishe];
        m_evtSheTrueTrkTotPhotons = nttrktotphotonscol[ishe];
        m_evtSheTrueTrkRWElecPhotons = nttrkrwelecphotonscol[ishe];
        m_evtSheTrueTrkRWTotPhotons = nttrkrwtotphotonscol[ishe];
      }
*/


      for( int itype = int(jmshower::kNull)+1; itype != int(jmshower::kLast); ++itype)
      { //type: 0-electron, 1-photon, 2-muon, 3-pion-zero, 4-inc-had, 5-proton, 6-neutron, 7-charged-pion; 
        m_evtSheDedxLLL[itype] = svcshowercol[ishe]->DedxLongLL(itype);
        m_evtSheDedxLLT[itype] = svcshowercol[ishe]->DedxTransLL(itype);
      }

      m_evtSheNPlane = svcshowercol[ishe]->NPlane();
      m_evtSheNMIPPlane = svcshowercol[ishe]->NMIPPlane();

    }

    m_evtShExclPE = 0; 
    m_evtShExclCentroid[0] = 0; 
    m_evtShExclCentroid[1] = 0; 
    m_evtShExclCentroid[2] = 0; 
    m_evtShExclRadius = 0;
/*
    if(svcexclshowercol.size()>0){
      m_evtShExclPE = svcexclshowercol[0].TotalPE();
      m_evtShExclCentroid[0] = svcexclshowercol[0].Centroid()[0];
      m_evtShExclCentroid[1] = svcexclshowercol[0].Centroid()[1];
      m_evtShExclCentroid[2] = svcexclshowercol[0].Centroid()[2];
      m_evtShExclRadius = shsvc->RadiusV(svcexclshowercol[0]);
    }
*/

    m_evtEtot = showeretot; 
    

//    m_evtPEtot = evtpetot; 
    fEvent->Fill();
}
    return; 
  }

  TVector3 ShowerAna::GetCellNodePos(unsigned int plane1, unsigned int cell1, unsigned int plane2, unsigned int cell2){
    double cellXYZ1[3]={0,0,0};
    double cellXYZ2[3]={0,0,0};
    art::ServiceHandle<geo::Geometry> geom;
    geom->Plane(plane1)->Cell(cell1)->GetCenter(cellXYZ1);
    geom->Plane(plane2)->Cell(cell2)->GetCenter(cellXYZ2);
    geo::View_t view1 = geom->Plane(plane1)->View();
    geo::View_t view2 = geom->Plane(plane2)->View();
    TVector3 nullpos(-9999,-9999,-9999);
    if(view1==view2)return nullpos;
    double x=0,y=0,z=0;
    if(view1 == geo::kX && view2 == geo::kY){
      x = cellXYZ1[0];
      y = cellXYZ2[1];
    }else if(view1 == geo::kY && view2 == geo::kX){
      x = cellXYZ2[0];
      y = cellXYZ1[1];
    }
    z = (cellXYZ1[2]+cellXYZ2[2])/2.;
    TVector3 nodepos(x,y,z);
    return nodepos; 
  }

  double ShowerAna::GetPointDoca(TVector3 vtx, TVector3 start, TVector3 stop){
    double d = 9999;
    double denorm = (start-stop).Mag();
    if(denorm<0.00001){d = (vtx-start).Mag();}else{
      d = ((vtx-start).Cross(vtx-stop)).Mag()/denorm;
    } 
    return d;
  }

  double ShowerAna::GetCellDistToTrk(unsigned int plane, unsigned int cell, TVector3 start, TVector3 stop){
    Double_t cellXYZ[3];
    art::ServiceHandle<geo::Geometry> geom;
    geom->Plane(plane)->Cell(cell)->GetCenter(cellXYZ);
    geo::View_t view = geom->Plane(plane)->View();

    double x1 = start[0];
    double y1 = start[1];
    double z1 = start[2];
    double p1 = (stop[0] - start[0])/(stop - start).Mag();
    double q1 = (stop[1] - start[1])/(stop - start).Mag();
    double r1 = (stop[2] - start[2])/(stop - start).Mag();

    double x2 = cellXYZ[0];
    double y2 = cellXYZ[1];
    double z2 = cellXYZ[2];

    double p2 = 1;
    double q2 = 0;
    double r2 = 0;
    if(view == geo::kX){
      p2 = 0;
      q2 = 1;
      r2 = 0;
    }else if(view == geo::kY){
      p2 = 1;
      q2 = 0;
      r2 = 0;
    }
    double ma = x2-x1;
    double mb = y2-y1;
    double mc = z2-z1;
    double md = p1;
    double me = q1;
    double mf = r1;
    double mg = p2;
    double mh = q2;
    double mi = r2;


    TMatrixD dDeNorm1(2,2);
    TMatrixD dDeNorm2(2,2);
    TMatrixD dDeNorm3(2,2);
    dDeNorm1[0][0] = p1;
    dDeNorm1[0][1] = q1;
    dDeNorm1[1][0] = p2;
    dDeNorm1[1][1] = q2;
    double det1 = dDeNorm1[0][0]*dDeNorm1[1][1] - dDeNorm1[0][1]*dDeNorm1[1][0];
    dDeNorm2[0][0] = q1;
    dDeNorm2[0][1] = r1;
    dDeNorm2[1][0] = q2;
    dDeNorm2[1][1] = r2;
    double det2 = dDeNorm2[0][0]*dDeNorm2[1][1] - dDeNorm2[0][1]*dDeNorm2[1][0];
    dDeNorm3[0][0] = r1;
    dDeNorm3[0][1] = p1;
    dDeNorm3[1][0] = r2;
    dDeNorm3[1][1] = p2;
    double det3 = dDeNorm3[0][0]*dDeNorm3[1][1] - dDeNorm3[0][1]*dDeNorm3[1][0];
    double det0 = ma*me*mi + mb*mf*mg + mc*md*mh - mc*me*mg - mb*md*mi - ma*mf*mh;
    double celldist = fabs(det0)/sqrt(det1*det1+det2*det2+det3*det3);
    return celldist;
  }


  unsigned int ShowerAna::GetTrkPlaneCell(unsigned int plane, art::Ptr<rb::Prong> trk){
    Double_t cellXYZ[3];
    Double_t cellXYZ1[3];
    Double_t cellXYZmax[3];
    art::ServiceHandle<geo::Geometry> geom;
    geom->Plane(plane)->Cell(0)->GetCenter(cellXYZ);
    geom->Plane(plane)->Cell(geom->Plane(plane)->Ncells()-1)->GetCenter(cellXYZ1);
    geom->Plane(plane)->Cell(geom->Plane(plane)->Ncells()-1)->GetCenter(cellXYZmax);
    double cellW = 2*geom->Plane(plane)->Cell(0)->HalfW();
    double cellwx = (cellXYZ1[0] - cellXYZ[0] + cellW)/geom->Plane(plane)->Ncells();
    double cellwy = (cellXYZ1[1] - cellXYZ[1] + cellW)/geom->Plane(plane)->Ncells();
    TVector3 trkdir=trk->Dir();
    TVector3 trkstart=trk->Start();
    TVector3 trkstop= trkstart + (trkdir * trk->TotalLength() ) ;
    double trkplanex=99999,trkplaney=99999;//, trkplanez=0;
//    double trkplanex0=99999,trkplaney0=99999;
//    double trkplanex1=99999,trkplaney1=99999;
    if(fabs(trkdir.z())>0.0001){
      trkplanex = trkstart.x()+(cellXYZ[2]-trkstart.z())*(trkdir.x()/trkdir.z());
      trkplaney = trkstart.y()+(cellXYZ[2]-trkstart.z())*(trkdir.y()/trkdir.z());
//      trkplanex0 = cellXYZ[0];
//      trkplaney0 = cellXYZ[1];
//      trkplanex1 = cellXYZmax[0];
//      trkplaney1 = cellXYZmax[1];
    }else{
      trkplanex = (trkstart.x()+ trkstop.x())/2.0;
      trkplaney = (trkstart.y()+ trkstop.y())/2.0;
    }
//    trkplanez = cellXYZ[2];
    geo::View_t view = geom->Plane(plane)->View();

/*
    if(view == geo::kX){
     std::cout<<"x view plane"<<std::endl;
    std::cout<<"x1-x0,cellW "<<" "<<cellXYZ1[0] - cellXYZ[0]<<" "<<cellW<<std::endl;
    }else if(view == geo::kY){
     std::cout<<"y view plane"<<std::endl;
    std::cout<<"y1-y0,cellW "<<" "<<cellXYZ1[1] - cellXYZ[1]<<" "<<cellW<<std::endl;
    }
*/

    int trkcellId = -9999;
//    int trkcellId0 = -9999;
//    int trkcellId1 = -9999;
//    int trkcellIdmax = geom->Plane(plane)->Ncells()-1;

    if(view == geo::kX)trkcellId = int((trkplanex+cellwx/2-cellXYZ[0])/cellwx);
    if(view == geo::kY)trkcellId = int((trkplaney+cellwy/2-cellXYZ[1])/cellwy);

//    if(view == geo::kX)trkcellId0 = int((trkplanex0+cellwx/2-cellXYZ[0])/cellwx);
//    if(view == geo::kY)trkcellId0 = int((trkplaney0+cellwy/2-cellXYZ[1])/cellwy);
//
//    if(view == geo::kX)trkcellId1 = int((trkplanex1+cellwx/2-cellXYZ[0])/cellwx);
//    if(view == geo::kY)trkcellId1 = int((trkplaney1+cellwy/2-cellXYZ[1])/cellwy);
    return unsigned(trkcellId);
  }
  
  std::vector<int> ShowerAna::getFaveIds(std::vector<cheat::NeutrinoWithIndex>& allNus, 
                                        std::vector< std::vector<cheat::NeutrinoEffPur> >& subevtNus)
  {
    // This method associates each subevt with its favorite neutrino. 
    // The first argument is all the neutrinos, the second has an vector for each subevt (in the same order)
    // The inner vectors contain all the neutrinos that made hits in the subevt
    // with efficiency and purity calculated by backtracker.  
    
    
    // We'll make an array with an entry for every subevt to store the unique identifier of 
    // the favourite neutrino within the subevt, with the correct size and all entries initialized to -1
    std::vector<int> faveIds(subevtNus.size(), -1);
    
    //loop over subevts
    // loop over nus within the subevts to find the one with the highest "metric"
    // if that slice has the highest "metric" value (efficiency for now)
    //store its uniqueId in the array.
    for(unsigned int iSubvt = 0; iSubvt < subevtNus.size(); ++iSubvt)
    {
      //for the moment we can just take the first neutrino in each subevt's vectors, 
      //since these will be sorted by efficiency.  
      // Later there might be another metric.
      if(subevtNus[iSubvt].size() > 0) {
        faveIds[iSubvt] = subevtNus[iSubvt][0].truthColIndex;
      }
    }
    
    
    // then loop over nus
    //  if a nuUniqueId exists more than once, 
    //determine which subevt has the highest efficiency
    // restore the other subevt entries in faveIds to -1.
    for(unsigned int iNu = 0; iNu < allNus.size(); ++iNu){
      std::vector<int> associatedSubevts;
      // loop over faveIds to find out how many times each nu was stored.
      for(unsigned int iSubvt = 0; iSubvt < faveIds.size(); ++iSubvt)
      {
        if((int)allNus[iNu].truthColIndex == faveIds[iSubvt])
        {
          associatedSubevts.push_back(iSubvt);
        }
        
      }
      
      
      if(associatedSubevts.size() > 1){// then there are subevts with the same faveNu.
        float maxEff = 0;
        int bestSubevt = -2; // -2 for debugging.
        // loop over all associatedSubevts to determine which has hightest efficiency        
        for(unsigned int iAssc = 0; iAssc < associatedSubevts.size(); ++iAssc){
          if(subevtNus[associatedSubevts[iAssc]][0].efficiency > maxEff){
            maxEff = subevtNus[associatedSubevts[iAssc]][0].efficiency;
            bestSubevt = associatedSubevts[iAssc];
          }//end if
        }// end for(iAssc ... )
        assert(bestSubevt >= 0 && "Best subevt not found"); 
        for(unsigned int iAssc = 0; iAssc < associatedSubevts.size(); ++iAssc){
          if(associatedSubevts[iAssc] != bestSubevt){ 
            // then this has been incorrectly identified.
            faveIds[associatedSubevts[iAssc]] = -1; 
          }//end if
        }// end for(iAssc ... )
        
      }// end if(#asscSubevts > 1)
      
    }
    
    return faveIds;
  }

}
 // namespace showerana 
//......................................................................

namespace showerana{
  DEFINE_ART_MODULE(ShowerAna);
}

